package sota;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.StreamTokenizer;

/**
 * 
 * bernimmt die Logging-Funktionalitt whrend der Testlufe fr Java-Programme.
 * 
 * Spezifikation des Logfiles
 * 
 * <PRE>
 * Block: 1. 00xxxxxx xxxxxxxx xxxxxxxx xxxxxxxx -&gt; 30bit-Integer codiert PathID (max &gt; 1 Milliarde)
 * 		  2. 01xxxxxx xxxxxxxx xxxxxxxx xxxxxxxx -&gt; 30bit-Integer codiert ID (max &gt; 1 Milliarden) 
 *      [ 3. 10yyyyyy xxxxxxxx xxxxxxxx xxxxxxxx -&gt; 6bit kodiert #Boolatome, 24bit Boolatome; false = 0, true = 1 
 *      [    xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx weitere 32bit Boolatome ] ] 
 *      [ 4. 11xxxxxx xxxxxxxx xxxxxxxx xxxxxxxx -&gt; noch undefiniert ]
 * 
 * 0xC0 = 11000000 
 * 0x80 = 10000000 
 * 0x40 = 01000000
 * </PRE>
 */
public class ASCLogger {

	private static boolean VERBOSE = false;

	private static boolean INFO = true;

	private static java.io.FileOutputStream os = null;

	// private static String ASCLogFile = "log";

	private static String iniFile = "ASCLogger.ini";
	private static String projectName = "";
	private static String testName = "";
	private static String testDesc = "";
	private static String iSchemeName = "";
	private static int timeStamp = 0;
	private static int nrBooleans = 0;
	private static int booleans = 0;
	private static boolean fileExists;
	private static boolean badFileName = false;

	private final static int PROJECT = 0;
	private final static int TESTCASE = 1;
	private final static int ISCHEME = 2;
	private final static int DESC = 3;

	/**
	 * Initalisierung des Loggers beim ersten Aufruf der Logfunktionen.
	 */
	private static void init() {
		int attribute = PROJECT;
		fileExists = false;

		if (INFO) {
			System.out.println("\n ASCLogger initialisation");
			System.out.println("\t Reading " + iniFile + " from " + (new File("").getAbsolutePath()));
		}
		try {
			StreamTokenizer tokenizer = new StreamTokenizer(new FileReader(iniFile));

			// solange nicht EOF
			while (tokenizer.nextToken() != StreamTokenizer.TT_EOF) {
				switch (tokenizer.ttype) {
				case StreamTokenizer.TT_WORD:
					if (tokenizer.sval.equalsIgnoreCase("project"))
						attribute = PROJECT;
					else if (tokenizer.sval.equalsIgnoreCase("testcase"))
						attribute = TESTCASE;
					else if (tokenizer.sval.equalsIgnoreCase("ischeme"))
						attribute = ISCHEME;
					else if (tokenizer.sval.equalsIgnoreCase("description"))
						attribute = DESC;
					break;
				case '=':
				case ',':
					break;
				case '"':
					inner: switch (attribute) {
					case PROJECT:
						projectName = tokenizer.sval;
						break inner;
					case TESTCASE:
						testName = tokenizer.sval;
						break inner;
					case ISCHEME:
						iSchemeName = tokenizer.sval;
						break inner;
					case DESC:
						testDesc = tokenizer.sval;
						break inner;
					}
				}
			}
			if (projectName.equals("") || testName.equals("") || iSchemeName.equals("")) {
				if (INFO)
					System.out.println("\n\t Ini-File corrupt!");
				return;
			}

		} catch (IOException e) {
			e.printStackTrace();
			if (INFO)
				System.out.println("\n\t Failed to read " + iniFile + "!");
			return;
		}

		if (INFO) {
			System.out.println("\t\t Project:  " + projectName);
			System.out.println("\t\t Testname: " + testName);
			System.out.println("\t\t Description: " + testDesc);
			System.out.println("\t\t IScheme:  " + iSchemeName);
			System.out.println("\t Opening Testlog " + (new File(testName + ".log")).getAbsoluteFile());
		}

		try {
			if (new File(testName + ".log").exists()) {
				System.out.println("\t Testlog exists! Appending new log data...");
				fileExists = true;
			}
			os = new java.io.FileOutputStream(testName + ".log", true);

		} catch (java.io.FileNotFoundException e) {
			badFileName = true;
			System.out.println("\n\t " + (new File(testName + ".log")).getAbsoluteFile() + " couldn't be created!");
			e.printStackTrace();
			return;
		}
		if (fileExists)
			log(0);
		log(projectName.getBytes().length);
		// System.out.println(iSchemeName.getBytes().length);
		log(projectName.getBytes());

		log(testDesc.getBytes().length);
		// System.out.println(iSchemeName.getBytes().length);
		log(testDesc.getBytes());

		log(iSchemeName.getBytes().length);
		// System.out.println(iSchemeName.getBytes().length);
		log(iSchemeName.getBytes());
	}

	/**
	 * Initalisiert das Logfile.
	 * 
	 * @param logFile
	 *            das Logfile
	 */
	private static void init(String logFile) {
		try {
			os = new java.io.FileOutputStream(logFile);
		} catch (java.io.FileNotFoundException e) {
			System.out.println("ASCLogger  File " + logFile + " couldn't be created.");
			return;
		}
	}

	/**
	 * Gibt eine Zeitstempel zurck, d.h. die ID fr Pfade.
	 * 
	 * @return der Int-Wert
	 */
	public static int getTimeStamp() {
		return ++timeStamp;
	}

	/**
	 * Loggt ein Integer.
	 * 
	 * @param a
	 *            das Integer
	 */
	public static void log(int a) {
		if (VERBOSE) {
			System.out.println(a);
		}
		log(int2byte(a));
	}

	/**
	 * Loggt ein ID mit Zeitstempel.
	 * 
	 * @param a
	 *            die ID
	 * @param b
	 *            der Zeitstempel
	 */
	public static void log(int a, int b) {
		if (VERBOSE)
			System.out.println("Log(" + a + ", " + b + ")");
		if (nrBooleans != 0)
			logBooleans();
		if (VERBOSE) {
			System.out.println(a + " (" + b + ")");
		}
		log(int2byte(b)); // 00 + PathID
		a |= (0x40 << 24); // setzt Headerbits auf 01
		log(int2byte(a)); // 01 + ID
	}

	// 10yyyyyy xxxxxxxxxx xxxxxxx9 87654321 <- boolwerte werden nach links
	// aufgefllt
	/**
	 * Speichert ein Booleanwert zwischen.
	 * 
	 * @param b
	 *            der Booleanwert
	 */
	public static void log(boolean b) {
		if (VERBOSE)
			System.out.println("new Boolean " + b + " (" + (nrBooleans + 1) + ". Boolean)");
		// System.out.println("Logging: " + b);
		if (b)
			booleans |= (1 << nrBooleans);
		nrBooleans++;
	}

	/**
	 * Loggt die zwischengespeicherten Booleans.
	 */
	public static void logBooleans() {
		if (VERBOSE)
			System.out.println("LogBooleans " + nrBooleans + " (" + booleans + ")");
		booleans |= (nrBooleans << 24);
		booleans |= 0x80000000;
		log(int2byte(booleans));
		booleans = 0;
		nrBooleans = 0;
	}

	/**
	 * Schreibt ein Byte-Array in das Logfile.
	 * 
	 * @param b
	 *            das Byte-Array
	 */
	private static void log(byte[] b) {
		if (badFileName)
			return;
		if (os == null) {
			init();
		}
		try {
			os.write(b);
		} catch (java.io.IOException e) {
			return;
		}
	}

	/**
	 * Wandelt Integer in ein Bytearray um.
	 * 
	 * @param a
	 *            das Integer
	 * @return der Byte-Array
	 */
	private static byte[] int2byte(int a) {
		byte b[] = new byte[4];
		for (int i = 0; i < 4; i++) {
			b[i] = (byte) (a & 0x000000FF);
			a = a >>> 8;
		}
		/*
		 * if (VERBOSE) System.out.println("\t Int " + s + " -> " + b[0] + " " + b[1] + " " + b[2] + " " + b[3]);
		 */
		return b;
	}

	/**
	 * Mainmethode, zum Testen.
	 * 
	 * @param args
	 */
	public static void main(String args[]) {
		init("test.log");
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("0"))
				log(false);
			else if (args[i].equals("1"))
				log(true);
			else
				log(1, 15);
		}
		logBooleans();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		if (nrBooleans != 0)
			logBooleans();
		super.finalize();
	}

}
