package sota.app;

import java.io.PipedInputStream;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.StatusLineContributionItem;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.text.Document;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferenceConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;

import sota.ast.util.ASTManager;
import sota.ast.util.ColorManager;
import sota.ast.util.ProjectConfig;

/**
 * This class controls all aspects of the application's execution
 */
public class Application extends AbstractUIPlugin implements IApplication {

	private static Application instance = null;

	private static ASTManager ast;

	public static final String PLUGIN_ID = "SOTA";

	private ColorManager colorManager;

	// private static FontRegistry fontRegistry = null;

	public static PipedInputStream in;

	private static Document fileDocument = null;

	private IStatusLineManager statusLineManager = null;

	/**
	 * Konstruktor des Singleton.
	 */
	public Application() {
		super();
		colorManager = new ColorManager();
		ast = new ASTManager(new ProjectConfig());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.equinox.app.IApplication#start(org.eclipse.equinox.app.IApplicationContext)
	 */
	public Object start(IApplicationContext context) {
		Display display = PlatformUI.createDisplay();
		PlatformUI.getPreferenceStore().setValue(IWorkbenchPreferenceConstants.SHOW_TRADITIONAL_STYLE_TABS, false);

		try {
			int returnCode = PlatformUI.createAndRunWorkbench(display, new ApplicationWorkbenchAdvisor());
			if (returnCode == PlatformUI.RETURN_RESTART) {
				return IApplication.EXIT_RESTART;
			}
			return IApplication.EXIT_OK;
		} finally {
			display.dispose();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.equinox.app.IApplication#stop()
	 */
	public void stop() {
		final IWorkbench workbench = PlatformUI.getWorkbench();
		if (workbench == null)
			return;
		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable() {
			public void run() {
				if (!display.isDisposed())
					workbench.close();
			}

		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeImageRegistry(org.eclipse.jface.resource.ImageRegistry)
	 */
	public void initializeImageRegistry(ImageRegistry reg) {
		Bundle bundle = Platform.getBundle(PLUGIN_ID);

		final String[] imageArray = {

				// Menus
				"antbuild", "icons/antFileIcon.gif", "cfg", "icons/cfgIcon.gif", "collapseall", "icons/minusIcon.gif",
				"configure", "icons/configureIcon.gif", "coverage", "icons/showCoverageIcon.gif", "createreport",
				"icons/reportIcon.gif", "delete", "icons/deleteIcon.gif", "expandall", "icons/plusIcon.gif", "flat",
				"icons/flatLayout.gif", "hierarchical", "icons/hierarchicalLayout.gif", "lock", "icons/locked.gif",
				"newproject", "icons/wizardIcon.gif",
				"newischeme",
				"icons/newIschemeIcon.gif",
				"manual",
				"icons/manualIcon.gif",
				"openproject",
				"icons/folderIcon.gif",
				"percent",
				"icons/percentIcon.gif",
				"pin",
				"icons/pinIcon.gif",
				"readlogs",
				"icons/readLogIcon.gif",
				"restarttest",
				"icons/restartTestIcon.gif",
				"restore",
				"icons/refreshIcon.gif",
				// "icons/refreshY.gif",
				"runproject", "icons/runScriptIcon.gif", "runargs", "icons/runScriptArgsIcon.gif", "saveproject", "icons/saveIcon.gif", "shownumbers",
				"icons/showNumbersIcon.gif", "sort", "icons/sortIcon.gif",
				"starttest",
				"icons/startTestIcon.gif",
				"stoptest",
				"icons/stopTestIcon.gif",
				"testview",
				"icons/testLogViewIcon.gif",
				"update",
				"icons/updateIcon.gif",
				"zoomin",
				"icons/zoomInIcon.gif",
				"zoomout",
				"icons/zoomOutIcon.gif",

				// Items
				"badlog", "icons/badLog.gif", "class", "icons/classIcon.gif", "dirty", "icons/dirtyFileIcon.gif",
				"djfile", "icons/dirtyjfile.gif", "dclass", "icons/dirtyClassIcon.gif", "diclass",
				"icons/dirtyInnerClassIcon.gif", "dpubmethod", "icons/dirtyPubMethodIcon.gif", "dprotmethod",
				"icons/dirtyProtMethodIcon.gif", "dprivmethod", "icons/dirtyPrivMethodIcon.gif", "file",
				"icons/fileIcon.gif", "jfile", "icons/jfileIcon.gif", "folder", "icons/folderIcon.gif", "function",
				"icons/functionIcon.gif", "ischeme", "icons/ischemeIcon.gif", "innerclass", "icons/innerClassIcon.gif",
				"filelvl0", "icons/filelvl0Icon.gif", "lvl0", "icons/lvl0Icon.gif", "lvl1", "icons/lvl1Icon.gif",
				"lvl2", "icons/lvl2Icon.gif", "lvl3", "icons/lvl3Icon.gif", "method", "icons/methodIcon.gif",
				"package", "icons/packageIcon.gif", "privatemethod", "icons/privMethodIcon.gif", "protectedmethod",
				"icons/protMethodIcon.gif", "publicmethod", "icons/pubMethodIcon.gif", "testlog",
				"icons/testLogIcon.gif", "testlog2", "icons/testLogIcon2.gif",

				"lvl2pubm", "icons/pubMlvl2Icon.gif", "lvl2prim", "icons/priMlvl2Icon.gif", "lvl2prom",
				"icons/proMlvl2Icon.gif", };

		for (int i = 0; i < imageArray.length; i++)
			reg.put(imageArray[i++], ImageDescriptor.createFromURL(FileLocator.find(bundle, new Path(imageArray[i]),
					null)));
	}

	/**
	 * Entscheidet, ob das Singleton instanziert wurde.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public static boolean hasInstance() {
		return instance != null;
	}

	/**
	 * Gibt die Instanz des Singletons zurck.
	 * 
	 * @return die Instanz
	 */
	public static Application getInstance() {
		return instance;
	}

	/**
	 * Erzeugt eine neue Instanz.
	 */
	public static void createInstance() {
		if (instance == null) {
			instance = new Application();
		}
	}

	/**
	 * Gibt den ASTManager zurck.
	 * 
	 * @return der ASTManager
	 */
	public ASTManager getASTManager() {
		return ast;
	}

	/**
	 * Gibt die aktuelle Konfiguration zurck.
	 * 
	 * @return die Konfiguration
	 */
	public ProjectConfig getConfiguration() {
		return ast.getConfiguration();
	}

	/**
	 * Gibt den ColorManager zurck.
	 * 
	 * @return der ColorManager
	 */
	public ColorManager getColorManager() {
		return colorManager;
	}

	/**
	 * Gibt die Farbe zu einer ID des PreferenceStores zurck.
	 * 
	 * @param s
	 *            der String
	 * @return die Farbe die Farbe
	 */
	static public Color getColorFromPreferenceStore(String s) {
		return getInstance().getColorManager().getColor(
				PreferenceConverter.getColor(PlatformUI.getPreferenceStore(), s));
	}

	/**
	 * Gibt die Farbe zu einem RGB-Wert zurck.
	 * 
	 * @param s
	 *            der RGB-Wert
	 * @return die Farbe
	 */
	static public Color getColor(RGB s) {
		return getInstance().getColorManager().getColor(s);
	}

	/**
	 * Setzt das aktuelle Dokument fr die Quellcodedarstellung.
	 * 
	 * @param fileDocument
	 *            das Dokument
	 */
	public static void setFileDocument(Document fileDocument) {
		Application.fileDocument = fileDocument;
	}

	/**
	 * Gibt das aktuelle Dokument fr die Quellcodedarstellung zurck.
	 * 
	 * @return das Dokument
	 */
	public static Document getFileDocument() {
		return fileDocument;
	}

	/**
	 * Fhrt ein Update der Statuszeile durch.
	 * 
	 * @param message
	 *            die Nachricht (ungenutzt)
	 * @param isError
	 *            true, wenn Fehler, sonst false (ungenutzt)
	 */
	public void updateStatusLine(String message, boolean isError) {

		if (getWorkbench() == null)
			return;
		ProjectConfig config = getConfiguration();
		if (statusLineManager == null) {
			IActionBars ab = ((IViewSite) getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart()
					.getSite()).getActionBars();
			if (ab != null) {
				statusLineManager = ab.getStatusLineManager();
			}
		}

		if (statusLineManager != null) {
			((StatusLineContributionItem) statusLineManager.find("Sourcecode")).setText("Status of source code: "
					+ (config.sourceStatus == ProjectConfig.DIRTY ? "  DIRTY" : "  CLEAN"));
			((StatusLineContributionItem) statusLineManager.find("Parsedcode")).setText("Status of parsed code: "
					+ (config.parsedSourceStatus == ProjectConfig.DIRTY ? "  DIRTY" : "  CLEAN"));

		}
	}
}
