package sota.app;

import java.util.ArrayList;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.StatusLineContributionItem;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

import sota.gui.action.BuildProjectAction;
import sota.gui.action.CloseProjectAction;
import sota.gui.action.ConfigureProjectAction;
import sota.gui.action.CreateReportAction;
import sota.gui.action.HelpAction;
import sota.gui.action.NewISchemeAction;
import sota.gui.action.NewProjectAction;
import sota.gui.action.OpenProjectAction;
import sota.gui.action.ReadLogsAction;
import sota.gui.action.RecoverProjectAction;
import sota.gui.action.RestoreSourcesAction;
import sota.gui.action.RunDropDownAction;
import sota.gui.action.RunProjectAction;
import sota.gui.action.RunProjectArgsAction;
import sota.gui.action.SaveProjectAction;
import sota.gui.action.SelectSourcesAction;
import sota.gui.action.ShowCoverageAction;
import sota.gui.action.StartTestAction;
import sota.gui.action.StopTestAction;
import sota.gui.action.SwitchPerspectiveAction;

/**
 * An action bar advisor is responsible for creating, adding, and disposing of the actions added to a workbench window.
 * Each window will be populated with new actions.
 */
public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	private IWorkbenchAction aboutAction;

	private IWorkbenchAction exitAction;

	private IWorkbenchAction newProjectAction;

	private IWorkbenchAction selectSourcesAction;

	private IWorkbenchAction repairProjectAction;

	private IWorkbenchAction readLogsAction;

	private IWorkbenchAction restoreProjectAction;

	private IWorkbenchAction newInstrumentationSchemeAction;

	private IWorkbenchAction openProjectAction;

	private IWorkbenchAction saveProjectAction;

	private IWorkbenchAction closeProjectAction;

	private IWorkbenchAction startTestAction;

	private IWorkbenchAction buildProjectAction;

	private IWorkbenchAction showCoverageAction;

	private IWorkbenchAction runProjectAction;
	
	private IWorkbenchAction runProjectArgsAction;
	
	private IAction runDropDownAction;

	private IWorkbenchAction createReportAction;

	private IWorkbenchAction preferenceAction;

	private IWorkbenchAction configureProjectAction;

	private IWorkbenchAction stopTestAction;

	private IWorkbenchAction helpAction;

	private IWorkbenchAction switchPerspectiveAction;

	private ArrayList<IWorkbenchAction> actionList;

	private boolean ENABLE_ALL = false;

	/**
	 * Der Konstruktor des Advisors.
	 * 
	 * @param configurer
	 *            der Configurer
	 */
	public ApplicationActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
		actionList = new ArrayList<IWorkbenchAction>();
	}

	
	
	
	protected void makeActions(final IWorkbenchWindow window) {

		// Actions des File-Mens
		exitAction = ActionFactory.QUIT.create(window);
		register(exitAction);
		aboutAction = ActionFactory.ABOUT.create(window);
		register(aboutAction);
		helpAction = new HelpAction(window);
		register(helpAction);
		switchPerspectiveAction = new SwitchPerspectiveAction(window);
		register(switchPerspectiveAction);
		newProjectAction = new NewProjectAction(window);
		register(newProjectAction);
		selectSourcesAction = new SelectSourcesAction(window);
		register(selectSourcesAction);

		// Actions des Tasks-Men
		repairProjectAction = new RecoverProjectAction(window);
		register(repairProjectAction);
		readLogsAction = new ReadLogsAction(window);
		register(readLogsAction);
		restoreProjectAction = new RestoreSourcesAction(window);
		register(restoreProjectAction);
		newInstrumentationSchemeAction = new NewISchemeAction(window);
		register(newInstrumentationSchemeAction);
		saveProjectAction = new SaveProjectAction();
		register(saveProjectAction);
		openProjectAction = new OpenProjectAction(window);
		register(openProjectAction);
		closeProjectAction = new CloseProjectAction(window);
		register(closeProjectAction);

		startTestAction = new StartTestAction(window);
		register(startTestAction);
		stopTestAction = new StopTestAction(window);
		register(stopTestAction);
		buildProjectAction = new BuildProjectAction(window);
		register(buildProjectAction);
		runProjectAction = new RunProjectAction(window);
		register(runProjectAction);
		runProjectArgsAction = new RunProjectArgsAction(window);
		register(runProjectArgsAction);
		
		runDropDownAction = new RunDropDownAction(runProjectAction,runProjectArgsAction) ;
		register(runDropDownAction);
		
		((RunProjectAction)runProjectAction).setDropDownAction((RunDropDownAction) runDropDownAction);
		((RunProjectArgsAction)runProjectArgsAction).setDropDownAction((RunDropDownAction) runDropDownAction);
		
		showCoverageAction = new ShowCoverageAction(window);
		register(showCoverageAction);

		createReportAction = new CreateReportAction(window);
		register(createReportAction);

		// Actions des Config-Mens
		preferenceAction = ActionFactory.PREFERENCES.create(window);
		register(preferenceAction);

		configureProjectAction = new ConfigureProjectAction(window);
		register(configureProjectAction);

		actionList.add(aboutAction);
		actionList.add(exitAction);
		actionList.add(newProjectAction);
		actionList.add(selectSourcesAction);
		actionList.add(repairProjectAction);
		actionList.add(readLogsAction);
		actionList.add(restoreProjectAction);
		actionList.add(newInstrumentationSchemeAction);
		actionList.add(openProjectAction);
		actionList.add(saveProjectAction);
		actionList.add(closeProjectAction);
		actionList.add(startTestAction);
		actionList.add(buildProjectAction);
		actionList.add(showCoverageAction);
		actionList.add(runProjectAction);
		actionList.add(createReportAction);
		actionList.add(preferenceAction);
		actionList.add(configureProjectAction);
		actionList.add(stopTestAction);
		actionList.add(helpAction);
		actionList.add(switchPerspectiveAction);


		if (ENABLE_ALL) {
			aboutAction.setEnabled(true);
			exitAction.setEnabled(true);
			newProjectAction.setEnabled(true);
			selectSourcesAction.setEnabled(true);
			repairProjectAction.setEnabled(true);
			readLogsAction.setEnabled(true);
			restoreProjectAction.setEnabled(true);
			newInstrumentationSchemeAction.setEnabled(true);
			openProjectAction.setEnabled(true);
			saveProjectAction.setEnabled(true);
			closeProjectAction.setEnabled(true);
			startTestAction.setEnabled(true);
			buildProjectAction.setEnabled(true);
			showCoverageAction.setEnabled(true);
			runProjectAction.setEnabled(true);
			createReportAction.setEnabled(true);
			preferenceAction.setEnabled(true);
			configureProjectAction.setEnabled(true);
			stopTestAction.setEnabled(true);
			helpAction.setEnabled(true);
			switchPerspectiveAction.setEnabled(true);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.ActionBarAdvisor#fillMenuBar(org.eclipse.jface.action.IMenuManager)
	 */
	protected void fillMenuBar(IMenuManager menuBar) {
		MenuManager fileMenu = new MenuManager("&Project", IWorkbenchActionConstants.M_FILE);
		fileMenu.add(newProjectAction);
		fileMenu.add(openProjectAction);
		fileMenu.add(saveProjectAction);
		fileMenu.add(closeProjectAction);
		fileMenu.add(new Separator());
		fileMenu.add(repairProjectAction);
		fileMenu.add(new Separator());
		fileMenu.add(exitAction);

		MenuManager launchMenu = new MenuManager("&Tasks", IWorkbenchActionConstants.M_LAUNCH);
		launchMenu.add(newInstrumentationSchemeAction);
		launchMenu.add(new Separator());
		launchMenu.add(startTestAction);
		launchMenu.add(stopTestAction);
		launchMenu.add(createReportAction);
		launchMenu.add(new Separator());
		launchMenu.add(buildProjectAction);
		//launchMenu.add(runProjectAction);
		launchMenu.add(runDropDownAction);
		launchMenu.add(restoreProjectAction);
		launchMenu.add(readLogsAction);
		launchMenu.add(new Separator());
		launchMenu.add(showCoverageAction);

		MenuManager preferencesMenu = new MenuManager("&Configuration", "configuration");

		preferencesMenu.add(configureProjectAction);
		preferencesMenu.add(selectSourcesAction);
		preferencesMenu.add(new Separator());
		preferencesMenu.add(preferenceAction);
		preferencesMenu.add(new Separator());
		preferencesMenu.add(switchPerspectiveAction);

		MenuManager helpMenu = new MenuManager("&Help", "help");
		helpMenu.add(helpAction);
		helpMenu.add(new Separator());
		helpMenu.add(aboutAction);

		menuBar.add(fileMenu);
		menuBar.add(launchMenu);
		menuBar.add(preferencesMenu);
		menuBar.add(helpMenu);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.ActionBarAdvisor#fillCoolBar(org.eclipse.jface.action.ICoolBarManager)
	 */
	protected void fillCoolBar(ICoolBarManager mgr) {
		super.fillCoolBar(mgr);

		IToolBarManager toolbar = new ToolBarManager(mgr.getStyle());
		mgr.add(toolbar);
		toolbar.add(newProjectAction);
		toolbar.add(openProjectAction);
		toolbar.add(saveProjectAction);
		toolbar.add(new Separator());
		toolbar.add(newInstrumentationSchemeAction);
		toolbar.add(new Separator());
		toolbar.add(startTestAction);
		toolbar.add(stopTestAction);
		toolbar.add(createReportAction);
		toolbar.add(new Separator());
		toolbar.add(buildProjectAction);
		toolbar.add(runDropDownAction);
		toolbar.add(restoreProjectAction);
		toolbar.add(readLogsAction);
		toolbar.add(new Separator());
		toolbar.add(showCoverageAction);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.ActionBarAdvisor#fillStatusLine(org.eclipse.jface.action.IStatusLineManager)
	 */
	@Override
	protected void fillStatusLine(IStatusLineManager statusLine) {

		StatusLineContributionItem slci;

		slci = new StatusLineContributionItem("Sourcecode");
		slci.setVisible(true);
		slci.setText("Status source code: CLEAN  ");
		statusLine.insertBefore(StatusLineManager.BEGIN_GROUP, slci);

		slci = new StatusLineContributionItem("Parsedcode");
		statusLine.insertAfter("SourceCode", slci);
		slci.setVisible(true);
		slci.setText("Status parsed sources: CLEAN  ");

		super.fillStatusLine(statusLine);
	}

}
