package sota.app;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import sota.Logger;
import sota.gui.preferences.PreferenceConstants;
import sota.gui.preferences.PreferenceInitializer;

public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {

	/**
	 * Der Konstruktor.
	 * 
	 * @param configurer
	 *            der Configurer
	 */
	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.application.WorkbenchWindowAdvisor#createActionBarAdvisor(org.eclipse.ui.application.
	 * IActionBarConfigurer)
	 */
	public ActionBarAdvisor createActionBarAdvisor(IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.WorkbenchWindowAdvisor#preWindowOpen()
	 */
	public void preWindowOpen() {

		Application.createInstance();
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(1024, 768));
		configurer.setShowMenuBar(true);
		configurer.setShowCoolBar(true);
		configurer.setShowStatusLine(true);
		configurer.setTitle("SOTA");
		configurer.setShowPerspectiveBar(false);

		/** HACK?: Initialisierung der defaults per Hand */
		new PreferenceInitializer().initializeDefaultPreferences();

		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_CREATE_LOG)) {

			String logFile;

			if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_OVERWRITE_LOG))
				logFile = "sota.log";
			else {
				Date dt = new Date();
				SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");

				logFile = "sota" + df.format(dt);

				if (new File(logFile + ".log").exists()) {
					int i = 1;
					while (new File((logFile + "_" + i + ".log")).exists())
						i++;
					logFile = (logFile + "_" + i + ".log");
				} else {
					logFile = logFile + ".log";
				}
			}
			Logger.setLogWriter(logFile, "SOTA coverage tool v1.0 ", "SOTA LogFile ");
		}

		if (!Application.getInstance().getASTManager().readLanguageSpecs()) {
			Logger.log("\n\t No appriopriate LanguageSpecification found!");
			Application.getInstance().stop();
		}
	}

}
