package sota.ast.nodes;

import sota.Logger;

/**
 * AST-Knoten fr Klassenstrukturen.
 * */
public abstract class ASTClassNode extends ASTScopeNode {

	/**
	 * Konstruktor fr die Knotenerstellung.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param content
	 *            der Inhalt
	 * @param scope
	 *            der Scope des Knotens
	 * @param name
	 *            der Name der Klasse
	 * @param parent
	 *            der bergeordnete ScopeNode
	 */
	public ASTClassNode(int ln, String content, String scope, String name, IScopeNode parent) {
		super(ln, scope, name, parent);
		this.content = content;
		setQualifiedName();
	}

	// Funktionen fr Instrumentierung

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#createASCContent(int)
	 */
	public void createASCContent(int level) {
		ascContent += content;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#setASCContent(java.lang.String)
	 */
	public void setASCContent(String s) {
		ascContent = s;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#appendToASCContent(java.lang.String)
	 */
	public void appendToASCContent(String s) {
		ascContent += s;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getASCContent()
	 */
	public String getASCContent() {
		return ascContent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.IScopeNode#getName()
	 */
	public String getName() {
		return name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printNode()
	 */
	public void printNode() {
		Logger.logln("---<" + id + "><Class:" + setOfClasseNodes.size() + "," + setOfFunctionNodes.size() + "><"
				+ fileNode.getName() + ": " + fileNode.getPackageName() + ". " + scopeName + "><" + name + ">---\n"
				+ content);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getNodeString()
	 */
	public String getNodeString() {
		return super.getNodeString() + "<Class->" + endNode.getID() + " (" + setOfClasseNodes.size() + ","
				+ setOfFunctionNodes.size() + ")><" + fileNode.getName() + ": " + fileNode.getPackageName() + ". "
				+ scopeName + "><" + name + ">";
	}
}