package sota.ast.nodes;

import sota.Logger;

/**
 * AST-Knoten fr Compoundblcke
 * 
 * <PRE>
 * 		..									 *		..
 * 		ASTCompoundNode		this			 * 		ASTCompoundNode		startNode
 * 		  &gt; ..							 *		  &gt; ..			
 * 		ASTCompoundNode		endNode			 *		ASTCompoundNode		this
 * 		..									 * 		..
 * </PRE>
 * */
public abstract class ASTCompoundNode extends ASTNode {

	/** Bei der Konstruktion zu setzende Informationen und Verknpfungen * */

	protected ASTCompoundNode endNode;

	protected ASTCompoundNode startNode;

	/**********************************************************************/

	/**
	 * Konstruktor fr den Blockknoten.
	 * 
	 * @param nl
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 */
	public ASTCompoundNode(int nl, String s) {
		super(nl, s);
		endNode = null;
	}

	/**
	 * Setzt den Endknoten.
	 * 
	 * @param n
	 *            der Endknoten
	 */
	public void setEndNode(ASTCompoundNode n) {
		endNode = n;
		n.setStartNode(this);
		return;
	}

	/**
	 * Setzt den Startknoten.
	 * 
	 * @param n
	 *            der Startknoten
	 */
	public void setStartNode(ASTCompoundNode n) {
		startNode = n;
	}

	/**
	 * Liefert den Endknoten.
	 * 
	 * @return der Endknoten
	 */
	public ASTCompoundNode getEndNode() {
		return endNode;
	}

	/**
	 * Liefert den Startknoten.
	 * 
	 * @return der Startknoten
	 */
	public ASTCompoundNode getStartNode() {
		return startNode;
	}

	/**
	 * Entscheidet, ob der Knoten der Startknoten des Blocks ist.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public boolean isStartNode() {
		return startNode == null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printNode()
	 */
	public void printNode() {
		Logger.log("---");
		printReach();
		Logger.log("<" + id + ">");
		if (endNode != null) {
			Logger.log("<Comp -> " + endNode.getID() + ">---\n" + content);
		} else if (startNode != null) {
			Logger.logln("<" + startNode.getID() + " -> Comp>---\n" + content);
		} else {
			Logger.logln("---\n" + content);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getNodeString()
	 */
	public String getNodeString() {
		String s = super.getNodeString();
		if (endNode != null)
			return s + "<Comp -> " + endNode.getID() + ">";
		if (startNode != null)
			return s + "<" + startNode.getID() + " -> Comp>";
		return "--- no compound partner ---";
	}
}