package sota.ast.nodes;

import sota.Logger;

/**
 * AST-Knoten fr Do-Knoten
 * */
public abstract class ASTDoNode extends ASTNode {
	protected ASTIterationNode iterationNode;

	/**
	 * Der Konstruktor.
	 * 
	 * @param nl
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 */
	public ASTDoNode(int nl, String s) {
		super(nl, s);
		iterationNode = null;
	}

	/**
	 * Setzt den Iterationsknoten.
	 * 
	 * @param in
	 *            der Iterationsknoten
	 */
	public void setIterationNode(ASTIterationNode in) {
		iterationNode = in;
	}

	/**
	 * Liefert den Iterationsknoten.
	 * 
	 * @return der Iterationsknoten
	 */
	public ASTIterationNode getIterationNode() {
		return iterationNode;
	}

	/**
	 * Liefert den Loopknoten.
	 * 
	 * @return der Loopknoten
	 */
	public ASTStatementNode getLoopNode() {
		return iterationNode.getLoopNode();
	}

	/**
	 * Liefert den Endknoten.
	 * 
	 * @return der Endknoten
	 */
	public ASTStatementNode getEndNode() {
		return iterationNode.getEndNode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printNode()
	 */
	public void printNode() {
		Logger.log("---");
		printReach();
		Logger.logln("<" + id + "><Do: " + iterationNode.getID() + ">---\n" + content);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getNodeString()
	 */
	public String getNodeString() {
		return super.getNodeString() + "<Do: " + iterationNode.getID() + ">";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printCFG()
	 */
	public void printCFG() {
		Logger.logln(" " + id + " <- " + iterationNode.getID());
		Logger.logln(" |");
	}

}