package sota.ast.nodes;

import java.util.ArrayList;
import java.util.Vector;

import sota.Logger;

/**
 * AST-Knoten fr Sprunganweisungen (break, continue, return, throw)
 * */
public abstract class ASTJumpNode extends ASTStatementNode {

	/** Bei der Konstruktion zu setzende Informationen und Verknpfungen * */

	protected ASTNode targetNode;

	/*
	 * Label steht immer direkt vor BranchStatement (while, do, for, if, switch, try) oder direkt vor einem
	 * CompoundNode!
	 */
	protected String label;

	protected boolean isReturn;

	/*
	 * funktioniert ausschlielich in Schleifen Continue springt immer zum IterationNode (auch im
	 * Try-Catch-Finally-Block)
	 */
	protected boolean isContinue;

	/*
	 * Break ohne Label funktioniert ausschlielich in Schleifen oder Switches Break mit Label springt immer zum endNode
	 * des BranchStatements oder CompoundBlocks (auch im Try-Catch-Finally-Block)
	 */
	protected boolean isBreak;

	protected boolean isThrow;

	protected boolean isGoto;

	/**********************************************************************************/

	/* wenn in Exception-Block, dann auf Finally aufpassen */
	protected ArrayList<ASTTryNode> listOfTryNodes = new ArrayList<ASTTryNode>();

	protected ArrayList<ASTNode> listOfFinallyNodes = null;

	/**
	 * Konstruktor fr Sprungknoten.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 * @param l
	 *            das Sprunglabel
	 */
	public ASTJumpNode(int ln, String s, String l) {
		super(ln, s);
		isReturn = false;
		isContinue = false;
		isBreak = false;
		isThrow = false;
		isGoto = false;
		label = l;
	}

	/**
	 * Legt fest, ob es sich um einen Sprung durch goto handelt.
	 * 
	 * @param b
	 *            true, wenn ja, sonst false
	 */
	public void isGoto(boolean b) {
		isGoto = b;
	}

	/**
	 * Entscheidet, ob es sich um einen goto-Sprung handelt.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public boolean isGoto() {
		return isGoto;
	}

	/**
	 * Legt fest, ob es sich um einen Sprung durch return handelt.
	 * 
	 * @param b
	 *            true, wenn ja, sonst false
	 */

	public void isReturn(boolean b) {
		isReturn = b;
	}

	/**
	 * Entscheidet, ob es sich um einen return-Sprung handelt.
	 * 
	 * @return true, wenn ja, sonst false
	 */

	public boolean isReturn() {
		return isReturn;
	}

	/**
	 * Legt fest, ob es sich um einen Sprung durch break handelt.
	 * 
	 * @param b
	 *            true, wenn ja, sonst false
	 */

	public void isBreak(boolean b) {
		isBreak = b;
	}

	/**
	 * Entscheidet, ob es sich um einen break-Sprung handelt.
	 * 
	 * @return true, wenn ja, sonst false
	 */

	public boolean isBreak() {
		return isBreak;
	}

	/**
	 * Legt fest, ob es sich um einen Sprung durch continue handelt.
	 * 
	 * @param b
	 *            true, wenn ja, sonst false
	 */

	public void isContinue(boolean b) {
		isContinue = b;
	}

	/**
	 * Entscheidet, ob es sich um einen continue-Sprung handelt.
	 * 
	 * @return true, wenn ja, sonst false
	 */

	public boolean isContinue() {
		return isContinue;
	}

	/**
	 * Legt fest, ob es sich um einen Sprung durch throw handelt.
	 * 
	 * @param b
	 *            true, wenn ja, sonst false
	 */

	public void isThrow(boolean b) {
		isThrow = b;
	}

	/**
	 * Entscheidet, ob es sich um einen throw-Sprung handelt.
	 * 
	 * @return true, wenn ja, sonst false
	 */

	public boolean isThrow() {
		return isThrow;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#isVirtual(boolean)
	 */
	public void isVirtual(boolean b) {
		isVirtual = b;
	}

	/**
	 * Setzt den Zielknoten fr den Sprung.
	 * 
	 * @param targetNode
	 *            der Zielknoten
	 */
	public void setTarget(ASTNode targetNode) {
		this.targetNode = targetNode;
	}

	/**
	 * Liefert den Zielknoten fr den Sprung.
	 * 
	 * @return der Zielknoten
	 */
	public ASTNode getTargetNode() {
		return targetNode;
	}

	/**
	 * Liefert das Label fr den Sprung.
	 * 
	 * @return das Label
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * Fgt eine Menge an TryKnoten hinzu.
	 * 
	 * @param n
	 *            die Menge
	 */
	public void addTryNodes(Vector<ASTTryNode> n) {
		for (int i = n.size() - 1; i >= 0; i--)
			listOfTryNodes.add(n.elementAt(i));
	}

	/**
	 * Liefert eine Liste an Finally-Knoten.
	 * 
	 * @return die Liste
	 */
	public ArrayList<ASTNode> getListOfFinallyNodes() {
		if (listOfFinallyNodes == null) {
			listOfFinallyNodes = new ArrayList<ASTNode>();
			for (ASTTryNode n : listOfTryNodes) {
				if (n.getFinallyNode() != null)
					listOfFinallyNodes.add(n.getFinallyNode());
			}
		}
		return listOfFinallyNodes;
	}

	/**
	 * Liefert die Liste an Try-Knoten.
	 * 
	 * @return die Liste
	 */
	public ArrayList<ASTTryNode> getListOfTryNodes() {
		return listOfTryNodes;
	}

	/** Prints ** */
	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTStatementNode#printNode()
	 */
	public void printNode() {
		Logger.log("---");
		printReach();
		if (targetNode == null)
			Logger.logln("<" + id + "><V-Jump> (no target!)");
		else {
			if (isVirtual)
				Logger.logln("<" + id + "><V-Jump -> " + targetNode.getID() + ">");
			else {
				Logger.log("<" + id + "><Jump");
				/*
				 * if (isThrow) { for (ASTTryNode tn : getListOfTryNodes()) { tn.print(); Logger.log(" -> " +
				 * tn.getID()); // Logger.log(" -> " + // tn.getFinallyNode().getID()); } Logger.log(">"); } else {
				 */
				for (ASTNode n : getListOfFinallyNodes())
					Logger.log(" -> " + n.getID());
				Logger.log(" -> " + targetNode.getID() + ">");
				// }
				Logger.log("<" + nrHits + ">");
				if (label != "")
					Logger.log("<" + label + ">");
				Logger.logln("---\n" + content);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTStatementNode#getNodeString()
	 */
	public String getNodeString() {
		String s = super.getNodeString();

		if (targetNode == null)
			return s + "<V-Jump> (no target!)";
		s += "<Jump";
		// for (ASTNode n : getListOfFinallyNodes())
		// s += " -> " + n.getID();
		s += " -> " + targetNode.getID() + ">";
		// }
		if (label != "")
			s += "<" + label + ">";

		return s;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTStatementNode#printCFG()
	 */
	public void printCFG() {
		Logger.logln("(" + id + "#" + nrHits + " -> " + targetNode.getID() + ")-P\n");
	}
}