package sota.ast.nodes;

import java.util.HashSet;

import sota.Logger;

/**
 * AST-Knoten fr Anweisungslabel
 * 
 * Label steht in Java immer direkt vor BranchStatement (while, do, for, if, switch, try) oder direkt vor einem
 * CompoundNode!
 */
public abstract class ASTLabelNode extends ASTNode {
	/** Bei der Konstruktion zu setzende Informationen und Verknpfungen * */

	protected String label;

	protected HashSet<ASTJumpNode> setOfJumpNodes;

	/********************************************************************/

	/**
	 * Konstruktor des eines Labelknotens.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 * @param l
	 *            das Label
	 */
	public ASTLabelNode(int ln, String s, String l) {
		super(ln, s);
		setOfJumpNodes = new HashSet<ASTJumpNode>();
		label = l;
	}

	/**
	 * Fgt einen ASTJumpNode hinzu, der zu diesem Label springt.
	 * 
	 * @param jn
	 *            der ASTJumpNode
	 */
	public void addJumpNode(ASTJumpNode jn) {
		setOfJumpNodes.add(jn);
	}

	/**
	 * Gibt die Menge der Sprungknoten zurck, die auf den Labelknoten verweisen.
	 * 
	 * @return die Menge der ASTJumpNodes
	 */
	public HashSet<ASTJumpNode> getSetOfJumpNodes() {
		return setOfJumpNodes;
	}

	/**
	 * Gibt das Label zurck.
	 * 
	 * @return das Label
	 */
	public String getLabel() {
		return label;
	}

	/** Print ** */
	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printNode()
	 */
	public void printNode() {
		Logger.log("---");
		printReach();
		Logger.log("<" + id + "><Label: " + label + "><JN: ");
		for (ASTNode n : setOfJumpNodes) {
			Logger.log(n.getID() + " ");
		}
		Logger.logln(">---\n" + content);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getNodeString()
	 */
	public String getNodeString() {
		String s = super.getNodeString() + "<Label: " + label + "><JN: ";
		for (ASTNode n : setOfJumpNodes) {
			s += n.getID() + " ";
		}
		return s + " >";
	}
}