package sota.ast.nodes;

import sota.Logger;

/**
 * AST-Knoten fr Anweisungen und damit auch Oberklasse fr Ableitungen, die berdeckungsinformation speichern.
 * 
 * */
public abstract class ASTStatementNode extends ASTNode implements IStatementNode {

	/** Bei der Konstruktion zu setzende Informationen und Verknpfungen * */

	// falls das Statement eine Klasssendefinition enthlt,dann verweist
	// endOfStatement auf das schlieende Ende der Anweisung
	protected ASTNode endOfStatement = null;

	/**********************************************************/

	protected int nrHits = 0;

	/**
	 * Konstruktor fr eine nichtvirtuelle Anweisung.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 */
	public ASTStatementNode(int ln, String s) {
		super(ln, s);
	}

	/**
	 * Konstruktor fr eine virtuelle Anweisung.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 */
	public ASTStatementNode(int ln) {
		super(ln);
	}

	/**
	 * Entscheidet, ob die Anweisung geteilt ist.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public boolean isSepareted() {
		return !(endOfStatement == null);
	}

	/**
	 * Setzt den Endknoten einer geteilten Anweisung.
	 * 
	 * @param n
	 *            der Endknoten
	 */
	public void setEndOfStatement(ASTNode n) {
		endOfStatement = n;
	}

	/**
	 * Gibt das Ende einer geteilten Anweisung zurck.
	 * 
	 * @return der Endknoten
	 */
	public ASTNode getEndOfStatement() {
		return endOfStatement;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.basic.ast.StatementNode#getNrHits()
	 */
	public int getNrHits() {
		return nrHits;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.basic.ast.StatementNode#setNrHits(int)
	 */
	public void setNrHits(int i) {
		nrHits = i;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.basic.ast.StatementNode#incNrHits()
	 */
	public int incNrHits() {
		return ++nrHits;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.IStatementNode#incNrHits(int)
	 */
	public int incNrHits(int i) {
		nrHits += i;
		return nrHits;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.IStatementNode#decNrHits(int)
	 */
	public int decNrHits(int i) {
		nrHits -= i;
		if (nrHits < 0)
			nrHits = 0;
		return nrHits;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.basic.ast.StatementNode#resetHits()
	 */
	public void resetHits() {
		nrHits = 0;
	}

	/** Print ** */
	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printNode()
	 */
	public void printNode() {
		Logger.log("---");
		printReach();
		if (isVirtual) {
			Logger.logln("<" + id + "><V><" + nrHits + ">---");
		} else {
			Logger.logln("<" + id + "><" + nrHits + ">---\n" + content);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#getNodeString()
	 */
	public String getNodeString() {
		String s = super.getNodeString();
		if (isSepareted())
			s += "<EOS: " + endOfStatement.getID() + ">";
		return s + "<" + nrHits + ">";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#printCFG()
	 */
	public void printCFG() {
		if (isVirtual)
			Logger.logln("[" + id + "#" + nrHits + "]");
		else
			Logger.logln("(" + id + "#" + nrHits + ")");
		Logger.logln(" |");
	}

}