package sota.ast.nodes;

import java.util.HashSet;
import java.util.TreeSet;

import sota.ast.util.Metrics;

/**
 * Superklasse fr alle Knoten mit Bedingungsauswertung
 * */
public interface IConditionNode {

	/**
	 * Numeriert die Bedingung, startend mit Parameterwert.
	 * 
	 * @param i
	 *            der Offset
	 * @return der neue Offset
	 */
	public int numerateCondition(int i);

	/**
	 * Gibt Bedingung zurck.
	 * 
	 * @return die Bedingung
	 */
	public Condition getCondition();

	/**
	 * Setzt alle berdeckungsdaten zurck.
	 */
	public void resetHits();

	/**
	 * Gibt alle Wahrheitsvektoren zurck.
	 * 
	 * @return die Menge der Wahrheitsvektoren
	 */
	public HashSet<String> getTruthVectors();

	/**
	 * Ausgabefunktion
	 */
	public void print();

	/**
	 * Gibt die ID des Knoten zurck.
	 * 
	 * @return
	 */
	public int getID();

	/**
	 * Berechnet die berdeckungsmae.
	 */
	public void computeConditionCoverage();

	/**
	 * Fgt Booleanarray aus Logfile in Knoten ein.
	 * 
	 * @param arrayList
	 *            die Auswertung der Atome
	 */
	public void fillConditions(TreeSet<String> arrayList);

	/**
	 * Berechnet aus einem Truthvector einen Belegungstring. Dabei werden nicht ausgewertete Variablen (bei
	 * short-circuit-evaluation) durch "-" gekennzeichnet. Bsp.: ((falsch && falsch) || wahr) Truthvector: 011 (zweite
	 * Wert wurde nicht geloggt, daher per default true) Eval.TV.: 0-1
	 * 
	 * @param s
	 *            der TruthVector, der umgewandelt werden soll
	 * @return der berechnete String mit den Bedingungsbelegungen
	 */
	public String getEvaluatedTVString(String s);

	/**
	 * Gibt die Metrik der Bedingung zurck.
	 * 
	 * @return die Metrik
	 */
	public Metrics getMetrics();

	/**
	 * Entscheidt, ob die Bedingung nichttrivial ist (kein true-, false-Atom oder ein leeres Atom).
	 * 
	 * @return true, wenn nicht-trivial, sonst false
	 */
	public boolean hasNonTrivialCondition();
}
