package sota.ast.nodes;

import java.util.TreeSet;

import sota.ast.util.Metrics;

/**
 * Interface fr die homogene Behandlung von ASTScopeNodes (Klasse und Funktion) sowie FileNode.
 * 
 */
public interface IScopeNode {

	/**
	 * Fgt eine Klasse hinzu.
	 * 
	 * @param cn
	 *            der ASTClassNode
	 */
	public abstract void addClassNode(ASTClassNode cn);

	/**
	 * Gib die Menge aller Klassen zurck.
	 * 
	 * @return die Menge der ASTClassNodes
	 */
	public abstract TreeSet<ASTClassNode> getClassNodes();

	/**
	 * Fgt eine Funktion hinzu.
	 * 
	 * @param cn
	 *            der ASTFunctionNode
	 */
	public abstract void addFunctionNode(ASTFunctionNode fn);

	/**
	 * Gib die Menge aller Funktionen zurck.
	 * 
	 * @return die Menge der ASTFunctionNodes
	 */
	public abstract TreeSet<ASTFunctionNode> getFunctionNodes();

	/**
	 * Gibt den Startknoten zurck.
	 * 
	 * @return der Startknoten
	 */
	public abstract ASTNode getStartNode();

	/**
	 * Setzt den Endknoten.
	 * 
	 * @param dn
	 *            der Endknoten
	 */
	public abstract void setEndNode(ASTNode dn);

	/**
	 * Gibt den Endknoten zurck.
	 * 
	 * @return der Endknoten
	 */
	public abstract ASTNode getEndNode();

	/**
	 * Gibt den Vaterknoten zurck, oder null.
	 * 
	 * @return der Vaterknoten oder null
	 */
	public abstract IScopeNode getParent();

	/**
	 * Entscheidet, ob der ScopeNode toplevel ist.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public abstract boolean isTopLevel();

	/**
	 * Gibt den zugehrigen FileNode zurck.
	 * 
	 * @return der FileNode
	 */
	public abstract FileNode getFileNode();

	/**
	 * Gibt den Scope fr diesen Knoten zurck.
	 * 
	 * @return der Scope
	 */
	public abstract String getScope();

	/**
	 * Gibt einen eindeutigen IDString zur Identifikation zurck.
	 * 
	 * @return der ID-String
	 */
	public abstract String getIDString();

	/**
	 * Gibt den Namen zurck.
	 * 
	 * @return der Name
	 */
	public abstract String getName();

	/**
	 * Liefert den vollqualifizierten Namen zurck.
	 * 
	 * @param showPackage
	 *            true, wenn Package vorangestellt werden soll, sonst false
	 * @param showFile
	 *            true, wenn File vorangestellt werden soll, sonst false
	 * @return der String
	 */
	public abstract String getFullyQualifiedName(boolean showPackage, boolean showFile);

	/**
	 * Gibt die Metrik der Bedingung zurck.
	 * 
	 * @return die Metrik
	 */
	public abstract Metrics getMetrics();

	/**
	 * Setzt die Metrik des Knoten.
	 * 
	 * @param die
	 *            Metrik
	 */
	public abstract void setMetrics(Metrics m);

	/**
	 * Gibt die erste Zeilennummer im Scope zurck.
	 * 
	 * @return die Zeilennummer
	 */
	public abstract int getFirstLineNr();

	/**
	 * Gibt die letzte Zeilennummer im Scope zurck.
	 * 
	 * @return die Zeilennummer
	 */
	public abstract int getLastLineNr();

	/**
	 * Erstellt fr den Scopee den instrumentierten Quellcode nach einem Instrumentationslevel.
	 * 
	 * @param currLevel
	 *            das Instrumentationslevel
	 */
	public abstract void createASCContent(int currLevel);
}