package sota.ast.util;

import java.util.*;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import sota.app.Application;
import sota.gui.preferences.PreferenceConstants;

/**
 * Definiert und verwaltet die verwendeten Farben.
 * 
 */
public class ColorManager {
	public static final RGB WHITE = new RGB(255, 255, 255);
	public static final RGB GREY = new RGB(220, 220, 220);
	public static final RGB BLACK = new RGB(0, 0, 0);
	public static final RGB DEFAULT = new RGB(0, 0, 0);
	public static final RGB BACKGROUND = new RGB(255, 255, 255);
	public static final RGB COMMENT = new RGB(64, 128, 128);
	public static final RGB KEYWORD = new RGB(128, 0, 96);
	public static final RGB NUMBER = new RGB(255, 0, 255);
	public static final RGB STRING = new RGB(0, 0, 255);
	public static final RGB ASCCOMMENT = new RGB(0, 160, 0);
	public static final RGB LGREEN = new RGB(216, 255, 216);
	public static final RGB GREEN = new RGB(200, 255, 200);
	public static final RGB DGREEN = new RGB(72, 160, 72);
	public static final RGB LBLUE = new RGB(228, 228, 255);
	public static final RGB BLUE = new RGB(160, 160, 255);
	public static final RGB GRAYBLUE = new RGB(210, 210, 230);
	public static final RGB LYELLOW = new RGB(255, 255, 216);
	public static final RGB YELLOW = new RGB(255, 255, 160);
	public static final RGB ORANGE = new RGB(255, 208, 208);
	public static final RGB LRED = new RGB(255, 208, 208);
	public static final RGB MRED = new RGB(255, 184, 184);
	public static final RGB RED = new RGB(255, 172, 172);
	public static final RGB DRED = new RGB(255, 112, 112);
	public static final RGB VDRED = new RGB(160, 72, 72);
	public static final RGB C1 = new RGB(130, 255, 130);
	public static final RGB C2 = new RGB(210, 255, 210);
	public static final RGB C3 = new RGB(255, 255, 170);
	public static final RGB C4 = new RGB(255, 210, 210);
	public static final RGB C5 = new RGB(255, 170, 170);
	public static final RGB C6 = new RGB(255, 130, 130);
	public static final RGB C7 = new RGB(255, 90, 90);
	public static final RGB C8 = new RGB(255, 50, 50);

	// Map to store created colors, with the corresponding RGB as key
	private HashMap<RGB, Color> colors = new HashMap<RGB, Color>();

	/**
	 * Liefert die Farbe zu einem RGB-Wert.
	 * 
	 * @param rgb
	 *            der RGB-Wert
	 * @return die Farbe
	 */
	public Color getColor(RGB rgb) {
		// Get the color from the map
		Color color = colors.get(rgb);
		if (color == null) {
			// Color hasn't been created yet; create and put in map
			color = new Color(Display.getCurrent(), rgb);
			colors.put(rgb, color);
		}
		return color;
	}

	/**
	 * Entfernt alle Farben.
	 */
	public void dispose() {
		for (Color c : colors.values())
			c.dispose();
	}

	/**
	 * Liefert anhand der Prferenzen zu jedem berdeckungsgrad eine Farbe.
	 * 
	 * @param p
	 *            der berdeckungsgrad
	 * @return die Farbe
	 */
	public Color percentage2Color(float p) {
		if (p == 100)
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_TOP_BAR_COLOR);
		if (p == 0)
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_BOTTOM_BAR_COLOR);
		if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FIRST_BAR))
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_FIRST_BAR_COLOR);
		if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_SECOND_BAR))
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_SECOND_BAR_COLOR);
		if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_THIRD_BAR))
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_THIRD_BAR_COLOR);
		if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FOURTH_BAR))
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_FOURTH_BAR_COLOR);
		if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FIFTH_BAR))
			return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_FIFTH_BAR_COLOR);
		return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_ABOVE_BOTTOM_BAR_COLOR);
	}
}