package sota.ast.util;

import java.util.HashMap;
import java.util.Set;

import sota.ast.nodes.IScopeNode;

/**
 * Reprsentiert eine Instrumentationsschema, d.h. eine Zuordnung von Projektstrukturen auf die Instrumentatiosnlevel
 * 0-3.
 * 
 */
public class IScheme implements Comparable<IScheme> {
	protected String name = "";

	protected String description = "";

	protected HashMap<String, Integer> levelMap;

	/**
	 * Der Konstruktor.
	 * 
	 * @param name
	 *            der Name des ISchemes
	 */
	public IScheme(String name) {
		this.name = name;
		levelMap = new HashMap<String, Integer>();
	}

	/**
	 * Liefert den Namen des ISchemes.
	 * 
	 * @return der Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setzt den Namen des ISchemes.
	 * 
	 * @param name
	 *            der neue Name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Liefert die Beschreibung
	 * 
	 * @return die Beschreibung
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Setzt die Beschreibung des ISchemes
	 * 
	 * @param description
	 *            die neue Beschreibung
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Liefert die Abbildung der Strukturen auf Instrumentationslevel.
	 * 
	 * @return die Map
	 */
	public HashMap<String, Integer> getLevelMap() {
		return levelMap;
	}

	/**
	 * Legt die Abbildung der Strukturen auf Instrumentationslevel fest.
	 * 
	 * @return die Map
	 */
	public void setLevelMap(HashMap<String, Integer> levelMap) {
		this.levelMap = levelMap;
	}

	/**
	 * Fgt ein Instrumentationslevel fr eine Struktur hinzu.
	 * 
	 * @param idString
	 *            die ID der Struktur
	 * @param level
	 *            das Instrumentationslevel
	 */
	public void addInstrumentationLevel(String idString, int level) {
		levelMap.put(idString, level);
	}

	/**
	 * Testet, ob eine Struktur ein Instrumentationslevel hat.
	 * 
	 * @param idString
	 *            die ID der Struktur
	 * @return true, wenn vorhanden, sonst false
	 */
	public boolean hasInstrumentationLevel(String idString) {
		return levelMap.containsKey(idString);
	}

	/**
	 * Liefert das Instrumentationslevel einer Struktur.
	 * 
	 * @param idString
	 *            die ID der Struktur
	 * @return das Level
	 */
	public int getInstrumentationLevel(String idString) {
		return levelMap.get(idString);
	}

	/**
	 * Liefert das Instrumentationslevel eines IScopeNodes.
	 * 
	 * @param node
	 *            der IScopeNode
	 * @return das Level
	 */
	public int getInstrumentationLevel(IScopeNode node) {
		if (this instanceof GlobalInstrumentationScheme)
			System.out.println("y");
		for (IScopeNode in = node; in != null; in = in.getParent()) {
			if (levelMap.keySet().contains(in.getIDString()))
				return levelMap.get(in.getIDString());
		}
		return 0;
	}

	/**
	 * Liefert eine Liste der Strukturen mit Instrumentationslevel.
	 * 
	 * @return die Liste
	 */
	public Set<String> getInstrumentationIds() {
		return levelMap.keySet();
	}

	/**
	 * Entfernt das Level einer Struktur.
	 * 
	 * @param string
	 *            die Struktur
	 */
	public void removeInstrumentierungsLevel(String string) {
		if (levelMap.containsKey(string))
			levelMap.remove(string);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(IScheme o) {
		return name.compareTo(o.name);
	}

}
