package sota.ast.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import sota.Logger;
import sota.app.Application;
import sota.ast.nodes.ASTBranchNode;
import sota.ast.nodes.ASTCaseNode;
import sota.ast.nodes.ASTCatchNode;
import sota.ast.nodes.ASTCompoundNode;
import sota.ast.nodes.ASTDoNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.ASTIfNode;
import sota.ast.nodes.ASTIterationNode;
import sota.ast.nodes.ASTJumpNode;
import sota.ast.nodes.ASTNode;
import sota.ast.nodes.ASTStatementNode;
import sota.ast.nodes.ASTSwitchNode;
import sota.ast.nodes.ASTTryNode;

/**
 * 
 * Klasse mit statischen Methoden zur Berechnung der BI-Pfadberdeckung.
 * 
 */
public class ModBIPathManager {

	protected static boolean INFO = false;
	protected static boolean VERBOSE = false;
	protected static boolean VERBOSE2 = false;
	protected static boolean VERBOSE3 = false;
	protected static boolean VERBOSE_COVER = false;

	protected static ASTFunctionNode currFunctionNode = null;

	/**
	 * Hlt die Informationen fr den Iterationszustand einer Schleife (fr Stack).
	 */
	protected static class ModBIPathInfo {

		// Anzahl der Pfade
		public long nrPaths = 1;

		// Abbildung einer Knoten-ID eines Sprungzielknotens auf die Menge der zu ihm fhrenden BI-Pfade ber break,
		// return, throw
		public HashMap<Integer, Long> closedPaths = new HashMap<Integer, Long>();
		// Abbildung einer Knoten-ID eines Sprungzielknotens auf die Menge der zu ihm fhrenden BI-Pfade ber continue
		public HashMap<Integer, Long> continuedPaths = new HashMap<Integer, Long>();
		// die Anzahl an Pfaden durch einen einfachen Schleifendurchlauf
		protected HashMap<Integer, Long> singleLoopMap = new HashMap<Integer, Long>();
		// die Anzahl an Pfaden durch einen doppelten Schleifendurchlauf
		protected HashMap<Integer, Long> multipleLoopMap = new HashMap<Integer, Long>();

		public String stringValue() {
			return "\n Path (" + nrPaths + ")\n\tclosedPaths: " + closedPaths + "\n\tcontinuedPaths: " + continuedPaths
					+ "\n\tsingleLoopMap: " + singleLoopMap + "\n\tmultipleLoopMap: " + multipleLoopMap + "\n";
		}
	}

	// Pfadinfo fr einen ModBIPath
	protected static class ModBIPath {
		long path = 0;
		int nrDecisions = 0;
		boolean isStillFunctionPath = true;
	}

	/************************************************************************************/
	/* Pfadmengenberechnung */
	/************************************************************************************/

	/**
	 * Berechnet die Anzahl der mglichen Pfade fr den Modifizierten Boundary-Interior-Pfadtest.
	 * 
	 * @param fn
	 *            der ASTFunctionNode der entsprechenden Funktion
	 */
	public static void computeNrModBIPaths(ASTFunctionNode fn) {
		currFunctionNode = fn;

		// berechnet rekursive die ModBI-Pfade
		ModBIPathInfo pi = computeModBIPaths(fn.getEntryNode(), fn.getEndNode());

		// Auswertung offener Sprungpfade
		if (pi.closedPaths.containsKey(fn.getEndNode().getID())) {
			pi.nrPaths += pi.closedPaths.remove(fn.getEndNode().getID());
		}
		if (INFO)
			System.out.println(fn.getIDString() + "\n" + pi.nrPaths);
		if (!pi.closedPaths.isEmpty())
			System.out.println("closedPath not empty!\n" + pi.closedPaths);
		if (!pi.continuedPaths.isEmpty())
			System.out.println("continuedPaths not empty!\n" + pi.continuedPaths);

		// Sicherung der Daten
		fn.getMetrics().mapOfModBIPaths.put(fn.getID(), pi.nrPaths);

		for (Integer i : pi.singleLoopMap.keySet()) {
			fn.getMetrics().mapOfModBIPaths.put(i, pi.singleLoopMap.get(i));
			pi.nrPaths += pi.singleLoopMap.get(i);
		}

		for (Integer i : pi.multipleLoopMap.keySet()) {
			if (fn.getMetrics().mapOfModBIPaths.containsKey(i))
				fn.getMetrics().mapOfModBIPaths.put(i, fn.getMetrics().mapOfModBIPaths.remove(i)
						+ pi.multipleLoopMap.get(i));
			else
				fn.getMetrics().mapOfModBIPaths.put(i, pi.multipleLoopMap.get(i));
			pi.nrPaths += pi.multipleLoopMap.get(i);
		}

		fn.getMetrics().nrModBIPaths = pi.nrPaths;
		if (INFO)
			System.out.println(pi.nrPaths);

	}

	/**
	 * Berechnet rekursiv die PfadInfo fr MBIPfade eines Quellcodeabschnitt
	 * 
	 * @param start
	 *            Beginn des Abschnittes
	 * @param end
	 *            Ende des Abschnittes
	 * @return die berechnete PfadInfo
	 */
	protected static ModBIPathInfo computeModBIPaths(ASTNode start, ASTNode end) {
		if (VERBOSE)
			System.out.println("----- computeModBIPaths " + start.getID() + " -> " + end.getID() + " -----");
		ModBIPathInfo pi = new ModBIPathInfo();
		ASTNode endNode = null;

		ASTNode n = start;
		while (n != end) {
			// IF: die parallelen True- und False-Pfade werden an den aktuellen Pfad angehngt
			if (n instanceof ASTIfNode) {
				endNode = ((ASTIfNode) n).getEndNode();
				ModBIPathInfo truePath = computeModBIPaths(((ASTIfNode) n).getTrueNode(), ((ASTIfNode) n).getElseNode());
				ModBIPathInfo falsePaths = computeModBIPaths(((ASTIfNode) n).getFalseNode(), endNode);
				pi = joinSequentialModBIPaths(pi, joinParallelModBIPaths(truePath, falsePaths), endNode);
				n = endNode;
				// normale Schleife: die Pfade der Schleife werden separat als IterationsPfade verarbeitet
			} else if (n instanceof ASTIterationNode && !((ASTIterationNode) n).hasDoNode()) {
				endNode = ((ASTIterationNode) n).getEndNode();

				ModBIPathInfo loopPaths = computeModBIPaths(((ASTIterationNode) n).getLoopNode(), endNode);

				long nrClosedPaths = 0;
				long nrContinuedPaths = 0;

				// Continue-Pfade fr diese Schleife werden gemerkt
				if (loopPaths.continuedPaths.containsKey(n.getID()))
					nrContinuedPaths = loopPaths.continuedPaths.remove(n.getID());

				// alle andere Continue-Pfade und Closed-Pfade werden als
				// Schleifenabbruch gezhlt
				for (Long l : loopPaths.closedPaths.values())
					nrClosedPaths += l;
				for (Long l : loopPaths.continuedPaths.values())
					nrClosedPaths += l;

				// einfachen Schleifendurchgang merken
				pi.singleLoopMap.put(n.getID(), loopPaths.nrPaths + nrContinuedPaths + nrClosedPaths);
				// doppelte Schleifendurchgnge merken
				pi.multipleLoopMap.put(n.getID(), (loopPaths.nrPaths + nrContinuedPaths)
						* (loopPaths.nrPaths + nrContinuedPaths + nrClosedPaths));

				transferLoopMaps(pi, loopPaths);

				n = endNode;
				// Do-While-Schleife: die Pfade der Schleife werden separat als IterationsPfade verarbeitet
			} else if (n instanceof ASTDoNode) {
				endNode = ((ASTDoNode) n).getEndNode();
				ASTIterationNode iterationNode = ((ASTDoNode) n).getIterationNode();
				ModBIPathInfo loopPaths = computeModBIPaths(((ASTDoNode) n).getLoopNode(), endNode);

				long nrClosedPaths = 0;
				long nrContinuedPathsToIterationNode = 0;

				// alle Pfade, die Iteration erneut beginnen
				if (loopPaths.continuedPaths.containsKey(iterationNode.getID()))
					nrContinuedPathsToIterationNode = loopPaths.continuedPaths.remove(iterationNode.getID());
				// alle Pfade aufsummieren, die Schleife verlassen
				for (Long l : loopPaths.closedPaths.values())
					nrClosedPaths += l;
				for (Long l : loopPaths.continuedPaths.values())
					nrClosedPaths += l;

				pi.multipleLoopMap.put(iterationNode.getID(), (loopPaths.nrPaths + nrContinuedPathsToIterationNode)
						* (loopPaths.nrPaths + nrContinuedPathsToIterationNode + nrClosedPaths));

				pi = joinSequentialModBIPaths(pi, loopPaths, endNode);

				n = endNode;
				// Switch: alle Cases werden als parallele Pfade verarbeitet
			} else if (n instanceof ASTSwitchNode) {
				endNode = ((ASTSwitchNode) n).getEndNode();
				ModBIPathInfo summedPaths = null;
				ModBIPathInfo tempPaths;
				for (ASTCaseNode cn : ((ASTSwitchNode) n).getListOfCases()) {
					tempPaths = computeModBIPaths(cn, cn.getSwitchNode().getEndNode());
					if (summedPaths == null)
						summedPaths = tempPaths;
					else
						summedPaths = joinParallelModBIPaths(summedPaths, tempPaths);
				}
				if (!((ASTSwitchNode) n).hasDefault())
					summedPaths.nrPaths++;
				pi = joinSequentialModBIPaths(pi, summedPaths, endNode);
				n = endNode;

				// Ausnahmebehandlung: Catches werden als parellele Pfade behandelt
			} else if (n instanceof ASTTryNode) {
				ASTTryNode tn = (ASTTryNode) n;
				endNode = tn.getEndNode();
				// die Pfade des Try-Blocks
				ModBIPathInfo tempPaths = computeModBIPaths(n.getNextNode(), tn.getEndOfTryBlock());
				// die Pfade der Exception-Auswertung (ohne Finally) werden als
				// parallel betrachtet - untere Schranke fr BI-Pfadtest
				for (ASTCatchNode cn : tn.getListOfCatches()) {
					if (!cn.isFinally()) {
						tempPaths = joinParallelModBIPaths(tempPaths, computeModBIPaths(cn.getNextNode(), cn
								.getEndOfCatchBlock()));
					}
				}
				if (tn.hasFinally()) {
					tempPaths = joinExceptionModBIPaths(tempPaths, computeModBIPaths(tn.getFinallyNode().getNextNode(),
							tn.getFinallyNode().getEndOfCatchBlock()), endNode);
				}
				pi = joinSequentialModBIPaths(pi, tempPaths, tn.getEndNode());
				n = tn.getEndNode();

				// Sprnge fhren zur ffnung von closed- bzw. continued Paths
			} else if (n instanceof ASTJumpNode && !n.isVirtual()) {
				ASTJumpNode jn = (ASTJumpNode) n;
				ASTNode tn = ((ASTJumpNode) n).getTargetNode();
				if (tn.getID() >= end.getID() || tn.getID() <= start.getID()) {
					// continues werden in continuedPaths gespeichert - treten
					// nur in Schleifen auf
					if (jn.isContinue()) {
						if (pi.continuedPaths.containsKey(tn.getID()))
							pi.continuedPaths.put(tn.getID(), pi.continuedPaths.remove(tn.getID()) + pi.nrPaths);
						else
							pi.continuedPaths.put(tn.getID(), pi.nrPaths);

						// throws werden immer! als return betrachtet, d.h.
						// untere Schranke der BIPfade
					} else if (jn.isThrow()) {
						if (pi.closedPaths.containsKey(currFunctionNode.getEndNode().getID()))
							pi.closedPaths.put(currFunctionNode.getEndNode().getID(), pi.closedPaths
									.remove(currFunctionNode.getEndNode().getID())
									+ pi.nrPaths);
						else
							pi.closedPaths.put(currFunctionNode.getEndNode().getID(), pi.nrPaths);
						// break und return werden zu ihrem Zielknoten verbunden
					} else {
						if (pi.closedPaths.containsKey(tn.getID()))
							pi.closedPaths.put(tn.getID(), pi.closedPaths.remove(tn.getID()) + pi.nrPaths);
						else
							pi.closedPaths.put(tn.getID(), pi.nrPaths);
					}
					pi.nrPaths = 0;
					return pi;
				}
				// 2. JumpNode beendet nur einen CompoundBlock oder springt zu
				// einem Label im Suchbereich
				n = ((ASTJumpNode) n).getTargetNode();

			} else if (n instanceof ASTCompoundNode && !((ASTCompoundNode) n).isVirtual()
					&& ((ASTCompoundNode) n).isStartNode() && ((ASTCompoundNode) n).getEndNode().getID() < end.getID()) {
				endNode = ((ASTCompoundNode) n).getEndNode();
				pi = joinSequentialModBIPaths(pi, computeModBIPaths(n.getNextNode(), endNode), endNode);
				n = endNode;
				// Behandlung von geteilten Statements (inkl. innerer Klasendefinition)
			} else if (n instanceof ASTStatementNode && ((ASTStatementNode) n).isSepareted())
				n = ((ASTStatementNode) n).getEndOfStatement();
			else
				n = n.getNextNode();
		}
		return pi;
	}

	/**
	 * Vereinigt die Pfadinfos zweier paralleler Pfade (true-false bei If, case bei Switch), addiert Pfade zusammen.
	 * 
	 * @param first
	 *            ein Pfad
	 * @param second
	 *            ein paralleler Pfad
	 * @return die Vereinigung der Pfade
	 */
	protected static ModBIPathInfo joinParallelModBIPaths(ModBIPathInfo first, ModBIPathInfo second) {
		if (VERBOSE)
			System.out.println("----- joinParallelModBIPaths -----" + first.stringValue() + " + "
					+ second.stringValue());

		// alle weiterhin geschlossenen Pfade vom ersten auf zweiten Pfadverlauf
		// bertragen
		for (Integer i : second.closedPaths.keySet()) {
			if (first.closedPaths.containsKey(i))
				first.closedPaths.put(i, first.closedPaths.remove(i) + second.closedPaths.get(i));
			else
				first.closedPaths.put(i, second.closedPaths.get(i));
		}

		// alle continue-Pfade von ersten auf zweiten Pfadverlauf bertragen
		for (Integer i : second.continuedPaths.keySet()) {
			if (first.continuedPaths.containsKey(i))
				first.continuedPaths.put(i, first.continuedPaths.remove(i) + second.continuedPaths.get(i));
			else
				first.continuedPaths.put(i, second.continuedPaths.get(i));
		}

		transferLoopMaps(first, second);

		first.nrPaths += second.nrPaths;

		if (VERBOSE)
			System.out.println(" = " + first.stringValue());

		return first;
	}

	/**
	 * Vereingt die Pfadinfos zweier aufeinanderfolgender Pfadverlufe an einem Endknoten. Offene Pfade zum Endknoten
	 * werden dort wieder eingefgt.
	 * 
	 * @param head
	 *            PfadInfo des Ursprungspfades
	 * @param tail
	 *            PfadInfo des Teilpfades
	 * @param endNode
	 *            der vereinigende Knoten des verzweigenden Konstrukts
	 * @return die vereinigte PfadInfo
	 */
	protected static ModBIPathInfo joinSequentialModBIPaths(ModBIPathInfo head, ModBIPathInfo tail, ASTNode endNode) {
		if (VERBOSE)
			System.out.println("----- joinSequentialPaths -----" + head.stringValue() + " -> " + tail.stringValue());

		// wenn abgeschlossenen Pfade im Tail auf den endNode verweisen, dann
		// diese wieder zu den offenen Pfaden hinzufgen
		if (!tail.closedPaths.isEmpty() && tail.closedPaths.keySet().contains(endNode.getID()))
			tail.nrPaths += tail.closedPaths.remove(endNode.getID());

		// alle weiterhin geschlossenen Pfade vom Tail auf Head bertragen,
		// dabei mit den
		// offenen Pfaden des Heads multiplizieren!
		for (Integer i : tail.closedPaths.keySet()) {
			if (head.closedPaths.containsKey(i))
				head.closedPaths.put(i, head.closedPaths.remove(i) + head.nrPaths * tail.closedPaths.get(i));
			else
				head.closedPaths.put(i, head.nrPaths * tail.closedPaths.get(i));
		}

		// alle continue-Pfade von Tail auf Head weitergeben, dabei mit den
		// offenen Pfaden des Heads multiplizieren!
		for (Integer i : tail.continuedPaths.keySet()) {
			if (head.continuedPaths.containsKey(i))
				head.continuedPaths.put(i, head.continuedPaths.remove(i) + head.nrPaths * tail.continuedPaths.get(i));
			else
				head.continuedPaths.put(i, head.nrPaths * tail.continuedPaths.get(i));
		}

		transferLoopMaps(head, tail);

		// offene Pfade vom Tail zum Head hinzufgen, d.h. multiplizieren
		head.nrPaths *= tail.nrPaths;

		if (VERBOSE)
			System.out.println(" =  " + head.stringValue());

		// alles Head und Tail vereinigt, Head weitergeben
		return head;
	}

	/**
	 * Vereingt die Pfadinfos der vereinigten Pfade von Try + allen Catches mit den folgenden finally-Pfaden. Ruft
	 * joinSequentialPaths() mit gleichen Parametern auf, aber sorgt vorher dafr, dass alle Sprungpfade aus dem
	 * Try-Konstrukt auch durch den finallyNode laufen.
	 * 
	 * @param head
	 *            PfadInfo des Ursprungspfades (Try-Catch)
	 * @param tail
	 *            PfadInfo des Finally-Pfades
	 * @param endNode
	 *            der vereinigende Knoten des Try-Konstrukts
	 * @return vereinigte PfadInfo
	 */
	protected static ModBIPathInfo joinExceptionModBIPaths(ModBIPathInfo head, ModBIPathInfo tail, ASTNode endNode) {
		if (VERBOSE)
			System.out.println("----- joinExceptionPaths -----" + head.stringValue() + " -> " + tail.stringValue());

		// alle Pfade, auch die breaks und returns laufen durch finally durch,
		// d.h. alle mssen mit Pfadanzahl im finallyNode multipliziert werden
		HashMap<Integer, Long> temp = new HashMap<Integer, Long>();
		for (Integer i : head.closedPaths.keySet())
			temp.put(i, head.closedPaths.get(i) * tail.nrPaths);
		head.closedPaths = temp;

		temp = new HashMap<Integer, Long>();
		for (Integer i : head.continuedPaths.keySet())
			temp.put(i, head.continuedPaths.remove(i) * tail.nrPaths);
		head.continuedPaths = temp;

		transferLoopMaps(head, tail);

		return joinSequentialModBIPaths(head, tail, endNode);

	}

	/**
	 * bertrgt die loopMaps von einer PathInfo auf die nchste, spart doppelte aus. (Switch-Catch ohne Break knnen
	 * Schleifen mehrmals auswerten)
	 * 
	 * @param first
	 *            die erste PathInfo
	 * @param second
	 *            die zweite PathInfo
	 * @return die erste PathInfo mit den Aufsummierten loopMaps der zweiten
	 */
	protected static ModBIPathInfo transferLoopMaps(ModBIPathInfo first, ModBIPathInfo second) {

		for (Integer i : second.singleLoopMap.keySet())
			if (!first.singleLoopMap.containsKey(i))
				first.singleLoopMap.put(i, second.singleLoopMap.get(i));
		for (Integer i : second.multipleLoopMap.keySet())
			if (!first.multipleLoopMap.containsKey(i))
				first.multipleLoopMap.put(i, second.multipleLoopMap.get(i));
		return first;
	}

	/************************************************************************************/
	/* Pfadberdeckungsauswertung */
	/************************************************************************************/

	/**
	 * Erstellt fr alle Pfade des aktuellen TestLogs die ModBiPfade.
	 * 
	 * @param ast
	 *            der aktuelle ASTManager
	 */
	public static void createCoveredModBIPathsSet(ASTManager ast) {
		if (VERBOSE_COVER)
			System.out.println("Begin Creating ModBIPathsSets.");
		for (TestLog t : ast.getConfiguration().getListOfTestLogs()) {
			for (TestCase tc : t.getListOfTestCases())
				for (ArrayList<Integer> list : tc.getListOfPaths()) {
					if (list.size() == 0)
						continue;
					createCoveredModBIPathsSetForFunction(ast, tc, list);
				}
		}
		if (VERBOSE_COVER)
			System.out.println("End of Creating ModBIPathsSets.");
	}

	/**
	 * Erstellt fr einen Pfad die ModBIPfade, d.h. fr den Funktion und fr die Iterationen. Ein Pfad wird durch eine
	 * long-Zahl reprsentiert.
	 * 
	 * @param ast
	 *            der ASTManager
	 * @param testCase
	 *            das entsprechende TestLog
	 * @param list
	 *            die Pfadliste
	 */

	protected static void createCoveredModBIPathsSetForFunction(ASTManager ast, TestCase testCase,
			ArrayList<Integer> list) {

		if (list.size() < 2)
			return;

		ASTFunctionNode functionNode = ast.getNodeByID(list.get(1)).getFunctionNode();
		ModBIPath modbiPath = new ModBIPath();

		ASTNode lastDecision = functionNode.getEntryNode();

		if (VERBOSE2)
			System.out.println("\n Convert to BIPaths");

		if (VERBOSE2) {
			System.out.println(list);
			System.out.println(pathToString(list));
		}

		for (int index = 2; index < list.size(); index++) {

			ASTNode indexNode = ast.getNodeByID(list.get(index));

			// nichtverzweigende Knoten uninteressant (Level3)
			if (indexNode.getControlingBranchNode() == null)
				continue;

			ASTBranchNode branchNode = indexNode.getControlingBranchNode();
			ASTDoNode doNode = null;
			if (branchNode instanceof ASTIterationNode && ((ASTIterationNode) branchNode).hasDoNode())
				doNode = ((ASTIterationNode) branchNode).getDoNode();

			// suche Entscheidung ohne geloggten Knoten (Switch ohne case, try ohne catc)
			// zwischen letzter und nchster Entscheidung:
			if (lastDecision != null) {
				ASTNode n = lastDecision;
				while (n != branchNode && n != null && n != doNode) {
					if (n instanceof ASTJumpNode)
						n = ((ASTJumpNode) n).getTargetNode();
					else if (n instanceof ASTStatementNode && ((ASTStatementNode) n).isSepareted())
						n = ((ASTStatementNode) n).getEndOfStatement();
					/*
					 * else if(n instanceof ASTTryNode) { int nrBranches = ((ASTTryNode)n).getListOfCatches().size() +
					 * (((ASTTryNode)n).hasFinally() ? 0 : 1); // Entscheidungsbreite aufaddieren int width = (int)
					 * Math.ceil(Math.log(nrBranches) / Math.log(2)); addDecisionToModBIPath(modbiPath, 0, width); }
					 */else if (n instanceof ASTCatchNode && !((ASTCatchNode) n).isFinally()) {
						if (((ASTCatchNode) n).getTryNode().hasFinally())
							n = ((ASTCatchNode) n).getTryNode().getFinallyNode();
						else
							n = ((ASTCatchNode) n).getTryNode().getEndNode();
					} else if (n instanceof ASTSwitchNode) {
						int nrBranches = ((ASTSwitchNode) n).getListOfCases().size()
								+ (((ASTSwitchNode) n).hasDefault() ? 0 : 1);

						if (VERBOSE2)
							Logger.logln("Switch: " + 0 + " / " + nrBranches);
						// decision ist 0 wenn Switch bersprungen, sonst Case-zahl

						// Entscheidungsbreite aufaddieren
						int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
						addDecisionToModBIPath(modbiPath, 0, width);
						n = ((ASTSwitchNode) n).getEndNode();
					} else if (n instanceof ASTIterationNode) {
						// Iteration muss nicht geloggt werden
						// addDecisionToModBIPath(modbiPath, 0, 1);
						n = ((ASTIterationNode) n).getEndNode();
					} else {
						n = n.getNextNode();
					}
				}
			}

			lastDecision = branchNode;

			if (!modbiPath.isStillFunctionPath) {
				if (branchNode instanceof ASTIterationNode)
					createCoveredModBIPathsSetForIteration(ast, testCase, list, index);
				continue;
			}

			// binre Entscheidung bei If
			if (branchNode instanceof ASTIfNode) {
				ASTIfNode in = (ASTIfNode) branchNode;

				if (indexNode == in.getTrueNode()) {
					if (VERBOSE2)
						Logger.logln("If-True " + indexNode.getID());
					addDecisionToModBIPath(modbiPath, 0, 1);
					continue;
				} else if (indexNode == in.getFalseNode()) {
					if (VERBOSE2)
						Logger.logln("If-False " + indexNode.getID());
					addDecisionToModBIPath(modbiPath, 1, 1);
					continue;
				}
			}

			// breite Entscheidung bei Switch
			if (branchNode instanceof ASTSwitchNode) {
				ASTSwitchNode sn = (ASTSwitchNode) branchNode;

				// Anzahl mglicher Verzweigungen
				int nrBranches = sn.getListOfCases().size() + (sn.hasDefault() ? 0 : 1);
				int decision = 0;
				int nrCases = 0;
				// nchste ID muss ein Case sein, oder ende von Switch
				for (ASTCaseNode cn : sn.getListOfCases()) {
					nrCases++;
					if (cn == indexNode) {
						// CaseNode gefunden
						decision = nrCases;
						break;
					}

				}
				if (VERBOSE2)
					Logger.logln("Switch: " + decision + " / " + nrBranches);
				// decision ist 0 wenn Switch bersprungen, sonst Case-zahl

				// Entscheidungsbreite aufaddieren
				int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
				addDecisionToModBIPath(modbiPath, decision, width);
				continue;

				// breite Entscheidung bei Try
			}

			if (branchNode instanceof ASTTryNode) {
				ASTTryNode tn = (ASTTryNode) branchNode;
				ArrayList<ASTCatchNode> listOfCatches = tn.getListOfCatches();

				// Anzahl mglicher Verzweigungen
				int nrBranches = listOfCatches.size() + (tn.hasFinally() ? 0 : 1);
				int decision = 0;
				int nrCatches = 0;

				for (ASTCatchNode cn : listOfCatches) {
					nrCatches++;
					if (cn == indexNode) {
						// Durchlauf = 0 als Entscheidung
						if (!cn.isFinally())
							decision = nrCatches;
					}
				}

				if (VERBOSE2)
					Logger.logln("Try: " + decision + " / " + nrBranches);

				// Entscheidungsbreite aufaddieren
				int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
				addDecisionToModBIPath(modbiPath, decision, width);
				continue;
			}
			// falls LoopNode - entscheide ob berhaupt noch Funktionspfad
			if (branchNode instanceof ASTIterationNode) {
				ASTIterationNode in = (ASTIterationNode) branchNode;
				if (in.hasDoNode()) {
					for (int j = index + 1; j < list.size(); j++) {
						// falls noch ein Loop auftaucht, dann kein Funtionspfad
						if (ast.getNodeByID(list.get(j)) == in.getLoopNode())
							modbiPath.isStillFunctionPath = false;
						// wenn Iteration verlassen wurde, dann schon
						if (!in.isWithinBranchConstruct(list.get(j)))
							break;
					}
				} else {
					// ansonsten auf jeden fall kein Funktionspfad mehr
					modbiPath.isStillFunctionPath = false;
				}
				if (!modbiPath.isStillFunctionPath)
					index = createCoveredModBIPathsSetForIteration(ast, testCase, list, index);
				continue;
			}
		}
		// printBIPath(functionNode.getID(), biPath);
		if (modbiPath.isStillFunctionPath)
			testCase.addCoveredModBIPath(functionNode.getID(), modbiPath.path);
		// printBIPath(functionID, biPath);

		return;

	}

	/**
	 * Erstellt fr eine abweisende Schleife die ModBIPfade.
	 * 
	 * @param ast
	 *            der ASTManager
	 * @param testCase
	 *            das entsprechende TestLog
	 * @param list
	 *            die Pfadliste
	 * @param start
	 *            der Indexnoten, der die Schleife beginnt
	 * @param return der neue Indexknoten,hinter der Schleife
	 * */
	protected static int createCoveredModBIPathsSetForIteration(ASTManager ast, TestCase testCase,
			ArrayList<Integer> list, int start) {

		ASTNode indexNode = ast.getNodeByID(list.get(start));
		ModBIPath modBIPath = new ModBIPath();
		ASTIterationNode in;
		ASTNode lastDecision = null;
		int loopCounter = 0;
		int index = start;
		if (indexNode.getControlingBranchNode() instanceof ASTIterationNode)
			in = (ASTIterationNode) indexNode.getControlingBranchNode();
		else
			return start;

		for (; index < list.size(); index++) {

			indexNode = ast.getNodeByID(list.get(index));
			// nichtverzweigende Knoten uninteressant (Level3)
			if (indexNode.getControlingBranchNode() == null)
				continue;

			// falls LoopNode kommt, dann hochzhlen
			if (indexNode.getControlingBranchNode() == in) {
				loopCounter++;
				// falls mehr als 2 Loops oder Iteration verlassen, dann Pfad fertig
				if (loopCounter > 2)
					break;

				addDecisionToModBIPath(modBIPath, 1, 1);
				continue;
			}

			if (!in.isWithinBranchConstruct(indexNode.getID())) {
				break;
			}

			ASTBranchNode branchNode = indexNode.getControlingBranchNode();
			ASTDoNode doNode = null;
			if (branchNode instanceof ASTIterationNode && ((ASTIterationNode) branchNode).hasDoNode())
				doNode = ((ASTIterationNode) branchNode).getDoNode();

			// suche Entscheidung ohne geloggten Knoten (Switch ohne case, try ohne catc)
			// zwischen letzter und nchster Entscheidung:
			if (lastDecision != null) {
				ASTNode n = lastDecision;
				while (n != branchNode && n != null && n != doNode) {
					if (n instanceof ASTJumpNode)
						n = ((ASTJumpNode) n).getTargetNode();
					else if (n instanceof ASTStatementNode && ((ASTStatementNode) n).isSepareted())
						n = ((ASTStatementNode) n).getEndOfStatement();
					/*
					 * else if(n instanceof ASTTryNode) { int nrBranches = ((ASTTryNode)n).getListOfCatches().size() +
					 * (((ASTTryNode)n).hasFinally() ? 0 : 1); // Entscheidungsbreite aufaddieren int width = (int)
					 * Math.ceil(Math.log(nrBranches) / Math.log(2)); addDecisionToModBIPath(modbiPath, 0, width); }
					 */else if (n instanceof ASTCatchNode && !((ASTCatchNode) n).isFinally()) {
						if (((ASTCatchNode) n).getTryNode().hasFinally())
							n = ((ASTCatchNode) n).getTryNode().getFinallyNode();
						else
							n = ((ASTCatchNode) n).getTryNode().getEndNode();
					} else if (n instanceof ASTSwitchNode) {
						int nrBranches = ((ASTSwitchNode) n).getListOfCases().size()
								+ (((ASTSwitchNode) n).hasDefault() ? 0 : 1);

						if (VERBOSE2)
							Logger.logln("Switch: " + 0 + " / " + nrBranches);
						// decision ist 0 wenn Switch bersprungen, sonst Case-zahl

						// Entscheidungsbreite aufaddieren
						int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
						addDecisionToModBIPath(modBIPath, 0, width);
						n = ((ASTSwitchNode) n).getEndNode();
					} else if (n instanceof ASTIterationNode) {
						n = ((ASTIterationNode) n).getEndNode();
					} else {
						n = n.getNextNode();
					}
				}
			}

			lastDecision = branchNode;

			// binre Entscheidung bei If
			if (branchNode instanceof ASTIfNode) {
				ASTIfNode ifn = (ASTIfNode) branchNode;

				if (indexNode == ifn.getTrueNode()) {
					if (VERBOSE2)
						Logger.logln("If-True " + indexNode.getID());
					addDecisionToModBIPath(modBIPath, 0, 1);
					continue;
				} else if (indexNode == ifn.getFalseNode()) {
					if (VERBOSE2)
						Logger.logln("If-False " + indexNode.getID());
					addDecisionToModBIPath(modBIPath, 1, 1);
					continue;
				}
			}

			// breite Entscheidung bei Switch
			if (branchNode instanceof ASTSwitchNode) {
				ASTSwitchNode sn = (ASTSwitchNode) branchNode;

				// Anzahl mglicher Verzweigungen
				int nrBranches = sn.getListOfCases().size() + (sn.hasDefault() ? 0 : 1);
				int decision = 0;
				int nrCases = 0;
				// nchste ID muss ein Case sein, oder ende von Switch
				for (ASTCaseNode cn : sn.getListOfCases()) {
					nrCases++;
					if (cn == indexNode) {
						// CaseNode gefunden
						decision = nrCases;
						break;
					}

				}
				if (VERBOSE2)
					Logger.logln("Switch: " + decision + " / " + nrBranches);
				// decision ist 0 wenn Switch bersprungen, sonst Case-zahl

				// Entscheidungsbreite aufaddieren
				int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
				addDecisionToModBIPath(modBIPath, decision, width);
				continue;

				// breite Entscheidung bei Try
			}

			if (branchNode instanceof ASTTryNode) {
				ASTTryNode tn = (ASTTryNode) branchNode;
				ArrayList<ASTCatchNode> listOfCatches = tn.getListOfCatches();

				// Anzahl mglicher Verzweigungen
				int nrBranches = listOfCatches.size() + (tn.hasFinally() ? 0 : 1);
				int decision = 0;
				int nrCatches = 0;

				for (ASTCatchNode cn : listOfCatches) {
					nrCatches++;
					if (cn == indexNode) {
						// Durchlauf = 0 als Entscheidung
						if (!cn.isFinally())
							decision = nrCatches;
					}
				}

				if (VERBOSE2)
					Logger.logln("Try: " + decision + " / " + nrBranches);

				// Entscheidungsbreite aufaddieren
				int width = (int) Math.ceil(Math.log(nrBranches) / Math.log(2));
				addDecisionToModBIPath(modBIPath, decision, width);
				continue;
			}
			// falls LoopNode - entscheide ob berhaupt noch Funktionspfad
			if (branchNode instanceof ASTIterationNode) {
				index = createCoveredModBIPathsSetForIteration(ast, testCase, list, index);
				lastDecision = null;
			}
		}
		if (in.hasDoNode() && loopCounter < 2)
			return index;
		testCase.addCoveredModBIPath(in.getID(), modBIPath.path);
		return index;
	}

	/**
	 * Fgt eine Entscheidung an den BIPfad an.
	 * 
	 * @param modBIPath
	 *            der aktuelle BIPfad
	 * @param i
	 *            die Entscheidung binr kodiert , 0/1 bei einfacher, sonst lnger
	 */
	protected static void addDecisionToModBIPath(ModBIPath modBIPath, int i, int width) {
		if (VERBOSE3)
			printModBIPath(modBIPath);
		modBIPath.path = modBIPath.path | (i << modBIPath.nrDecisions);
		if (VERBOSE3) {
			String s = "";
			for (int j = 0; j < modBIPath.nrDecisions; j++) {
				if ((j % 8) == 0)
					s += " ";
				s += " ";
			}
			System.out.println(s + "<- adding: " + i);
			printModBIPath(modBIPath);
		}
		modBIPath.nrDecisions += width;
	}

	/**
	 * Kommandozeilenausgabe eines ModBIPath.
	 * 
	 * @param p
	 *            der Pfad
	 */
	protected static void printModBIPath(ModBIPath p) {
		int z = 1;
		int c = 0;
		for (int j = 0; j < 64; j++) {
			if ((c % 8) == 0)
				System.out.print(" ");
			System.out.print(((p.path & z) == 0) ? "0" : "1");
			z = z << 1;
			c++;
		}
		System.out.println();
	}

	/**
	 * Erzeugt aus einer Liste aus Knoten-IDs einen informativen String ber den Pfad.
	 * 
	 * @param l
	 *            die Liste der IDs
	 * @return der erstellte String
	 */
	protected static String pathToString(ArrayList<Integer> l) {
		List<ASTNode> al = Application.getInstance().getASTManager().getAllNodes();
		HashSet<ASTNode> trues = new HashSet<ASTNode>();
		HashSet<ASTNode> falses = new HashSet<ASTNode>();
		HashSet<ASTNode> loops = new HashSet<ASTNode>();
		HashSet<ASTNode> iexits = new HashSet<ASTNode>();
		String s = "[";

		for (Integer i : l) {
			ASTNode n = al.get(i);
			if (n instanceof ASTIterationNode) {
				s += "It (" + i + "), ";
				loops.add(((ASTIterationNode) n).getLoopNode());
				iexits.addAll(((ASTIterationNode) n).getSetOfExitNodes());
			} else if (n instanceof ASTIfNode) {
				s += "If (" + i + "), ";
				trues.add(((ASTIfNode) n).getTrueNode());
				falses.add(((ASTIfNode) n).getFalseNode());
			} else if (n instanceof ASTSwitchNode) {
				s += "Sw (" + i + "), ";
			} else if (n instanceof ASTCaseNode) {
				s += "Cs (" + i + "), ";
			} else if (n instanceof ASTCatchNode) {
				s += "Ct (" + i + "), ";
			} else if (n instanceof ASTTryNode) {
				s += "Ty (" + i + "), ";
			} else if (n instanceof ASTFunctionNode) {
				s += "Fu (" + i + "), ";
			} else if (trues.contains(n)) {
				s += "tr (" + i + "), ";
			} else if (falses.contains(n)) {
				s += "fa (" + i + "), ";
			} else if (iexits.contains(n)) {
				s += "x (" + i + "), ";
			} else if (loops.contains(n)) {
				s += "l (" + i + "), ";
			}
		}
		return s + "]";
	}

}
