package sota.ast.util;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Scanner;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.ui.PlatformUI;

import sota.Logger;
import sota.app.Application;
import sota.gui.preferences.PreferenceConstants;

/**
 * Kompiliert ein Projekt anhand einer Ant-Skriptdatei mit einem Progressmonitor.
 * 
 */
public class ProjectBuilder implements IRunnableWithProgress {

	boolean flag[];

	/**
	 * Der Konstruktor, der eine Flag zum Abbruch des ProgressMonitors erhlt.
	 * 
	 * @param flag
	 *            die Flag
	 */
	public ProjectBuilder(boolean flag[]) {
		this.flag = flag;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		File scriptFile = null;
		try {
			monitor.beginTask("Building Project", IProgressMonitor.UNKNOWN);
			Logger.logln("\t Building project ...");
			String antLocation = PlatformUI.getPreferenceStore().getString(PreferenceConstants.GENERAL_ANT_LOCATION);

			scriptFile = new File(antLocation);
			if (!scriptFile.exists()) {
				Logger.logln("\t Ant not found at: " + antLocation);
				Application.getInstance().getConfiguration().exceptionMessage = "Failed to build project.\n\n"
						+ "\t Ant not found at: " + antLocation;
				return;
			}

			ProcessBuilder processBuilder = new ProcessBuilder(scriptFile.getAbsolutePath(), "clean", "build", "-f",
					Application.getInstance().getConfiguration().antBuildFile);
			processBuilder.directory(scriptFile.getParentFile());
			processBuilder.redirectErrorStream(true);
			Process process = processBuilder.start();

			Scanner scanner = new Scanner(process.getInputStream());
			while (scanner.hasNextLine()) {
				// System.out.println(flag[0]);
				if (flag.length > 0 && flag[0]) {
					throw new InterruptedException();
				}
				Logger.logln(scanner.nextLine());
			}
			scanner.close();
			Logger.logln(process.waitFor());

		} catch (IOException e1) {
			Logger.logln("ioexception: " + e1.toString());
			Application.getInstance().getConfiguration().exceptionMessage = "Failed to build project with:\n\n"
					+ scriptFile.getAbsolutePath() + " clean build -f "
					+ Application.getInstance().getConfiguration().antBuildFile + ".\n\n" + e1.getMessage();
		} catch (InterruptedException e) {
			Logger.logln("interrupted exception: " + e.toString());
			/*
			 * Application.getInstance().getConfiguration().exceptionMessage = "Failed to build project with:\n\n" +
			 * scriptFile.getAbsolutePath() + " clean build -f " +
			 * Application.getInstance().getConfiguration().antBuildFile + ".\n\n" + e.getMessage();
			 */
		} finally {
			monitor.done();
		}
	}
}