package sota.ast.util;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Scanner;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;

import sota.Logger;
import sota.app.Application;

/**
 * Startet ein Projekt anhand eines bergebenes Startskript mit einem Progressmonitor.
 */
public class ProjectRunner implements IRunnableWithProgress {

	// erlaubt den Abbruch durch den Progressmonitor
	private boolean[] flag;

	/**
	 * Der Konstruktor.
	 * 
	 * @param flag
	 *            die Abbruchflag (momentan ungenutzt)
	 */
	public ProjectRunner(boolean[] flag) {
		this.flag = flag;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {

		try {
			ProjectConfig config = Application.getInstance().getConfiguration();
			monitor.beginTask("Running Project", IProgressMonitor.UNKNOWN);
			String arguments[] = { config.runScript, config.runArgs};

			Logger.logln("\t Running project ...");
			ProcessBuilder processBuilder = new ProcessBuilder(arguments);
			processBuilder.directory(new File(Application.getInstance().getConfiguration().runScript).getParentFile());
			Process process = processBuilder.start();
			Scanner scanner = new Scanner(process.getInputStream());
			while (scanner.hasNextLine()) {
				Logger.logln(scanner.nextLine());
			}
			scanner.close();
			Logger.logln(process.waitFor());

		} catch (IOException e1) {
			Logger.logln("ioexception: " + e1.toString());
			Application.getInstance().getConfiguration().exceptionMessage = "Failed to run project with:\n\n"
					+ Application.getInstance().getConfiguration().runScript + "\n\n" + e1.getMessage();
		} catch (InterruptedException e) {
			Logger.logln("interrupted exception: " + e.toString());
		} finally {
			monitor.done();
		}
	}

}