package sota.ast.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Formatter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.PlatformUI;

import sota.Logger;
import sota.app.Application;
import sota.ast.nodes.ASTClassNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.FileNode;
import sota.gui.preferences.PreferenceConstants;

/**
 * Erstellt und formatiert den Report im Html-Format anhand der spezifizierten Testlogs und der Einstellungen in den
 * Prferenzen.
 * 
 */
public class ReportCreator {
	static ASTManager ast;

	static String s;

	static int fontSize;
	static boolean useAllTests, overwriteReport, useColors, showTests, showFunctions, showClasses, showInnerClasses;

	/**
	 * Erstellt den Report anhand der bergebenen Daten.
	 * 
	 * @param _ast
	 *            der ASTManager
	 * @param fileName
	 *            die Reportdatei
	 * @param _fontSize
	 *            die Fontgre
	 * @param _useAllTests
	 *            ob alle Testlogs genutzt werden sollen, oder lediglich die ausgewhlten
	 * @param _overwriteReport
	 *            ob die Reportdatei berschrieben oder ein neuer Name generiert werden soll
	 * @param _useColors
	 *            ob der Report farbig sein soll
	 * @param _showTests
	 *            ob die Tests aufgelistet werden sollen
	 * @param _showFunctions
	 *            ob die Funktionsbersicht angezeigt werden soll
	 * @param _showClasses
	 *            ob die Klassenbersicht angezeigt werdens oll
	 * @param _showInnerClasses
	 *            ob innere Klasen angezeigt werden sollen
	 * @return true, wenn erfolgreich, sonst false
	 */
	public static boolean createReport(ASTManager _ast, String fileName, int _fontSize, boolean _useAllTests,
			boolean _overwriteReport, boolean _useColors, boolean _showTests, boolean _showFunctions,
			boolean _showClasses, boolean _showInnerClasses) {
		ast = _ast;
		useAllTests = _useAllTests;
		overwriteReport = _overwriteReport;
		fontSize = _fontSize;
		useColors = _useColors;
		showTests = _showTests;
		showFunctions = _showFunctions;
		showClasses = _showClasses;
		showInnerClasses = _showInnerClasses;

		String reportFile;
		String path = ast.getConfiguration().projectDir + "\\";
		try {
			if (fileName != null) {
				reportFile = fileName.substring(fileName.lastIndexOf("\\") + 1);
				path = fileName.substring(0, fileName.lastIndexOf("\\") + 1);
			} else if (overwriteReport)
				reportFile = "report.html";
			else {
				Date dt = new Date();
				SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");

				reportFile = "report_" + df.format(dt);

				if (new File(path + reportFile + ".html").exists()) {
					int i = 1;
					while (new File((path + reportFile + "_" + i + ".html")).exists())
						i++;
					reportFile = (reportFile + "_" + i + ".html");
				} else {
					reportFile = reportFile + ".html";
				}
			}

			Logger.log("\t Writing Report to " + path + reportFile + " ... ");
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(path + reportFile)));

			s = createHeader();
			out.write(s, 0, s.length());

			s = createProjectTable();
			out.write(s, 0, s.length());

			if (showTests) {
				s = createTestLogTable();
				out.write(s, 0, s.length());
			}

			if (showClasses) {

				s = createClassTable();
				out.write(s, 0, s.length());
			}

			if (showFunctions) {

				s = "<BR><H2> Detailed Coverage </H2>";
				out.write(s, 0, s.length());
				for (FileNode f : ast.getFileNodes())
					for (ASTClassNode c : f.getClassNodes()) {

						if (c.isTopLevel() || showInnerClasses) {
							s = createTable(c.getIDString());
							s += createTargetClassLine(c);
							for (ASTFunctionNode fn : c.getFunctionNodes()) {
								s += createFunctionLine(fn);
							}
							s += createTableEnd();
							out.write(s, 0, s.length());
						}
					}
			}

			s = createEnd();
			out.write(s, 0, s.length());
			out.close();
			Logger.log("OK\n");
		} catch (IOException e) {
			Logger.log("FAILED\n");
			return false;
		}
		return true;
	}

	/**
	 * Erstellt den Header der Reportdatei.
	 * 
	 * @return der erstellte String
	 */
	private static String createHeader() {
		return "<HTML><HEAD><META HTTP-EQUIV=\"CONTENT-TYPE\" CONTENT=\"text/html; charset=windows-1252\">\n"
				+ "<TITLE>SOTA Coverage Report</TITLE>\n"
				+ "<STYLE TYPE=\"text/css\">\n"
				+ " TABLE.i {border-color:black; border-style:none;  border-width:0px 0px 0px 0px; width:100%; padding=\"10\"; empty-cells:show; border-collapse:collapse;}\n"
				+ " TABLE.k {border-color:black; border-style:solid; border-width:1px 1px 1px 1px; width:100%; padding=\"10\"; empty-cells:show; border-collapse:collapse;}\n"
				+ " TD.tl   {border-color:black; border-style:solid; border-width:1px 1px 0px 1px; text-align:left;}\n"
				+ " TD.l    {border-color:black; border-style:solid; border-width:0px 1px 0px 1px; text-align:left;}\n"
				+ " TD.bl   {border-color:black; border-style:solid; border-width:0px 1px 1px 1px; text-align:left;}\n"
				+ " TD.tr   {border-color:black; border-style:solid; border-width:1px 1px 0px 0px; text-align:right;}\n"
				+ " TD.r    {border-color:black; border-style:solid; border-width:0px 1px 0px 0px; text-align:right;}\n"
				+ " TD.br   {border-color:black; border-style:solid; border-width:0px 1px 1px 0px; text-align:right;}\n"
				+ " TD.c    {border-color:black; border-style:solid; border-width:1px 1px 1px 1px; text-align:center;}\n"
				+ " TD.fl   {border-color:black; border-style:solid; border-width:1px 1px 1px 1px; text-align:left;}\n"
				+ " P,H1,H2,H3,TH {font-family:verdana,arial,sans-serif;}\n"
				+ " P {font-size:"
				+ fontSize
				+ "pt;}\n"
				+ " H1 {font-size:"
				+ (fontSize + 2)
				+ "pt; font-weight:bold; }\n "
				+ " H2 {font-size:"
				+ (fontSize + 2)
				+ "pt;}\n"
				+ "</STYLE></HEAD>\n"
				+ "<BODY LANG=\"de-DE\">\n"
				+ "<TABLE style=\"width:100%; border-color:#000000; padding:0; border-spacing:0px;border-top-width:2px;  border-bottom-width:2px; border-top-style:solid; border-bottom-style:solid;\">\n"
				+ "<TR><TD style=\"width:100%; color:black; background-color:" + rgb2hex(210, 210, 255) + ";\">\n"
				+ "<H1 style=\"margin-left: 0.5cm; font-size: 12pt\" >SOTA Coverage Report</H1></TD></TR></TABLE>\n";
	}

	/**
	 * Erstellt das Ende der Reportdatei.
	 * 
	 * @return der erstellte String
	 */
	private static String createEnd() {
		return "<P>\n<BR>\n<BR>\n</P>\n</BODY>\n	</HTML>\n";
	}

	/**
	 * Erstellt ein Tabellenende.
	 * 
	 * @return der erstellte String
	 */
	private static String createTableEnd() {
		return "</TABLE>\n\n";
	}

	/**
	 * Erstellt die Tabelle fr die TestLogs.
	 * 
	 * @return der erstellte String
	 */
	private static String createTestLogTable() {
		boolean firstLine = true;
		String iSd = "";
		Collection<TestLog> l;
		if (useAllTests)
			l = ast.getConfiguration().getListOfTestLogs();
		else
			l = ast.getConfiguration().listOfSelectedTestLogs;

		String s = "<BR>\n\n" + "<H2>Tests</H2>\n" + "<TABLE class=\"i\" cellpadding=4 cellspacing=0>\n"
				+ "<COLGROUP>\n" + "\t<COL WIDTH=\"20%\">\n" + "\t<COL WIDTH=\"80%\">\n" + "</COLGROUP>\n";

		for (TestLog t : l) {
			iSd = "";
			for (IScheme is : ast.getConfiguration().getListOfISchemes())
				if (is.getName().equals(t.getISchemeName())) {
					iSd = is.getDescription();
					break;
				}

			s += "<TR>\n" + "\t<TD class = \"" + (firstLine ? "tl" : "l") + "\" STYLE=\"background-color:"
					+ rgb2hex(240, 240, 255) + ";\">" + "<P>" + t.getName() + "</P></TD>\n" + "\t<TD class = \""
					+ (firstLine ? "tl" : "l") + "\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\">"
					+ "<P>" + t.getDescription() + "</P></TD>\n" + "<TR>\n" + "\t<TD class = \"bl\"><P>"
					+ t.getISchemeName() + "</P></TD>\n" + "\t<TD class = \"bl\"><P>" + iSd + "</P></TD>\n";
			firstLine = false;
		}
		return s + "</TR>\n</TABLE>\n";
	}

	/**
	 * Erstellt die Projektbersicht als Tabelle.
	 * 
	 * @return der erstellte String
	 */
	private static String createProjectTable() {
		Date dt = new Date();
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

		Metrics m = ast.getProjectMetrics();
		return "<BR>\n\n" + "<H2>Project: " + ast.getConfiguration().getName() + "</H2>\n" + "<P>created: "
				+ df.format(dt) + "</P>\n" + "<TABLE class=\"i\" cellpadding=4 cellspacing=0>\n" + "<COLGROUP>\n"
				+ "\t<COL WIDTH=\"20%\">\n" + "\t<COL WIDTH=\"8%\">\n" + "\t<COL WIDTH=\"3%\">\n"
				+ "\t<COL WIDTH=\"20%\">\n" + "\t<COL WIDTH=\"8%\">\n" + "</COLGROUP>\n" + "<TR>\n"
				+ "\t<TD class = \"tl\" STYLE=\"background-color:" + rgb2hex(240, 240, 255)
				+ ";\"><P>Function Entry-Exit Coverage (FEEC)</P></TD>\n"
				+ "\t<TD class = \"tr\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>"
				+ float2string(m.feec) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"tl\" STYLE=\"background-color:" + rgb2hex(240, 240, 255)
				+ ";\"><P># Files</P></TD>\n" + "\t<TD class = \"tr\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P>" + m.nrFiles + "</P></TD>\n" + "</TR>\n" + "<TR>\n"
				+ "\t<TD class = \"l\"><P>Statement Coverage (C0)</P></TD>\n" + "\t<TD class = \"r\"><P>"
				+ float2string(m.c0) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"l\"><P># Classes (TopLevel- + inner Classes)</P></TD>\n" + "\t<TD class = \"r\"><P>"
				+ m.nrClasses + " (" + (m.nrClasses - m.nrInnerClasses) + " + " + m.nrInnerClasses + ") "
				+ "</P></TD>\n" + "</TR>\n" + "<TR>\n" + "\t<TD class = \"l\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P>Decision Coverage (C1)</P></TD>\n"
				+ "\t<TD class = \"r\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>"
				+ float2string(m.c1) + "</P></TD>\n" + "\t<TD></TD>" + "\t<TD class = \"l\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P># Functions</P></TD>\n"
				+ "\t<TD class = \"r\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>" + m.nrFunctions
				+ "</P></TD>" + "</TR>\n" + "<TR>\n" + "\t<TD class = \"l\"><P>Condition Coverage (C2)</P></TD>\n"
				+ "\t<TD class = \"r\"><P>" + float2string(m.c2) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"l\"><P># Lines</P></TD>\n" + "\t<TD class = \"r\"><P>" + m.nrLines + "</P></TD>"
				+ "</TR>\n" + "<TR>\n" + "\t<TD class = \"l\" STYLE=\"background-color:" + rgb2hex(240, 240, 255)
				+ ";\"><P>Minimal Multiple Decision Coverage (MMDC)</P></TD>\n"
				+ "\t<TD class = \"r\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>"
				+ float2string(m.mmdc) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"l\" STYLE=\"background-color:" + rgb2hex(240, 240, 255)
				+ ";\"><P># Statements</P></TD>\n" + "\t<TD class = \"r\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P>" + m.nrStatements + "</P></TD>" + "</TR>\n<TR>"
				+ "\t<TD class = \"l\"><P>Modified Condition Decision Coverage (MCDC)</P></TD>\n"
				+ "\t<TD class = \"r\"><P>" + float2string(m.mcdc) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"l\"><P># Conditions</P></TD>\n" + "\t<TD class = \"r\"><P>" + m.nrConditions
				+ "</P></TD>\n" + "</TR>\n" + "<TR>\n" + "\t<TD class = \"l\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P>Multiple Condition Coverage (C3)</P></TD>\n"
				+ "\t<TD class = \"r\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>"
				+ float2string(m.c3) + "</P></TD>\n" + "\t<TD></TD>" + "\t<TD class = \"l\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P> </P></TD>\n" + "\t<TD class = \"r\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P> </P></TD>" + "</TR>\n" + "<TR>\n"
				+ "\t<TD class = \"l\"><P>Modified Boundary-Interior Path Coverage (ModBI)</P></TD>\n"
				+ "\t<TD class = \"r\"><P>" + float2string(m.mbi) + "</P></TD>\n" + "\t<TD></TD>"
				+ "\t<TD class = \"l\"><P> </P></TD>\n" + "\t<TD class = \"r\"><P> </P></TD>\n" + "</TR>\n" + "<TR>\n"
				+ "<TD class = \"bl\" STYLE=\"background-color:" + rgb2hex(240, 240, 255)
				+ ";\"><P>Boundary-Interior Path Coverage (BI)</P></TD>\n"
				+ "<TD class = \"br\" STYLE=\"background-color:" + rgb2hex(240, 240, 255) + ";\"><P>"
				+ float2string(m.mbi) + "</P></TD>\n" + "<TD></TD>" + "<TD class = \"bl\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P> </P></TD>\n" + "<TD class = \"br\" STYLE=\"background-color:"
				+ rgb2hex(240, 240, 255) + ";\"><P> </P></TD>\n" + "</TR>\n" + "</TABLE>\n";
	}

	/**
	 * Erstellt die Tabelle fr die Klassenbersicht.
	 * 
	 * @return der erstellte String
	 */
	private static String createClassTable() {

		s = "<BR><H2> Coverage of Classes </H2>\n" + createTable("Classes") + "<TR VALIGN=TOP>\n"
				+ "<TD class=\"fl\" bgcolor = " + rgb2hex(210, 210, 255) + ">" + "<P> Project "
				+ ast.getConfiguration().getName() + "</a></P>" + createCoverageCells(ast.getProjectMetrics())
				+ "</TR>\n";

		for (FileNode f : ast.getFileNodes())
			for (ASTClassNode c : f.getClassNodes()) {
				if (c.isTopLevel() || showInnerClasses)
					s += createClassLine(c);
			}
		s += createTableEnd();
		return s;

	}

	/**
	 * Erstellt einen Tabellenkopf fr einen Klassenknoten.
	 * 
	 * @param ref
	 *            der Klassenknoten
	 * @return der erstellte String
	 */
	private static String createTable(String ref) {
		return "<BR><TABLE class=\"k\" CELLPADDING=4 CELLSPACING=0>\n	" + "<COLGROUP>" + "<COL WIDTH=\"37%\">\n	"
				+ "<COL WIDTH=\"7%\">\n	" + "<COL WIDTH=\"7%\">\n	" + "<COL WIDTH=\"7%\">\n	" + "<COL WIDTH=\"7%\">\n	"
				+ "<COL WIDTH=\"7%\">\n	" + "<COL WIDTH=\"7%\">\n	" + "<COL WIDTH=\"7%\">" + "<COL WIDTH=\"7%\">"
				+ "<COL WIDTH=\"7%\"></COLGROUP>" + "<TR VALIGN=TOP>" + "<TD class=\"fl\"><P><a name=\""
				+ id2anchor(ref) + "\">" + "<a href=\"#TOP\"> top </a></a></P>	</TD>\n	"
				+ createHeaderCell("FEEC", rgb2hex(210, 210, 255)) + createHeaderCell("C0", rgb2hex(210, 210, 255))
				+ createHeaderCell("C1", rgb2hex(210, 210, 255)) + createHeaderCell("C2", rgb2hex(210, 210, 255))
				+ createHeaderCell("MMDC", rgb2hex(210, 210, 255)) + createHeaderCell("MCDC", rgb2hex(210, 210, 255))
				+ createHeaderCell("C3", rgb2hex(210, 210, 255)) + createHeaderCell("ModBI", rgb2hex(210, 210, 255))
				+ createHeaderCell("BI", rgb2hex(210, 210, 255)) + "</TR>";

	}

	/**
	 * Erstellt einen Zeile der Klassentabelle.
	 * 
	 * @param c
	 *            der Klassenknoten
	 * @return der erstellte String
	 */
	private static String createClassLine(ASTClassNode c) {
		String s = "<TR VALIGN=TOP>\n" + "<TD class=\"fl\" bgcolor = " + rgb2hex(240, 240, 255) + ">"
				+ "<P><a href=\"#" + id2anchor(c.getIDString()) + "\">" + " Class "
				+ c.getFullyQualifiedName(true, false) + "</a></P>" + createCoverageCells(c.getMetrics()) + "</TR>\n";
		return s;
	}

	/**
	 * Erstellt einen Class-Zeile in der Funktionstabelle.
	 * 
	 * @param c
	 *            der Klassenknoten
	 * @return der erstellte String
	 */
	private static String createTargetClassLine(ASTClassNode c) {
		return "<TR>\n" + "<TD class=\"fl\" bgcolor = " + rgb2hex(240, 240, 255) + ">" + "<P> <a href=\"#"
				+ id2anchor(c.getIDString()) + "\">" + " Class " + c.getFullyQualifiedName(true, false) + "</a></P>"
				+ createCoverageCells(c.getMetrics()) + "</TR>\n";
	}

	/**
	 * Erstellt einen Zeile der Funktionstabelle.
	 * 
	 * @param c
	 *            der Funktionsknoten
	 * @return der erstellte String
	 */
	private static String createFunctionLine(ASTFunctionNode c) {
		return "<TR VALIGN=TOP>\n" + "<TD class=\"fl\">" + "<P>   -   " + c.getName().replace(",", ", ") + "</P>"
				+ createCoverageCells(c.getMetrics()) + "</TR>\n";
	}

	/**
	 * Erstellt die berdeckungsmaezellen der Tabelle.
	 * 
	 * @param m
	 *            die Metrik
	 * @return der erstellte String
	 */
	private static String createCoverageCells(Metrics m) {
		String s = "";

		if (m.nrFunctions == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.feec);

		if (m.nrStatements == 0)
			s += createCell("---");
		else

			s += createPercentageCell(m.c0);
		if (m.nrBranches == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.c1);

		if (m.nrAtoms == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.c2);

		if (m.nrAtoms == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.mmdc);

		if (m.nrAtoms == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.mcdc);

		if (m.nrAtoms == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.c3);

		if (m.nrBIPaths == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.mbi);

		if (m.nrBIPaths == 0)
			s += createCell("---");
		else
			s += createPercentageCell(m.bi);

		return s;
	}

	/**
	 * Erstellt Headerzelle.
	 * 
	 * @param s
	 *            der Inhalt
	 * @param c
	 *            die Farbe
	 * @return der erstellte String
	 */
	private static String createHeaderCell(String s, String c) {
		return "<TD class = \"c\" bgcolor = " + c + "><P>" + s + "</P>	</TD>\n	";
	}

	/**
	 * Erstellt eine Tabellenzelle.
	 * 
	 * @param s
	 *            der Inhalt
	 * @return der erstellte String
	 */
	private static String createCell(String s) {
		return "<TD class=\"fl\"><P>" + s + "</P>	</TD>\n	";
	}

	/**
	 * Erstellt eine Zelle mit prozentualem Wert.
	 * 
	 * @param f
	 *            der Wert
	 * @return der erstellte String
	 */
	private static String createPercentageCell(float f) {

		String CELL_START = "<TD class=\"c\" bgcolor = " + float2hexColor(f) + "><P>";
		StringBuilder sb = new StringBuilder();
		Formatter formatter = new Formatter(sb);
		formatter.format("%3.2f", f);
		String CELL_END = "</P>	</TD>\n	";
		return CELL_START + sb.toString() + CELL_END;
	}

	/**
	 * Ermittelt fr einen berdeckungsgrad den Html-Farbwert als String.
	 * 
	 * @param f
	 *            der berdeckungsgrad
	 * @return der erstellte String mit dem Farbwert
	 */
	private static String float2hexColor(float f) {
		if (Application.hasInstance())
			return color2hex(percentage2Color(f));
		if (f >= 100)
			return "#82ff82";
		if (f >= 90)
			return "#ffffaa";
		if (f >= 75)
			return "#ffd2d2";
		if (f >= 50)
			return "#ffaaaa";
		if (f >= 25)
			return "#ff8282";
		if (f > 0)
			return "#ff5a5a";
		return "#ff3232";
	}

	/**
	 * Erstellt aus einer Float-Zahl eine Prozentangabe.
	 * 
	 * @param f
	 *            der Float-Wert
	 * @return die erstellte String
	 */
	private static String float2string(float f) {

		StringBuilder sb = new StringBuilder();
		Formatter formatter = new Formatter(sb);
		formatter.format("%3.2f", f);
		return sb.toString() + "%";
	}

	/**
	 * Wandelt eine Farbe in ihren Html-Farbcode um.
	 * 
	 * @param c
	 *            die Farbe
	 * @return der erstellte String
	 */
	private static String color2hex(Color c) {
		String s = "#";
		String s2 = Integer.toHexString(c.getRed());
		if (s2.length() == 1)
			s += "0" + s2;
		else
			s += s2;
		s2 = Integer.toHexString(c.getGreen());
		if (s2.length() == 1)
			s += "0" + s2;
		else
			s += s2;
		s2 = Integer.toHexString(c.getBlue());
		if (s2.length() == 1)
			s += "0" + s2;
		else
			s += s2;
		return s;
	}

	/**
	 * Wandelt einen RGB-Wert in eine hexadezimale Stringdarstellung um.
	 * 
	 * @param r
	 *            der Rot-Wert
	 * @param g
	 *            der Grn-Wert
	 * @param b
	 *            der Blau-Wert
	 * @return der erstellte String
	 */
	private static String rgb2hex(int r, int g, int b) {
		return "#" + Integer.toHexString(r) + Integer.toHexString(g) + Integer.toHexString(b);
	}

	/**
	 * Ermittelt fr einen berdeckungsgrad die zu verwendende Farbe.
	 * 
	 * @param p
	 *            der berdeckungsgrad
	 * @return die Farbe
	 */
	public static Color percentage2Color(float p) {
		if (!useColors)
			return Application.getColor(ColorManager.WHITE);

		if (Application.hasInstance()) {
			if (p == 100)
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_TOP_BAR_COLOR);
			if (p == 0)
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_BOTTOM_BAR_COLOR);
			if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FIRST_BAR))
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_FIRST_BAR_COLOR);
			if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_SECOND_BAR))
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_SECOND_BAR_COLOR);
			if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_THIRD_BAR))
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_THIRD_BAR_COLOR);
			if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FOURTH_BAR))
				return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_FOURTH_BAR_COLOR);
			if (p >= PlatformUI.getPreferenceStore().getInt(PreferenceConstants.COVERAGE_FIFTH_BAR))
				return Application.getColorFromPreferenceStore(PreferenceConstants.COVERAGE_FIFTH_BAR_COLOR);
			return Application.getColorFromPreferenceStore(PreferenceConstants.REPORT_ABOVE_BOTTOM_BAR_COLOR);
		}

		return Application.getColor(ColorManager.WHITE);
	}

	/**
	 * Formattiert eine ID-Referenz
	 * 
	 * @param id
	 *            die ID
	 * @return der erstellte String
	 */
	private static String id2anchor(String id) {
		String s = id.replace('\\', '_').replace('.', '_').replace(':', '_');
		return s;
	}

}
