package sota.ast.util;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Kapselt alle Testverlufe aus einer Testdatei, d.h. alle dort registrierten Testflle in Form von TestCases.
 * 
 */
public class TestLog implements ITestContainer, Comparable<TestLog> {

	private String projectName = "";
	private String name = "";
	private String description = "";
	private String iSchemeName = "";
	private String fileName = "";

	private ArrayList<TestCase> listOfTestCases = new ArrayList<TestCase>();
	private ArrayList<String> listOfBadTestCases = new ArrayList<String>();

	/**
	 * Erstellt ein Testlog anhand der bergebenen Daten.
	 * 
	 * @param fileName
	 *            der Dateiname
	 * @param projectName
	 *            der Projektname
	 * @param testName
	 *            der Testname
	 * @param description
	 *            die Beschreibung
	 * @param iScheme
	 *            das verwendete IScheme
	 */
	public TestLog(String fileName, String projectName, String testName, String description, String iScheme) {
		this.fileName = fileName;
		this.projectName = projectName;
		this.description = description;
		name = testName;
		this.iSchemeName = iScheme;
		listOfTestCases = new ArrayList<TestCase>();
	}

	/**
	 * Liefert den Dateinamen.
	 * 
	 * @return der Dateiname
	 */
	public String getFileName() {
		return fileName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.util.ITestContainer#getDescription()
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Legt die Beschreibung des Tests fest.
	 * 
	 * @param description
	 *            die Beschreibung
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.util.ITestContainer#getISchemeName()
	 */
	public String getISchemeName() {
		return iSchemeName;
	}

	/**
	 * Legt das IScheme fest.
	 * 
	 * @param iScheme
	 *            der IScheme-Name
	 */
	public void setISchemeName(String iScheme) {
		this.iSchemeName = iScheme;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.util.ITestContainer#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * Liefert den Projektnamen.
	 * 
	 * @return der Projektname
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * Fgt einen neuen Testfall hinzu.
	 * 
	 * @param testCase
	 *            der Testfall
	 */
	public void addTestCase(TestCase testCase) {
		listOfTestCases.add(testCase);

	}

	/**
	 * Liefert eine Liste der Testflle.
	 * 
	 * @return die Liste
	 */
	public ArrayList<TestCase> getListOfTestCases() {
		return listOfTestCases;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.util.ITestContainer#getInvokationMap()
	 */
	public HashMap<Integer, int[]> getInvokationMap() {

		HashMap<Integer, int[]> map = new HashMap<Integer, int[]>();
		for (TestCase tc : listOfTestCases)
			for (Integer k : tc.getInvokationMap().keySet()) {
				int[] dataTc = tc.getInvokationMap().get(k);
				int[] dataSum;
				if (map.containsKey(k))
					dataSum = map.get(k);
				else
					dataSum = new int[dataTc.length];
				for (int i = 0; i < dataTc.length; i++)
					dataSum[i] += dataTc[i];
				map.put(k, dataSum);
			}
		return map;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.util.ITestContainer#getNrOfPaths()
	 */
	public int getNrOfPaths() {
		int nr = 0;
		for (TestCase tc : listOfTestCases)
			nr += tc.getListOfPaths().size();
		return nr;

	}

	/**
	 * Berechnet die Aufruftabelle der Testflle.
	 */
	public void computeInvokationMap() {
		for (TestCase tc : listOfTestCases)
			tc.computeInvokationMap();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(TestLog o) {
		return name.compareTo(o.getName());
	}

	/**
	 * Fgt invalide Testfalle hinzu.
	 * 
	 * @param list
	 *            die Liste der invaliden Testflle
	 */
	public void addBadTestCases(ArrayList<String> list) {
		this.listOfBadTestCases = list;
	}

	/**
	 * Liefert die Menge der invaliden Testflle
	 * 
	 * @return die Liste der invaliden Testflle
	 */
	public ArrayList<String> getListOfBadTestCases() {
		return listOfBadTestCases;
	}
}
