package sota.gui.action;

import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.Logger;
import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectBuilder;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.preferences.PreferenceConstants;

/**
 * Leitet das Kompilieren des Projektes ein.
 * 
 */
public class BuildProjectAction extends Action implements IWorkbenchAction, GuiUpdateListener, IPropertyChangeListener {
	public final static String ID = "sota.gui.buildprojectaction";

	// fr Interprozesskommunikation: wird auf true gesetzt,wenn Dialog gecancelt wird
	public static boolean[] flag = new boolean[1];

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public BuildProjectAction(IWorkbenchWindow window) {
		setId(ID);
		setText("&Build Project");
		setToolTipText("Build Project");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("antbuild"));
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		PlatformUI.getPreferenceStore().addPropertyChangeListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		Logger.logln("\t Building project ...");
		ProgressMonitorDialog pmd;
		flag[0] = false;
		Application.getInstance().getConfiguration().buildSuccessFlag = false;
		try {
			pmd = new ProgressMonitorDialog(Display.getCurrent().getActiveShell()) {

				@Override
				protected void cancelPressed() {
					flag[0] = true;
					super.cancelPressed();
				}
			};
			pmd.setCancelable(true);
			// pmd.setBlockOnOpen(true);
			pmd.run(true, true, new ProjectBuilder(flag));
		} catch (InvocationTargetException exception) {
			Logger.logln("Invocation: " + exception.getTargetException().getMessage());
			return;
		} catch (InterruptedException exception) {
			System.out.println("Interrupted: " + exception.getMessage());
			return;
		} finally {

			if (Application.getInstance().getConfiguration().exceptionMessage != null) {
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.ICON_ERROR);
				mb.setText("Error");
				mb.setMessage(Application.getInstance().getConfiguration().exceptionMessage);
				mb.open();
				Application.getInstance().getConfiguration().exceptionMessage = null;
				return;
			}
		}
		Application.getInstance().getConfiguration().buildSuccessFlag = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		String antFile = PlatformUI.getPreferenceStore().getString(PreferenceConstants.GENERAL_ANT_LOCATION);
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED && ast.getConfiguration().antBuildFile != null
				&& (new File(ast.getConfiguration().antBuildFile).length() > 0) && antFile != null
				&& antFile.length() > 0 && (new File(antFile)).canExecute());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		ASTManager ast = Application.getInstance().getASTManager();
		String antFile = PlatformUI.getPreferenceStore().getString(PreferenceConstants.GENERAL_ANT_LOCATION);
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED && ast.getConfiguration().antBuildFile != null
				&& (new File(ast.getConfiguration().antBuildFile).length() > 0) && antFile != null
				&& antFile.length() > 0 && (new File(antFile)).canExecute());

	}
}
