package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.dialogs.ConfigureProjectDialog;

/**
 * Konfiguriert die Daten des Projektes.
 * 
 */
public class ConfigureProjectAction extends Action implements IWorkbenchAction, GuiUpdateListener {

	public final static String ID = "sota.gui.configureprojectsaction";

	static IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public ConfigureProjectAction(IWorkbenchWindow window) {
		ConfigureProjectAction.window = window;
		setId(ID);
		setText("&Configure Project");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("configure"));
		setToolTipText("Configure Project");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ConfigureProjectDialog dlg = new ConfigureProjectDialog(window.getShell());
		if (dlg.open() == Window.OK) {
			Application.getInstance().getASTManager().saveProject();
			Application.getInstance().getASTManager().fireToGuiUpdateListeners();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.CONFIGURED);
	}
}