package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.ast.util.ReportCreator;
import sota.ast.util.TestCase;
import sota.ast.util.TestLog;
import sota.gui.GuiUpdateListener;
import sota.gui.preferences.PreferenceConstants;

/**
 * Erstellt fr das Projekt und die ausgewhlten Testdaten eine Reportdatei im Html-Format.
 * 
 */
public class CreateReportAction extends Action implements IWorkbenchAction, GuiUpdateListener, IPropertyChangeListener {

	public final static String ID = "sota.gui.addfilesaction";

	@SuppressWarnings("unused")
	private IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public CreateReportAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&Create Report");
		setToolTipText("Create Report");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("createreport"));

		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		PlatformUI.getPreferenceStore().addPropertyChangeListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ASTManager ast = Application.getInstance().getASTManager();

		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_USE_ALL_TESTS)) {

			ast.resetHits();

			for (TestLog t : ast.getConfiguration().getListOfTestLogs())
				for (TestCase tc : t.getListOfTestCases())
					ast.applyTestCase(tc);

			ast.computeCoverage();
			ast.fireToGuiUpdateListeners();

		}
		String s = null;
		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_PROMPT_FILENAME)) {
			FileDialog fd = new FileDialog(Display.getCurrent().getActiveShell());
			fd.setOverwrite(true);
			fd.setText("Select file name for report.");
			fd.setFilterPath(Application.getInstance().getConfiguration().projectDir);
			fd.setFileName("report.html");
			s = fd.open();
			if (s == null)
				return;
		}
		ReportCreator.createReport(ast, s,
				PlatformUI.getPreferenceStore().getInt(PreferenceConstants.REPORT_FONT_SIZE), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_USE_ALL_TESTS), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_OVERWRITE_REPORT), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_USE_COLORS), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_SHOW_TESTS), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_SHOW_FUNCTIONS), PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_SHOW_CLASSES), !PlatformUI
						.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_IGNORE_INNER_CLASSES));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		ProjectConfig config = ast.getConfiguration();
		setEnabled((PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_USE_ALL_TESTS) && !config
				.getListOfTestLogs().isEmpty())
				|| (config.listOfSelectedTestCases != null && config.listOfSelectedTestCases.size() > 0));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		ProjectConfig config = Application.getInstance().getASTManager().getConfiguration();
		setEnabled((PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.REPORT_USE_ALL_TESTS) && !config
				.getListOfTestLogs().isEmpty())
				|| (config.listOfSelectedTestCases != null && config.listOfSelectedTestCases.size() > 0));
	}
}