package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.IScheme;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.dialogs.NewISchemeDialog;

/**
 * Erstellt ein neues Ischeme.
 * 
 */
public class NewISchemeAction extends Action implements GuiUpdateListener, IWorkbenchAction {
	private final IWorkbenchWindow window;
	public final static String ID = "sota.gui.newinstrumentationschemeaction";

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public NewISchemeAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&New InstrumentationScheme");
		setToolTipText("Create New Instrumentationscheme (IScheme)");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("newischeme"));
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		Application.getInstance().getConfiguration().setCurrentInstrumentationScheme(new IScheme(""));
		NewISchemeDialog dlg = new NewISchemeDialog(window.getShell());
		dlg.open();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

}
