package sota.gui.action;

import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.GlobalInstrumentationScheme;
import sota.ast.util.IScheme;
import sota.gui.dialogs.NewProjectWizard;

/**
 * ffnet den Wizard zur Projekterstellung.
 * 
 */
public class NewProjectAction extends Action implements IWorkbenchAction {
	public final static String ID = "sota.gui.newprojectaction";
	IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public NewProjectAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&New Project");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("newproject"));
		setToolTipText("Create New Project");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ASTManager ast = Application.getInstance().getASTManager();
		WizardDialog wdlg = new WizardDialog(Display.getCurrent().getActiveShell(), new NewProjectWizard());
		if (wdlg.open() != 0)
			return;
		ast.reinit(ast.getConfiguration());
		ast.parseProject(window);
		TreeSet<IScheme> list = ast.getConfiguration().getListOfISchemes();
		list.add(new GlobalInstrumentationScheme("Level 1 instrumentation", 1));
		list.add(new GlobalInstrumentationScheme("Level 2 instrumentation", 2));
		list.add(new GlobalInstrumentationScheme("Level 3 instrumentation", 3));

		ast.backupProject();
		Application.getInstance().getASTManager().saveProject();
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setText(
				"SOTA - Project " + ast.getConfiguration().getName());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}

}
