package sota.gui.action;

import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.dialogs.ReadLogsDialog;

/**
 * Liest Logdateien fr das Projekt ein.
 * 
 */
public class ReadLogsAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.readlogsaction";
	static IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public ReadLogsAction(IWorkbenchWindow window) {
		ReadLogsAction.window = window;
		setId(ID);
		setText("&Read Testlogs");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("readlogs"));
		setToolTipText("Read Testlogs");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ReadLogsDialog dlg = new ReadLogsDialog(window.getShell());
		dlg.open();
		TreeSet<String> set = Application.getInstance().getConfiguration().getListOfTestFiles();
		if (set.size() > 0) {
			if (Application.getInstance().getASTManager().getParsedSourceStatus() == ProjectConfig.DIRTY) {
				Application.getInstance().getASTManager().restoreProject();
				Application.getInstance().getASTManager().reinit(
						Application.getInstance().getASTManager().getConfiguration());
				Application.getInstance().getASTManager().parseProject(window);
			}

			Application.getInstance().getASTManager().readLog(set, window);
			set.clear();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED);
	}
}