package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.preferences.PreferenceConstants;

/**
 * berfhrt die Quellen des aktuellen Projektes wieder in den Originalzustand.
 * 
 */
public class RestoreSourcesAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.restoresourcesaction";
	public IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public RestoreSourcesAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&Restore Sources");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("restore"));
		setToolTipText("Restoring Original Sources");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ASTManager ast = Application.getInstance().getASTManager();
		ast.restoreProject();
		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_PARSE_DIRTY)) {
			Application.getInstance().getASTManager().reinit(
					Application.getInstance().getASTManager().getConfiguration());
			ast.parseProject(window);
		}
		ast.fireToProjectChangeListeners();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED && ast.getSourceStatus() == ProjectConfig.DIRTY);
	}
}