package sota.gui.action;

import java.io.File;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;

/**
 * Startet das Project ber eine der beiden untergeordneten Actions RunProjectAction bzw. RunProjectArgsAction.
 * 
 */
public class RunDropDownAction extends Action implements IWorkbenchAction, IMenuCreator, GuiUpdateListener {

	public final static String ID = "sota.gui.action.rundropdownaction";
	private Menu fMenu;
	private IWorkbenchAction currentAction, alternateAction;

	/**
	 * Konstruktor der DropDown-Aktion, erhlt die Subactions.
	 * 
	 * @param action1
	 *            aktuelle Aktion
	 * @param action2
	 *            alternativ Aktion
	 */
	public RunDropDownAction(IWorkbenchAction action1, IWorkbenchAction action2) {
		super("&Run project", Action.AS_DROP_DOWN_MENU);

		currentAction = action1;
		alternateAction = action2;

		setId(ID);
		setToolTipText("Run project");
		setEnabled(false);

		setImageDescriptor(currentAction.getImageDescriptor());
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setMenuCreator(this);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		super.run();
		currentAction.run();
	}

	/**
	 * Legt die aktuell gewhlte Run-Action fest.
	 * 
	 * @param die
	 *            gewhlte Run-Action
	 */
	public void setAction(IWorkbenchAction action) {
		if (currentAction != action) {
			IWorkbenchAction temp = alternateAction;
			alternateAction = currentAction;
			currentAction = temp;

			setImageDescriptor(currentAction.getImageDescriptor());
			setToolTipText(currentAction.getToolTipText());
			setText(currentAction.getText());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		if (fMenu != null) {
			fMenu.dispose();
			fMenu = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Control)
	 */
	public Menu getMenu(Control parent) {
		if (fMenu != null)
			fMenu.dispose();

		fMenu = parent.getMenu();
		if (fMenu == null)
			fMenu = new Menu(parent);
		new ActionContributionItem((Action) currentAction).fill(fMenu, -1);
		new ActionContributionItem((Action) alternateAction).fill(fMenu, -1);
		return fMenu;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Menu)
	 */
	public Menu getMenu(Menu parent) {
		if (fMenu != null)
			fMenu.dispose();

		fMenu = new Menu(parent);
		new ActionContributionItem((Action) currentAction).fill(fMenu, -1);
		new ActionContributionItem((Action) alternateAction).fill(fMenu, -1);
		return fMenu;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED && ast.getConfiguration().runScript != null
				&& (new File(ast.getConfiguration().runScript).length() > 0));
	}

}