package sota.gui.action;

import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.Logger;
import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.ast.util.ProjectRunner;
import sota.gui.GuiUpdateListener;

/**
 * Leitet das Starten des Projektes mit Argumente ein, die ber einen InputDialog abgefragt werden. Subaction von RunDropDownAction.
 * 
 */
public class RunProjectArgsAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.runprojectargsaction";
	private boolean[] flag = new boolean[1];
	private RunDropDownAction runDropDownAction;
	IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public RunProjectArgsAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&Run Project with Arguments");
		setToolTipText("Run Project with Arguments");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("runargs"));
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);

	}

	/**
	 * bergibt die zugehrige DropDown-Action fr die Statusaktualisierung.
	 * @param die DropDown-Action
	 */
	public void setDropDownAction(RunDropDownAction action) {
		runDropDownAction = action;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {

		ProjectConfig config = Application.getInstance().getConfiguration();
		runDropDownAction.setAction(this);
		
		if(config.runArgs == null)
			config.runArgs = "";

		InputDialog dlg = new InputDialog(window.getShell(), "Run Project with Arguments", "Arguments", config.runArgs,
				null);

		if (dlg.open() != Dialog.OK)
			return;
		else
			config.runArgs = dlg.getValue();
		
		try {
			flag[0] = false;
			new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run(true, true, new ProjectRunner(flag));
		} catch (InvocationTargetException exception) {
			Logger.logln("Invocation: " + exception.getTargetException().getMessage());
		} catch (InterruptedException exception) {
			Logger.logln("Interrupted: " + exception.getMessage());
		} finally {

			if (Application.getInstance().getConfiguration().exceptionMessage != null) {
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.ICON_ERROR);
				mb.setText("Error");
				mb.setMessage(Application.getInstance().getConfiguration().exceptionMessage);
				mb.open();
				Application.getInstance().getConfiguration().exceptionMessage = null;
				return;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED && ast.getConfiguration().runScript != null
				&& (new File(ast.getConfiguration().runScript).length() > 0));
	}

}
