package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;

/**
 * Schaltet die Darstellung der berdeckungen an und aus.
 * 
 */
public class ShowCoverageAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.showcoverageaction";

	static IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public ShowCoverageAction(IWorkbenchWindow window) {
		super("Show Coverage", IAction.AS_CHECK_BOX);
		ShowCoverageAction.window = window;

		setId(ID);
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("coverage"));
		setToolTipText("Show Sourcecode Coverage");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		Application.getInstance().getConfiguration().invertCoverage();
		Application.getInstance().getASTManager().fireToGuiUpdateListeners();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		if (ast.getParsedSourceStatus() == ProjectConfig.DIRTY) {
			setEnabled(false);
			setChecked(false);
		} else {
			setEnabled(ast.getConfiguration().getListOfTestLogs().size() > 0);
			setChecked(ast.getConfiguration().showCoverage());
		}
	}
}