package sota.gui.action;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.Logger;
import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectBuilder;
import sota.ast.util.ProjectConfig;
import sota.ast.util.ProjectRunner;
import sota.gui.GuiUpdateListener;
import sota.gui.dialogs.StartTestDialog;
import sota.gui.preferences.PreferenceConstants;

/**
 * ffnet den Dialog zum Starten einens Tests.
 * 
 */
public class StartTestAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.starttestsaction";
	static IWorkbenchWindow window;

	// fr Interprozesskommunikation: wird auf true gesetzt,wenn Dialog
	// gecancelt wird
	private boolean[] flag = new boolean[1];

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public StartTestAction(IWorkbenchWindow window) {
		StartTestAction.window = window;
		setId(ID);
		setText("&Start Test");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("starttest"));
		setToolTipText("Start Test");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		String iSchemeName = null;
		BufferedWriter writer;
		ProgressMonitorDialog pmd = null;

		StartTestDialog dlg = new StartTestDialog(window.getShell());
		ASTManager ast = Application.getInstance().getASTManager();
		ProjectConfig config = ast.getConfiguration();
		String execDir = config.execDir.replace("\\", "/");
		if (!execDir.endsWith("/"))
			execDir += "/";
		if (dlg.open() != 0)
			return;

		if (config.newTest) {

			// Versuche aktuellen IScheme-Namen aus ASCLogger.ini zu lesen
			if (config.rerunProject) {
				try {
					StreamTokenizer tokenizer = new StreamTokenizer(new FileReader(execDir + "ASCLogger.ini"));

					// Suche nach ISchemeName
					while (tokenizer.nextToken() != StreamTokenizer.TT_EOF) {
						// Suche Key
						if (tokenizer.ttype != StreamTokenizer.TT_WORD)
							continue;
						if (!tokenizer.sval.equalsIgnoreCase("ischeme"))
							continue;
						// folgende von =
						tokenizer.nextToken();
						if (tokenizer.ttype != '=')
							continue;
						// lies String mit ISchemeName ein
						tokenizer.nextToken();
						if (tokenizer.ttype != '"')
							continue;
						iSchemeName = tokenizer.sval;
						break;
					}
				} catch (IOException e) {
				}

				if (iSchemeName == null)
					iSchemeName = ast.getConfiguration().iSchemeName;
			} else
				iSchemeName = ast.getConfiguration().iSchemeName;

			// erstelle ASCLogger.ini
			try {
				writer = new BufferedWriter(new FileWriter(execDir + "ASCLogger.ini"));
				writer.write("project = \"" + ast.getConfiguration().getName() + "\"");
				writer.newLine();

				writer.write("testcase = \"" + ast.getConfiguration().testName + "\"");
				writer.newLine();

				writer.write("description = \"" + ast.getConfiguration().testDesc + "\"");
				writer.newLine();

				writer.write("ischeme = \"" + iSchemeName + "\"");
				writer.newLine();
				writer.close();
				ast.getConfiguration().newTest = false;
				if (!config.rerunProject) {
					ast.instrumentProject();
					if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_PARSE_DIRTY)) {
						ast.reinit(config);
						ast.parseProject(window);
					}
					ast.checkSourceStatus();
					ast.fireToGuiUpdateListeners();
				}

			} catch (IOException e) {
				System.out.println("IOException@StartTestAction");
				e.printStackTrace();
			}

			config.buildSuccessFlag = false;

			// versuche Projekt mittels Ant zu kompilieren
			if (config.buildProject && config.antBuildFile.length() > 0) {

				try {
					flag[0] = false;
					pmd = new ProgressMonitorDialog(Display.getCurrent().getActiveShell()) {

						@Override
						protected void cancelPressed() {
							flag[0] = true;
							super.cancelPressed();
						}
					};
					pmd.setCancelable(true);
					// pmd.setBlockOnOpen(true);
					pmd.run(true, true, new ProjectBuilder(flag));

				} catch (InvocationTargetException exception) {
					Logger.logln("Invocation: " + exception.getTargetException().getMessage());
					return;
				} catch (InterruptedException exception) {
					System.out.println("Interrupted: " + exception.getMessage());
					return;
				} finally {

					if (Application.getInstance().getConfiguration().exceptionMessage != null) {
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
								SWT.ICON_ERROR);
						mb.setText("Error");
						mb.setMessage(Application.getInstance().getConfiguration().exceptionMessage);
						mb.open();
						Application.getInstance().getConfiguration().exceptionMessage = null;
						return;
					}

					if (pmd.getReturnCode() == 0)
						Application.getInstance().getConfiguration().buildSuccessFlag = true;
				}
			}

			// versuche Projekt zu starten
			if ((!config.buildProject || config.buildSuccessFlag) && config.runProject
					&& new File(config.runScript).length() > 0) {
				config.buildSuccessFlag = false;

				try {
					flag[0] = false;
					pmd = new ProgressMonitorDialog(Display.getCurrent().getActiveShell()) {

						@Override
						protected void cancelPressed() {
							flag[0] = true;
							super.cancelPressed();
						}
					};
					pmd.setCancelable(true);
					// pmd.setBlockOnOpen(true);
					pmd.run(true, true, new ProjectRunner(flag));
				} catch (InvocationTargetException exception) {
					Logger.logln("Invocation: " + exception.getTargetException().getMessage());
					return;
				} catch (InterruptedException exception) {
					System.out.println("Interrupted: " + exception.getMessage());
					return;
				} finally {

					if (Application.getInstance().getConfiguration().exceptionMessage != null) {
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
								SWT.ICON_ERROR);
						mb.setText("Error");
						mb.setMessage(Application.getInstance().getConfiguration().exceptionMessage);
						mb.open();
						Application.getInstance().getConfiguration().exceptionMessage = null;
						return;
					}
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		if (ast.getSourceStatus() == ProjectConfig.DIRTY) {
			setText("&Restart Test");
			setToolTipText("Restart Test");
			setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("restarttest"));
		} else {
			setText("&Start Test");
			setToolTipText("Start Test");
			setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("starttest"));
		}
		setEnabled(ast.getProjectStatus() >= ProjectConfig.PARSED);

	}
	
}