package sota.gui.action;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import sota.Logger;
import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ProjectBuilder;
import sota.ast.util.ProjectConfig;
import sota.gui.GuiUpdateListener;
import sota.gui.dialogs.ReadLogsDialog;
import sota.gui.preferences.PreferenceConstants;

/**
 * Beendet einen Test.
 * 
 */
public class StopTestAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.stoptestsaction";

	public IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public StopTestAction(IWorkbenchWindow window) {
		this.window = window;
		setId(ID);
		setText("&Stop Test");
		setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("stoptest"));
		setToolTipText("Stop Test");
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		setEnabled(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ASTManager ast = Application.getInstance().getASTManager();
		ast.restoreProject();
		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_PARSE_DIRTY)) {
			Application.getInstance().getASTManager().reinit(
					Application.getInstance().getASTManager().getConfiguration());
			ast.parseProject(window);
		}

		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_AUTO_REBUILD)
				&& new File(ast.getConfiguration().antBuildFile).length() > 0)
			try {
				new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run(true, true, new ProjectBuilder(
						new boolean[1]));
			} catch (InvocationTargetException exception) {
				Logger.logln("Invocation: " + exception.getTargetException().getMessage());
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "Error", exception.getMessage());
			} catch (InterruptedException exception) {
				Logger.logln("Interrupted: " + exception.getMessage());
				MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "Cancelled", exception
						.getMessage());
			}

		ReadLogsDialog dlg = new ReadLogsDialog(window.getShell());
		dlg.open();
		TreeSet<String> set = Application.getInstance().getConfiguration().getListOfTestFiles();
		if (set.size() > 0) {
			// Application.getInstance().getASTManager().showCoverage(true);
			Application.getInstance().getASTManager().readLog(set, window);
			set.clear();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		setEnabled(ast.getProjectStatus() > 0 && ast.getSourceStatus() == ProjectConfig.DIRTY);
	}
}