package sota.gui.action;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.internal.registry.PerspectiveDescriptor;

import sota.app.Application;
import sota.app.DebugPerspective;
import sota.app.SimplePerspective;
import sota.ast.util.ASTManager;
import sota.gui.GuiUpdateListener;
import sota.gui.preferences.PreferenceConstants;

/**
 * Wechselt die Perspektive zwischen Debug- und normaler Perspektive.
 * 
 */
public class SwitchPerspectiveAction extends Action implements IWorkbenchAction, GuiUpdateListener {
	public final static String ID = "sota.gui.switchperspectiveaction";

	static IWorkbenchWindow window;

	/**
	 * Der Konstruktor der Aktion.
	 * 
	 * @param window
	 *            das zugehrige Window
	 */
	public SwitchPerspectiveAction(IWorkbenchWindow window) {
		super("Debug Modus", IAction.AS_CHECK_BOX);
		SwitchPerspectiveAction.window = window;

		setId(ID);
		setEnabled(PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_ENABLE_DEBUG));
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		final IWorkbench workBench = Application.getInstance().getWorkbench();
		// String debugPerspectiveId = TestPerspective.PERSPECTIVE_ID;
		String debugPerspectiveId = DebugPerspective.PERSPECTIVE_ID;
		String simplePerspectiveId = SimplePerspective.PERSPECTIVE_ID;
		PerspectiveDescriptor debugPerspectiveDesc = (PerspectiveDescriptor) workBench.getPerspectiveRegistry()
				.findPerspectiveWithId(debugPerspectiveId);
		PerspectiveDescriptor simplePerspectiveDesc = (PerspectiveDescriptor) workBench.getPerspectiveRegistry()
				.findPerspectiveWithId(simplePerspectiveId);
		window = workBench.getActiveWorkbenchWindow();
		if (window != null) {
			IWorkbenchPage page = window.getActivePage();
			if (page.getPerspective() == debugPerspectiveDesc)
				page.setPerspective(simplePerspectiveDesc);
			else
				page.setPerspective(debugPerspectiveDesc);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		if (PlatformUI.getPreferenceStore().getBoolean(PreferenceConstants.GENERAL_ENABLE_DEBUG)) {
			setEnabled(true);

		} else {
			setEnabled(false);
			setChecked(false);
			final IWorkbench workBench = Application.getInstance().getWorkbench();
			window = workBench.getActiveWorkbenchWindow();
			PerspectiveDescriptor simplePerspectiveDesc = (PerspectiveDescriptor) workBench.getPerspectiveRegistry()
					.findPerspectiveWithId(SimplePerspective.PERSPECTIVE_ID);

			IWorkbenchPage page = window.getActivePage();

			if (window != null && page.getPerspective() != simplePerspectiveDesc) {
				page.setPerspective(simplePerspectiveDesc);
			}
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.actions.ActionFactory.IWorkbenchAction#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}
}