package sota.gui.dialogs;

import java.io.File;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.ast.util.ProjectConfig;

/**
 * Der Dialog zur Konfiguration des Projekts.
 * 
 */
public class ConfigureProjectDialog extends TitleAreaDialog {

	private Text antFileText, runScriptText, projectDirText, execDirText;

	private Button button;

	/**
	 * Der Konstruktor.
	 * 
	 * @param shell
	 *            die Shell, in der der Dialog zu konstruieren ist
	 */
	public ConfigureProjectDialog(Shell shell) {
		super(shell);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("Configure Project");
		setMessage("Configure Project " + Application.getInstance().getConfiguration().name);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);
		GridData data;

		final ProjectConfig config = Application.getInstance().getASTManager().getConfiguration();

		Composite area = new Composite(contents, SWT.NONE);
		area.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = 20;
		layout.marginWidth = 20;
		layout.makeColumnsEqualWidth = false;
		area.setLayout(layout);

		// Projektverzeichnis - Text
		new Label(area, 0).setText("Project directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);

		projectDirText = new Text(area, SWT.SINGLE | SWT.BORDER);
		projectDirText.setLayoutData(data);
		projectDirText.setText(config.projectDir);
		projectDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
			}
		});

		// Projektverzeichnis - Button
		final Button projectDirButton = new Button(area, SWT.PUSH);
		projectDirButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		projectDirButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(projectDirText.getText());
				dlg.setText("Project Directory:");
				dlg.setMessage("Select directory for this project");
				String dir = dlg.open();
				if (dir != null) {
					projectDirText.setText(dir);
				}
			}
		});

		// Sourceverzeichnis - Text
		/*
		 * new Label(area, 0).setText("Source directory:"); data = new GridData(GridData.FILL_HORIZONTAL); srcDirText =
		 * new Text(area, SWT.SINGLE | SWT.BORDER); srcDirText.setText(config.srcDir);
		 * 
		 * srcDirText.addModifyListener(new ModifyListener() { public void modifyText(ModifyEvent arg0) { } });
		 * srcDirText.setLayoutData(data);
		 */

		// Sourceverzeichnis - Button
		/*
		 * final Button srcDirButton = new Button(area, SWT.PUSH);
		 * srcDirButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		 * srcDirButton.addSelectionListener(new SelectionAdapter() { public void widgetSelected(SelectionEvent event) {
		 * DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent() .getActiveShell());
		 * if(srcDirText.getText()==null || srcDirText.getText().length()==0)
		 * dlg.setFilterPath(projectDirText.getText()); else dlg.setFilterPath(srcDirText.getText());
		 * dlg.setText("Binary Directory:"); dlg.setMessage("Select source directory of the project"); String dir =
		 * dlg.open(); if (dir != null) { srcDirText.setText(dir); } } });
		 */

		// Execverzeichnis - Text
		new Label(area, 0).setText("Execution directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		execDirText = new Text(area, SWT.SINGLE | SWT.BORDER);
		execDirText.setText(config.execDir);
		execDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
			}
		});
		execDirText.setLayoutData(data);

		// Executionverzeichnis - Button
		final Button execDirButton = new Button(area, SWT.PUSH);
		execDirButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		execDirButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				if (execDirText.getText() == null || execDirText.getText().length() == 0)
					dlg.setFilterPath(projectDirText.getText());
				else
					dlg.setFilterPath(execDirText.getText());
				dlg.setText("Execution Directory:");
				dlg.setMessage("Select directory of program execution");
				String dir = dlg.open();
				if (dir != null) {
					execDirText.setText(dir);
				}
			}
		});

		// Separator
		Label separator = new Label(area, SWT.HORIZONTAL | SWT.SEPARATOR);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		separator.setLayoutData(data);

		// Antbuildfile
		new Label(area, 0).setText("Ant build.file:");
		data = new GridData(GridData.FILL_HORIZONTAL);

		antFileText = new Text(area, SWT.SINGLE | SWT.BORDER);
		antFileText.setLayoutData(data);
		antFileText.setText(config.antBuildFile);
		antFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				getButton(IDialogConstants.OK_ID).setEnabled(
						(runScriptText.getText().trim().length() == 0 || (new File(runScriptText.getText())).exists())
								&& (antFileText.getText().trim().length() == 0 || (new File(antFileText.getText()))
										.exists()));
			}
		});

		button = new Button(area, SWT.PUSH);
		button.setImage(Application.getInstance().getImageRegistry().get("folder"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(config.projectDir);
				dlg.setFilterNames(new String[] { "*.xml", "*.*" });
				dlg.setFilterExtensions(new String[] { "*.xml", "*.*" });
				dlg.setText("Ant build-file:");
				dlg.setText("Select ant buildfile for this project");
				String file = dlg.open();
				if (file != null) {
					antFileText.setText(file);
				}
			}
		});

		// Runscript
		new Label(area, 0).setText("Run Script:");
		data = new GridData(GridData.FILL_HORIZONTAL);

		runScriptText = new Text(area, SWT.SINGLE | SWT.BORDER);
		runScriptText.setLayoutData(data);
		runScriptText.setText(config.runScript);
		runScriptText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				getButton(IDialogConstants.OK_ID).setEnabled(
						(runScriptText.getText().trim().length() == 0 || (new File(runScriptText.getText())).exists())
								&& (antFileText.getText().trim().length() == 0 || (new File(antFileText.getText()))
										.exists()));
			}
		});

		button = new Button(area, SWT.PUSH);
		// button.setText("Browse...");
		button.setImage(Application.getInstance().getImageRegistry().get("folder"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(config.projectDir);
				dlg.setFilterNames(new String[] { "*.bat", "*.*" });
				dlg.setFilterExtensions(new String[] { "*.bat", "*.*" });
				dlg.setText("Run Script:");
				dlg.setText("Select run script for this project");
				String file = dlg.open();
				if (file != null) {
					runScriptText.setText(file);
				}
			}
		});

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		ProjectConfig config = Application.getInstance().getASTManager().getConfiguration();
		if ((new File(projectDirText.getText())).exists())
			config.projectDir = projectDirText.getText();

		/*
		 * if ((new File(srcDirText.getText())).exists()) config.srcDir = srcDirText.getText();
		 */

		if ((new File(execDirText.getText())).exists())
			config.execDir = execDirText.getText();

		if (antFileText.getText().trim().length() == 0)
			config.antBuildFile = "";
		else if (new File(antFileText.getText()).exists())
			config.antBuildFile = antFileText.getText();

		if (runScriptText.getText().trim().length() == 0)
			config.runScript = "";
		else if (new File(runScriptText.getText()).exists())
			config.runScript = runScriptText.getText();
		super.okPressed();
	}
}
