package sota.gui.dialogs;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.ast.util.ProjectConfig;
import sota.lang.LanguageSpecification;

/**
 * Die erste Wizardseite zur Projekterstellung.
 * 
 */
public class FirstWizardPage extends WizardPage {
	private ProjectConfig config;
	private boolean execDirHasBeenSelected = false;

	/**
	 * Der Konstrukor der Seite.
	 * 
	 * @param config
	 *            die Projektconfiguration
	 */
	public FirstWizardPage(ProjectConfig config) {
		super("First Page");
		this.config = config;
		setTitle("Project configuration");
		setDescription("Please enter properties of the project.");
		// setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("readlogs"));
		config.language = "Java";
		config.prefix = "asc";
		config.backupExtension = "backup";

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridData data;

		composite.setLayout(new GridLayout(3, false));

		new Label(composite, 0).setText("Project name:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text projectName = new Text(composite, SWT.SINGLE | SWT.BORDER);
		projectName.setLayoutData(data);
		projectName.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent arg0) {
				config.name = projectName.getText();
				setPageComplete(config.firstPageComplete());
			}
		});
		new Label(composite, 0).setText("");

		// ProjectDir
		new Label(composite, 0).setText("Project directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text projectDirText = new Text(composite, SWT.SINGLE | SWT.BORDER);

		projectDirText.setLayoutData(data);
		final Button projectDirButton = new Button(composite, SWT.PUSH);
		projectDirButton.setImage(Application.getInstance().getImageRegistry().get("folder"));

		// ExecutionDir
		new Label(composite, 0).setText("Execution directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text execDirText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		execDirText.setEnabled(false);
		execDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				config.execDir = execDirText.getText();
				setPageComplete(config.firstPageComplete());
			}
		});
		execDirText.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent e) {
				execDirHasBeenSelected = true;
			}

			@Override
			public void focusLost(FocusEvent e) {
			}

		});
		execDirText.setLayoutData(data);
		final Button execDirButton = new Button(composite, SWT.PUSH);
		execDirButton.setEnabled(false);
		execDirButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		execDirButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				execDirHasBeenSelected = true;
				if (execDirText.getText() == null || execDirText.getText().length() == 0)
					dlg.setFilterPath(projectDirText.getText());
				else
					dlg.setFilterPath(execDirText.getText());
				dlg.setText("Execution Directory:");
				dlg.setMessage("Select directory of program execution");
				String dir = dlg.open();
				if (dir != null) {
					execDirText.setText(dir);
					setPageComplete(config.firstPageComplete());
				}
			}
		});

		// ProjectDir again
		projectDirButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(projectDirText.getText());
				dlg.setText("Project Directory:");
				dlg.setMessage("Select directory for this project");
				String dir = dlg.open();
				if (dir != null) {
					projectDirText.setText(dir);

					if (!execDirButton.isEnabled()) {
						execDirText.setText(dir);
						execDirButton.setEnabled(true);
						execDirText.setEnabled(true);
					}

					config.srcDir = dir;

					setPageComplete(config.firstPageComplete());
				}
			}
		});
		projectDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				config.projectDir = projectDirText.getText();
				// if (srcDirText.getText().length() == 0)
				((SecondWizardPage) getNextPage()).setProjectDir(config.projectDir);
				if (!execDirHasBeenSelected) {
					execDirText.setEnabled(true);
					execDirButton.setEnabled(true);
					execDirText.setText(config.projectDir);
				}
				setPageComplete(config.firstPageComplete());
			}
		});

		new Label(composite, 0).setText("Language:");
		data = new GridData();
		data.horizontalSpan = 2;
		data.minimumWidth = 50;
		final Combo combo = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY); // SWT.BORDER);
		for (LanguageSpecification ls : Application.getInstance().getConfiguration().getLangSpecs())
			combo.add(ls.name);
		combo.select(0);
		config.language = combo.getText();
		combo.setLayoutData(data);
		combo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				config.language = combo.getText();
				setPageComplete(config.firstPageComplete());
			}
		});

		Label label = new Label(composite, SWT.HORIZONTAL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		label.setLayoutData(data);

		Label separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		separator.setLayoutData(data);

		Label label2 = new Label(composite, SWT.HORIZONTAL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		label2.setLayoutData(data);

		new Label(composite, 0).setText("Ant build.file:");
		data = new GridData(GridData.FILL_HORIZONTAL);

		final Text antBuildFile = new Text(composite, SWT.SINGLE | SWT.BORDER);
		antBuildFile.setLayoutData(data);
		antBuildFile.setText("");
		antBuildFile.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				config.antBuildFile = antBuildFile.getText();
				setPageComplete(config.firstPageComplete());
			}
		});
		Button antButton = new Button(composite, SWT.PUSH);
		antButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		antButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(projectDirText.getText());
				dlg.setFilterNames(new String[] { "*.xml", "*.*" });
				dlg.setFilterExtensions(new String[] { "*.xml", "*.*" });
				dlg.setText("Ant build-file:");
				dlg.setText("Select ant buildfile for this project");
				String file = dlg.open();
				if (file != null) {
					antBuildFile.setText(file);
					config.antBuildFile = antBuildFile.getText();
					setPageComplete(config.firstPageComplete());
				}
			}
		});

		new Label(composite, 0).setText("Run script:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text runScript = new Text(composite, SWT.SINGLE | SWT.BORDER);
		runScript.setLayoutData(data);
		runScript.setText("");
		runScript.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				config.runScript = runScript.getText();
				setPageComplete(config.firstPageComplete());
			}
		});

		Button runButton = new Button(composite, SWT.PUSH);
		runButton.setImage(Application.getInstance().getImageRegistry().get("folder"));
		runButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(projectDirText.getText());
				dlg.setFilterNames(new String[] { "*.bat", "*.*" });
				dlg.setFilterExtensions(new String[] { "*.bat", "*.*" });
				dlg.setText("Run script:");
				dlg.setText("Select a script to running the project:");
				String file = dlg.open();
				if (file != null) {
					runScript.setText(file);
					config.runScript = runScript.getText();
					setPageComplete(config.firstPageComplete());
				}
			}
		});

		Label label3 = new Label(composite, SWT.HORIZONTAL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		label3.setLayoutData(data);

		composite.pack();
		setPageComplete(config.firstPageComplete());
		setControl(composite);
	}

}
