package sota.gui.dialogs;

import java.util.Comparator;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.IScheme;
import sota.gui.provider.IProjectTreeView;
import sota.gui.provider.InstrumentationLabelProvider;
import sota.gui.provider.ProjectContentProvider;

/**
 * Der Dialog zur Erstellung eines neuen ISchemes.
 * 
 */
public class NewISchemeDialog extends TitleAreaDialog implements IProjectTreeView {
	private TreeViewer fileViewer;

	private IScheme scheme;

	/**
	 * Der Konstruktor des Dialogs.
	 * 
	 * @param shell
	 *            die bergeordnete Shell
	 */
	public NewISchemeDialog(Shell shell) {
		super(shell);
		scheme = new IScheme("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("New IScheme");
		setMessage("Create new instrumentation scheme");
		
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {
		Composite contents = (Composite) super.createDialogArea(parent);
		Composite area = new Composite(contents, SWT.NONE);
		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		area.setLayoutData(data);

		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		layout.verticalSpacing = 10;
		layout.horizontalSpacing = 0;
		layout.makeColumnsEqualWidth = true;
		area.setLayout(layout);

		new Label(area, 0).setText("IScheme name:");
		final Text schemeName = new Text(area, SWT.BORDER | SWT.SINGLE);
		schemeName.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent arg0) {
				scheme.setName(schemeName.getText());
				getButton(IDialogConstants.OK_ID).setEnabled(schemeName.getText().trim().length() != 0);
			}

		});
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		schemeName.setLayoutData(data);
		// new Label(area, 0).setText("");

		new Label(area, 0).setText("Description:");
		final Text schemeDescription = new Text(area, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		schemeDescription.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent arg0) {
				scheme.setDescription(schemeDescription.getText());
			}

		});

		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 4;
		data.verticalSpan = 3;
		schemeDescription.setLayoutData(data);

		new Label(area, 0).setText("");
		new Label(area, 0).setText("");

		new Label(area, 0).setText("Project hierarchy:");

		new Label(area, 0).setText("");
		new Label(area, 0).setText("");

		Button expandButton = new Button(area, SWT.PUSH);
		expandButton.setText("Expand all");
		expandButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				fileViewer.expandAll();
			}
		});
		Button collapseButton = new Button(area, SWT.PUSH);
		collapseButton.setText("Collapse all");
		collapseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				fileViewer.collapseAll();
			}
		});

		/*
		 * Label emptyLabel = new Label(area, SWT.NONE); data = new GridData(GridData.FILL_HORIZONTAL);
		 * data.horizontalSpan = 3; emptyLabel.setLayoutData(data);
		 */

		// FileViewer
		final ProjectContentProvider icp = new ProjectContentProvider(this);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 5;
		data.heightHint = 200;
		fileViewer = new TreeViewer(area, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		fileViewer.getTree().setLayoutData(data);
		fileViewer.setContentProvider(icp);
		fileViewer.setLabelProvider(new InstrumentationLabelProvider(scheme));
		fileViewer.setInput(Application.getInstance().getASTManager());

		Label label = new Label(area, 0);
		data = new GridData(GridData.FILL_HORIZONTAL);
		label.setText("Set selection to: ");
		label.setLayoutData(data);

		Button button = new Button(area, SWT.PUSH);
		button.setText("Level 0");
		button.setImage(Application.getInstance().getImageRegistry().get("lvl0"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				IStructuredSelection selection = (IStructuredSelection) fileViewer.getSelection();
				IScopeNode sn;
				for (Object o : selection.toArray()) {
					sn = (IScopeNode) o;
					scheme.addInstrumentationLevel(sn.getIDString(), 0);
				}
				fileViewer.refresh();
			}
		});

		button = new Button(area, SWT.PUSH);
		button.setText("Level 1");
		button.setImage(Application.getInstance().getImageRegistry().get("lvl1"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				IStructuredSelection selection = (IStructuredSelection) fileViewer.getSelection();
				IScopeNode sn;
				for (Object o : selection.toArray()) {
					sn = (IScopeNode) o;
					scheme.addInstrumentationLevel(sn.getIDString(), 1);
					removeInstrumentierungsLevelOfChildren(o);
				}
				fileViewer.refresh();
			}

			private void removeInstrumentierungsLevelOfChildren(Object o) {
				for (Object x : icp.getChildren(o)) {
					scheme.removeInstrumentierungsLevel(((IScopeNode) x).getIDString());
					removeInstrumentierungsLevelOfChildren(x);
				}
			}

		});

		button = new Button(area, SWT.PUSH);
		button.setText("Level 2");
		button.setImage(Application.getInstance().getImageRegistry().get("lvl2"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				IStructuredSelection selection = (IStructuredSelection) fileViewer.getSelection();
				IScopeNode sn;
				for (Object o : selection.toArray()) {
					sn = (IScopeNode) o;
					scheme.addInstrumentationLevel(sn.getIDString(), 2);
					removeInstrumentierungsLevelOfChildren(o);
				}
				fileViewer.refresh();
			}

			private void removeInstrumentierungsLevelOfChildren(Object o) {
				for (Object x : icp.getChildren(o)) {
					scheme.removeInstrumentierungsLevel(((IScopeNode) x).getIDString());
					removeInstrumentierungsLevelOfChildren(x);
				}
			}
		});

		button = new Button(area, SWT.PUSH);
		button.setText("Level 3");
		button.setImage(Application.getInstance().getImageRegistry().get("lvl3"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				IStructuredSelection selection = (IStructuredSelection) fileViewer.getSelection();
				IScopeNode sn;
				for (Object o : selection.toArray()) {
					sn = (IScopeNode) o;
					scheme.addInstrumentationLevel(sn.getIDString(), 3);
					removeInstrumentierungsLevelOfChildren(o);
				}
				fileViewer.refresh();
			}

			private void removeInstrumentierungsLevelOfChildren(Object o) {
				for (Object x : icp.getChildren(o)) {
					scheme.removeInstrumentierungsLevel(((IScopeNode) x).getIDString());
					removeInstrumentierungsLevelOfChildren(x);
				}
			}
		});

		// IScheme-Level-Beschreibung

		Group group = new Group(area, SWT.SHADOW_ETCHED_OUT);
		group.setLayout(new GridLayout(5, false));
		group.setText("IScheme Level");
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 5;
		group.setLayoutData(data);

		CLabel lvl0Label = new CLabel(group, SWT.NONE);
		lvl0Label.setImage(Application.getInstance().getImageRegistry().get("lvl0"));
		lvl0Label.setText("Level 0");
		Label lvl0descritption = new Label(group, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		lvl0descritption.setText("No instrumentation, leaves sourcecode as it is.");
		lvl0descritption.setLayoutData(data);

		CLabel lvl1Label = new CLabel(group, SWT.NONE);
		lvl1Label.setImage(Application.getInstance().getImageRegistry().get("lvl1"));
		lvl1Label.setText("Level 1");
		Label lvl1descritption = new Label(group, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		lvl1descritption.setText("Instruments cfg-relevant blocks, standard coverage analysis.");
		lvl1descritption.setLayoutData(data);

		CLabel lvl2Label = new CLabel(group, SWT.NONE);
		lvl2Label.setImage(Application.getInstance().getImageRegistry().get("lvl2"));
		lvl2Label.setText("Level 2");
		Label lvl2descritption = new Label(group, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		lvl2descritption.setText("Instruments also conditions, enables analysis of condition coverage.");
		lvl2descritption.setLayoutData(data);

		CLabel lvl3Label = new CLabel(group, SWT.NONE);
		lvl3Label.setImage(Application.getInstance().getImageRegistry().get("lvl3"));
		lvl3Label.setText("Level 3");
		Label lvl3descritption = new Label(group, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		lvl3descritption.setText("Full instrumentation, every single statement is instrumented.");
		lvl3descritption.setLayoutData(data);

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		super.okPressed();
		Application.getInstance().getASTManager().addInstrumentationScheme(scheme);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#getComparator()
	 */
	@SuppressWarnings("unchecked")
	public Comparator getComparator() {
		return new Comparator() {
			public int compare(Object firstObject, Object secondObject) {
				if (!(firstObject instanceof IScopeNode && secondObject instanceof IScopeNode))
					return 0;
				IScopeNode first = (IScopeNode) firstObject, second = (IScopeNode) secondObject;
				return first.getFullyQualifiedName(showPackageNames(), showFiles()).compareTo(
						second.getFullyQualifiedName(showPackageNames(), showFiles()));
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#select(java.lang.Object)
	 */
	public void select(Object o) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showClasses()
	 */
	public boolean showClasses() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFiles()
	 */
	public boolean showFiles() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFlat()
	 */
	public boolean showFlat() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showInnerClasses()
	 */
	public boolean showInnerClasses() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showPackageNames()
	 */
	public boolean showPackageNames() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showProjectLine()
	 */
	public boolean showProjectLine() {
		return false;
	}
}
