package sota.gui.dialogs;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.gui.provider.LogTableLabelProvider;

/**
 * Dialog zum Einlesen der Logdateien.
 * 
 */
public class ReadLogsDialog extends TitleAreaDialog {

	private CheckboxTableViewer _ctv;

	/**
	 * Der Konstruktor des Dialogs.
	 * 
	 * @param shell
	 *            die bergeordnete Shell
	 */
	public ReadLogsDialog(Shell shell) {
		super(shell);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("Read TestLogs");
		setMessage("Please select testlogs to import.");
		setTitleImage(Application.getInstance().getImageRegistry().get("readlogs"));
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);
		GridData data;

		Composite area = new Composite(contents, SWT.NONE);
		area.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = 20;
		layout.marginWidth = 20;
		layout.makeColumnsEqualWidth = false;
		area.setLayout(layout);

		new Label(area, 0).setText("Directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text text = new Text(area, SWT.SINGLE | SWT.BORDER);
		String dir = Application.getInstance().getConfiguration().execDir;
		if (dir.length() > 0 && (new File(dir).exists()))
			text.setText(Application.getInstance().getConfiguration().execDir);
		else
			text.setText(Application.getInstance().getConfiguration().projectDir);

		text.setLayoutData(data);
		Button button = new Button(area, SWT.PUSH);
		button.setText("Browse...");

		final CheckboxTableViewer ctv = CheckboxTableViewer.newCheckList(area, SWT.BORDER | SWT.V_SCROLL);
		_ctv = ctv;
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 5;
		ctv.getTable().setLayoutData(data);

		ctv.setContentProvider(new IStructuredContentProvider() {

			public void dispose() {
			}

			public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
			}

			public Object[] getElements(Object o) {
				File file = null;
				if (!(o instanceof String) || !((file = new File((String) o)).exists()))
					return new Object[0];

				ArrayList<File> fileList = new ArrayList<File>();
				for (File f : file.listFiles()) {
					if (f.getName().endsWith(".log"))
						fileList.add(f);
				}
				return fileList.toArray();
			}
		});
		ctv.setLabelProvider(new LogTableLabelProvider());
		if (dir.length() > 0 && (new File(dir).exists()))
			ctv.setInput(Application.getInstance().getConfiguration().execDir);
		else
			ctv.setInput(Application.getInstance().getConfiguration().projectDir);
		ctv.getTable().addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
			}

			public void mouseDown(MouseEvent e) {
			}

			public void mouseUp(MouseEvent e) {
				getButton(IDialogConstants.OK_ID).setEnabled(ctv.getCheckedElements().length > 0);
			}

		});

		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(text.getText());
				dlg.setText("Execution Directory:");
				dlg.setMessage("Select directory of Logfiles.");
				String dir = dlg.open();
				if (dir != null) {
					text.setText(dir);
					ctv.setInput(dir);
				}
			}
		});

		Button checkAllButton = new Button(area, SWT.PUSH);
		checkAllButton.setText("Check all");
		checkAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				ctv.setAllChecked(true);
				getButton(IDialogConstants.OK_ID).setEnabled(ctv.getCheckedElements().length > 0);
			}
		});
		Button uncheckAllButton = new Button(area, SWT.PUSH);
		uncheckAllButton.setText("Uncheck all");
		uncheckAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				ctv.setAllChecked(false);
				getButton(IDialogConstants.OK_ID).setEnabled(ctv.getCheckedElements().length > 0);
			}
		});

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		if (_ctv.getCheckedElements().length == 0)
			return;

		for (Object o : _ctv.getCheckedElements()) {
			Application.getInstance().getConfiguration().addTestFile(((File) o).getAbsolutePath());
			// Logger.logln("RLDialog: " +
			// Application.getInstance().getConfiguration().getListOfTestFiles().toString());
		}
		super.okPressed();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#getInitialSize()
	 */
	protected Point getInitialSize() {
		return new Point(340, 420);

	}
}
