package sota.gui.dialogs;

import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.ast.util.ProjectConfig;
import sota.gui.provider.FileTreeContentProvider;
import sota.gui.provider.FileTreeLabelProvider;

/**
 * Die zweite Wizardseite zur Projekterstellung.
 * 
 */
public class SecondWizardPage extends WizardPage {
	private ProjectConfig config;
	private Object[] checkedFiles;
	private Text _text;
	private CheckboxTreeViewer _cbtv;

	/**
	 * Der Konstruktor der Seite.
	 * 
	 * @param config
	 *            die ProjektConfiguration
	 */
	public SecondWizardPage(ProjectConfig config) {
		super("Second Page");
		this.config = config;
		setTitle("Source Files");
		setDescription("Please select the source files to import into this project.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridData data;
		GridLayout layout = new GridLayout(3, false);

		composite.setLayout(layout);

		final CheckboxTreeViewer cbtv;

		new Label(composite, 0).setText("Project directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text text = new Text(composite, SWT.SINGLE | SWT.BORDER);
		_text = text;
		text.setLayoutData(data);

		Button button = new Button(composite, SWT.PUSH);
		button.setImage(Application.getInstance().getImageRegistry().get("folder"));

		final FileTreeContentProvider tcp = new FileTreeContentProvider();
		cbtv = new CheckboxTreeViewer(composite);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 3;

		cbtv.getTree().setLayoutData(data);
		cbtv.setContentProvider(tcp);

		cbtv.setLabelProvider(new FileTreeLabelProvider());
		_cbtv = cbtv;
		cbtv.setInput("");

		cbtv.addCheckStateListener(new ICheckStateListener() {

			public void checkStateChanged(CheckStateChangedEvent event) {
				Object element = event.getElement();

				if (event.getChecked()) {
					cbtv.setSubtreeChecked(element, true);
				} else
					cbtv.setSubtreeChecked(element, false);

				checkedFiles = cbtv.getCheckedElements();
				setPageComplete(config.secondPageComplete() && (cbtv.getCheckedElements() != null)
						&& (cbtv.getCheckedElements().length > 0));
			}
		});
		setControl(composite);

		Button expandButton = new Button(composite, SWT.PUSH);
		expandButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		expandButton.setText("Expand all");
		expandButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				cbtv.expandAll();
			}
		});
		Button collapseButton = new Button(composite, SWT.PUSH);
		collapseButton.setText("Collapse all");
		collapseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				cbtv.collapseAll();
			}
		});

		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(text.getText());
				dlg.setText("Source Directory:");
				dlg.setMessage("Select directory of sourcefiles");
				String dir = dlg.open();
				if (dir != null) {
					text.setText(dir);
					cbtv.setInput(dir);
					cbtv.refresh();
					// config.srcDir = text.getText();
					setPageComplete(config.secondPageComplete() && (cbtv.getCheckedElements() != null)
							&& (cbtv.getCheckedElements().length > 0));
				}
			}
		});

		text.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent event) {
			}

			public void focusLost(FocusEvent event) {
				cbtv.setInput(text.getText());
				cbtv.refresh();
				config.srcDir = text.getText();
				setPageComplete(config.isComplete());
			}
		});
		setPageComplete(config.secondPageComplete() && (cbtv.getCheckedElements() != null)
				&& (cbtv.getCheckedElements().length > 0));
	}

	/**
	 * Liefert die ausgewhlten Dateinamen.
	 * 
	 * @return die Dateinamen
	 */
	public Object[] getFileNames() {
		return checkedFiles;
	}

	/**
	 * Legt das Projektverzeichnis fest.
	 * 
	 * @param projectDir
	 *            das Projektverzeichnis
	 */
	public void setProjectDir(String projectDir) {
		_text.setText(projectDir);
		_cbtv.setInput(projectDir);
	}

}
