package sota.gui.dialogs;

import java.io.File;
import java.util.Comparator;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import sota.app.Application;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.IScheme;
import sota.ast.util.ProjectConfig;
import sota.gui.preferences.PreferenceConstants;
import sota.gui.provider.IProjectTreeView;
import sota.gui.provider.InstrumentationLabelProvider;
import sota.gui.provider.ProjectContentProvider;

/**
 * Dialog zum Teststart.
 * 
 */
public class StartTestDialog extends TitleAreaDialog implements IProjectTreeView {

	private Text testName, testDesc, runArgs;
	private Combo combo;
	private TreeViewer tv = null;
	private String text = "";
	private int selection = 0;
	private Button runButton, buildButton, rerunButton, expandButton, collapseButton;
	private Label label1, label2;
	private CLabel lvl0Label, lvl1Label, lvl2Label, lvl3Label;
	private Label lvl0Text, lvl1Text, lvl2Text, lvl3Text, emptyLabel;
	private Group iSchemeGroup, testOptionGroup;

	/**
	 * Der Konstruktor.
	 * 
	 * @param shell
	 *            die bergeordnete Shell
	 */
	public StartTestDialog(Shell shell) {
		super(shell);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.DIALOG_TRIM | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("Start test");
		setMessage("Please select a name for the testcase.");
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(final Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);

		// Init DialogArea
		final Composite area = new Composite(contents, SWT.NONE);
		area.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

		// Layout
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		layout.makeColumnsEqualWidth = false;
		area.setLayout(layout);

		// erstelle Widgets
		createTestDataFields(area);
		createISchemeInfos(area);
		createTestOptionGroup(area);

		// blende Details aus
		showISchemeDetails(false);
		area.getParent().getParent().getParent().pack(true);

		updateButtonStatus();

		return contents;
	}

	/**
	 * Erstellt die Widgets zur Eingabe der Testparamenter.
	 * 
	 * @param area
	 *            das zugehrige Composite
	 */
	private void createTestDataFields(final Composite area) {
		// Test name
		final Label l1 = new Label(area, 0);
		l1.setText("Test name:");
		testName = new Text(area, SWT.BORDER | SWT.SINGLE);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		testName.setLayoutData(data);
		testName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				text = testName.getText().trim();
				updateDialogStatus();
			}
		});

		// Test description
		new Label(area, 0).setText("Test description:");
		testDesc = new Text(area, SWT.BORDER | SWT.MULTI);
		data = new GridData(GridData.FILL_BOTH);
		data.verticalSpan = 3;
		data.horizontalSpan = 2;
		testDesc.setLayoutData(data);
		new Label(area, 0);
		new Label(area, 0);

		// IScheme Dropdown
		new Label(area, 0).setText("Select IScheme:");
		final Combo combo = new Combo(area, SWT.BORDER | SWT.DROP_DOWN | SWT.READ_ONLY);
		this.combo = combo;
		combo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		combo.add(" ");
		for (IScheme scheme : Application.getInstance().getASTManager().getConfiguration().setOfISchemes) {
			combo.add(scheme.getName());
		}
		combo.select(0);

		final Button detailsButton = new Button(area, SWT.TOGGLE);
		new Label(area, 0);
		detailsButton.setText("Details");

		// Verhalten des Detailsbutton: blendet alle vorherigen Elemente bis zum Detailsbutton aus
		detailsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				showISchemeDetails(detailsButton.getSelection());
				area.getParent().getParent().getParent().getParent().pack(true);
			}
		});

	}

	protected void updateDialogStatus() {
		getButton(IDialogConstants.OK_ID)
				.setEnabled(text.length() > 0 && (rerunButton.getSelection() || selection > 0));
		if (text.length() == 0)
			setMessage("Please select a name for the testcase.");
		else if (selection == 0)
			setMessage("Please select an IScheme for testing.");
		else
			setMessage("Press OK to instrument the project.");
	}

	/**
	 * Erstellt die Widgets fr die (versteckten) IScheme-Infos.
	 * 
	 * @param area
	 *            das zugehrige Composite
	 */
	private void createISchemeInfos(Composite area) {

		// Project Treeviewer
		GridData data = new GridData(GridData.FILL_BOTH);
		data.heightHint = 200;
		data.horizontalSpan = 3;
		final TreeViewer tv = new TreeViewer(area, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		tv.getTree().setLayoutData(data);
		tv.setContentProvider(new ProjectContentProvider(this));
		tv.setLabelProvider(new InstrumentationLabelProvider());
		tv.setInput(Application.getInstance().getASTManager());
		this.tv = tv;

		combo.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {
				selection = 0;
				getButton(IDialogConstants.OK_ID).setEnabled(
						text.length() > 0 && (rerunButton.getSelection() || selection > 0));

			}

			public void widgetSelected(SelectionEvent event) {
				String iSchemeName = combo.getItem(combo.getSelectionIndex());
				selection = combo.getSelectionIndex();
				
				updateDialogStatus();
				
				Application.getInstance().getASTManager().getConfiguration().setCurrentInstrumentationScheme(iSchemeName);
				tv.setLabelProvider(new InstrumentationLabelProvider(Application.getInstance().getASTManager()
						.getConfiguration().getCurrentIScheme()));
			}

		});

		// Button Expand all
		expandButton = new Button(area, SWT.PUSH);
		expandButton.setText("Expand all");
		expandButton.setLayoutData(new GridData());
		expandButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				tv.expandAll();
			}
		});

		// Button Collapse all
		collapseButton = new Button(area, SWT.PUSH);
		collapseButton.setText("Collapse all");
		collapseButton.setLayoutData(new GridData());
		collapseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				tv.collapseAll();
			}
		});

		emptyLabel = new Label(area, 0);
		emptyLabel.setLayoutData(new GridData());

		// die Info-Group
		createISchemeDetailGroup(area);

	}

	/**
	 * Erstellt die Widget-Group fr die (versteckten) ISchemedetails.
	 * 
	 * @param area
	 *            das zugehreige Composite
	 */
	private void createISchemeDetailGroup(Composite area) {
		iSchemeGroup = new Group(area, SWT.SHADOW_ETCHED_OUT);
		iSchemeGroup.setLayout(new GridLayout(2, false));
		iSchemeGroup.setText("IScheme Level");
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		iSchemeGroup.setLayoutData(data);

		// Instrumentation level Labels
		lvl0Label = new CLabel(iSchemeGroup, SWT.NONE);
		lvl0Label.setImage(Application.getInstance().getImageRegistry().get("lvl0"));
		lvl0Label.setText("Level 0");
		lvl0Label.setLayoutData(new GridData());
		lvl0Text = new Label(iSchemeGroup, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 1;
		lvl0Text.setLayoutData(data);
		lvl0Text.setText("No instrumentation, leaves sourcecode as it is.");

		lvl1Label = new CLabel(iSchemeGroup, SWT.NONE);
		lvl1Label.setImage(Application.getInstance().getImageRegistry().get("lvl1"));
		lvl1Label.setText("Level 1");
		lvl1Label.setLayoutData(new GridData());
		lvl1Text = new Label(iSchemeGroup, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 1;
		lvl1Text.setLayoutData(data);
		lvl1Text.setText("Instruments cfg-relevant blocks, standard coverage analysis.");

		lvl2Label = new CLabel(iSchemeGroup, SWT.NONE);
		lvl2Label.setImage(Application.getInstance().getImageRegistry().get("lvl2"));
		lvl2Label.setText("Level 2");
		lvl2Label.setLayoutData(new GridData());
		lvl2Text = new Label(iSchemeGroup, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 1;
		lvl2Text.setLayoutData(data);
		lvl2Text.setText("Instruments also conditions, enables analysis of condition coverage.");

		lvl3Label = new CLabel(iSchemeGroup, SWT.NONE);
		lvl3Label.setImage(Application.getInstance().getImageRegistry().get("lvl3"));
		lvl3Label.setText("Level 3");
		lvl3Label.setLayoutData(new GridData());
		lvl3Text = new Label(iSchemeGroup, SWT.NONE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 1;
		lvl3Text.setLayoutData(data);
		lvl3Text.setText("Full instrumentation, every single statement is instrumented.");
	}

	/**
	 * Erstellt die Widgets fr die Testoptionen.
	 * 
	 * @param area
	 *            das zugehrige Composite
	 */
	private void createTestOptionGroup(Composite area) {
		testOptionGroup = new Group(area, SWT.SHADOW_ETCHED_OUT);
		testOptionGroup.setLayout(new GridLayout(2, false));
		testOptionGroup.setText("Test Options");
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		testOptionGroup.setLayoutData(data);

		// Rerun Button
		final Button rerunButton = new Button(testOptionGroup, SWT.CHECK);
		this.rerunButton = rerunButton;
		rerunButton.setText("Rerun configuration. Don't change sources or binaries.");
		rerunButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				updateButtonStatus();
			}
		});
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		rerunButton.setLayoutData(data);

		// falls im Test, dann Rerun Button nicht nderbar
		if (Application.getInstance().getASTManager().getSourceStatus() == ProjectConfig.DIRTY) {
			rerunButton.setEnabled(false);
			rerunButton.setSelection(true);
		}

		// Build Button
		final Button buildButton = new Button(testOptionGroup, SWT.CHECK);
		this.buildButton = buildButton;
		buildButton.setText("Build project");
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		buildButton.setLayoutData(data);
		buildButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				updateButtonStatus();
			}
		});

		// Run Button
		final Button runButton = new Button(testOptionGroup, SWT.CHECK);
		this.runButton = runButton;
		runButton.setText("Run project        Arguments:");
		data = new GridData(0);
		data.horizontalSpan = 1;
		runButton.setLayoutData(data);
		runButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				updateButtonStatus();
			}
		});

		// aktualisiere Auswahl des Run Buttons mit letzter Wahl
		if (Application.getInstance().getASTManager().getSourceStatus() == ProjectConfig.DIRTY) {
			runButton.setEnabled(true);
			runButton.setSelection(Application.getInstance().getConfiguration().runProject);
		}

		runArgs = new Text(testOptionGroup, SWT.BORDER | SWT.SINGLE);
		data = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		data.horizontalSpan = 1;
		runArgs.setLayoutData(data);

		// Starttest Info
		final Label infoLabel1 = new Label(testOptionGroup, 0);
		this.label1 = infoLabel1;
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		data.widthHint = 420;
		infoLabel1.setLayoutData(data);

		final Label infoLabel2 = new Label(testOptionGroup, 0);
		this.label2 = infoLabel2;
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		infoLabel2.setLayoutData(data);

	}

	/**
	 * Versteckt bzw. zeigt die Detailinfos zum gewhlten Instrumentationsschema.
	 * 
	 * @param b
	 *            true, falls Infos gezeigt werden sollen, sonst false
	 */
	protected void showISchemeDetails(boolean b) {
		tv.getTree().setVisible(b);
		expandButton.setVisible(b);
		collapseButton.setVisible(b);
		emptyLabel.setVisible(b);
		iSchemeGroup.setVisible(b);
		lvl0Label.setVisible(b);
		lvl1Label.setVisible(b);
		lvl2Label.setVisible(b);
		lvl3Label.setVisible(b);
		lvl0Text.setVisible(b);
		lvl1Text.setVisible(b);
		lvl2Text.setVisible(b);
		lvl3Text.setVisible(b);

		((GridData) tv.getTree().getLayoutData()).exclude = !b;
		((GridData) expandButton.getLayoutData()).exclude = !b;
		((GridData) collapseButton.getLayoutData()).exclude = !b;
		((GridData) emptyLabel.getLayoutData()).exclude = !b;
		((GridData) iSchemeGroup.getLayoutData()).exclude = !b;
		((GridData) lvl0Label.getLayoutData()).exclude = !b;
		((GridData) lvl1Label.getLayoutData()).exclude = !b;
		((GridData) lvl2Label.getLayoutData()).exclude = !b;
		((GridData) lvl3Label.getLayoutData()).exclude = !b;
		((GridData) lvl0Text.getLayoutData()).exclude = !b;
		((GridData) lvl1Text.getLayoutData()).exclude = !b;
		((GridData) lvl2Text.getLayoutData()).exclude = !b;
		((GridData) lvl3Text.getLayoutData()).exclude = !b;
	}

	/**
	 * Aktualisiert den Status der Auswahlbuttons.
	 */
	private void updateButtonStatus() {
		// Labelinfos
		if (rerunButton.getSelection()) {
			label1.setText("Rerun a test - changes the name of the current test, but leaves the sourcecode");
			label2.setText("and the binaries untouched." + (runButton.getSelection() ? " Runs project." : ""));
		} else {
			label1.setText("Instruments the project with the chosen IScheme"
					+ (buildButton.getSelection() ? (runButton.getSelection() ? ", " : " and")
							+ (buildButton.getSelection() ? " compiles" : "")
							+ (runButton.getSelection() ? " and runs project." : " project.") : "."));
			label2.setText("");
		}

		// Button Status
		// Falls rerun
		if (rerunButton.getSelection()) {
			// kein Build mglich
			buildButton.setSelection(false);
			buildButton.setEnabled(false);
			// wenn runScript da, dann dieses ermglichen
			runButton.setEnabled(Application.getInstance().getConfiguration().runScript.length() > 0);
			selection = 0;
			combo.deselectAll();
			combo.setEnabled(false);
			tv.refresh();
			// falls kein rerun
		} else {
			combo.setEnabled(true);
			// build erlauben, wenn antfile da
			buildButton.setEnabled((new File(PlatformUI.getPreferenceStore().getString(
					PreferenceConstants.GENERAL_ANT_LOCATION))).length() > 0
					&& new File(Application.getInstance().getConfiguration().antBuildFile).length() > 0);
			// falls build gewhlt
			if (buildButton.isEnabled() && buildButton.getSelection()) {
				// wenn runScript da, dann dieses ermglichen
				runButton.setEnabled(new File(Application.getInstance().getConfiguration().runScript).length() > 0);
			} else {
				// kein run mglich
				runButton.setSelection(false);
				runButton.setEnabled(false);
			}
		}

		runArgs.setEnabled(runButton.getSelection());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		if (testName.getText().equals(""))
			return;
		Application.getInstance().getASTManager().getConfiguration().setNewTest(testName.getText(), testDesc.getText(),
				combo.getSelectionIndex() >= 0 ? combo.getItem(combo.getSelectionIndex()) : null,
				rerunButton.getSelection(), buildButton.getSelection(), runButton.getSelection(), runArgs.getText());
		super.okPressed();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#getComparator()
	 */
	@SuppressWarnings("unchecked")
	public Comparator getComparator() {
		return new Comparator() {
			public int compare(Object firstObject, Object secondObject) {
				if (!(firstObject instanceof IScopeNode && secondObject instanceof IScopeNode))
					return 0;
				IScopeNode first = (IScopeNode) firstObject, second = (IScopeNode) secondObject;
				return first.getFullyQualifiedName(showPackageNames(), showFiles()).compareTo(
						second.getFullyQualifiedName(showPackageNames(), showFiles()));
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#select(java.lang.Object)
	 */
	public void select(Object o) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showClasses()
	 */
	public boolean showClasses() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFiles()
	 */
	public boolean showFiles() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFlat()
	 */
	public boolean showFlat() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showInnerClasses()
	 */
	public boolean showInnerClasses() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showPackageNames()
	 */
	public boolean showPackageNames() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showProjectLine()
	 */
	public boolean showProjectLine() {
		return false;
	}
}
