package sota.gui.dialogs;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeColumn;

import sota.app.Application;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.util.ITestContainer;
import sota.ast.util.TestCase;

/**
 * Dialog zur Darstellung eines TestCases bzw. TestLogs.
 * 
 */
public class TestLogDialog extends TitleAreaDialog {

	private ITestContainer container;
	private TestLogSortCriteria sortCriterium = TestLogSortCriteria.name;
	private boolean reverseSort = false;

	/**
	 * Die Sortierungskriterien der Tabelle.
	 * 
	 */
	public enum TestLogSortCriteria {
		name, nrPaths, lvl1, lvl2, lvl3
	}

	/**
	 * Der Konstruktor des Dialogs.
	 * 
	 * @param shell
	 *            die bergeordnete Shell
	 * @param container
	 *            der darzustellende TestContainer
	 */
	public TestLogDialog(Shell shell, ITestContainer container) {
		super(shell);
		this.container = container;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		if (container instanceof TestCase)
			setTitle("TestCase: " + container.getName());
		else
			setTitle("TestLog: " + container.getName());
		getButton(IDialogConstants.OK_ID).setEnabled(true);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);
		GridData data;

		Composite area = new Composite(contents, SWT.NONE);
		area.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = 20;
		layout.marginWidth = 20;
		layout.makeColumnsEqualWidth = false;
		area.setLayout(layout);

		new Label(area, 0).setText("Test name:");
		Label testName = new Label(area, SWT.BORDER | SWT.SINGLE);
		testName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		testName.setText(" " + container.getName());
		// testName.setEditable(false);

		new Label(area, 0).setText("Test description:");
		Label testDesc = new Label(area, SWT.BORDER);// | SWT.MULTI);
		data = new GridData(GridData.FILL_BOTH);
		data.verticalSpan = 3;
		testDesc.setLayoutData(data);
		testDesc.setText(" " + container.getDescription());
		// testDesc.setEditable(false);

		new Label(area, 0);
		new Label(area, 0);

		new Label(area, 0).setText("IScheme:");
		Label ischemeLabel = new Label(area, SWT.BORDER);
		ischemeLabel.setText(" " + container.getISchemeName());
		ischemeLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		new Label(area, 0).setText("Nr. of Paths in Test:");
		Label pathLabel = new Label(area, SWT.BORDER);
		pathLabel.setText(" " + container.getNrOfPaths());
		pathLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		new Label(area, 0);
		new Label(area, 0);

		data = new GridData(GridData.FILL_BOTH);
		data.heightHint = 300;
		data.widthHint = 540;
		data.horizontalSpan = 2;
		final TreeViewer fileViewer = new TreeViewer(area, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		fileViewer.getTree().setLayoutData(data);

		TreeColumn column = new TreeColumn(fileViewer.getTree(), SWT.NONE);
		column.setText("Function");
		column.setWidth(380);
		column.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof TreeColumn && ((TreeColumn) event.widget).getText().equals("Function")) {
					if (sortCriterium == TestLogSortCriteria.name)
						reverseSort = !reverseSort;
					sortCriterium = TestLogSortCriteria.name;
					fileViewer.refresh();
				}
			}
		});

		TreeColumn column1 = new TreeColumn(fileViewer.getTree(), SWT.NONE);
		column1.setText("#Paths");
		column1.setWidth(55);
		column1.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof TreeColumn && ((TreeColumn) event.widget).getText().equals("#Paths")) {
					if (sortCriterium == TestLogSortCriteria.nrPaths)
						reverseSort = !reverseSort;
					sortCriterium = TestLogSortCriteria.nrPaths;

					fileViewer.refresh();
				}
			}
		});

		TreeColumn column2 = new TreeColumn(fileViewer.getTree(), SWT.NONE);
		column2.setText("Lvl 1");
		column2.setWidth(40);
		column2.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof TreeColumn && ((TreeColumn) event.widget).getText().equals("Lvl 1")) {
					if (sortCriterium == TestLogSortCriteria.lvl1)
						reverseSort = !reverseSort;
					sortCriterium = TestLogSortCriteria.lvl1;

					fileViewer.refresh();
				}
			}
		});

		TreeColumn column3 = new TreeColumn(fileViewer.getTree(), SWT.NONE);
		column3.setText("Lvl 2");
		column3.setWidth(40);
		column3.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof TreeColumn && ((TreeColumn) event.widget).getText().equals("Lvl 2")) {
					if (sortCriterium == TestLogSortCriteria.lvl2)
						reverseSort = !reverseSort;
					sortCriterium = TestLogSortCriteria.lvl2;

					fileViewer.refresh();
				}
			}
		});

		TreeColumn column4 = new TreeColumn(fileViewer.getTree(), SWT.NONE);
		column4.setText("Lvl 3");
		column4.setWidth(40);
		column4.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof TreeColumn && ((TreeColumn) event.widget).getText().equals("Lvl 3")) {
					if (sortCriterium == TestLogSortCriteria.lvl3)
						reverseSort = !reverseSort;
					sortCriterium = TestLogSortCriteria.lvl3;

					fileViewer.refresh();
				}
			}
		});

		fileViewer.getTree().setHeaderVisible(true);

		fileViewer.setContentProvider(new ITreeContentProvider() {

			@Override
			public Object[] getChildren(Object parentElement) {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public Object getParent(Object element) {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public boolean hasChildren(Object element) {
				// TODO Auto-generated method stub
				return false;
			}

			@SuppressWarnings("unchecked")
			public Object[] getElements(Object arg0) {
				if (arg0 instanceof ITestContainer) {
					container = (ITestContainer) arg0;
					HashMap<Integer, int[]> map = container.getInvokationMap();
					ArrayList<ASTFunctionNode> fl = new ArrayList<ASTFunctionNode>();

					for (ASTFunctionNode fn : Application.getInstance().getASTManager().getFunctionNodes()) {
						if (map.containsKey(fn.getEntryNode().getID()))
							fl.add(fn);
					}
					Object[] array = fl.toArray();
					Vector<Object> v = new Vector<Object>();
					Arrays.sort(array, new Comparator() {

						public int compare(Object firstObject, Object secondObject) {

							if (!(firstObject instanceof ASTFunctionNode && secondObject instanceof ASTFunctionNode))
								return 0;

							ASTFunctionNode first = (ASTFunctionNode) firstObject, second = (ASTFunctionNode) secondObject;

							boolean showPackages = false;
							boolean showFiles = false;
							HashMap<Integer, int[]> map = container.getInvokationMap();

							int factor = reverseSort ? -1 : 1;

							switch (sortCriterium) {
							case nrPaths:
								return factor
										* (((map.get(first.getEntryNode().getID()))[0] - (map.get(second.getEntryNode()
												.getID()))[0]));
							case lvl1:
								return factor
										* (((map.get(first.getEntryNode().getID()))[1] - (map.get(second.getEntryNode()
												.getID()))[1]));
							case lvl2:
								return factor
										* (((map.get(first.getEntryNode().getID()))[2] - (map.get(second.getEntryNode()
												.getID()))[2]));
							case lvl3:
								return factor
										* (((map.get(first.getEntryNode().getID()))[3] - (map.get(second.getEntryNode()
												.getID()))[3]));
							case name:
								return factor
										* first.getFullyQualifiedName(showPackages, showFiles).compareTo(
												second.getFullyQualifiedName(showPackages, showFiles));
							}

							return 0;
						}
					});
					v.addAll(Arrays.asList(array));
					return v.toArray();
				}
				return new Object[0];

			}

			@Override
			public void dispose() {
				// TODO Auto-generated method stub

			}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				// TODO Auto-generated method stub

			}

		});
		// }TestLogContentProvider());
		fileViewer.setLabelProvider(new ITableLabelProvider() {

			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof ASTFunctionNode) {
					if (columnIndex != 0) {
						return (container.getInvokationMap().get(((ASTFunctionNode) element).getEntryNode().getID()))[columnIndex - 1]
								+ "";
					}
					String s;
					s = ((ASTFunctionNode) element).getFullyQualifiedName(false, false);
					return s;
				}
				return null;
			}

			@Override
			public void addListener(ILabelProviderListener listener) {
				// TODO Auto-generated method stub

			}

			@Override
			public void dispose() {
				// TODO Auto-generated method stub

			}

			@Override
			public boolean isLabelProperty(Object element, String property) {
				// TODO Auto-generated method stub
				return false;
			}

			@Override
			public void removeListener(ILabelProviderListener listener) {
				// TODO Auto-generated method stub

			}

		});
		fileViewer.setInput(container);

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		super.okPressed();
	}

}
