package sota.gui.dialogs;

import java.io.File;
import java.util.TreeSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import sota.app.Application;
import sota.ast.util.ProjectConfig;
import sota.gui.provider.FileTreeContentProvider;
import sota.gui.provider.FileTreeLabelProvider;

/**
 * Dialog zum Aktualisieren der dem Projekt zugehrigen Quellen.
 * 
 */
public class UpdateSourcesDialog extends TitleAreaDialog {
	Text testName;

	private CheckboxTreeViewer _cbtv;

	/**
	 * Der Konstruktor des Dialogs.
	 * 
	 * @param shell
	 *            die zugehrige Shell
	 */
	public UpdateSourcesDialog(Shell shell) {
		super(shell);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#setShellStyle(int)
	 */
	protected void setShellStyle(int newShellStyle) {
		// TODO Auto-generated method stub
		super.setShellStyle(newShellStyle | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("Select sources of project");
		setMessage("Please select sourcesfiles to import.");
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);
		GridData data;

		final ProjectConfig config = Application.getInstance().getConfiguration();
		final CheckboxTreeViewer cbtv;

		Composite area = new Composite(contents, SWT.NONE);
		area.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = 20;
		layout.marginWidth = 20;
		layout.makeColumnsEqualWidth = false;
		area.setLayout(layout);

		new Label(area, 0).setText("Project directory:");
		data = new GridData(GridData.FILL_HORIZONTAL);
		final Text text = new Text(area, SWT.SINGLE | SWT.BORDER);
		text.setLayoutData(data);
		text.setText(config.projectDir);

		Button button = new Button(area, SWT.PUSH);
		button.setImage(Application.getInstance().getImageRegistry().get("folder"));

		final FileTreeContentProvider tcp = new FileTreeContentProvider();
		cbtv = new CheckboxTreeViewer(area, SWT.BORDER | SWT.V_SCROLL);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 5;
		data.verticalSpan = 5;

		cbtv.getTree().setLayoutData(data);
		cbtv.setContentProvider(tcp);

		cbtv.setLabelProvider(new FileTreeLabelProvider());
		_cbtv = cbtv;
		cbtv.setInput(config.projectDir);

		cbtv.addCheckStateListener(new ICheckStateListener() {

			private Object[] checkedFiles;

			public void checkStateChanged(CheckStateChangedEvent event) {
				Object element = event.getElement();

				if (event.getChecked()) {
					cbtv.setSubtreeChecked(element, true);
				} else
					cbtv.setSubtreeChecked(element, false);

				checkedFiles = cbtv.getCheckedElements();
				getButton(IDialogConstants.OK_ID).setEnabled(checkedFiles.length > 0);
			}
		});

		Button expandButton = new Button(area, SWT.PUSH);
		expandButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		expandButton.setText("Expand all");
		expandButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				cbtv.expandAll();
			}
		});
		Button collapseButton = new Button(area, SWT.PUSH);
		collapseButton.setText("Collapse all");
		collapseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				cbtv.collapseAll();
			}
		});

		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());
				dlg.setFilterPath(text.getText());
				dlg.setText("Source Directory:");
				dlg.setMessage("Select directory of sourcefiles");
				String dir = dlg.open();
				if (dir != null) {
					text.setText(dir);
					cbtv.setInput(dir);
					cbtv.refresh();
					config.srcDir = text.getText();
				}
			}
		});

		text.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent event) {
			}

			public void focusLost(FocusEvent event) {
				cbtv.setInput(text.getText());
				cbtv.refresh();
				config.srcDir = text.getText();
			}
		});

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {

		TreeSet<String> setOfFileNames = new TreeSet<String>();

		for (Object o : _cbtv.getCheckedElements()) {
			if (((File) o).isFile())
				setOfFileNames.add(((File) o).getAbsolutePath());
		}
		ProjectConfig config = Application.getInstance().getConfiguration();
		config.setSetOfFileNames(setOfFileNames);
		Application.getInstance().getASTManager().reinit(config);
		// System.out.println(((File)o).getName());

		super.okPressed();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#getInitialSize()
	 */
	protected Point getInitialSize() {
		return new Point(460, 640);

	}
}