package sota.gui.graph;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.graph.Edge;
import org.eclipse.ui.PlatformUI;

import sota.app.Application;
import sota.gui.preferences.PreferenceConstants;

/**
 * Stellt eine Kante im Graph dar.
 * 
 */
public class GraphEdge {
	GraphNode sourceNode;
	GraphNode targetNode;
	Edge edge;
	Label hitLabel;
	PolylineConnection connection;

	// Rckwrtsgerichtete Kante
	boolean isInverted = false;

	// Kante mit zwei Pfeilen
	boolean isTwoSided = false;

	// Kante, die den Weg durch eine Ausnahme reprsentiert
	boolean isExceptionEdge = false;

	/**
	 * Der Konstruktor fr die Kante.
	 * 
	 * @param from
	 *            der Urpsrungsknoten
	 * @param to
	 *            der ZielKnoten
	 */
	public GraphEdge(GraphNode from, GraphNode to) {
		sourceNode = from;
		targetNode = to;
	}

	/**
	 * Fhrt ein Update der Darstellung anhand der Prferenzen durch. (Die gesonderte Darstellung der Ausnahmekanten
	 * wurde aktiviert)
	 */
	public void updateStyle() {
		String style;

		if (isExceptionEdge)
			style = PlatformUI.getPreferenceStore().getString(PreferenceConstants.CFG_EDGE_EXCEPTION_STYLE);
		else
			style = PlatformUI.getPreferenceStore().getString(PreferenceConstants.CFG_EDGE_HIT_STYLE);
		/*
		 * else style = PlatformUI.getPreferenceStore().getString( PreferenceConstants.CFG_EDGE_MISS_STYLE);
		 */

		/** BUG: LINE_DOT-style fhrt bei groen Graphen zum Systemcrash! */
		/*
		 * if (style.equals("dot")) connection.setLineStyle(Graphics.LINE_DOT); else if (style.equals("dash"))
		 * connection.setLineStyle(Graphics.LINE_DASH); else if (style.equals("dash-dot"))
		 * connection.setLineStyle(Graphics.LINE_DASHDOT); else if (style.equals("solid"))
		 */
		connection.setLineStyle(Graphics.LINE_SOLID);

		connection.setLineWidth(PlatformUI.getPreferenceStore().getInt(PreferenceConstants.CFG_EDGE_WIDTH));
	}

	/**
	 * Legt die Anzahl der Kantendurchlufe fest und bestimmt daraus die berdeckung.
	 * 
	 * @param i
	 *            die Anzahl der Kantendurchlufe
	 */
	public void showWithTraversals(int i) {
		updateStyle();
		if (Application.getInstance().getConfiguration().showPathNumbers())
			hitLabel.setText(i + "     ");
		else
			hitLabel.setText("");
		if (i > 0)
			connection.setForegroundColor(Application
					.getColorFromPreferenceStore(PreferenceConstants.CFG_EDGE_HIT_COLOR));
		else
			connection.setForegroundColor(Application
					.getColorFromPreferenceStore(PreferenceConstants.CFG_EDGE_MISS_COLOR));
	}

	/**
	 * Zeigt die Kante wieder im berdeckungslosen Modus an.
	 */
	public void showDefault() {
		String style;
		connection.setLineWidth(PlatformUI.getPreferenceStore().getInt(PreferenceConstants.CFG_EDGE_WIDTH));
		connection.setForegroundColor(Application.getColorFromPreferenceStore(PreferenceConstants.CFG_EDGE_DEF_COLOR));

		if (isExceptionEdge)
			style = PlatformUI.getPreferenceStore().getString(PreferenceConstants.CFG_EDGE_EXCEPTION_STYLE);

		else
			style = PlatformUI.getPreferenceStore().getString(PreferenceConstants.CFG_EDGE_HIT_STYLE);

		/** BUG: LINE_DOT-style fhrt bei groen Graphen zum Systemcrash! */
		/*
		 * if (style.equals("dot")) connection.setLineStyle(Graphics.LINE_DOT); else if (style.equals("dash"))
		 * connection.setLineStyle(Graphics.LINE_DASH); else if (style.equals("dash-dot"))
		 * connection.setLineStyle(Graphics.LINE_DASHDOT); else if (style.equals("solid"))
		 */
		connection.setLineStyle(Graphics.LINE_SOLID);
	}
}
