package sota.gui.graph;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import sota.ast.nodes.ASTCaseNode;
import sota.ast.nodes.ASTCatchNode;
import sota.ast.nodes.ASTDoNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.ASTIfNode;
import sota.ast.nodes.ASTIterationNode;
import sota.ast.nodes.ASTJumpNode;
import sota.ast.nodes.ASTSwitchNode;
import sota.ast.nodes.ASTTryNode;
import sota.ast.nodes.Condition;
import sota.ast.nodes.IConditionNode;
import sota.ast.nodes.IStatementNode;
import sota.ast.util.Metrics;
import sota.gui.provider.NodeDialogLabelProvider;

/**
 * Der Dialog fr die Darstellung der Knotendetails.
 * 
 */
public class NodeInfoDialog extends TitleAreaDialog {
	private GraphNode graphNode;

	/**
	 * Der Konstruktor des Dialogs fr einen GraphNode.
	 * 
	 * @param shell
	 *            die Shell
	 * @param graphNode
	 *            der zugehrige GraphNode
	 */
	public NodeInfoDialog(Shell shell, GraphNode graphNode) {
		super(shell);
		setShellStyle(SWT.CLOSE | SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL | SWT.RESIZE);
		this.graphNode = graphNode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitleArea();
		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent) {

		Composite contents = (Composite) super.createDialogArea(parent);
		Composite area = new Composite(contents, SWT.NONE);
		area.setLayout(new GridLayout(3, true));
		createCoverageLabels(area);
		if (graphNode.astNode instanceof IConditionNode && ((IConditionNode) graphNode.astNode).getCondition() != null)
			createConditionTable(contents);

		return contents;
	}

	/**
	 * Erstellt die berdeckungslabel.
	 * 
	 * @param contents
	 *            der Composite
	 */
	protected void createCoverageLabels(Composite contents) {
		GridData data = new GridData();
		Metrics m = null;

		data = new GridData();
		data.horizontalSpan = 3;
		(new Label(contents, SWT.NONE)).setLayoutData(data);
		if (graphNode.astNode instanceof IStatementNode) {
			new Label(contents, SWT.NONE).setText("\t nrHits: ");
			new Label(contents, SWT.NONE).setText(" " + ((IStatementNode) graphNode.astNode).getNrHits());
			new Label(contents, SWT.NONE).setText("");
			if (graphNode.astNode instanceof ASTIfNode) {
				new Label(contents, SWT.NONE).setText("\t trueNode.nrHits: ");
				new Label(contents, SWT.NONE).setText(" " + ((ASTIfNode) graphNode.astNode).getTrueNode().getNrHits());
				new Label(contents, SWT.NONE).setText("");
				new Label(contents, SWT.NONE).setText("\t falseNode.nrHits: ");
				new Label(contents, SWT.NONE).setText(" " + ((ASTIfNode) graphNode.astNode).getFalseNode().getNrHits());
				new Label(contents, SWT.NONE).setText("");

			} else if (graphNode.astNode instanceof ASTIterationNode) {
				new Label(contents, SWT.NONE).setText("\t nrSkips: ");
				new Label(contents, SWT.NONE).setText(" " + ((ASTIterationNode) graphNode.astNode).getNrSkips());
				new Label(contents, SWT.NONE).setText("");

				new Label(contents, SWT.NONE).setText("\t nrSingleLoops: ");
				new Label(contents, SWT.NONE).setText(" " + ((ASTIterationNode) graphNode.astNode).getNrSingleLoops());
				new Label(contents, SWT.NONE).setText("");

				new Label(contents, SWT.NONE).setText("\t nrMultipleLoops: ");
				new Label(contents, SWT.NONE)
						.setText(" " + ((ASTIterationNode) graphNode.astNode).getNrMultipleLoops());
				new Label(contents, SWT.NONE).setText("");

				new Label(contents, SWT.NONE).setText("\t nrLoops: ");
				new Label(contents, SWT.NONE).setText(" " + ((ASTIterationNode) graphNode.astNode).getNrLoops());
				new Label(contents, SWT.NONE).setText("");

			} else if (graphNode.astNode instanceof ASTSwitchNode) {
				for (ASTCaseNode cn : ((ASTSwitchNode) graphNode.astNode).getListOfCases()) {
					if (cn.isDefault())
						new Label(contents, SWT.NONE).setText("\t default:");
					else
						new Label(contents, SWT.NONE).setText("\t " + cn.getContent().trim());

					new Label(contents, SWT.NONE).setText(" " + cn.getNrSelects());
					new Label(contents, SWT.NONE).setText(" nrHits: " + cn.getNrHits());
				}
			} else if (graphNode.astNode instanceof ASTTryNode) {
				new Label(contents, SWT.NONE).setText("\t w/o Exception: ");
				new Label(contents, SWT.NONE).setText(" "
						+ ((ASTTryNode) graphNode.astNode).getNrHitsWithoutException());
				new Label(contents, SWT.NONE);

				data = new GridData();
				data.horizontalSpan = 3;
				(new Label(contents, SWT.NONE)).setLayoutData(data);
				for (ASTCatchNode cn : ((ASTTryNode) graphNode.astNode).getListOfCatches()) {
					if (cn.isFinally())
						new Label(contents, SWT.NONE).setText("\t finally:");
					else
						new Label(contents, SWT.NONE).setText("\t " + cn.getParameter().trim() + ":");
					new Label(contents, SWT.NONE).setText(" " + cn.getNrHits());
					new Label(contents, SWT.NONE);
				}
			}
			data = new GridData();
			data.horizontalSpan = 3;
			(new Label(contents, SWT.NONE)).setLayoutData(data);
		}
		if (graphNode.astNode instanceof ASTFunctionNode) {
			ASTFunctionNode fn = (ASTFunctionNode) graphNode.astNode;
			m = fn.getMetrics();
			new Label(contents, SWT.NONE).setText("\t FEEC:   ");
			new Label(contents, SWT.NONE).setText(" " + m.feec + "%");
			new Label(contents, SWT.NONE).setText("\t (" + (m.nrReachedExits + (fn.getNrHits() > 0 ? 1 : 0)) + "/"
					+ (m.nrFunctionExits + 1) + ")");
			new Label(contents, SWT.NONE).setText("\t C0:     ");
			new Label(contents, SWT.NONE).setText(" " + m.c0 + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrCoveredStatements + "/" + m.nrStatements + ")");
			new Label(contents, SWT.NONE).setText("\t C1:     ");
			new Label(contents, SWT.NONE).setText(" " + m.c1 + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrCoveredBranches + "/" + m.nrBranches + ")");
		}
		if ((graphNode.astNode instanceof IConditionNode && ((IConditionNode) graphNode.astNode)
				.hasNonTrivialCondition())
				|| (graphNode.astNode instanceof ASTFunctionNode && ((ASTFunctionNode) graphNode.astNode).getMetrics().nrAtoms > 0)) {
			if (graphNode.astNode instanceof IConditionNode)
				m = ((IConditionNode) graphNode.astNode).getMetrics();
			new Label(contents, SWT.NONE).setText("\t C2:     ");
			new Label(contents, SWT.NONE).setText(" " + m.c2 + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrAtomEvaluations + "/" + m.nrAtoms * 2 + ")");
			new Label(contents, SWT.NONE).setText("\t MMDC:   ");
			new Label(contents, SWT.NONE).setText(" " + m.mmdc + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrConditionEvaluations + "/" + m.nrConditions * 2 + ")");
			new Label(contents, SWT.NONE).setText("\t MCDC:   ");
			new Label(contents, SWT.NONE).setText(" " + m.mcdc + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrMcdcPairs + "/" + m.nrAtoms + ")");
			new Label(contents, SWT.NONE).setText("\t C3:     ");
			new Label(contents, SWT.NONE).setText(" " + m.c3 + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrAtomCombinationEvaluations + "/" + m.nrAtomCombinations
					+ ")");

		}

		if (graphNode.astNode instanceof ASTIterationNode) {
			m = graphNode.astNode.getFunctionNode().getMetrics();
			new Label(contents, SWT.NONE).setText("\t MBI:     ");
			new Label(contents, SWT.NONE).setText(" " + 100 * m.getNrCoveredModBIPaths(graphNode.astNode.getID())
					/ m.getNrModBIPaths(graphNode.astNode.getID()) + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.getNrCoveredModBIPaths(graphNode.astNode.getID()) + "/"
					+ m.getNrModBIPaths(graphNode.astNode.getID()) + ")");
		} else if (graphNode.astNode instanceof ASTFunctionNode) {
			new Label(contents, SWT.NONE).setText("\t MBI:    ");
			new Label(contents, SWT.NONE).setText(" " + m.mbi + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrCoveredModBIPaths + "/" + m.nrModBIPaths + ")");

			new Label(contents, SWT.NONE).setText("\t    - Function:");
			new Label(contents, SWT.NONE).setText(" " + 100.0 * m.getNrCoveredModBIPaths(graphNode.astNode.getID())
					/ m.getNrModBIPaths(graphNode.astNode.getID()) + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.getNrCoveredModBIPaths(graphNode.astNode.getID()) + "/"
					+ m.getNrModBIPaths(graphNode.astNode.getID()) + ")");

			new Label(contents, SWT.NONE).setText("\t    - Iterations:");
			if (m.nrModBIPaths == m.getNrModBIPaths(graphNode.astNode.getID()))
				new Label(contents, SWT.NONE).setText(" ---");
			else
				new Label(contents, SWT.NONE).setText(" " + 100.0
						* (m.nrCoveredModBIPaths - m.getNrCoveredModBIPaths(graphNode.astNode.getID()))
						/ (m.nrModBIPaths - m.getNrModBIPaths(graphNode.astNode.getID())) + "%");
			new Label(contents, SWT.NONE).setText("\t ("
					+ (m.nrCoveredModBIPaths - m.getNrCoveredModBIPaths(graphNode.astNode.getID())) + "/"
					+ (m.nrModBIPaths - m.getNrModBIPaths(graphNode.astNode.getID())) + ")");

			new Label(contents, SWT.NONE).setText("\t BI:     ");
			new Label(contents, SWT.NONE).setText(" " + m.bi + "%");
			new Label(contents, SWT.NONE).setText("\t (" + m.nrCoveredBIPaths + "/" + m.nrBIPaths + ")");
		}

		new Label(contents, SWT.NONE);
	}

	/**
	 * Erstellt die Bedingungstabelle.
	 * 
	 * @param contents
	 *            der Composite
	 */
	protected void createConditionTable(Composite contents) {

		final Condition cond = ((IConditionNode) graphNode.astNode).getCondition();
		final IConditionNode n = (IConditionNode) graphNode.astNode;

		new Label(contents, SWT.NONE);
		Text condContent = new Text(contents, SWT.NONE);
		condContent.setText("   Condition : " + trimString(cond.getConditionContent()));
		condContent.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		condContent.setEditable(false);
		new Label(contents, SWT.NONE);

		final TableViewer tv = new TableViewer(contents);
		tv.getTable().setLayoutData(new GridData(GridData.FILL_BOTH));
		TableColumn column;

		final int nrAtoms = cond.countNrAtoms();
		final TableColumn[] columns = new TableColumn[nrAtoms + 1];

		column = new TableColumn(tv.getTable(), SWT.NONE);
		column.setText("TruthVector");
		column.setWidth(80);
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				graphNode.firstMCDC = "";
				graphNode.secondMCDC = "";
				tv.refresh();
			}
		});
		columns[0] = column;

		column = new TableColumn(tv.getTable(), SWT.NONE);
		column.setText("evaluates to");
		column.setWidth(80);
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				graphNode.firstMCDC = "";
				graphNode.secondMCDC = "";
				tv.refresh();
			}
		});

		tv.getTable().setHeaderVisible(true);
		String content;
		for (int i = 0; i < nrAtoms; i++) {
			content = trimString(cond.getAtomCondition(i).getContent());
			column = new TableColumn(tv.getTable(), SWT.NONE);
			columns[i + 1] = column;
			column.setText(content);
			if (content.length() > 6)
				column.setWidth(content.length() * 6);
			else
				column.setWidth(40);
			column.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent event) {
					graphNode.firstMCDC = "";
					graphNode.secondMCDC = "";
					x: for (int i = 0; i < nrAtoms; i++)
						if (columns[i + 1] == event.widget)
							for (String s1 : n.getTruthVectors())
								for (String s2 : n.getTruthVectors())
									if (cond.areMcdcPairs(s1, s2, i)) {
										graphNode.firstMCDC = s1;
										graphNode.secondMCDC = s2;
										break x;
									}

					tv.refresh();
				}
			});

		}
		tv.setContentProvider(new IStructuredContentProvider() {

			public Object[] getElements(Object input) {
				if (input instanceof IConditionNode) {
					return ((IConditionNode) input).getTruthVectors().toArray();
				}
				return new Object[0];
			}

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}

		});

		tv.setLabelProvider(new NodeDialogLabelProvider(graphNode));
		tv.setInput(graphNode.astNode);

	}

	/**
	 * Bestimmt den Inhalt des TitleAreas.
	 */
	protected void setTitleArea() {
		setTitle("Node Information");
		String s = "Statement-Node";
		if (graphNode.astNode instanceof ASTIfNode) {
			s = "If-Node";
		} else if (graphNode.astNode instanceof ASTIterationNode) {
			s = "Iteration-Node";
		} else if (graphNode.astNode instanceof ASTDoNode) {
			s = "Do-Node";
		} else if (graphNode.astNode instanceof ASTSwitchNode) {
			s = "Switch-Node";
		} else if (graphNode.astNode instanceof ASTCaseNode) {
			s = "Case-Node";
		} else if (graphNode.astNode instanceof ASTTryNode) {
			s = "Try-Node";
		} else if (graphNode.astNode instanceof ASTCatchNode) {
			s = "Catch-Node";
		} else if (graphNode.astNode instanceof ASTFunctionNode) {
			s = "Function-Node";
		} else if (graphNode.astNode instanceof ASTJumpNode) {
			s = "Jump-Node";
		}
		setMessage(s + " (" + graphNode.astNode.getID() + ") " + "\nLine " + graphNode.astNode.getLineNr() + " in "
				+ graphNode.astNode.getFunctionNode().getFileNode().getName() + ", Function: "
				+ graphNode.astNode.getFunctionNode().getName());
	}

	/**
	 * Trimmt einen String.
	 * 
	 * @param s
	 *            der String
	 * @return der getrimmte String.
	 */
	private String trimString(String s) {
		return s.replaceAll("[\t\n]", "");
	}

}
