package sota.gui.graph;

/**
 * Verwaltet die verschiedenen Zoom-Stufen der Darstellung des CFG.
 * 
 */
public class ZoomManager {

	private int[] nodeHeight = { 40, 30, 25, 20, 15, 12, 8, 6 };
	private int[] nodeWidth = { 50, 50, 45, 40, 35, 12, 8, 6 };

	private int[] condYOffset = { 4, 2, 1, 0, 0, 0, 0, 0 };
	private int[] condYDist = { 8, 7, 6, 5, 0, 0, 0, 0 };
	private int[] condHeight = { 7, 6, 5, 4, 0, 0, 0, 0 };
	private int[] condWidth = { 7, 6, 5, 4, 0, 0, 0, 0 };

	private int[] arrowXScale = { 6, 6, 5, 5, 5, 4, 4, 3 };
	private int[] arrowYScale = { 3, 3, 3, 3, 2, 2, 2, 2 };

	// private String[] fontName = { "font0", "font1", "font2", "font3", "font4", "", ""};

	// private int[] padding = {24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 5};
	private int[] padding = { 16, 14, 12, 10, 10, 10, 8, 4 };

	private int maxZoomLevel = 7;
	private int zoomLevel = 0;

	/**
	 * Liefert den Innenabstand.
	 * 
	 * @return der Innenabstand
	 */
	public int getPadding() {
		return padding[zoomLevel];
	}

	/**
	 * Zoomt in den Graphen hinein.
	 * 
	 * @return true, wenn Level vergrert werden konnte, sonst false
	 */
	public boolean zoomIn() {
		if (zoomLevel > 0) {
			zoomLevel--;
			return true;
		}
		return false;
	}

	/**
	 * Zoomt aus dem Graphen heraus.
	 * 
	 * @return true, wenn Level veringert werden konnte, sonst false
	 */
	public boolean zoomOut() {
		if (zoomLevel < maxZoomLevel) {
			zoomLevel++;
			return true;
		}
		return false;
	}

	/**
	 * Liefert das aktuelle Zoomlevel.
	 * 
	 * @return das aktuelle Zoomlevel
	 */
	public int getZoomLevel() {
		return zoomLevel;
	}

	/**
	 * Liefert die Knotenhhe.
	 * 
	 * @return die Knotenhhe
	 */
	public int getNodeHeight() {
		return nodeHeight[zoomLevel];
	}

	/**
	 * Liefert die Knotenbreite.
	 * 
	 * @return die Knotenbreite
	 */
	public int getNodeWidth() {
		return nodeWidth[zoomLevel];
	}

	/**
	 * Liefert den y-Offset der Bedingungslabels.
	 * 
	 * @return der y-Offset
	 */
	public int getCondYOffset() {
		return condYOffset[zoomLevel];
	}

	/**
	 * Liefert den x-Offset der Bedingungslabels.
	 * 
	 * @return der x-Offset
	 */
	public int getCondYDist() {
		return condYDist[zoomLevel];
	}

	/**
	 * Liefert die Hhe der Bedingungslabels.
	 * 
	 * @return die Hhe
	 */
	public int getCondHeight() {
		return condHeight[zoomLevel];
	}

	/**
	 * Liefert die Breite der Bedingungslabels.
	 * 
	 * @return die Breite
	 */
	public int getCondWidth() {
		return condWidth[zoomLevel];
	}

	/**
	 * Liefert die x-Achsen-Skalierung der Pfeile.
	 * 
	 * @return die x-Achsen-Skalierung
	 */
	public int getArrowXScale() {
		return arrowXScale[zoomLevel];
	}

	/**
	 * Liefert die y-Achsen-Skalierung der Pfeile.
	 * 
	 * @return die y-Achsen-Skalierung
	 */
	public int getArrowYScale() {
		return arrowYScale[zoomLevel];
	}

	/**
	 * Gibt vor, ob der Buttontext angezeigt werden soll.
	 * 
	 * @return true, wenn ja, sonst false
	 */
	public boolean showButtonText() {
		return zoomLevel <= 4;
	}

	/**
	 * Liefert die maximale Zoomstufe.
	 * 
	 * @return die maximale Zoomstufe
	 */
	public int getMaxZoomLevel() {
		return maxZoomLevel;
	}

}
