package sota.gui.preferences;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.ui.PlatformUI;

import sota.ast.util.ColorManager;

/**
 * Klasse zur Initialisierung der Prferenzen.
 * 
 */
public class PreferenceInitializer extends AbstractPreferenceInitializer {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = PlatformUI.getPreferenceStore();

		// SourceView
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_LINE_HIT_COLOR, ColorManager.LGREEN);
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_LINE_MISS_COLOR, ColorManager.MRED);
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_ASC_COLOR, ColorManager.ASCCOMMENT);
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_KEYWORD_COLOR, ColorManager.KEYWORD);
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_COMMENT_COLOR, ColorManager.COMMENT);
		PreferenceConverter.setDefault(store, PreferenceConstants.SOURCE_STRING_COLOR, ColorManager.STRING);
		store.setDefault(PreferenceConstants.SOURCE_INDENT_SIZE, 10);

		// CFG-View
		store.setDefault(PreferenceConstants.CFG_SHOW_CONDITIONCOVERAGE_LABELS, true);
		store.setDefault(PreferenceConstants.CFG_SHOW_PATHCOVERAGE_LABELS, false);
		store.setDefault(PreferenceConstants.CFG_SHOW_BASICOVERAGE_LABELS, false);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_NODE_DEF_COLOR, ColorManager.GRAYBLUE);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_NODE_HIT_COLOR, ColorManager.LGREEN);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_NODE_MISS_COLOR, ColorManager.RED);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_BRANCHINIG_NODE_COLOR, ColorManager.YELLOW);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_EDGE_DEF_COLOR, ColorManager.BLACK);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_EDGE_HIT_COLOR, ColorManager.DGREEN);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_EDGE_MISS_COLOR, ColorManager.DRED);
		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_CURR_LINE_COLOR, ColorManager.YELLOW);

		PreferenceConverter.setDefault(store, PreferenceConstants.CFG_DEF_COV_LABEL, ColorConstants.lightGray.getRGB());

		store.setDefault(PreferenceConstants.CFG_SHOW_BRANCHING_NODE_COLOR, true);
		store.setDefault(PreferenceConstants.CFG_EDGE_WIDTH, "1");
		store.setDefault(PreferenceConstants.CFG_EDGE_HIT_STYLE, "solid");
		store.setDefault(PreferenceConstants.CFG_EDGE_MISS_STYLE, "dash");
		store.setDefault(PreferenceConstants.CFG_EDGE_EXCEPTION_STYLE, "dot");

		store.setDefault(PreferenceConstants.COVERAGE_FIRST_BAR, 99);
		store.setDefault(PreferenceConstants.COVERAGE_SECOND_BAR, 90);
		store.setDefault(PreferenceConstants.COVERAGE_THIRD_BAR, 75);
		store.setDefault(PreferenceConstants.COVERAGE_FOURTH_BAR, 50);
		store.setDefault(PreferenceConstants.COVERAGE_FIFTH_BAR, 25);

		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_TOP_BAR_COLOR, ColorManager.C1);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_FIRST_BAR_COLOR, ColorManager.C2);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_SECOND_BAR_COLOR, ColorManager.C3);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_THIRD_BAR_COLOR, ColorManager.C4);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_FOURTH_BAR_COLOR, ColorManager.C5);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_FIFTH_BAR_COLOR, ColorManager.C6);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_ABOVE_BOTTOM_BAR_COLOR, ColorManager.C7);
		PreferenceConverter.setDefault(store, PreferenceConstants.COVERAGE_BOTTOM_BAR_COLOR, ColorManager.C8);

		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_TOP_BAR_COLOR, ColorManager.C1);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_FIRST_BAR_COLOR, ColorManager.C2);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_SECOND_BAR_COLOR, ColorManager.C3);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_THIRD_BAR_COLOR, ColorManager.C4);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_FOURTH_BAR_COLOR, ColorManager.C5);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_FIFTH_BAR_COLOR, ColorManager.C6);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_ABOVE_BOTTOM_BAR_COLOR, ColorManager.C7);
		PreferenceConverter.setDefault(store, PreferenceConstants.REPORT_BOTTOM_BAR_COLOR, ColorManager.C8);

		store.setDefault(PreferenceConstants.REPORT_PROMPT_FILENAME, true);
		store.setDefault(PreferenceConstants.REPORT_OVERWRITE_REPORT, true);
		store.setDefault(PreferenceConstants.REPORT_SHOW_STATISTICS, true);
		store.setDefault(PreferenceConstants.REPORT_SHOW_OVERVIEW, true);
		store.setDefault(PreferenceConstants.REPORT_SHOW_CLASSES, true);
		store.setDefault(PreferenceConstants.REPORT_SHOW_FUNCTIONS, true);
		store.setDefault(PreferenceConstants.REPORT_SHOW_TESTS, true);
		store.setDefault(PreferenceConstants.REPORT_USE_COLORS, true);
		store.setDefault(PreferenceConstants.REPORT_FONT_SIZE, 10);
		store.setDefault(PreferenceConstants.REPORT_USE_ALL_TESTS, true);
		store.setDefault(PreferenceConstants.REPORT_IGNORE_INNER_CLASSES, true);

		store.setDefault(PreferenceConstants.GENERAL_CREATE_LOG, true);
		store.setDefault(PreferenceConstants.GENERAL_OVERWRITE_LOG, true);
		// store.setDefault(PreferenceConstants.GENERAL_LIST_FILE, false);
		store.setDefault(PreferenceConstants.GENERAL_PARSE_DIRTY, true);
		store.setDefault(PreferenceConstants.GENERAL_ENABLE_DEBUG, false);

	}

}
