package sota.gui.provider;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

import sota.app.Application;
import sota.ast.nodes.FileNode;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;
import sota.ast.util.IScheme;
import sota.gui.GuiUpdateListener;

/**
 * Der LabelProvider fr die Ansicht der Instrumentationsschemata.
 * 
 */
public class InstrumentationLabelProvider implements ILabelProvider, GuiUpdateListener {

	private IScheme scheme;

	/**
	 * Konstrukor des Labelproviders.
	 * 
	 * @param scheme
	 *            das aktuelle gewhlte IScheme
	 */
	public InstrumentationLabelProvider(IScheme scheme) {
		this.scheme = scheme;
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
	}

	/**
	 * Konstruktor des Labelproviders
	 */
	public InstrumentationLabelProvider() {
		scheme = Application.getInstance().getASTManager().getConfiguration().getCurrentIScheme();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object object) {
		IScopeNode n;
		if (object instanceof IScheme)
			return Application.getInstance().getImageRegistry().get("ischeme");
		if (object instanceof String)
			return Application.getInstance().getImageRegistry().get("lvl0");
		if (object instanceof IScopeNode) {
			n = (IScopeNode) object;
			if (scheme != null && scheme.hasInstrumentationLevel(n.getIDString())) {
				switch (scheme.getInstrumentationLevel(n.getIDString())) {
				case 0:
					return Application.getInstance().getImageRegistry().get("lvl0");
				case 1:
					return Application.getInstance().getImageRegistry().get("lvl1");
				case 2:
					return Application.getInstance().getImageRegistry().get("lvl2");
				case 3:
					return Application.getInstance().getImageRegistry().get("lvl3");
				default:
					return Application.getInstance().getImageRegistry().get("lvl0");
				}
			} else if (n.getParent() != null) {
				return getImage(n.getParent());
			}
		}
		return Application.getInstance().getImageRegistry().get("lvl0");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object object) {

		if (object instanceof IScheme)
			return ((IScheme) object).getName();
		if (object instanceof String)
			return (String) object;
		if (object instanceof FileNode) {
			return ((FileNode) object).getName();
		}
		if (object instanceof IScopeNode)
			return ((IScopeNode) object).getName();

		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener arg0) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object arg0, String arg1) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener arg0) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		scheme = ast.getConfiguration().getCurrentIScheme();
	}

}
