package sota.gui.provider;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Image;

import sota.app.Application;
import sota.ast.nodes.ASTClassNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.FileNode;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.Metrics;

/**
 * Der LabelProvider fr die MetricsView.
 * 
 */
public class MetricsViewLabelProvider implements ITableLabelProvider {

	private static String EMPTY = "  ---  ";

	private IProjectTreeView projectTreeView;

	/**
	 * Der Konstruktor.
	 * 
	 * @param projectTreeView
	 *            der Layoutprivder
	 */
	public MetricsViewLabelProvider(IProjectTreeView projectTreeView) {
		super();
		this.projectTreeView = projectTreeView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
	 */
	public Image getColumnImage(Object element, int columnIndex) {
		if (columnIndex != 0)
			return null;
		if (element instanceof ASTFunctionNode) {
			if (((ASTFunctionNode) element).isPrivate())
				return Application.getInstance().getImageRegistry().get("privatemethod");
			if (((ASTFunctionNode) element).isProtected())
				return Application.getInstance().getImageRegistry().get("protectedmethod");
			return Application.getInstance().getImageRegistry().get("method");

		}

		if (element instanceof ASTClassNode) {
			if (((ASTClassNode) element).isTopLevel())
				return Application.getInstance().getImageRegistry().get("class");
			return Application.getInstance().getImageRegistry().get("innerclass");
		}

		if (element instanceof FileNode)
			return Application.getInstance().getImageRegistry().get("jfile");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	public String getColumnText(Object element, int columnIndex) {
		Metrics m;
		boolean showFlat = projectTreeView.showFlat();
		boolean showPackages = projectTreeView.showPackageNames();
		boolean showFiles = projectTreeView.showFiles();

		// Zeilenname
		if (columnIndex == 0) {
			if (element instanceof String)
				return "Project " + (String) element;

			if (element instanceof IScopeNode) {
				if (showFlat) {
					return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
				} else {
					if (showPackages) {
						if (element instanceof FileNode)
							return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
						if (!showFiles && element instanceof ASTClassNode && ((ASTClassNode) element).isTopLevel())
							return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
					}
					return ((IScopeNode) element).getName();

				}
			}
		}

		if (element instanceof String)
			m = Application.getInstance().getASTManager().getProjectMetrics();
		else if (element instanceof IScopeNode)
			m = ((IScopeNode) element).getMetrics();
		else
			return EMPTY;

		switch (columnIndex) {
		case 1:
			if (m.cyclomaticComplexity == 0)
				return EMPTY;
			if (element instanceof ASTFunctionNode)
				return m.cyclomaticComplexity + "";
			return "< " + (m.cyclomaticComplexity + 1);
		case 2:
			if (m.essentialComplexity == 0)
				return EMPTY;
			if (element instanceof ASTFunctionNode)
				return m.essentialComplexity + "";
			return "< " + (m.essentialComplexity + 1);
		case 3:
			if (m.nrLines == 0)
				return EMPTY;
			return m.nrLines + "";
		case 4:
			if (m.nrStatements == 0)
				return EMPTY;
			return m.nrStatements + "";
		case 5:
			if (m.nrBranches == 0)
				return EMPTY;
			return m.nrBranches + "";
		case 6:
			if (m.nrModBIPaths == 0)
				return EMPTY;
			return m.nrModBIPaths + "";
		case 7:
			if (m.nrBIPaths == 0)
				return EMPTY;
			return m.nrBIPaths + "";
		case 8:
			if (m.nrConditionNodes == 0)
				return EMPTY;
			return m.nrConditionNodes + "";
		case 9:
			if (m.nrAtoms == 0)
				return EMPTY;
			return m.nrAtoms + "";
		case 10:
			if (m.nrConditions == 0)
				return EMPTY;
			return m.nrConditions + "";

		default:
			return EMPTY;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}

}
