package sota.gui.provider;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import sota.app.Application;
import sota.ast.nodes.ASTClassNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.ASTScopeNode;
import sota.ast.nodes.FileNode;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;

/**
 * ContentProvider fr alle Ansichten mit Projektstrukturen.
 * 
 */
public class ProjectContentProvider implements ITreeContentProvider {

	private IProjectTreeView projectTreeView;

	/**
	 * Der Konstrukor
	 * 
	 * @param projectTreeView
	 *            die zugehrige IProjectTreeView
	 */
	public ProjectContentProvider(IProjectTreeView projectTreeView) {
		super();
		this.projectTreeView = projectTreeView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		if (projectTreeView.showFlat())
			return null;
		if (parentElement instanceof FileNode) {
			ArrayList<ASTClassNode> al = new ArrayList<ASTClassNode>();
			for (ASTClassNode cn : ((FileNode) parentElement).getClassNodes()) {
				if (cn.isTopLevel())
					al.add(cn);
			}
			return al.toArray();
		}
		if (parentElement instanceof ASTScopeNode) {
			ArrayList<IScopeNode> l = new ArrayList<IScopeNode>();
			l.addAll(((ASTScopeNode) parentElement).getFunctionNodes());
			if (projectTreeView.showInnerClasses())
				l.addAll(((ASTScopeNode) parentElement).getClassNodes());
			return l.toArray();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		if (projectTreeView.showFlat())
			return null;
		if (element instanceof IScopeNode) {
			if (projectTreeView.showFiles() || !(((IScopeNode) element).getParent() instanceof FileNode))
				return ((IScopeNode) element).getParent();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		if (projectTreeView.showFlat())
			return false;
		if (element instanceof ASTManager)
			return true;
		if (element instanceof IScopeNode) {
			if (element instanceof FileNode)
				return ((IScopeNode) element).getClassNodes().size() + ((IScopeNode) element).getFunctionNodes().size() > 0;
			if (element instanceof ASTClassNode) {
				if (((ASTClassNode) element).getFunctionNodes().size() > 0)
					return true;
				if (projectTreeView.showInnerClasses())
					return ((ASTClassNode) element).getClassNodes().size() > 0;
			}
			if (element instanceof ASTFunctionNode) {
				if (projectTreeView.showInnerClasses())
					return ((ASTFunctionNode) element).getClassNodes().size() > 0;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public Object[] getElements(Object inputElement) {
		if (!(inputElement instanceof ASTManager))
			return new Object[0];

		Vector<Object> v = new Vector<Object>();
		if (projectTreeView.showProjectLine()) {
			v.add(Application.getInstance().getConfiguration().getName());
		}
		Vector<IScopeNode> fns = new Vector<IScopeNode>();

		if (projectTreeView.showFlat()) {
			// Files sind Elemente, auer wenn explizit abgeschaltet
			if (projectTreeView.showFiles())
				fns.addAll(((ASTManager) inputElement).getFileNodes());
			// Klasses sind Elemente, wenn angeschaltet und FlatLayout ist oder
			// keine Files gezeigt werden
			if (projectTreeView.showClasses()) {
				if (projectTreeView.showInnerClasses())
					fns.addAll(((ASTManager) inputElement).getClassNodes());
				else
					fns.addAll(((ASTManager) inputElement).getTopLevelClassNodes());
			}
			if (projectTreeView.showInnerClasses())
				fns.addAll(((ASTManager) inputElement).getFunctionNodes());
			else
				fns.addAll(((ASTManager) inputElement).getTopLevelFunctionNodes());
		} else {
			if (projectTreeView.showFiles())
				fns.addAll(((ASTManager) inputElement).getFileNodes());
			else
				fns.addAll(((ASTManager) inputElement).getTopLevelClassNodes());
		}

		Object[] array = fns.toArray();
		Arrays.sort(array, projectTreeView.getComparator());
		v.addAll(Arrays.asList(array));
		return v.toArray();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
	 * java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}
}
