package sota.gui.provider;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

import sota.app.Application;
import sota.ast.nodes.ASTClassNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.FileNode;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;

/**
 * Der LabelProvider fr alle Ansichten mit Projektstrukturen.
 * 
 */
public class ProjectViewLabelProvider implements ILabelProvider {

	private IProjectTreeView projectTreeView;

	/**
	 * Der Konstruktor
	 * 
	 * @param projectTreeView
	 *            die zugehrige View
	 */
	public ProjectViewLabelProvider(IProjectTreeView projectTreeView) {
		super();
		this.projectTreeView = projectTreeView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object object) {

		if (object instanceof String) {
			if (ASTManager.isInstrumented((String) object))
				return Application.getInstance().getImageRegistry().get("djfile");
			return Application.getInstance().getImageRegistry().get("dfile");
		}

		if (object instanceof FileNode) {
			if (((FileNode) object).isDirty())
				return Application.getInstance().getImageRegistry().get("djfile");
			return Application.getInstance().getImageRegistry().get("jfile");
		}

		if (object instanceof ASTFunctionNode) {
			if (((ASTFunctionNode) object).isPrivate()) {
				if (((ASTFunctionNode) object).getFileNode().isDirty())
					return Application.getInstance().getImageRegistry().get("dprivmethod");
				return Application.getInstance().getImageRegistry().get("privatemethod");
			}
			if (((ASTFunctionNode) object).isProtected()) {
				if (((ASTFunctionNode) object).getFileNode().isDirty())
					return Application.getInstance().getImageRegistry().get("dprotmethod");
				return Application.getInstance().getImageRegistry().get("protectedmethod");
			}
			if (((ASTFunctionNode) object).getFileNode().isDirty())
				return Application.getInstance().getImageRegistry().get("dpubmethod");
			return Application.getInstance().getImageRegistry().get("method");
		}
		if (object instanceof ASTClassNode) {
			if (((ASTClassNode) object).isTopLevel()) {
				if (((ASTClassNode) object).getFileNode().isDirty())
					return Application.getInstance().getImageRegistry().get("dclass");
				return Application.getInstance().getImageRegistry().get("class");
			}
			if (((ASTClassNode) object).getFileNode().isDirty())
				return Application.getInstance().getImageRegistry().get("diclass");
			return Application.getInstance().getImageRegistry().get("innerclass");
		}
		return Application.getInstance().getImageRegistry().get("jfile");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element) {
		boolean showFlat = projectTreeView.showFlat();
		boolean showPackages = projectTreeView.showPackageNames();
		boolean showFiles = projectTreeView.showFiles();

		if (element instanceof String)
			return (String) element;

		if (element instanceof IScopeNode) {
			if (showFlat) {
				return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
			} else {
				if (showPackages) {
					if (element instanceof FileNode)
						return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
					if (!showFiles && element instanceof ASTClassNode && ((ASTClassNode) element).isTopLevel())
						return ((IScopeNode) element).getFullyQualifiedName(showPackages, showFiles);
				}
				return ((IScopeNode) element).getName();

			}
		}
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener arg0) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object arg0, String arg1) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener arg0) {
		// TODO Auto-generated method stub

	}

}
