package sota.gui.views;

import java.util.Comparator;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;
import sota.gui.GuiUpdateListener;
import sota.gui.ProjectChangeListener;
import sota.gui.provider.CoverageViewLabelProvider;
import sota.gui.provider.IProjectTreeView;
import sota.gui.provider.ProjectContentProvider;

/**
 * Stellt die Ansicht fr die tabellarische berdeckungsauswertung dar.
 * 
 */
public class CoverageView extends ViewPart implements GuiUpdateListener, ProjectChangeListener,
		IPropertyChangeListener, IProjectTreeView {
	public static final String ID = "sota.coverageview";

	private boolean showFlat = false;
	private boolean showPackages = false;
	private boolean showFiles = true;
	private boolean showClasses = true;
	private boolean showInnerClasses = true;
	private boolean reverseSort = false;
	private CoverageSortCriteria sortCriterium = CoverageSortCriteria.name;

	private boolean showPercentage = true;
	private int initColumnSize[] = { 200, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60 };
	private TreeViewer tv;
	private Action showFlatAction;
	private Action showHierarchicalAction;
	private Action changeInfoAction;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {

		Application.getInstance().getASTManager().addProjectChangeListener(this);
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		Application.getInstance().getASTManager().addProjectTreeView(this);
		PlatformUI.getPreferenceStore().addPropertyChangeListener(this);

		createActions();

		tv = new TreeViewer(parent);
		tv.setContentProvider(new ProjectContentProvider(this));
		tv.setLabelProvider(new CoverageViewLabelProvider(this));
		final TreeViewer ftv = tv;

		TreeColumn column;

		/*
		 * C0 = Anweisungsberdeckung C1 = Zweigberdeckung BI = Boundary-Interior-berdeckung C2 = einfache
		 * Bedingungsberdeckung C3 = mehrfache Bedingungsberdeckung MMDC = minimal mehrfache Bedingungsberdeckung
		 */

		final String[] columns = { "Name  ", "FEEC  ", "C0  ", "C1  ", "C2  ", "MMDC  ", "MCDC  ", "C3  ", "MBI  ",
				"BI  " };

		int columnIndex = 0;
		for (String s : columns) {
			column = new TreeColumn(tv.getTree(), SWT.NONE);
			column.setAlignment(SWT.RIGHT);
			column.setText(s);

			column.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					if (event.widget instanceof TreeColumn) {
						for (int i = 0; i < columns.length; i++) {
							if (((TreeColumn) event.widget).getText().equals(columns[i])) {
								switch (i) {
								case 0:

									if (sortCriterium == CoverageSortCriteria.name)
										;
									reverseSort = !reverseSort;
									sortCriterium = CoverageSortCriteria.name;
									break;
								case 1:
									if (sortCriterium == CoverageSortCriteria.feec)
										reverseSort = !reverseSort;
									sortCriterium = CoverageSortCriteria.feec;
									break;
								case 2:
									if (sortCriterium == CoverageSortCriteria.c0)
										reverseSort = !reverseSort;
									sortCriterium = CoverageSortCriteria.c0;
									break;
								case 3:
									if (sortCriterium == CoverageSortCriteria.c1)
										reverseSort = !reverseSort;
									sortCriterium = CoverageSortCriteria.c1;
									break;
								case 4:
									if (sortCriterium == CoverageSortCriteria.c2)
										reverseSort = !reverseSort;
									sortCriterium = CoverageSortCriteria.c2;
									break;
								case 5:
									if (sortCriterium == CoverageSortCriteria.mmdc)
										reverseSort = !reverseSort;

									sortCriterium = CoverageSortCriteria.mmdc;
									break;
								case 6:
									if (sortCriterium == CoverageSortCriteria.mcdc)
										reverseSort = !reverseSort;

									sortCriterium = CoverageSortCriteria.mcdc;
									break;
								case 7:
									if (sortCriterium == CoverageSortCriteria.c3)
										reverseSort = !reverseSort;

									sortCriterium = CoverageSortCriteria.c3;
									break;
								case 8:
									if (sortCriterium == CoverageSortCriteria.mbi)
										reverseSort = !reverseSort;

									sortCriterium = CoverageSortCriteria.mbi;
									break;
								case 9:
									if (sortCriterium == CoverageSortCriteria.bi)
										reverseSort = !reverseSort;

									sortCriterium = CoverageSortCriteria.bi;
									break;
								}
							}
						}
						ftv.getTree().setRedraw(false);
						ftv.collapseAll();
						ftv.getTree().setRedraw(true);
						ftv.refresh();

					}
				}
			});
			column.setWidth(initColumnSize[columnIndex++]);
		}
		tv.getTree().setHeaderVisible(true);

		tv.setInput(Application.getInstance().getASTManager());

		tv.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				Application.getInstance().getASTManager().select(
						((IStructuredSelection) event.getSelection()).getFirstElement());
			}
		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.ProjectChangeListener#projectChanged(sota.ast.util.ASTManager)
	 */
	public void projectChanged(ASTManager ast) {
		tv.setInput(ast);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		tv.refresh();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().startsWith("Coverage")) {
			tv.refresh();
		}
	}

	/**
	 * Erstellt die Aktionen fr diese Ansicht.
	 */
	private void createActions() {

		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();

		IMenuManager mm = getViewSite().getActionBars().getMenuManager();

		changeInfoAction = new Action("Change Info", IAction.AS_CHECK_BOX) {
			public void run() {
				showPercentage = !showPercentage;
				tv.refresh();
			}
		};
		changeInfoAction.setChecked(showPercentage);

		Action expandAllAction = new Action("Expand All") {
			public void run() {
				tv.getTree().setRedraw(false);
				tv.expandAll();
				tv.getTree().setRedraw(true);
			}
		};

		Action collapseAllAction = new Action("Collapse All") {
			public void run() {
				tv.getTree().setRedraw(false);
				tv.collapseAll();
				tv.getTree().setRedraw(true);
			}
		};

		Action showPackageAction = new Action("Show PackageNames", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showPackages = !showPackages;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showFilesAction = new Action("Show Files", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showFiles = !showFiles;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showClassesAction = new Action("Show Classes", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showClasses = !showClasses;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showInnerClassesAction = new Action("Show Inner Classes", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showInnerClasses = !showInnerClasses;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		showFlatAction = new Action("Flat Presentation", IAction.AS_RADIO_BUTTON) {
			public void run() {
				showFlat(true);
			}
		};

		showHierarchicalAction = new Action("Hierarchical Presentation", IAction.AS_RADIO_BUTTON) {
			public void run() {
				showFlat(false);
			}
		};

		showFlatAction.setChecked(showFlat);
		showHierarchicalAction.setChecked(!showFlat);

		showPackageAction.setChecked(showPackages);
		showFilesAction.setChecked(showFiles);
		showClassesAction.setChecked(showClasses);
		showInnerClassesAction.setChecked(showInnerClasses);

		expandAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("expandall"));
		collapseAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("collapseall"));
		showPackageAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("package"));
		showFilesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("jfile"));
		showClassesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("class"));
		showInnerClassesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor(
				"innerclass"));
		showFlatAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("flat"));
		showHierarchicalAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor(
				"hierarchical"));
		changeInfoAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("percent"));

		mm.add(showHierarchicalAction);
		mm.add(showFlatAction);
		mm.add(new Separator());
		mm.add(showPackageAction);
		mm.add(showFilesAction);
		mm.add(showClassesAction);
		mm.add(showInnerClassesAction);
		// mm.add(changeInfoAction);

		tbm.add(changeInfoAction);
		tbm.add(expandAllAction);
		tbm.add(collapseAllAction);
	}

	/**
	 * Informiert darber, ob die Ansicht die prozentual anzeigen soll.
	 * 
	 * @return true, wenn prozentual eingestellt wurde, sonst false.
	 */
	public boolean showPercentage() {
		return showPercentage;
	}

	/**
	 * Legt fest, ob die Anzeige der Strukturen hierarchisch oder flach sein soll.
	 * 
	 * @param b
	 *            true, wenn sie flach sein soll, sonst false
	 */
	private void showFlat(boolean b) {
		tv.getTree().setRedraw(false);
		showFlat = b;
		showFlatAction.setChecked(b);
		showHierarchicalAction.setChecked(!b);
		tv.refresh();
		// tv.setInput(Application.getInstance().getASTManager());
		tv.getTree().setRedraw(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showProjectLine()
	 */
	public boolean showProjectLine() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showPackageNames()
	 */
	public boolean showPackageNames() {
		return showPackages;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFiles()
	 */
	public boolean showFiles() {
		return showFiles;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showClasses()
	 */
	public boolean showClasses() {
		return showClasses;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showInnerClasses()
	 */
	public boolean showInnerClasses() {
		return showInnerClasses;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFlat()
	 */
	public boolean showFlat() {
		return showFlat;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#getComparator()
	 */
	@SuppressWarnings("unchecked")
	public Comparator getComparator() {
		return new Comparator() {

			@Override
			public int compare(Object firstObject, Object secondObject) {
				if (!(firstObject instanceof IScopeNode && secondObject instanceof IScopeNode))
					return 0;

				IScopeNode first = (IScopeNode) firstObject, second = (IScopeNode) secondObject;
				int factor = 1;
				if (reverseSort) {
					factor = -1;
				}
				switch (sortCriterium) {
				case name:
					return factor
							* first.getFullyQualifiedName(showPackageNames(), showFiles()).compareTo(
									second.getFullyQualifiedName(showPackageNames(), showFiles()));
				case feec:
					return factor * new Float(first.getMetrics().feec).compareTo(new Float(second.getMetrics().feec));
				case c0:
					return factor * new Float(first.getMetrics().c0).compareTo(new Float(second.getMetrics().c0));
				case c1:
					return factor * new Float(first.getMetrics().c1).compareTo(new Float(second.getMetrics().c1));
				case c2:
					return factor * new Float(first.getMetrics().c2).compareTo(new Float(second.getMetrics().c2));
				case mmdc:
					return factor * new Float(first.getMetrics().mmdc).compareTo(new Float(second.getMetrics().mmdc));
				case mcdc:
					return factor * new Float(first.getMetrics().mcdc).compareTo(new Float(second.getMetrics().mcdc));
				case c3:
					return factor * new Float(first.getMetrics().c3).compareTo(new Float(second.getMetrics().c3));
				case mbi:
					return factor * new Float(first.getMetrics().mbi).compareTo(new Float(second.getMetrics().mbi));
				case bi:
					return factor * new Float(first.getMetrics().bi).compareTo(new Float(second.getMetrics().bi));
				}
				return 0;
			}
		};
	}

	/**
	 * Die Sortierungskriterien der Ansicht.
	 * 
	 */
	public enum CoverageSortCriteria {
		name, feec, c0, c1, c2, mmdc, mcdc, c3, mbi, bi
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#select(java.lang.Object)
	 */
	@Override
	public void select(Object o) {
		tv.setSelection(new StructuredSelection(o), true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite, org.eclipse.ui.IMemento)
	 */
	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		super.init(site, memento);
		if (memento == null)
			return;
		for (int i = 0; i < initColumnSize.length; i++) {
			if (memento.getInteger("initCoverageViewColumnSize_" + i) != null)
				initColumnSize[i] = memento.getInteger("initCoverageViewColumnSize_" + i);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.ViewPart#saveState(org.eclipse.ui.IMemento)
	 */
	@Override
	public void saveState(IMemento memento) {
		int i = 0;
		for (TreeColumn tc : tv.getTree().getColumns()) {
			memento.putInteger("initCoverageViewColumnSize_" + i++, tc.getWidth());
		}
		super.saveState(memento);
	}

}
