package sota.gui.views;

import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.GlobalInstrumentationScheme;
import sota.ast.util.IScheme;
import sota.gui.GuiUpdateListener;
import sota.gui.ProjectChangeListener;
import sota.gui.dialogs.EditISchemeDialog;
import sota.gui.provider.InstrumentationLabelProvider;

/**
 * Die Ansicht fr die Instrumentationsschemata.
 * 
 */
public class ISchemeView extends ViewPart implements ProjectChangeListener, GuiUpdateListener {
	public static final String ID = "sota.ischemeview";

	private TreeViewer tv;
	private IStructuredSelection selection = null;
	private IScheme ischeme = null;
	private Action deleteAction = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		createActions();
		tv = new TreeViewer(parent, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
		tv.setContentProvider(new ITreeContentProvider() {

			public Object[] getElements(Object inputElement) {
				if (inputElement instanceof ASTManager)
					return ((ASTManager) inputElement).getConfiguration().getListOfISchemes().toArray();
				return null;
			}

			public void dispose() {
			}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}

			public Object[] getChildren(Object parentElement) {
				return null;
			}

			public Object getParent(Object element) {
				return null;
			}

			public boolean hasChildren(Object element) {
				return false;
			}
		});
		tv.setLabelProvider(new InstrumentationLabelProvider());
		tv.setInput(Application.getInstance().getASTManager());
		Application.getInstance().getASTManager().addProjectChangeListener(this);
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		Application.getInstance().getASTManager().getConfiguration().setCurrentInstrumentationScheme("");
		getSite().setSelectionProvider(tv);
		tv.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				selection = (IStructuredSelection) tv.getSelection();
				ischeme = (IScheme) selection.getFirstElement();

				if (selection != null && (selection.getFirstElement() instanceof IScheme)) {
					if (!(ischeme instanceof GlobalInstrumentationScheme)) {
						deleteAction.setEnabled(true);
						return;
					}
				}
				deleteAction.setEnabled(false);
			}

		});

		tv.addDoubleClickListener(new IDoubleClickListener() {

			@Override
			public void doubleClick(DoubleClickEvent event) {
				IStructuredSelection ss = (IStructuredSelection) event.getSelection();
				if (ss.getFirstElement() instanceof IScheme)
					(new EditISchemeDialog(getViewSite().getShell(), (IScheme) ss.getFirstElement())).open();
			}
		});
	}

	/**
	 * Erstellt die Aktionen fr diese Ansicht.
	 */
	private void createActions() {
		deleteAction = new Action("Delete IScheme") {
			public void run() {

				int result = (new MessageDialog(getViewSite().getShell(), "Confirm Delete", null,
						"Are you sure you want to delete IScheme " + ischeme.getName() + "?", MessageDialog.QUESTION,
						new String[] { "Ok", "Cancel" }, 1)).open();
				if (result == 0) {
					TreeSet<IScheme> l = Application.getInstance().getConfiguration().getListOfISchemes();
					l.remove(ischeme);
					ischeme = null;
					selection = null;
					deleteAction.setEnabled(false);
					Application.getInstance().getConfiguration().currInstrumentationScheme = null;
					Application.getInstance().getConfiguration().setListOfISchemes(l);
					Application.getInstance().getASTManager().saveProject();
					tv.setInput(Application.getInstance().getASTManager());
				}
			}
		};

		deleteAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("delete"));
		deleteAction.setEnabled(false);
		IToolBarManager tmgr = getViewSite().getActionBars().getToolBarManager();
		tmgr.add(deleteAction);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.ProjectChangeListener#projectChanged(sota.ast.util.ASTManager)
	 */
	public void projectChanged(ASTManager ast) {
		tv.setInput(ast);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		tv.refresh();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		tv.getControl().setFocus();
	}

	/**
	 * Liefert die aktuelle Auswahl der ISchemes.
	 * 
	 * @return die Auswahl
	 */
	public ISelection getSelection() {
		return tv.getSelection();
	}
}