package sota.gui.views;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.TreeSet;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;
import sota.ast.util.TestCase;
import sota.ast.util.TestLog;
import sota.gui.ProjectChangeListener;

/**
 * Die Klasse stellt die (Debug-)Ansicht dar, die die detailierte Auflistung der Testdaten bernimmt.
 * 
 */
public class LogView extends ViewPart implements ProjectChangeListener, ISelectionListener {

	public static final String ID = "sota.logview";

	private final StringBuffer content = new StringBuffer("");
	private Text text;
	private Action refreshAction;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		createActions();

		text = new Text(parent, SWT.V_SCROLL | SWT.BORDER);
		text.setLayoutData(new GridData(GridData.FILL_BOTH));
		text.setEditable(false);

		Application.getInstance().getASTManager().addProjectChangeListener(this);
		getViewSite().getPage().addSelectionListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.ProjectChangeListener#projectChanged(sota.ast.util.ASTManager)
	 */
	public void projectChanged(ASTManager ast) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		text.setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
	 * org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {

		/*
		 * if (part instanceof ProjectView && selection instanceof IStructuredSelection) {
		 * 
		 * scopeNode = (IScopeNode) (((IStructuredSelection) selection).getFirstElement()); } if (part instanceof
		 * TestView && selection instanceof IStructuredSelection) { logList = new ArrayList<TestCase>(); for (Object s :
		 * ((IStructuredSelection) selection).toList()) { if (s instanceof TestLog) { for (TestCase tc : ((TestLog)
		 * s).getListOfTestCases()) logList.add(tc); } else if (s instanceof TestCase) { logList.add((TestCase) s); } }
		 * } if(logList!=null) System.out.println(logList.size());
		 */
	}

	/**
	 * Ermittelt die Loginformationen mit einem Progressmonitor.
	 * 
	 */
	class LogViewCreator implements IRunnableWithProgress {

		private IScopeNode scopeNode;

		private TestCase log;

		/**
		 * Der Konstruktor.
		 */
		public LogViewCreator() {// ArrayList<TestCase> logList, IScopeNode scopeNode) {
			Object scopeNodeSelection = Application.getInstance().getConfiguration().getScopeNodeSelection();
			TreeSet<TestCase> testCaseSet = Application.getInstance().getConfiguration().listOfSelectedTestCases;
			TreeSet<TestLog> testLogSet = Application.getInstance().getConfiguration().listOfSelectedTestLogs;
			log = null;

			if (scopeNodeSelection instanceof IScopeNode || scopeNodeSelection instanceof IStructuredSelection) {
				if (scopeNodeSelection instanceof IScopeNode)
					scopeNode = (IScopeNode) scopeNodeSelection;
				else if (((IStructuredSelection) scopeNodeSelection).size() > 0
						&& ((IStructuredSelection) scopeNodeSelection).getFirstElement() instanceof IScopeNode)
					scopeNode = (IScopeNode) ((IStructuredSelection) scopeNodeSelection).getFirstElement();

			}

			for (TestLog tl : testLogSet) {
				testCaseSet.addAll(tl.getListOfTestCases());
			}

			if (scopeNodeSelection == null || testCaseSet == null || testCaseSet.size() == 0)
				return;
			else {
				TestCase first = testCaseSet.first();
				log = new TestCase(first.getNumber(), first.getFileName(), first.getProjectName(), first.getName(),
						first.getDescription(), first.getISchemeName());
				for (TestCase tc : testCaseSet) {
					log.sumUp(tc);
				}
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
		 */
		public void run(IProgressMonitor monitor) {

			try {

				if (Application.getInstance().getConfiguration().projectStatus <= 3)
					return;

				if (scopeNode == null || log == null)
					return;
				monitor.beginTask("Reading TestLogs", log.getListOfPaths().size());
				content.setLength(0);
				int x = 0;
				String s = "Name: " + log.getName() + "\t\tProject: " + log.getProjectName() + "\t\tIScheme: "
						+ log.getISchemeName() + "\nDescription: " + log.getDescription() + "\n";
				for (ArrayList<Integer> l : log.getListOfPaths()) {
					monitor.worked(1);
					if (l.size() > 0) {
						if (l.get(1) > scopeNode.getStartNode().getID() && l.get(0) < scopeNode.getEndNode().getID()) {

							s += "\nPath: "
									+ x++
									+ "          "
									+ Application.getInstance().getASTManager().getNodeByID(l.get(1)).getFunctionNode()
											.getName() + ", Level: " + l.get(0) + "\n";

							for (int i = 1; i < l.size(); i++)
								s += l.get(i) + " ";
						}
					}
				}
				s += "\n\nHitMap:\n";
				for (Integer i : log.getHitMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getHitMap().get(i) + "), ";
				}

				s += "\n\n TruthVectorMap:\n";
				for (Integer i : log.getTvMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getTvMap().get(i) + "), ";
				}

				s += "\n\nModBIPaths:\n";
				for (Integer i : log.getMapOfCoveredModBIPaths().keySet()) {
					if ((scopeNode instanceof ASTFunctionNode && ((ASTFunctionNode) scopeNode).getID() == i)
							|| i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " ([" + TestCase.getPathString(log.getMapOfCoveredModBIPaths().get(i)) + "]), ";
				}

				s += "\n\nBIPaths:\n";
				// System.out.println("\n ->" + log.getMapOfCoveredBIPaths());
				for (Integer i : log.getMapOfCoveredBIPaths().keySet()) {
					if ((scopeNode instanceof ASTFunctionNode && ((ASTFunctionNode) scopeNode).getID() == i)
							|| i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " ([" + TestCase.getPathString(log.getMapOfCoveredBIPaths().get(i)) + "]), ";
				}

				s += "\n\nSelectionMap:\n";
				for (Integer i : log.getSelectionMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getSelection(i) + "), ";
				}

				s += "\n\nSkipMap:\n";
				for (Integer i : log.getIterationSkipsMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getIterationSkipsMap().get(i) + "), ";
				}

				s += "\n\nLoopMap:\n";
				for (Integer i : log.getIterationSingleLoopsMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getIterationSingleLoopsMap().get(i) + "), ";
				}

				s += "\n\nDoubleLoopMap:\n";
				for (Integer i : log.getIterationDoubleLoopsMap().keySet()) {
					if (i > scopeNode.getStartNode().getID() && i < scopeNode.getEndNode().getID())
						s += i + " (" + log.getIterationDoubleLoopsMap().get(i) + "), ";
				}

				content.append(s);
			} finally {
				monitor.done();
			}

		}

	}

	/**
	 * Erstellt die Aktionen fr diese Ansicht.
	 */
	private void createActions() {
		refreshAction = new Action("Refresh") {
			public void run() {
				if (Application.getInstance().getConfiguration().projectStatus <= 3)
					return;
				try {
					new ProgressMonitorDialog(getViewSite().getWorkbenchWindow().getShell()).run(true, true,
							new LogViewCreator());
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				text.setText(content.toString());
			}
		};

		refreshAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("restore"));

		IToolBarManager tmgr = getViewSite().getActionBars().getToolBarManager();

		tmgr.add(refreshAction);
	}
}