package sota.gui.views;

import java.util.Comparator;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.nodes.IScopeNode;
import sota.ast.util.ASTManager;
import sota.gui.GuiUpdateListener;
import sota.gui.ProjectChangeListener;
import sota.gui.provider.IProjectTreeView;
import sota.gui.provider.MetricsViewLabelProvider;
import sota.gui.provider.ProjectContentProvider;

/**
 * Die Klasse fr die Darstellung der statischen Metriken in Tabellenform in einer Ansicht.
 * 
 */
public class MetricsView extends ViewPart implements GuiUpdateListener, ProjectChangeListener, IProjectTreeView {

	public static final String ID = "sota.metricsview";

	private TreeViewer tv;
	private boolean showFlat = false;
	private boolean showPackages = false;
	private boolean showFiles = true;
	private boolean showClasses = true;
	private boolean showInnerClasses = true;
	private boolean reverseSort = false;
	private MetricsSortCriteria sortCriterium = MetricsSortCriteria.name;

	private int initColumnSize[] = { 200, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60 };

	Action showHierarchicalAction = null;
	Action showFlatAction = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		Application.getInstance().getASTManager().addProjectChangeListener(this);
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		Application.getInstance().getASTManager().addProjectTreeView(this);

		createActions();
		tv = new TreeViewer(parent);
		tv.setContentProvider(new ProjectContentProvider(this));
		tv.setLabelProvider(new MetricsViewLabelProvider(this));
		final TreeViewer ftv = tv;

		TreeColumn column;

		final String[] columnNames = { "Name", "Cycl.Compl.", "Ess.Compl.", "LOC", "#Statements", "#Branches",
				"#ModBIP", "#BIP", "#ConditionStmts.", "#Atoms", "#Conditions" };

		int columnIndex = 0;

		for (String s : columnNames) {
			column = new TreeColumn(tv.getTree(), SWT.NONE);
			column.setAlignment(SWT.RIGHT);
			column.setText(s);
			column.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					if (event.widget instanceof TreeColumn) {

						for (int i = 0; i < columnNames.length; i++) {
							if (((TreeColumn) event.widget).getText().equals(columnNames[i])) {

								switch (i) {
								case 0:
									if (sortCriterium == MetricsSortCriteria.name)
										reverseSort = !reverseSort;
									sortCriterium = MetricsSortCriteria.name;
									break;
								case 1:
									if (sortCriterium == MetricsSortCriteria.cc)
										reverseSort = !reverseSort;
									sortCriterium = MetricsSortCriteria.cc;
									break;
								case 2:
									if (sortCriterium == MetricsSortCriteria.ec)
										reverseSort = !reverseSort;
									sortCriterium = MetricsSortCriteria.ec;
									break;
								case 3:
									if (sortCriterium == MetricsSortCriteria.loc)
										reverseSort = !reverseSort;
									sortCriterium = MetricsSortCriteria.loc;
									break;
								case 4:
									if (sortCriterium == MetricsSortCriteria.statements)
										reverseSort = !reverseSort;
									sortCriterium = MetricsSortCriteria.statements;
									break;
								case 5:
									if (sortCriterium == MetricsSortCriteria.branches)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.branches;
									break;
								case 6:
									if (sortCriterium == MetricsSortCriteria.modbip)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.modbip;
									break;
								case 7:
									if (sortCriterium == MetricsSortCriteria.bip)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.bip;
									break;
								case 8:
									if (sortCriterium == MetricsSortCriteria.condstatments)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.condstatments;
									break;
								case 9:
									if (sortCriterium == MetricsSortCriteria.atoms)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.atoms;
									break;
								case 10:
									if (sortCriterium == MetricsSortCriteria.conditions)
										reverseSort = !reverseSort;

									sortCriterium = MetricsSortCriteria.conditions;
									break;

								}
							}
						}
						ftv.getTree().setRedraw(false);
						ftv.collapseAll();
						ftv.getTree().setRedraw(true);
						ftv.refresh();
					}
				}
			});

			column.setWidth(initColumnSize[columnIndex++]);
		}
		tv.getTree().setHeaderVisible(true);

		tv.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				Application.getInstance().getASTManager().select(
						((IStructuredSelection) event.getSelection()).getFirstElement());
			}
		});

		tv.setInput(Application.getInstance().getASTManager());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.ProjectChangeListener#projectChanged(sota.ast.util.ASTManager)
	 */
	public void projectChanged(ASTManager ast) {
		tv.setInput(ast);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		tv.refresh();
	}

	/**
	 * Erstellt die Aktionen fr diese Ansicht.
	 */
	private void createActions() {

		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();

		IMenuManager mm = getViewSite().getActionBars().getMenuManager();

		Action expandAllAction = new Action("Expand All") {
			public void run() {
				tv.getTree().setRedraw(false);
				tv.expandAll();
				tv.getTree().setRedraw(true);
			}
		};

		Action collapseAllAction = new Action("Collapse All") {
			public void run() {
				tv.getTree().setRedraw(false);
				tv.collapseAll();
				tv.getTree().setRedraw(true);
			}
		};

		Action showPackageAction = new Action("Show PackageNames", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showPackages = !showPackages;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showFilesAction = new Action("Show Files", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showFiles = !showFiles;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showClassesAction = new Action("Show Classes", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showClasses = !showClasses;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		Action showInnerClassesAction = new Action("Show Inner Classes", IAction.AS_CHECK_BOX) {
			public void run() {
				tv.getTree().setRedraw(false);
				showInnerClasses = !showInnerClasses;
				tv.refresh();
				tv.getTree().setRedraw(true);
			}
		};

		showFlatAction = new Action("Flat Presentation", IAction.AS_RADIO_BUTTON) {
			public void run() {
				showFlat(true);
			}
		};

		showHierarchicalAction = new Action("Hierarchical Presentation", IAction.AS_RADIO_BUTTON) {
			public void run() {
				showFlat(false);
			}
		};

		showFlatAction.setChecked(showFlat);
		showHierarchicalAction.setChecked(!showFlat);

		showPackageAction.setChecked(showPackages);
		showFilesAction.setChecked(showFiles);
		showClassesAction.setChecked(showClasses);
		showInnerClassesAction.setChecked(showInnerClasses);

		expandAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("expandall"));
		collapseAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("collapseall"));
		showPackageAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("package"));
		showFilesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("jfile"));
		showClassesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("class"));
		showInnerClassesAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor(
				"innerclass"));
		showFlatAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("flat"));
		showHierarchicalAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor(
				"hierarchical"));

		mm.add(showHierarchicalAction);
		mm.add(showFlatAction);
		mm.add(new Separator());
		mm.add(showPackageAction);
		mm.add(showFilesAction);
		mm.add(showClassesAction);
		mm.add(showInnerClassesAction);

		tbm.add(expandAllAction);
		tbm.add(collapseAllAction);
	}

	/**
	 * Legt fest, ob die Strukturen hierarchisch oder flach dargestellt werden sollen.
	 * 
	 * @param b
	 *            true, fr flache Darstellung, sonst false
	 */
	private void showFlat(boolean b) {
		tv.getTree().setRedraw(false);
		showFlat = b;
		showFlatAction.setChecked(b);
		showHierarchicalAction.setChecked(!b);
		tv.refresh();
		// tv.setInput(Application.getInstance().getASTManager());
		tv.getTree().setRedraw(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showProjectLine()
	 */
	public boolean showProjectLine() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showPackageNames()
	 */
	public boolean showPackageNames() {
		return showPackages;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFiles()
	 */
	public boolean showFiles() {
		return showFiles;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showClasses()
	 */
	public boolean showClasses() {
		return showClasses;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showInnerClasses()
	 */
	public boolean showInnerClasses() {
		return showInnerClasses;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#showFlat()
	 */
	public boolean showFlat() {
		return showFlat;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#getComparator()
	 */
	@SuppressWarnings("unchecked")
	public Comparator getComparator() {
		return new Comparator() {

			@Override
			public int compare(Object firstObject, Object secondObject) {
				if (!(firstObject instanceof IScopeNode && secondObject instanceof IScopeNode))
					return 0;

				IScopeNode first = (IScopeNode) firstObject, second = (IScopeNode) secondObject;
				int factor = 1;
				if (reverseSort) {
					factor = -1;
				}
				switch (sortCriterium) {
				case name:
					return factor
							* first.getFullyQualifiedName(showPackageNames(), showFiles()).compareTo(
									second.getFullyQualifiedName(showPackageNames(), showFiles()));
				case cc:
					return factor
							* (first.getMetrics().cyclomaticComplexity - second.getMetrics().cyclomaticComplexity);
				case ec:
					return factor * (first.getMetrics().essentialComplexity - second.getMetrics().essentialComplexity);
				case loc:
					return factor * (first.getMetrics().nrLines - second.getMetrics().nrLines);
				case statements:
					return factor * (first.getMetrics().nrStatements - second.getMetrics().nrStatements);
				case branches:
					return factor * (first.getMetrics().nrBranches - second.getMetrics().nrBranches);
				case modbip:
					return factor
							* (new Long(first.getMetrics().nrModBIPaths)).compareTo(new Long(
									second.getMetrics().nrModBIPaths));
				case bip:
					return factor
							* (new Long(first.getMetrics().nrBIPaths))
									.compareTo(new Long(second.getMetrics().nrBIPaths));
				case condstatments:
					return factor * (first.getMetrics().nrConditionNodes - second.getMetrics().nrConditionNodes);
				case atoms:
					return factor * (first.getMetrics().nrAtoms - second.getMetrics().nrAtoms);
				case conditions:
					return factor * (first.getMetrics().nrConditions - second.getMetrics().nrConditions);
				}
				return 0;
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.gui.provider.IProjectTreeView#select(java.lang.Object)
	 */
	@Override
	public void select(Object o) {
		tv.setSelection(new StructuredSelection(o), true);
	}

	/**
	 * Die Sortierungskriterien fr die Tabelle dieser Ansicht.
	 * 
	 */
	public enum MetricsSortCriteria {
		name, cc, ec, loc, statements, branches, modbip, bip, condstatments, atoms, conditions
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite, org.eclipse.ui.IMemento)
	 */
	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		super.init(site, memento);
		if (memento == null)
			return;
		for (int i = 0; i < initColumnSize.length; i++) {
			if (memento.getInteger("initMetricsViewColumnSize_" + i) != null)
				initColumnSize[i] = memento.getInteger("initMetricsViewColumnSize_" + i);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.ViewPart#saveState(org.eclipse.ui.IMemento)
	 */
	@Override
	public void saveState(IMemento memento) {
		int i = 0;
		for (TreeColumn tc : tv.getTree().getColumns()) {
			memento.putInteger("initMetricsViewColumnSize_" + i++, tc.getWidth());
		}
		super.saveState(memento);
	}

}
