package sota.gui.views;

import java.util.Vector;

import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.text.source.VerticalRuler;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.nodes.ASTClassNode;
import sota.ast.nodes.ASTDoNode;
import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.ASTIfNode;
import sota.ast.nodes.ASTIterationNode;
import sota.ast.nodes.ASTNode;
import sota.ast.nodes.ASTStatementNode;
import sota.ast.nodes.ASTSwitchNode;
import sota.ast.nodes.ASTTryNode;
import sota.ast.nodes.FileNode;
import sota.ast.nodes.IConditionNode;
import sota.ast.util.ColorManager;

/**
 * Klasse fr die (Debug-)Ansicht, die die Knotenliste fr eine Datei auflistet.
 * 
 */
public class NodeView extends ViewPart implements ISelectionListener {

	public static String ID = "sota.nodeview";

	private Document fileDocument = new Document();
	private SourceViewer sourceViewer;
	private Vector<Integer> classLines = new Vector<Integer>();
	private Vector<Integer> functionLines = new Vector<Integer>();
	private Vector<Integer> nodeLines = new Vector<Integer>();
	private Vector<Integer> branchLines = new Vector<Integer>();
	private Vector<Integer> statementLines = new Vector<Integer>();
	private Vector<Integer> condLines = new Vector<Integer>();
	private Display display;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		// Logger.logln("createpartcontrol");
		display = PlatformUI.getWorkbench().getDisplay();

		sourceViewer = new SourceViewer(parent, new VerticalRuler(10), SWT.V_SCROLL | SWT.H_SCROLL);
		sourceViewer.getTextWidget().setWordWrap(false);
		sourceViewer.getTextWidget().setFont(new Font(display, new FontData("Courier New", 9, 0)));

		sourceViewer.setDocument(fileDocument);
		sourceViewer.getTextWidget().setIndent(10);

		getViewSite().getPage().addSelectionListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// Logger.logln("setfocus");
		sourceViewer.getTextWidget().getShell().setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
	 * org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		// Logger.logln("selectionchanged nv");
		if (selection instanceof IStructuredSelection)
			if (part instanceof ProjectView) {
				Object e = ((IStructuredSelection) selection).getFirstElement();
				if (e instanceof FileNode) {
					classLines.clear();
					functionLines.clear();
					nodeLines.clear();
					condLines.clear();
					branchLines.clear();
					statementLines.clear();

					FileNode fileNode = (FileNode) e;
					ASTNode sn = fileNode.getStartNode();
					ASTNode en = fileNode.getEndNode();
					StringBuffer s = new StringBuffer();
					int length;

					ASTNode n = sn;
					while (n != en) {
						length = s.length();
						if (n instanceof ASTFunctionNode)
							functionLines.add(length);
						else if (n instanceof ASTClassNode)
							classLines.add(length);
						else if ((n instanceof ASTIfNode) || (n instanceof ASTIterationNode)
								|| (n instanceof ASTSwitchNode) || (n instanceof ASTDoNode)
								|| (n instanceof ASTTryNode))
							branchLines.add(length);
						else if (n instanceof ASTStatementNode)
							statementLines.add(length);
						else
							nodeLines.add(length);

						if (n instanceof IConditionNode)
							condLines.add(length);
						s.append(n.getNodeString() + n.getContent() + "\n");
						if (n instanceof IConditionNode)
							condLines.add(s.length());
						n = n.getNextNode();
					}
					s.append(n.getNodeString() + n.getContent() + "\n\n");

					fileDocument.set(s.toString());

					for (Integer x : classLines)
						sourceViewer.getTextWidget().setLineBackground(sourceViewer.getTextWidget().getLineAtOffset(x),
								1, Application.getInstance().getColorManager().getColor(ColorManager.BLUE));
					for (Integer x : functionLines)
						sourceViewer.getTextWidget().setLineBackground(sourceViewer.getTextWidget().getLineAtOffset(x),
								1, Application.getInstance().getColorManager().getColor(ColorManager.DRED));
					for (Integer x : nodeLines)
						sourceViewer.getTextWidget().setLineBackground(sourceViewer.getTextWidget().getLineAtOffset(x),
								1, Application.getInstance().getColorManager().getColor(ColorManager.LYELLOW));
					for (Integer x : branchLines)
						sourceViewer.getTextWidget().setLineBackground(sourceViewer.getTextWidget().getLineAtOffset(x),
								1, Application.getInstance().getColorManager().getColor(ColorManager.RED));
					for (Integer x : statementLines)
						sourceViewer.getTextWidget().setLineBackground(sourceViewer.getTextWidget().getLineAtOffset(x),
								1, Application.getInstance().getColorManager().getColor(ColorManager.ORANGE));
					for (int i = 0; i < condLines.size(); i = i + 2) {
						length = sourceViewer.getTextWidget().getLineAtOffset(condLines.get(i + 1))
								- sourceViewer.getTextWidget().getLineAtOffset(condLines.get(i));
						sourceViewer.getTextWidget().setLineBackground(
								sourceViewer.getTextWidget().getLineAtOffset(condLines.get(i)) + 1, length - 2,
								Application.getInstance().getColorManager().getColor(ColorManager.LBLUE));
					}
				}
			}
	}
}
