package sota.gui.views;

import java.io.File;
import java.util.ArrayList;
import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import sota.app.Application;
import sota.ast.util.ASTManager;
import sota.ast.util.ITestContainer;
import sota.ast.util.ProjectConfig;
import sota.ast.util.TestCase;
import sota.ast.util.TestLog;
import sota.gui.GuiUpdateListener;
import sota.gui.ProjectChangeListener;
import sota.gui.dialogs.TestLogDialog;

/**
 * Stellt die Testflle in eier Ansicht dar.
 * 
 */
public class TestView extends ViewPart implements GuiUpdateListener, ISelectionListener, ProjectChangeListener {

	public static final String ID = "sota.testview";
	private TreeViewer lv;
	private Action markAllAction, unmarkAllAction, lockAction, deleteAction;
	private boolean locked = true;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {

		createActions();

		lv = new TreeViewer(parent, /* SWT.CHECK | */SWT.MULTI | SWT.V_SCROLL | SWT.BORDER);
		// lv.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));
		getSite().setSelectionProvider(lv);
		getViewSite().getPage().addSelectionListener(this);
		Application.getInstance().getASTManager().addGuiUpdateListener(this);
		Application.getInstance().getASTManager().addProjectChangeListener(this);

		lv.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {

				IStructuredSelection ss = (IStructuredSelection) event.getSelection();
				if (ss.getFirstElement() instanceof ITestContainer)
					new TestLogDialog(getViewSite().getShell(), (ITestContainer) ss.getFirstElement()).open();
			}

		});

		lv.setLabelProvider(new ILabelProvider() {

			public Image getImage(Object arg0) {
				if (arg0 instanceof TestCase)
					return Application.getInstance().getImageRegistry().get("testlog");
				if (arg0 instanceof TestLog) {
					if (((TestLog) arg0).getListOfBadTestCases().size() == 0)
						return Application.getInstance().getImageRegistry().get("testlog");
					if (((TestLog) arg0).getListOfTestCases().size() > 0)
						return Application.getInstance().getImageRegistry().get("testlog2");
				}
				return Application.getInstance().getImageRegistry().get("badlog");
			}

			public String getText(Object o) {
				if (o instanceof TestCase) {
					String s = ((TestCase) o).getNumber() + "";
					s = s.substring(s.lastIndexOf('/') + 1);
					return s + "   (" + ((TestCase) o).getISchemeName() + ")";
				} else if (o instanceof TestLog) {
					String s = ((TestLog) o).getName();
					s = s.substring(s.lastIndexOf('/') + 1);
					return s;// + "   (" + ((TestLog) o).getISchemeName() + ")";
				} else if (o instanceof String) {
					String s = ((String) o);
					s = s.substring(s.lastIndexOf("\\") + 1);
					if (Application.getInstance().getConfiguration().getMapOfBadTestFileProjects().containsKey(
							(String) o))
						return s
								+ "   (Project: "
								+ Application.getInstance().getConfiguration().getMapOfBadTestFileProjects().get(
										(String) o) + ")";
					return s + "   (invalid log)";
				}
				return null;
			}

			public void addListener(ILabelProviderListener arg0) {
			}

			public void dispose() {
			}

			public boolean isLabelProperty(Object arg0, String arg1) {
				return false;
			}

			public void removeListener(ILabelProviderListener arg0) {
			}

		});
		lv.setContentProvider(new ITreeContentProvider() {

			public void dispose() {
			}

			public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
			}

			public Object[] getElements(Object o) {
				if (o instanceof ASTManager) {
					TreeSet<TestLog> l1 = ((ASTManager) o).getConfiguration().getListOfTestLogs();
					ArrayList<String> l2 = ((ASTManager) o).getConfiguration().getListOfBadTestFiles();
					ArrayList<Object> l = new ArrayList<Object>();
					l.addAll(l1);
					l.addAll(l2);
					return l.toArray();
				}
				return null;
			}

			@Override
			public Object[] getChildren(Object parentElement) {
				if (!locked && parentElement instanceof TestLog) {
					ArrayList<Object> al = new ArrayList<Object>();
					al.addAll(((TestLog) parentElement).getListOfTestCases());
					al.addAll(((TestLog) parentElement).getListOfBadTestCases());
					return al.toArray();
				}
				return null;
			}

			@Override
			public Object getParent(Object element) {
				return null;
			}

			@Override
			public boolean hasChildren(Object element) {
				if (!locked && element instanceof TestLog)
					return ((TestLog) element).getListOfTestCases().size() > 0;
				return false;
			}

		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
	 * org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (part == this) {
			Application.getInstance().getASTManager().resetHits();

			TreeSet<TestCase> tcl = new TreeSet<TestCase>();
			TreeSet<TestLog> tll = new TreeSet<TestLog>();
			ArrayList<String> lbad = new ArrayList<String>();

			for (Object o : ((IStructuredSelection) lv.getSelection()).toArray()) {
				if (o instanceof TestCase)
					tcl.add((TestCase) o);
				else if (o instanceof TestLog) {
					tll.add((TestLog) o);
					for (TestCase tc : ((TestLog) o).getListOfTestCases())
						tcl.add(tc);
				} else if (o instanceof String)
					lbad.add((String) o);
			}
			for (TestCase tc : tcl)
				Application.getInstance().getASTManager().applyTestCase(tc);

			Application.getInstance().getConfiguration().listOfSelectedTestLogs = tll;
			Application.getInstance().getConfiguration().listOfSelectedTestCases = tcl;
			Application.getInstance().getConfiguration().listOfSelectedBadTestFiles = lbad;
			Application.getInstance().getASTManager().computeCoverage();
			Application.getInstance().getASTManager().fireToGuiUpdateListeners();
		}

	}

	/**
	 * Erstellt die Aktionen fr diese Ansicht.
	 */
	private void createActions() {

		lockAction = new Action("Lock Testlogs", IAction.AS_CHECK_BOX) {
			public void run() {
				locked = isChecked();
				if (isChecked())
					lv.collapseAll();
				lv.refresh();
			}

		};
		markAllAction = new Action("Mark All") {
			public void run() {
				if (Application.getInstance().getASTManager().getSourceStatus() == ProjectConfig.DIRTY) {
					lv.setSelection(null, true);
					return;
				}
				Application.getInstance().getASTManager().resetHits();

				lv.getTree().selectAll();

				TreeSet<TestCase> tcl = new TreeSet<TestCase>();
				TreeSet<TestLog> tll = new TreeSet<TestLog>();
				ArrayList<String> lbad = new ArrayList<String>();

				for (Object o : ((IStructuredSelection) lv.getSelection()).toArray()) {
					if (o instanceof TestCase)
						tcl.add((TestCase) o);
					else if (o instanceof TestLog) {
						tll.add((TestLog) o);
						for (TestCase tc : ((TestLog) o).getListOfTestCases())
							tcl.add(tc);
					} else if (o instanceof String)
						lbad.add((String) o);
				}
				for (TestCase tc : tcl)
					Application.getInstance().getASTManager().applyTestCase(tc);

				Application.getInstance().getConfiguration().listOfSelectedTestLogs = tll;
				Application.getInstance().getConfiguration().listOfSelectedTestCases = tcl;
				Application.getInstance().getConfiguration().listOfSelectedBadTestFiles = lbad;
				Application.getInstance().getASTManager().computeCoverage();
				Application.getInstance().getASTManager().fireToGuiUpdateListeners();
			}
		};

		unmarkAllAction = new Action("Unmark All") {
			public void run() {
				lv.getTree().deselectAll();

				if (Application.getInstance().getASTManager().getSourceStatus() == ProjectConfig.DIRTY) {
					lv.setSelection(null, true);
					return;
				}
				Application.getInstance().getASTManager().resetHits();

				Application.getInstance().getASTManager().computeCoverage();
				Application.getInstance().getASTManager().fireToGuiUpdateListeners();
			}
		};

		deleteAction = new Action("Delete TestLogs") {
			public void run() {
				int result = (new MessageDialog(getViewSite().getShell(), "Confirm Delete", null,
						"Are you sure you want to delete the selected TestLogs?", MessageDialog.QUESTION, new String[] {
								"Ok", "Cancel" }, 1)).open();
				if (result == 0) {
					ProjectConfig config = Application.getInstance().getConfiguration();
					ArrayList<String> lbad = config.listOfSelectedBadTestFiles;
					TreeSet<TestLog> l = config.listOfSelectedTestLogs;
					for (TestLog t : l) {
						if ((new File(t.getFileName())).delete())
							config.removeTestLog(t);
					}
					for (String s : lbad) {
						if ((new File(s)).delete())
							config.removeBadTestFile(s);
					}
					deleteAction.setEnabled(false);
					config.listOfSelectedBadTestFiles = null;
					config.listOfSelectedTestLogs = null;
					Application.getInstance().getASTManager().fireToGuiUpdateListeners();
				}
			}
		};

		deleteAction.setEnabled(false);
		// lockAction.setEnabled(false);
		lockAction.setChecked(locked);

		lockAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("lock"));
		deleteAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("delete"));
		markAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("expandall"));
		unmarkAllAction.setImageDescriptor(Application.getInstance().getImageRegistry().getDescriptor("collapseall"));

		IToolBarManager tmgr = getViewSite().getActionBars().getToolBarManager();

		tmgr.add(deleteAction);
		tmgr.add(markAllAction);
		tmgr.add(unmarkAllAction);
		tmgr.add(lockAction);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.GuiUpdateListener#guiUpdate(sota.ast.util.ASTManager)
	 */
	public void guiUpdate(ASTManager ast) {
		// lockAction.setEnabled(ast.getConfiguration().getListOfTestLogs().size()>0);
		deleteAction
				.setEnabled((ast.getConfiguration().listOfSelectedBadTestFiles != null && ast.getConfiguration().listOfSelectedBadTestFiles
						.size() > 0)
						|| (ast.getConfiguration().listOfSelectedTestLogs != null && ast.getConfiguration().listOfSelectedTestLogs
								.size() > 0));
		lv.refresh();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		lv.getControl().setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.java.ast.util.ProjectChangeListener#projectChanged(sota.ast.util.ASTManager)
	 */
	@Override
	public void projectChanged(ASTManager ast) {
		lv.setInput(Application.getInstance().getASTManager());
		// lockAction.setEnabled(ast.getConfiguration().getListOfTestLogs().size()>0);
	}

}