package sota.lang;

import java.util.List;

import org.eclipse.jface.text.rules.IPartitionTokenScanner;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

import sota.ast.util.MetricManager;
import sota.ast.util.ProjectConfig;
import sota.ast.util.TestLogManager;
import sota.gui.graph.CFGBuilder;

/**
 * Liefert die Implementation der in der Sprachspezifikation gelisteten Klassen zurck.
 * 
 */
public class LangClassProvider {

	// obligatorisch

	/**
	 * Liefert die ASTBuilder-Klasse.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der ASTBuilder
	 */
	public static Class<IASTBuilder> getASTBuilder(ProjectConfig c) {

		Class<IASTBuilder> astBuilder;

		try {
			astBuilder = (Class<IASTBuilder>) Class.forName(c.getLangSpecification(c.language).astBuilder);
			return astBuilder;
		} catch (Exception e) {
		}
		return null;
	}

	/**
	 * Liefert die Dateierweiterung fr eine Sprache.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return eine Liste der Erweiterungen
	 */
	public static List<String> getExtensions(ProjectConfig c) {
		return c.getLangSpecification(c.language).extensions;
	}

	// optional

	/**
	 * Liefert einen LogReader-Ableitung.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der LogReader
	 */
	public static String getLogReader(ProjectConfig c) {
		return c.getLangSpecification(c.language).logReader;
	}

	/**
	 * Lieferte eine TestLogManager-Ableitung.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der TestLogManager
	 */
	public static Class<TestLogManager> getTestLogManager(ProjectConfig c) {

		Class<TestLogManager> hitListManagerClass;
		try {
			hitListManagerClass = (Class<TestLogManager>) Class
					.forName(c.getLangSpecification(c.language).testLogManager);
			return hitListManagerClass;
		} catch (Exception e) {
			return TestLogManager.class;
		}
	}

	/**
	 * Liefert die CFGBuilder-Implementation.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der CFG-Builder
	 */
	public static Class<CFGBuilder> getCFGBuilder(ProjectConfig c) {

		Class<CFGBuilder> cfgBuilderClass;
		try {
			cfgBuilderClass = (Class<CFGBuilder>) Class.forName(c.getLangSpecification(c.language).cfgBuilder);
			return cfgBuilderClass;
		} catch (Exception e) {
			return CFGBuilder.class;
		}
	}

	/**
	 * Liefert eine MetricManager-Ableitung.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der MetricManager
	 */
	public static Class<MetricManager> getMetricManager(ProjectConfig c) {

		Class<MetricManager> metricManagerClass;
		try {
			metricManagerClass = (Class<MetricManager>) Class.forName(c.getLangSpecification(c.language).metricManager);
			return metricManagerClass;
		} catch (Exception e) {
			return MetricManager.class;
		}
	}

	/**
	 * Liefert eine SourceViewerConfiguration.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return die SourceViewerConfiguration
	 */
	public static Class<SourceViewerConfiguration> getSourceViewerConfiguration(ProjectConfig c) {

		try {
			Class<SourceViewerConfiguration> sourceViewerClass = (Class<SourceViewerConfiguration>) Class.forName(c
					.getLangSpecification(c.language).sourceViewerConfiguration);
			return sourceViewerClass;
		} catch (Exception e) {

		}
		return null;
	}

	/**
	 * Liefert einen PartitionScanner
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return der PartitionScanner
	 */
	public static Class<IPartitionTokenScanner> getPartitionScanner(ProjectConfig c) {
		try {
			Class<IPartitionTokenScanner> partitionScannerClass = (Class<IPartitionTokenScanner>) Class.forName(c
					.getLangSpecification(c.language).partitionScanner);
			return partitionScannerClass;
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Liefert die Typen des PartitionScanners.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return die Typen
	 */
	public static String[] getPartitionScannerTypes(ProjectConfig c) {
		try {
			Class a = Class.forName(c.getLangSpecification(c.language).partitionScanner);
			return ((IPartitionScanner) a.newInstance()).getTypes();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Liefert den Partitioningstring.
	 * 
	 * @param c
	 *            die Projektkonfiguration
	 * @return das Partitioning
	 */
	public static String getPartitioning(ProjectConfig c) {
		try {
			Class a = Class.forName(c.getLangSpecification(c.language).sourceViewerConfiguration);
			return ((ISourceViewerConfiguration) a.newInstance()).getConfiguredDocumentPartitioning();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

}
