package sota.lang;

import java.util.ArrayList;
import java.util.Arrays;

import sota.Logger;
import sota.ast.util.LogReader;
import sota.ast.util.MetricManager;
import sota.ast.util.TestLogManager;

/**
 * Container fr eine Sprachspezifikation.
 * 
 */
public class LanguageSpecification {

	// obligatorisch
	public String name = "";
	public ArrayList<String> extensions = new ArrayList<String>();
	public String astBuilder = "";

	// optional
	public String cfgBuilder = "sota.gui.graph.CFGBuilder";
	public String logReader = "sota.ast.util.LogReader";
	public String sourceViewerConfiguration = "";
	public String partitionScanner = "";
	public String metricManager = "sota.ast.util.MetricManager";
	public String testLogManager = "sota.ast.util.TestLogManager";

	/**
	 * Testet die Spezifikation auf Vollstndigkeit.
	 * 
	 * @return true, wenn vollstndig, sonst false
	 * 
	 */
	public boolean checkSpecification() {
		boolean match = false;
		try {
			if (name.length() < 1) {
				Logger.log("\n\t\tNo Name found! ");
				return false;
			}

			// IASTBuilder
			try {
				if (!Arrays.asList(Class.forName(astBuilder).getInterfaces()).contains(IASTBuilder.class)) {
					Logger.log("\n\t\tNo IASTBuilder Implementation found for " + name + "! ");
					return false;
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo IASTBuilder Implementation found for " + name + "! ");
				return false;
			}

			// LogReader
			try {
				if (!Class.forName(logReader).isInstance(new LogReader())) {
					Logger.log("\n\t\tNo LogReader Implementation found for " + name + "! ");
					return false;
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo LogReader Implementation found for " + name + " !");
				return false;
			}

			// MetricManager
			try {
				if (!Class.forName(metricManager).isInstance(new MetricManager())) {
					Logger.log("\n\t\tNo MetricManager Implementation found for " + name + "! ");
					return false;
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo MetricManager Implementation found for " + name + "! ");
				return false;
			}

			// TestLogManager
			try {
				if (!Class.forName(testLogManager).isInstance(new TestLogManager())) {
					Logger.log("\n\t\tNo TestLogManager Implementation found for " + name + "! ");
					return false;
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo TestLogManager Implementation found for " + name + "! ");
				return false;
			}

			// ISourceViewerConfiguration
			match = false;
			try {
				Class<?>[] interfaces = Class.forName(sourceViewerConfiguration).getInterfaces();
				for (Class<?> x : interfaces)
					if (x.equals(ISourceViewerConfiguration.class)) {
						match = true;
						break;
					}

				if (!match) {
					Logger.log("\n\t\tNo ISourceViewerConfiguration found for " + name + "! ");
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo ISourceViewerConfiguration Implementation found for " + name + "! ");
			}

			// IPartitionScanner
			match = false;
			try {
				Class<?>[] interfaces = Class.forName(partitionScanner).getInterfaces();
				for (Class<?> x : interfaces)
					if (x.equals(IPartitionScanner.class)) {
						match = true;
						break;
					}
				if (!match) {
					Logger.log("\n\t\tNo IPartitionScanner Implementation found for " + name + "! ");
				}
			} catch (ClassNotFoundException e) {
				Logger.log("\n\t\tNo IPartitionScanner Implementation found for " + name + "! ");
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
}
