package sota.lang.java.ast.nodes;

import sota.ast.nodes.ASTFunctionNode;
import sota.ast.nodes.ASTNode;
import sota.ast.nodes.IScopeNode;
import sota.lang.java.ast.util.JavaASCFormatter;

/**
 * AST-Knoten fr eine Funktion
 * */
public class JavaASTFunctionNode extends ASTFunctionNode {

	/**
	 * Der Konstruktor
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param content
	 *            der Inhalt
	 * @param scope
	 *            der Scope
	 * @param modifiers
	 *            die Modifier der Funktion
	 * @param name
	 *            der Name der Funktion
	 * @param arguments
	 *            die Argumente
	 * @param parent
	 *            der bergeordnete ScopeNode
	 */
	public JavaASTFunctionNode(int ln, String content, String scope, String modifiers, String name, String arguments,
			IScopeNode parent) {
		super(ln, content, scope, modifiers, name, arguments, parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTFunctionNode#createASCContent(int)
	 */
	public void createASCContent(int level) {
		appendToASCContent(content);
		switch (level) {
		case 0:
			break;
		case 1:
			entryNode.setASCContent(JavaASCFormatter.insertPIDInit()
					+ JavaASCFormatter.logId(level, "Instrumentation Level")
					+ JavaASCFormatter.logId(entryNode.getID(), "FunctionStart"));
			for (ASTNode n : setOfExitNodes)
				n.appendToASCContent(JavaASCFormatter.logId(n.getID(), "Function-Exit"));
			break;

		case 2:
			entryNode.setASCContent(JavaASCFormatter.insertPIDInit()
					+ JavaASCFormatter.logId(level, "Instrumentation Level")
					+ JavaASCFormatter.logId(entryNode.getID(), "FunctionStart"));
			if (nrAtoms.size() > 0)
				entryNode.appendToASCContent(JavaASCFormatter.declarationOfBooleans(nrPrimaryConditions, nrAtoms));
			for (ASTNode n : setOfExitNodes)
				n.appendToASCContent(JavaASCFormatter.logId(n.getID(), "Function-Exit"));
			break;

		case 3:
			entryNode.setASCContent(JavaASCFormatter.insertPIDInit()
					+ JavaASCFormatter.logId(level, "Instrumentation Level")
					+ JavaASCFormatter.logId(entryNode.getID(), "FunctionStart"));
			if (nrAtoms.size() > 0)
				entryNode.appendToASCContent(JavaASCFormatter.declarationOfBooleans(nrPrimaryConditions, nrAtoms));
			for (ASTNode n : setOfExitNodes)
				if (n.isReachable()) {
					if (n == endNode)
						n.setASCContent(JavaASCFormatter.logId(n.getID(), "Function-Exit"));
					else
						n.appendToASCContent(JavaASCFormatter.insertComment("Function-Exit"));
				}
			break;
		}
	}

}