package sota.lang.java.ast.nodes;

import sota.ast.nodes.ASTIfNode;
import sota.ast.nodes.ASTNode;
import sota.ast.nodes.Condition;
import sota.lang.java.ast.util.JavaASCFormatter;

/**
 * AST-Knoten fr If-Anweisung
 * */
public class JavaASTIfNode extends ASTIfNode {
	/**
	 * Der Konstruktor fr einen If-Knoten
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 * @param cond
	 *            die Bedingung
	 */
	public JavaASTIfNode(int ln, String s, Condition cond) {
		super(ln, s, cond);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#createASCContent(int)
	 */
	public void createASCContent(int level) {
		switch (level) {
		case 0:
			appendToASCContent(content);
			break;
		case 1:
			appendToASCContent(JavaASCFormatter.insertComment("If-Entry(" + id + ")") + content);
			trueNode.setASCContent(JavaASCFormatter.startCompound()
					+ JavaASCFormatter.logId(trueNode.getID(), "If-True(" + trueNode.getID() + ", " + id + ")"));
			if (elseNode.isVirtual())
				elseNode.setASCContent(JavaASCFormatter.endCompound() + JavaASCFormatter.insert("else "));
			else
				elseNode.setASCContent(JavaASCFormatter.endCompound() + elseNode.getASCContent());

			falseNode.setASCContent(JavaASCFormatter.startCompound()
					+ JavaASCFormatter.logId(falseNode.getID(), "If-False(" + falseNode.getID() + ", " + id + ")"));
			endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());

			for (ASTNode n : setOfExitNodes) {
				n.setASCContent(JavaASCFormatter.insertComment("If-Exit(" + n.getID() + ", " + id + ")")
						+ n.getASCContent());
			}
			break;
		case 2:
		case 3:
			entryNode.setASCContent(JavaASCFormatter.insertComment("If-Entry(" + id + ")"));
			this.setASCContent(JavaASCFormatter.logCondition(id, "If-Node(" + id + ")", cond));
			this.appendToASCContent(JavaASCFormatter.insertIfStatement("if(", cond, ")", "If-Condition(" + id + ")"));

			trueNode.setASCContent(JavaASCFormatter.startCompound()
					+ JavaASCFormatter.logId(trueNode.getID(), "If-True(" + trueNode.getID() + ", " + id + ")"));

			if (elseNode.isVirtual())
				elseNode.setASCContent(JavaASCFormatter.endCompound() + JavaASCFormatter.insert("else "));
			else
				elseNode.setASCContent(JavaASCFormatter.endCompound() + elseNode.getASCContent());

			falseNode.setASCContent(JavaASCFormatter.startCompound()
					+ JavaASCFormatter.logId(falseNode.getID(), "If-False(" + falseNode.getID() + ", " + id + ")"));
			endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());

			for (ASTNode n : setOfExitNodes) {
				if (n == endNode) {
					if (level == 3)
						n.appendToASCContent(JavaASCFormatter.logId(endNode.getID(), "If-End(" + n.getID() + ", " + id
								+ ")"));
					else
						n.appendToASCContent(JavaASCFormatter.insertComment("Iteration-End(" + n.getID() + ", " + id
								+ ")"));
				} else
					n.setASCContent(JavaASCFormatter.insertComment("If-Exit(" + n.getID() + ", " + id + ")")
							+ n.getASCContent());
			}
			break;
		}
		return;
	}

}