package sota.lang.java.ast.nodes;

import sota.Logger;
import sota.ast.nodes.ASTIterationNode;
import sota.ast.nodes.ASTNode;
import sota.ast.nodes.Condition;
import sota.lang.java.ast.util.JavaASCFormatter;

/**
 * AST-Knoten fr iterierende Anweisungen (Schleifen).
 * */
public class JavaASTIterationNode extends ASTIterationNode {

	/**
	 * Konstruktof fr While-Schleifen.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 * @param cond
	 *            die Bedingung
	 */
	public JavaASTIterationNode(int ln, String s, Condition cond) {
		super(ln, s, cond);
	}

	/**
	 * Konstruktor fr For-Schleifen.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 * @param _forInit
	 *            die Initialisierung
	 * @param _cond
	 *            die Bedingung
	 * @param _forUpdate
	 *            der Update-Anweisung
	 */
	public JavaASTIterationNode(int ln, String s, String _forInit, Condition _cond, String _forUpdate) {
		super(ln, s, _forInit, _cond, _forUpdate);
	}

	/**
	 * Konstruktof fr Foreach-Schleifen.
	 * 
	 * @param ln
	 *            die Zeilennummer
	 * @param s
	 *            der Inhalt
	 */
	public JavaASTIterationNode(int ln, String s) {
		super(ln, s);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTNode#createASCContent(int)
	 */
	public void createASCContent(int level) {
		if (isForNode)
			createForASCContent(level);
		else if (hasDoNode)
			createDoWhileASCContent(level);
		else
			createWhileASCContent(level);
		return;
	}

	/**
	 * Erstellt die Instrumentierung fr For-Schleifen.
	 * 
	 * @param level
	 *            das Instrumentationslevel
	 */
	public void createForASCContent(int level) {
		switch (level) {
		case 0:
			appendToASCContent(content);
			break;
		case 1:

			entryNode.setASCContent(JavaASCFormatter.insertComment("Iteration-Entry(" + id + ")")
					+ entryNode.getASCContent());

			this.setASCContent(content + JavaASCFormatter.startCompound());

			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());

			for (ASTNode n : setOfExitNodes) {
				n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
						+ n.getASCContent());
			}
			break;
		case 2:
		case 3:

			// foreach - wird nicht geloggt
			if (cond == null) {
				entryNode.setASCContent((level == 2 ? JavaASCFormatter.insertComment("Iteration-Entry(" + id + ")")
						: JavaASCFormatter.logId(id, "Iteration-Node(" + id + ")"))
						+ entryNode.getASCContent());
				this.setASCContent(content + JavaASCFormatter.startCompound());
				endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());
				// empty Bedingung : for( Init;;Update)
			} else if (cond.isEmpty()) {
				entryNode.setASCContent(JavaASCFormatter.startCompound()
						+ (level == 2 ? JavaASCFormatter.insertComment("Iteration-Entry(" + id + ")")
								: JavaASCFormatter.logId(id, "Iteration-Node(" + id + ")"))
						+ JavaASCFormatter.insert("boolean ast_for_entry_" + id + " = true;\n")
						+ JavaASCFormatter.insertForInit(forInit) + entryNode.getASCContent());

				this.setASCContent(JavaASCFormatter.insert("while(true)") + JavaASCFormatter.startCompound());

				if (forUpdate != null && !(forUpdate.length() < 1)) {
					this.appendToASCContent(JavaASCFormatter.insert("if(ast_for_entry_" + id + ") ")
							+ JavaASCFormatter.insert("ast_for_entry_" + id + " = false;")
							+ JavaASCFormatter.insert("else") + JavaASCFormatter.insertComment("for-update")
							+ JavaASCFormatter.insert(forUpdate + ";"));
				}

				endNode.setASCContent(JavaASCFormatter.endCompound() + JavaASCFormatter.endCompound()
						+ endNode.getASCContent());

				// echte Bedingung for(Init; Cond; Update)
			} else {
				entryNode.setASCContent(JavaASCFormatter.startCompound()
						+ JavaASCFormatter.insert("boolean ast_for_entry_" + id + " = true;\n")
						+ JavaASCFormatter.insertForInit(forInit)
						+ JavaASCFormatter.insertComment("Iteration-Entry(" + id + ")")

						+ entryNode.getASCContent());

				this.setASCContent(JavaASCFormatter.insert("while(true)") + JavaASCFormatter.startCompound());

				if (forUpdate != null && !(forUpdate.length() < 1)) {
					this.appendToASCContent(JavaASCFormatter.insert("if(ast_for_entry_" + id + ") ")
							+ JavaASCFormatter.insert("ast_for_entry_" + id + " = false;")
							+ JavaASCFormatter.insert("else") + JavaASCFormatter.insertComment("for-update")
							+ JavaASCFormatter.insert(forUpdate + ";"));
				}

				this.appendToASCContent(JavaASCFormatter.logCondition(id, "Iteration-Node(" + id + ")", cond)
						+ JavaASCFormatter.insert("if(!" + JavaASCFormatter.getBooleanName(cond, 0) + ")")
						+ JavaASCFormatter.insert("\tbreak;"));

				endNode.setASCContent(JavaASCFormatter.endCompound() + JavaASCFormatter.endCompound()
						+ endNode.getASCContent());

			}

			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			for (ASTNode n : setOfExitNodes) {

				if (n == endNode) {
					if (level == 3)
						n.appendToASCContent(JavaASCFormatter.logId(endNode.getID(), "Iteration-End(" + n.getID()
								+ ", " + id + ")"));
					else
						n.appendToASCContent(JavaASCFormatter.insertComment("Iteration-End(" + n.getID() + ", " + id
								+ ")"));
				} else
					n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
							+ n.getASCContent());
			}
			break;
		}
		return;
	}

	/**
	 * Erstellt die Instrumentierung fr Do-While-Schleifen.
	 * 
	 * @param level
	 *            das Instrumentationslevel
	 */
	public void createDoWhileASCContent(int level) {
		switch (level) {
		case 0:
			appendToASCContent(content);
			break;
		case 1:

			entryNode.setASCContent(JavaASCFormatter.insertComment("Iteration-Entry(" + id + ")")
					+ entryNode.getASCContent());

			doNode.appendToASCContent(JavaASCFormatter.startCompound());

			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			for (ASTNode n : setOfExitNodes) {
				n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
						+ n.getASCContent());
			}

			this.setASCContent(JavaASCFormatter.endCompound() + content);

			break;

		case 2:
		case 3:

			entryNode.setASCContent(JavaASCFormatter.insertComment("IterationEntry ( " + id + ")")
					+ entryNode.getASCContent());

			doNode.appendToASCContent(JavaASCFormatter.startCompound());

			// logge Beginn eines Schleifendurchlaufes
			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			for (ASTNode n : setOfExitNodes) {
				if (n == endNode) {
					if (level == 3)
						n.appendToASCContent(JavaASCFormatter.logId(endNode.getID(), "Iteration-End(" + n.getID()
								+ ", " + id + ")"));
					else
						n.appendToASCContent(JavaASCFormatter.insertComment("Iteration-End(" + n.getID() + ", " + id
								+ ")"));
				} else
					n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
							+ n.getASCContent());
			}

			if (cond.isTrueAtom())

				this.setASCContent(JavaASCFormatter.insertComment("Iterating(" + id + ")")
						+ JavaASCFormatter.endCompound() + JavaASCFormatter.insert("while(true);", ""));
			else
				this.setASCContent(JavaASCFormatter.insertComment("Iterating(" + id + ")")
						+ JavaASCFormatter.logCondition(id, "Iteration-Node(" + id + ")", cond)
						+ JavaASCFormatter.insert("if(!" + JavaASCFormatter.getBooleanName(cond, 0) + ")")
						+ JavaASCFormatter.insert("break;") + JavaASCFormatter.endCompound()
						+ JavaASCFormatter.insert("while(true);", ""));

		}
		return;
	}

	/**
	 * Erstellt die Instrumentierung fr While-Schleifen.
	 * 
	 * @param level
	 *            das Instrumentationslevel
	 */
	public void createWhileASCContent(int level) {
		switch (level) {
		case 0:
			appendToASCContent(content);
			break;
		case 1:
			entryNode.setASCContent(JavaASCFormatter.insertComment("IterationEntry ( " + id + ")")
					+ entryNode.getASCContent());

			this.setASCContent(content + JavaASCFormatter.startCompound());

			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());

			for (ASTNode n : setOfExitNodes) {
				n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
						+ n.getASCContent());
			}
			break;
		case 2:
		case 3:

			if (cond.isTrueAtom())
				this.setASCContent(JavaASCFormatter.insert("while(true)") + JavaASCFormatter.startCompound()
						+ JavaASCFormatter.logCondition(id, "Iteration-Node(" + id + ")", cond));
			else
				this.setASCContent(JavaASCFormatter.insert("while(true)") + JavaASCFormatter.startCompound()
						+ JavaASCFormatter.logCondition(id, "Iteration-Node(" + id + ")", cond)
						+ JavaASCFormatter.insert("if(!" + JavaASCFormatter.getBooleanName(cond, 0) + ")")
						+ JavaASCFormatter.insert("\tbreak;"));

			// logge Beginn eines Schleifendurchlaufes
			loopNode.setASCContent(JavaASCFormatter.logId(loopNode.getID(), "Iteration-Loop(" + loopNode.getID() + ", "
					+ id + ")"));

			endNode.setASCContent(JavaASCFormatter.endCompound() + endNode.getASCContent());

			for (ASTNode n : setOfExitNodes) {
				if (n == endNode) {
					if (level == 3)
						n.appendToASCContent(JavaASCFormatter.logId(endNode.getID(), "Iteration-End(" + n.getID()
								+ ", " + id + ")"));
					else
						n.appendToASCContent(JavaASCFormatter.insertComment("Iteration-End(" + n.getID() + ", " + id
								+ ")"));
				} else
					n.setASCContent(JavaASCFormatter.insertComment("Iteration-Exit(" + n.getID() + ", " + id + ")")
							+ n.getASCContent());
			}

		}
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.ASTIterationNode#printNode()
	 */
	public void printNode() {

		Logger.log("---");
		printReach();
		if (hasDoNode) {
			Logger.log("<" + id + "><Iterate " + entryNode.getID() + " , Do: " + doNode.getID() + " -> "
					+ endNode.getID() + ">");
		} else {
			Logger.logln("<" + id + "><Iterate " + entryNode.getID() + " -> " + endNode.getID() + ">");
		}
		printExits();
		Logger.log("<" + nrHits + "><hits: " + loopNode.getNrHits() + ", double: " + nrMultipleLoops + ", skips: "
				+ nrSkips + ">\n");

		if (cond != null) {
			cond.print();
		}
		if (setOfTruthVectors.size() > 0) {
			Logger.log(" <TruthVectors: ");
			for (String s : setOfTruthVectors) {
				Logger.log(s + " ");
			}
			Logger.log(">");
		}
		Logger.logln("----------\n" + content);
	}

}