package sota.lang.java.ast.nodes;

import java.util.ArrayList;

import sota.Logger;
import sota.ast.nodes.Condition;

/**
 * AST-Knoten fr Konditionknoten
 */
public class JavaCondition extends Condition {
	/**
	 * Konstruktor fr zusammengesetzte Bedingung (binrer Operator).
	 * 
	 * @param lC
	 *            linke Bedingung
	 * @param op
	 *            der Operator
	 * @param rC
	 *            rechte Bedingung
	 */
	public JavaCondition(Condition lC, String op, Condition rC) {
		super(lC, op, rC);
	}

	/**
	 * Konstruktor fr zusammengesezte Bedingung (Unr -> !)
	 * 
	 * @param op
	 *            der Operator
	 * @param rC
	 *            rechte Bedingung
	 */
	public JavaCondition(String op, Condition rC) {
		super(op, rC);
	}

	/**
	 * Konstruktor fr atomare Bedingung.
	 * 
	 * @param atom
	 *            das Atom
	 */
	public JavaCondition(String atom) {
		super(atom);
	}

	/**
	 * Konstruktor fr eine leere, d.h. wahre Bedingung.
	 */
	public JavaCondition() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#toASCString(java.lang.String)
	 */
	public String toASCString(String s) {
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#evaluateTruthVector(java.lang.String)
	 */
	@Override
	public boolean evaluateTruthVector(String s) {
		// Logger.logln(atomId + " " + s);
		if (isEmpty)
			return true;
		if (isTrueAtom)
			return true;
		if (isFalseAtom)
			return false;
		if (isAtom) {
			return s.substring(atomId).startsWith("1");
		}
		if (leftCondition == null)
			if (rightCondition != null)
				return !rightCondition.evaluateTruthVector(s);

		if (content.equals("||"))
			return leftCondition.evaluateTruthVector(s) || rightCondition.evaluateTruthVector(s);
		if (content.equals("|"))
			return leftCondition.evaluateTruthVector(s) | rightCondition.evaluateTruthVector(s);
		if (content.equals("&&"))
			return leftCondition.evaluateTruthVector(s) && rightCondition.evaluateTruthVector(s);
		if (content.equals("&"))
			return leftCondition.evaluateTruthVector(s) & rightCondition.evaluateTruthVector(s);
		if (content.equals("^"))
			return leftCondition.evaluateTruthVector(s) ^ rightCondition.evaluateTruthVector(s);
		Logger.logln("Condition not evaluated! Unknown Type.");
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#applyTruthVector(java.lang.String)
	 */
	@Override
	public boolean applyTruthVector(String s) {
		boolean b;
		if (isEmpty)
			return true;
		if (isTrueAtom)
			return true;
		if (isFalseAtom)
			return false;
		if (isAtom) {
			b = s.substring(atomId).startsWith("1");
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		if (leftCondition == null)
			if (rightCondition != null) {
				b = !rightCondition.applyTruthVector(s);
				if (b)
					nrTrueEvaluations++;
				else
					nrFalseEvaluations++;
				return b;
			}

		if (content.equals("||")) {
			b = leftCondition.applyTruthVector(s) || rightCondition.applyTruthVector(s);
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		if (content.equals("|")) {
			b = leftCondition.applyTruthVector(s) | rightCondition.applyTruthVector(s);
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		if (content.equals("&&")) {
			b = leftCondition.applyTruthVector(s) && rightCondition.applyTruthVector(s);
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		if (content.equals("&")) {
			b = leftCondition.applyTruthVector(s) & rightCondition.applyTruthVector(s);
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		if (content.equals("^")) {
			b = leftCondition.applyTruthVector(s) ^ rightCondition.applyTruthVector(s);
			if (b)
				nrTrueEvaluations++;
			else
				nrFalseEvaluations++;
			return b;
		}
		Logger.logln("Condition not evaluated! Unknown Type.");
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#getAtomEvaluation(java.util.ArrayList)
	 */
	public ArrayList<Integer[]> getAtomEvaluation(ArrayList<Integer[]> l) {
		if (isAtom) {
			if (l == null)
				l = new ArrayList<Integer[]>();
			l.add(new Integer[] { nrTrueEvaluations, nrFalseEvaluations });
			return l;
		}
		if (leftCondition != null)
			l = leftCondition.getAtomEvaluation(l);
		if (rightCondition != null)
			l = rightCondition.getAtomEvaluation(l);
		return l;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#getConditionEvaluation(java.util.ArrayList)
	 */
	public ArrayList<Integer[]> getConditionEvaluation(ArrayList<Integer[]> l) {

		if (l == null)
			l = new ArrayList<Integer[]>();

		l.add(new Integer[] { nrTrueEvaluations, nrFalseEvaluations });

		if (leftCondition != null)
			l = leftCondition.getConditionEvaluation(l);
		if (rightCondition != null)
			l = rightCondition.getConditionEvaluation(l);

		return l;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.basic.nodes.Condition#getEvaluatedTVString(java.lang.String)
	 */
	public String getEvaluatedTVString(String s) {
		String returnString = "";

		if (isEmpty)
			return returnString;
		if (isTrueAtom)
			return returnString + "1";
		if (isFalseAtom)
			return returnString + "0";
		if (isAtom) {
			return returnString + s.charAt(atomId) + "";
		}

		if (leftCondition == null) {
			if (rightCondition != null)
				return returnString + rightCondition.getEvaluatedTVString(s);
		}

		if (content.equals("||")) {
			returnString += leftCondition.getEvaluatedTVString(s);
			if (leftCondition.evaluateTruthVector(s) == true) {
				for (int i = 0; i < rightCondition.countNrAtoms(); i++)
					returnString += "-";
				return returnString;
			}
			return returnString + rightCondition.getEvaluatedTVString(s);
		}

		if (content.equals("&&")) {
			returnString = returnString + leftCondition.getEvaluatedTVString(s);
			if (leftCondition.evaluateTruthVector(s) == false) {
				for (int i = 0; i < rightCondition.countNrAtoms(); i++)
					returnString += "-";
				return returnString;
			}
			return returnString + rightCondition.getEvaluatedTVString(s);
		}

		if (content.equals("&") || content.equals("|") || content.equals("^"))
			return returnString + leftCondition.getEvaluatedTVString(s) + rightCondition.getEvaluatedTVString(s);

		Logger.logln("Condition not evaluated! Unknown Type.");
		return "X";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#countNrAtoms()
	 */
	public int countNrAtoms() {
		if (isAtom)
			return 1;
		if (isOperator) {
			if (leftCondition != null)
				return leftCondition.countNrAtoms() + rightCondition.countNrAtoms();
			return rightCondition.countNrAtoms();
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#getAtomCondition(int)
	 */
	@Override
	public Condition getAtomCondition(int i) {
		Condition c;
		if (isAtom && atomId == i)
			return this;

		if (leftCondition != null)
			if ((c = leftCondition.getAtomCondition(i)) != null)
				return c;

		if (rightCondition != null)
			if ((c = rightCondition.getAtomCondition(i)) != null)
				return c;

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.ast.nodes.Condition#computeNrAtomCombinations()
	 */
	@Override
	public int[] computeNrAtomCombinations() {
		int[] i = new int[2];
		int[] lc, rc;

		if (isEmpty)
			return i;
		if (isTrueAtom) {
			i[0] = 1;
		} else if (isFalseAtom) {
			i[1] = 1;
		} else if (isAtom) {
			i[0] = 1;
			i[1] = 1;
		} else if (leftCondition == null) {
			rc = rightCondition.computeNrAtomCombinations();
			i[0] = rc[1];
			i[1] = rc[0];

		} else {
			lc = leftCondition.computeNrAtomCombinations();
			rc = rightCondition.computeNrAtomCombinations();
			if (content.equals("&")) {
				i[0] = lc[0] * rc[0];
				i[1] = lc[0] * rc[1] + lc[1] * rc[0] + lc[1] * rc[1];
			} else if (content.equals("&&")) {
				i[0] = lc[0] * rc[0];
				i[1] = lc[0] * rc[1] + lc[1];
			} else if (content.equals("|")) {
				i[0] = lc[0] * rc[0] + lc[0] * rc[1] + lc[1] * rc[0];
				i[1] = lc[1] * rc[1];
			} else if (content.equals("||")) {
				i[0] = lc[0] + lc[1] * rc[0];
				i[1] = lc[1] * rc[1];
			} else if (content.equals("^")) {
				i[0] = lc[0] * rc[1] + lc[1] * rc[0];
				i[1] = lc[1] * rc[1] + lc[0] * rc[0];
			}
		}
		return i;
	}
}