package sota.lang.java.ast.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Stack;

import sota.Logger;

/**
 * Verwaltet die Symboltabelle (momentan ungenutzt) und den aktuellen Scope.
 * */
class JavaSymbolTable {

	private static boolean VERBOSE = false;

	private static boolean VERBOSE2 = false;

	private static boolean PRINT_ERROR = false;

	/* Globale Variablen */
	private ArrayList<String> listOfClassPaths; // CLASSPATH der Umgebung

	private HashMap<String, String> idTable; // globale Symboltabelle:

	// fullyQualifiedId -> fullyQualifiedType
	private HashMap<String, String> extensionTable; // fullyQualifiedType ->

	// fullyQualifiedSuperType

	/* Dateilokale Variablen */
	private HashSet<String> setOfImportedPackages; // Packagename von: import

	// packagename.*;

	private HashSet<String> setOfImportedClasses; // vollqu., imp. Klassen

	// von: import
	// package.class;

	private int unnamedScopeId; // Counter fr namenlose Scopes
	// sowie verifizierte Klassen aus importPackages!

	protected HashSet<String> setOfTypeParsedClasses; // alle Klassen, die
	// schon getypeparsed wurden

	protected String currPackage;

	protected String currScope; // classname::function(args)

	protected Stack<Object> dataStack; // zum Auslagern dateilokaler Variablen

	public JavaSymbolTable() {
		dataStack = new Stack<Object>();
		idTable = new HashMap<String, String>();
		setOfTypeParsedClasses = new HashSet<String>();
		extensionTable = new HashMap<String, String>();
		listOfClassPaths = new ArrayList<String>();
		listOfClassPaths.add("/home/Ronny/development/dipl/");
		setOfImportedPackages = new HashSet<String>();
		setOfImportedClasses = new HashSet<String>();
		currPackage = "";
		currScope = "";
		unnamedScopeId = 0;
		// init();
	}

	/** dateilokaler Reset; vor dem Parsen einer neuen Datei notwendig */
	public void reset() {
		unnamedScopeId = 0;
		currPackage = "";
		currScope = "";
	}

	/** ffnet ein Packet */
	public void openPackage(String _name) {
		currPackage = _name;
		currScope = "";
		if (VERBOSE2) {
			Logger.logln("\n\topening package: " + currPackage + "\n");
		}
	}

	/** fgt Importierte Pakete und Klassen hinzu */
	public void addImports(HashSet<String> _setOfImportedPackages, HashSet<String> _setOfImportedClasses) {
		setOfImportedPackages = _setOfImportedPackages;
		setOfImportedClasses = _setOfImportedClasses;
	}

	/** fgt Importierte Pakete hinzu */
	public void addImportedPackage(String importedPackage) {
		setOfImportedPackages.add(importedPackage);
	}

	/** fgt Importierte Pakete hinzu */
	public void addImportedClass(String importedClass) {
		setOfImportedClasses.add(importedClass);
	}

	public String getCurrPackage() {
		return currPackage;
	}

	/** ffnet benannten Scope */
	public void openScope(String _name) {
		// bisher nur Package im Scope
		if (currScope.equals("")) {
			currScope = _name;
		} else {
			currScope += "::" + _name;
		}
		if (VERBOSE2) {
			Logger.logln("\n\topening scope (" + currPackage + ".) " + currScope);
		}
	}

	/** ffnet unbenannten Scope */
	public void openScope() {
		// bisher nur Package im Scope
		if (currScope.equals("")) {
			currScope = "" + unnamedScopeId++;
		} else {
			currScope += "::" + unnamedScopeId++;
		}
		if (VERBOSE2) {
			Logger.logln("\n\topening scope (" + currPackage + ".) " + currScope);
		}
	}

	/** gibt aktuellen Scope zurck */
	public String getCurrScope() {
		return currScope;
	}

	/** schliet aktuellen Scope und kehr in den darberliegenden zurck */
	public boolean closeScope() {
		int i;
		if (currScope.equals("")) {
			return false;
		} else if ((i = currScope.lastIndexOf("::")) == -1) {
			currScope = "";
		} else {
			currScope = currScope.substring(0, i);
		}
		if (VERBOSE2) {
			Logger.logln("\tback in scope (" + currPackage + ".) " + currScope + "\n");
		}
		return true;
	}

	// fgt ID mit Types in aktuellen Scope ein
	public void putId(String _name, String _type) {
		String str;

		if (currScope.equals("")) {
			str = currPackage + "." + _name;
		} else {
			str = currPackage + "." + currScope + "::" + _name;

		}
		idTable.put(str, _type);
		if (VERBOSE) {
			Logger.logln("Putting Id " + str + " (" + _type + ")");
		}
		return;
	}

	/** fgt Type in aktuellen Scope ein */
	public void putType(String _name) {
		String str;
		if (currScope.equals("")) {
			str = currPackage + "." + _name;
		} else {
			str = currPackage + "." + currScope + "::" + _name;
		}
		if (VERBOSE) {
			Logger.logln("Putting Type " + str);
		}
		idTable.put(str, "0");
		return;
	}

	/** fgt Type eines Packages ein */
	public void putType(String _package, String _name) {
		String str = _package + "." + _name;

		if (VERBOSE) {
			Logger.logln("Putting Type " + str);
		}
		idTable.put(str, "0");
		return;
	}

	/** fgt vollqualifizierten Typ ein */
	public void putFullyQualifiedType(String _type) {
		if (VERBOSE) {
			Logger.logln("Putting Type " + _type);
		}
		idTable.put(_type, "0");
	}

	/** hinzufgen einer Superklassenbeziehung */
	public void putExtension(String _class, String _superClass) {
		String c = checkType(_class);
		String sc = checkType(_superClass);
		extensionTable.put(c, sc);
		if (VERBOSE) {
			Logger.logln("Putting Extension " + sc + " -> " + c);
		}
	}

	/** liefert den Typen der Id */
	public String lookUp(String _name) {
		String type = "", str, absScope;
		int i = 0;

		if (VERBOSE) {
			Logger.logln("Looking up Id " + _name + " ...");

			// ist Id in der Tabelle?
		}
		if (idTable.containsKey(_name)) {
			type = idTable.get(_name);
			if (VERBOSE) {
				Logger.logln("\t ... type: " + type);
			}
			return type;
		}

		// ist Id ein Array? (_name[]) gib Type des Arrays zurck
		if (_name.endsWith("[]")) {
			if ((type = lookUp(_name.substring(0, _name.length() - 2))) == null) {
				return null;
			}
			if (type.endsWith("[]")) {
				type = type.substring(0, type.length() - 2);
			}
			if (VERBOSE) {
				Logger.logln("\t ... " + _name + " is Array of " + type);
			}
			return type;
		}

		// ist Id ist eine new-Allocation? (new _name(...))
		if (_name.startsWith("new")) {
			type = _name.substring(3, _name.length()).trim();
			if ((i = type.indexOf('(')) != -1) {
				type = type.substring(0, i);
			}
			if ((type = checkType(type)) == null) {
				return null;
			}
			if (VERBOSE) {
				Logger.logln("\t" + _name + " is new Allocation of " + type);
			}
			return type;
		}

		if (currScope.equals("")) {
			absScope = currPackage;
			str = absScope + "." + _name;
		} else {
			absScope = currPackage + "." + currScope;
			str = absScope + "::" + _name;
		}

		// schlage Id aufsteigend vom aktuellen Scope nach
		do {
			if (VERBOSE2) {
				Logger.log("\t... looking up " + str);

			}
			if (idTable.containsKey(str)) {
				type = idTable.get(str);
				if (VERBOSE) {
					Logger.logln(" ... " + str + " has type: " + type);
				}
				return type;
			}
			if (VERBOSE2) {
				Logger.logln(" ... failed");

			}
			if ((i = absScope.lastIndexOf("::")) != -1) {
				absScope = absScope.substring(0, i);

			}
			if (absScope.equals(currPackage)) {
				str = absScope + "." + _name;
			} else {
				str = absScope + "::" + _name;
			}
		} while (i != -1);

		// schau importierte Klassen nach
		for (String s : setOfImportedClasses) {
			if ((i = s.lastIndexOf('.')) == -1) {
				continue;
			}
			s = s.substring(i + 1, s.length());
			if (VERBOSE2) {
				Logger.log("\t... looking up " + s + " (" + _name + ")");
			}
			if (_name.equals(s)) {
				type = idTable.get(s);
				if (VERBOSE) {
					Logger.logln(" ... " + s + " has type" + type);
				}
				return type;
			}
			if (VERBOSE2) {
				Logger.logln(" ... failed");
			}
		}

		// ist Id ein Member?
		if ((i = _name.lastIndexOf('.')) != -1) {
			if ((type = lookUp(_name.substring(0, i))) == null) {
				return null;
			}
			if (!type.equals("0")) {
				str = _name.substring(i + 1, _name.length());
				if (VERBOSE) {
					Logger.logln("\t ... memberrelation " + str + " is member of " + type);
				}
				if (type.endsWith("[]") && str.equals("length")) {
					return "int";
				}
				if ((type = lookUpFullyQualifiedId(type + "::" + str)) == null) {
					return null;
				}
				return type;
			}
		}

		if (PRINT_ERROR) {
			System.out.println("Error@SymbolTable.lookUp(): Couldn't find Type for " + _name + "!");
		}
		return null;
	}

	/** liefert den Type einer vollqualifizierten Id */
	public String lookUpFullyQualifiedId(String _name) {
		String type = null;

		if (VERBOSE) {
			Logger.logln("Looking up fully qualified ID: " + _name);

			// _name in idTable?
		}
		if (idTable.containsKey(_name)) {
			type = idTable.get(_name);
			if (VERBOSE) {
				Logger.logln("\t ... type: " + type);
			}
			return type;
		}

		if (PRINT_ERROR) {
			Logger.logln("Error@SymbolTable.lookUpFullyQualifiedID():" + " Couldn't find Type for " + _name + "!");
		}
		return null;
	}

	/** testet, ob t Type ist und gibt vollqualifizierten Type von t zurck */
	public String checkType(String _type) {
		String str, absScope, type;
		int i = 0;

		if (VERBOSE) {
			Logger.logln("Checking Type of " + _type);

			// ist es ein Builtin Type ?
		}
		if (VERBOSE2) {
			Logger.log("\t... checking builtin");
		}
		if (idTable.containsKey(_type)) {
			str = idTable.get(_type);
			if (str.equals("0") || str.equals("1")) {
				if (VERBOSE) {
					Logger.logln(" ... is type: " + _type);
				}
				return _type;
			}
			if (VERBOSE2) {
				Logger.logln("\nError@SymbolTable.checkType(): " + _type + " is not a Type!");
			}
			return null;

		} else if (VERBOSE2) {
			Logger.logln(" - failed");

		}
		if (currScope.equals("")) {
			absScope = currPackage;
			str = absScope + "." + _type;
		} else {
			absScope = currPackage + "." + currScope;
			str = absScope + "::" + _type;
		}

		// schau im Scope aufwrts nach
		do {
			if (VERBOSE2) {
				Logger.log("\t... checking " + str);
			}
			if (idTable.containsKey(str)) {
				type = idTable.get(str);
				if (type.equals("0") || type.equals("1")) {
					if (VERBOSE) {
						Logger.logln(" ... is type: " + str);
					}
					return str;
				}
				if (VERBOSE2 || PRINT_ERROR) {
					Logger.logln("\nError@SymbolTable.checkType(): " + _type + " is not a Type!");
				}
				return null;
			}
			if (VERBOSE2) {
				Logger.logln(" - failed");
			}
			if ((i = absScope.lastIndexOf("::")) != -1) {
				absScope = absScope.substring(0, i);

			}
			if (absScope.equals(currPackage)) {
				str = absScope + "." + _type;
			} else {
				str = absScope + "::" + _type;
			}
		} while (i != -1);

		// teste importierte Klassen
		if (VERBOSE2) {
			Logger.log("\t... checking imported classes");
		}
		for (String s : setOfImportedClasses) {
			if ((i = s.lastIndexOf('.')) != -1) {
				if (_type.equals(s.substring(i + 1, s.length()))) {
					if (VERBOSE) {
						Logger.logln(" ... full qualified type: " + s);
					}
					return s;
				}
			}
		}
		if (VERBOSE2) {
			Logger.logln(" failed");
		}

		if (PRINT_ERROR) {
			Logger.logln("Error@SymbolTable.checkType(): Type " + _type + " not found");
		}
		return null;
	}

	/** initialisiert Symboltabelle mit Standardtypen */
	public void init() {
		idTable.put("void", "0");
		idTable.put("boolean", "0");
		idTable.put("byte", "0");
		idTable.put("char", "0");
		idTable.put("short", "0");
		idTable.put("int", "0");
		idTable.put("long", "0");
		idTable.put("float", "0");
		idTable.put("double", "0");
		idTable.put("String", "0");

		idTable.put("true", "boolean");
		idTable.put("false", "boolean");
		idTable.put("null", "void");
	}

	/** gibt Symboltabelle aus */
	public void print() {
		for (String s : idTable.keySet())
			Logger.logln(s + " (" + idTable.get(s) + ")");
	}

	/*
	 * public void putTypes(HashSet<String> h) { }
	 * 
	 * public HashSet<String> getSetOfTypes() { return null; }
	 */
}