package sota.lang.java.gui.util;

import java.util.ArrayList;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWhitespaceDetector;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;

import sota.app.Application;
import sota.ast.util.ColorManager;
import sota.gui.preferences.PreferenceConstants;

/**
 * Scannt den Quellcode nach Strings und Schlsselworten.
 * 
 */
public class JavaCodeScanner extends RuleBasedScanner {
	public JavaCodeScanner() {
		IToken keyword = new Token(new TextAttribute(Application
				.getColorFromPreferenceStore(PreferenceConstants.SOURCE_KEYWORD_COLOR), null, SWT.BOLD));
		IToken string = new Token(Application.getColorFromPreferenceStore(PreferenceConstants.SOURCE_STRING_COLOR));
		IToken other = new Token(Application.getColor(ColorManager.DEFAULT));
		setDefaultReturnToken(other);

		ArrayList<IRule> rules = new ArrayList<IRule>();

		rules.add(new SingleLineRule("\"", "\"", string, '\\'));
		rules.add(new SingleLineRule("'", "'", string, '\\'));
		rules.add(new WhitespaceRule(new IWhitespaceDetector() {
			public boolean isWhitespace(char c) {
				return Character.isWhitespace(c);
			}
		}));

		WordRule wordRule = new WordRule(new JavaWordDetector(), other);
		for (String keywordString : JavaSyntax.KEYWORDS)
			wordRule.addWord(keywordString, keyword);
		rules.add(wordRule);
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}