package sota.lang.java.gui.util;

import java.util.ArrayList;

import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;

import sota.lang.IPartitionScanner;

/**
 * Beschreibt die Regeln, nach denen der Quellcode in partitioniert werden soll, die verschiedenfarbig behandelt bzw.
 * dargestellt werden sollen.
 * 
 */
public class JavaPartitionScanner extends RuleBasedPartitionScanner implements IPartitionScanner {

	public static final String JAVA_DOC = "javadoc";
	public static final String MULTILINE_COMMENT = "Multi-line comment";
	public static final String SINGLELINE_COMMENT = "Single-line comment";
	public static final String STRING = "String";
	public static final String ASCLINE = "ASC-Line";
	public static final String[] PARTITION_TYPES = { ASCLINE, JAVA_DOC, MULTILINE_COMMENT, SINGLELINE_COMMENT, STRING };

	/**
	 * Legt die Regeln des PartitionScanners fest.
	 */
	public JavaPartitionScanner() {
		super();
		IToken javadoc = new Token(JAVA_DOC);
		IToken multilineComment = new Token(MULTILINE_COMMENT);
		IToken singlelineComment = new Token(SINGLELINE_COMMENT);
		IToken string = new Token(STRING);
		IToken ascLine = new Token(ASCLINE);

		ArrayList<IPredicateRule> list = new ArrayList<IPredicateRule>();

		list.add(new MultiLineRule("/* ASC", "*/", ascLine, (char) 0, true));
		list.add(new MultiLineRule("/**", "*/", javadoc, (char) 0, true));
		list.add(new MultiLineRule("/*", "*/", multilineComment, (char) 0, true));
		list.add(new SingleLineRule("\"", "\"", string, '\\'));
		list.add(new EndOfLineRule("//", singlelineComment));

		IPredicateRule[] rules = new IPredicateRule[list.size()];
		list.toArray(rules);

		setPredicateRules(rules);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.interfaces.IPartitionScanner#getTypes()
	 */
	public String[] getTypes() {
		return PARTITION_TYPES;
	}
}
