package sota.lang.java.gui.util;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

import sota.lang.ISourceViewerConfiguration;

/**
 * Definiert die Partitionierung eines Document und ihre Scanner.
 * 
 */
public class JavaSourceViewerConfiguration extends SourceViewerConfiguration implements ISourceViewerConfiguration {

	static private String PARTITIONING = "JavaPartitioning";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(org.eclipse.jface.text.source
	 * .ISourceViewer)
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();
		reconciler.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(new JavaCommentScanner());
		reconciler.setDamager(dr, JavaPartitionScanner.MULTILINE_COMMENT);
		reconciler.setRepairer(dr, JavaPartitionScanner.MULTILINE_COMMENT);

		dr = new DefaultDamagerRepairer(new JavaASCScanner());
		reconciler.setDamager(dr, JavaPartitionScanner.ASCLINE);
		reconciler.setRepairer(dr, JavaPartitionScanner.ASCLINE);

		dr = new DefaultDamagerRepairer(new JavaStringScanner());
		reconciler.setDamager(dr, JavaPartitionScanner.STRING);
		reconciler.setRepairer(dr, JavaPartitionScanner.STRING);

		dr = new DefaultDamagerRepairer(new JavaCodeScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(new JavaCommentScanner());
		reconciler.setDamager(dr, JavaPartitionScanner.JAVA_DOC);
		reconciler.setRepairer(dr, JavaPartitionScanner.JAVA_DOC);

		dr = new DefaultDamagerRepairer(new JavaCommentScanner());
		reconciler.setDamager(dr, JavaPartitionScanner.SINGLELINE_COMMENT);
		reconciler.setRepairer(dr, JavaPartitionScanner.SINGLELINE_COMMENT);
		return reconciler;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.text.source.SourceViewerConfiguration#getTabWidth(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return 4;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sota.lang.interfaces.ISourceViewerConfiguration#getConfiguredDocumentPartitioning()
	 */
	public String getConfiguredDocumentPartitioning() {
		return PARTITIONING;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(org.eclipse.jface.text.source
	 * .ISourceViewer)
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, JavaPartitionScanner.STRING,
				JavaPartitionScanner.JAVA_DOC, JavaPartitionScanner.MULTILINE_COMMENT,
				JavaPartitionScanner.SINGLELINE_COMMENT };
	}
}
