/* Generated By:JavaCC: Do not edit this line. JavaParser.java */
package sota.lang.java.ast.parser;

import sota.ast.nodes.*;
import sota.lang.java.ast.nodes.*;

import sota.lang.java.ast.util.JavaASTBuilder;

import sota.lang.java.ast.parser.JavaParser;
import sota.lang.java.ast.parser.JavaParserConstants;
import sota.lang.java.ast.parser.JavaParserTokenManager;
import sota.lang.java.ast.parser.ParseException;
import sota.lang.java.ast.parser.SimpleCharStream;
import sota.lang.java.ast.parser.Token;

import sota.Logger;

import java.util.ArrayList;
import java.util.Stack;
import java.io.FileInputStream;

/**
 * Grammar to parse Java version 1.5
 * 
 * @author Sreenivasa Viswanadha - Simplified and enhanced for 1.5
 * @author Jlio Vilmar Gesser - bug fixes and added ast generation
 * @author Ronny Treysse - removed ast generation and added ast generation via JavaASTBuilder
 */

public final class JavaParser implements JavaParserConstants {

	public static boolean VERBOSE = false;
	public static boolean verbose_parsing = false; // Ausgabe des Parsevorganges?
	public static boolean verbose_expression_parsing = false; // Ausgabe incl. Expressions?
	public static boolean print_stream = false; // Ausgabe des Token-Streams?
	public static boolean print_ast_nodes = false; // Ausgabe des ASTs?
	public static boolean print_cfg = false; // Ausgabe des CFG?
	public static boolean create_asc_file = false; // erstellt *.asc.cpp
	public static boolean log_reading_modus = false;

	public static int nrParses = 0;

	public static JavaParser parser;

	public static JavaASTBuilder jab;

	public static FileInputStream stream;

	public static Token pToken;

	public static int nrLines = 0;

	/*
	 * Parse-Funktion
	 */

	public static void parse(FileInputStream stream, JavaASTBuilder jab) throws ParseException {
		if (nrParses++ == 0)
			parser = new JavaParser(stream);
		else
			parser.ReInit(stream);

		JavaParser.jab = jab;

		parser.CompilationUnit();
	}

	/*
	 * Funktionen zum Loggen whrend des Parsevorganges
	 */

	private static void log(String s) {
		if (verbose_parsing)
			Logger.log("> " + s + " ");
	}

	private static void expr_log(String s) {
		if (verbose_expression_parsing)
			log(s);
	}

	/*
	 * Verknpft zwei Bedingungen mit Operator
	 */

	private static JavaCondition joinConditions(Token first, // erste Token der ersten Bedingung
			JavaCondition cond1, // erste Bedingung, kann null sein
			Token op, // Operator-Token
			JavaCondition cond2, // zweite Bedingung, kann null sein
			Token last) // letzte Token der zweiten Bedingung
	{
		if (cond1 == null)
			cond1 = new JavaCondition(op.sumUpX(first));
		if (cond2 == null)
			cond2 = new JavaCondition(last.sumUp(op.next));
		return new JavaCondition(cond1, op.image, cond2);
	}

	/*****************************************
	 * THE JAVA LANGUAGE GRAMMAR STARTS HERE *
	 *****************************************/

	/*
	 * Program structuring syntax follows.
	 */
	final public void CompilationUnit() throws ParseException {
		log("CompilationUnit");

		pToken = getToken(1);
		if (pToken.allToString().startsWith(" /* ASC"))
			jab.isInstrumented();
		boolean hasPackageDeclaration = false;
		switch (jj_nt.kind) {
		case PACKAGE:
			PackageDeclaration();
			hasPackageDeclaration = true;
			break;
		default:
			jj_la1[0] = jj_gen;
			;
		}
		/*
		 * Hinter dem ersten Knoten wird der Logger-Import eingefgt. Ist erster Knoten keine Packagedeclaration, dann
		 * wird ein virtueller Knoten dafr eingefgt.
		 */
		if (!hasPackageDeclaration)
			jab.appendNode(getToken(0).beginLine);
		label_1: while (true) {
			switch (jj_nt.kind) {
			case IMPORT:
				;
				break;
			default:
				jj_la1[1] = jj_gen;
				break label_1;
			}
			ImportDeclaration();
		}
		label_2: while (true) {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case CLASS:
			case ENUM:
			case FINAL:
			case INTERFACE:
			case NATIVE:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case STATIC:
			case STRICTFP:
			case SYNCHRONIZED:
			case TRANSIENT:
			case VOLATILE:
			case SEMICOLON:
			case AT:
				;
				break;
			default:
				jj_la1[2] = jj_gen;
				break label_2;
			}
			TypeDeclaration();
		}
		jab.appendNode(getToken(0).beginLine, getToken(1).sumUp(pToken)); // EOF

		jj_consume_token(0);
	}

	final public void PackageDeclaration() throws ParseException {
		log("PackageDeclaration");
		String s;

		Token t = getToken(1);
		jj_consume_token(PACKAGE);
		s = Name();
		jj_consume_token(SEMICOLON);
		if (verbose_parsing)
			Logger.logln("In Package: " + s);

		jab.openPackage(s);
		jab.appendNode(getToken(0).beginLine, getToken(0).sumUp(t));
		pToken = getToken(1);
	}

	final public void ImportDeclaration() throws ParseException {
		log("ImportDeclaration");
		String s;

		boolean isClass = true;
		jj_consume_token(IMPORT);
		switch (jj_nt.kind) {
		case STATIC:
			jj_consume_token(STATIC);
			break;
		default:
			jj_la1[3] = jj_gen;
			;
		}
		s = Name();
		switch (jj_nt.kind) {
		case DOT:
			jj_consume_token(DOT);
			jj_consume_token(STAR);
			jab.addImportedPackage(s);
			isClass = false;
			break;
		default:
			jj_la1[4] = jj_gen;
			;
		}
		jj_consume_token(SEMICOLON);
		if (isClass) {
			jab.addTypeName(s);
			jab.addImportedClass(s);
		}
	}

	/*
	 * Modifiers. We match all modifiers in a single rule to reduce the chances of syntax errors for simple modifier
	 * mistakes. It will also enable us to give better error messages.
	 */
	final public String Modifiers() throws ParseException {
		String s = " ";
		label_3: while (true) {
			if (jj_2_1(2)) {
				;
			} else {
				break label_3;
			}
			switch (jj_nt.kind) {
			case PUBLIC:
				jj_consume_token(PUBLIC);
				break;
			case STATIC:
				jj_consume_token(STATIC);
				break;
			case PROTECTED:
				jj_consume_token(PROTECTED);
				break;
			case PRIVATE:
				jj_consume_token(PRIVATE);
				break;
			case FINAL:
				jj_consume_token(FINAL);
				break;
			case ABSTRACT:
				jj_consume_token(ABSTRACT);
				break;
			case SYNCHRONIZED:
				jj_consume_token(SYNCHRONIZED);
				break;
			case NATIVE:
				jj_consume_token(NATIVE);
				break;
			case TRANSIENT:
				jj_consume_token(TRANSIENT);
				break;
			case VOLATILE:
				jj_consume_token(VOLATILE);
				break;
			case STRICTFP:
				jj_consume_token(STRICTFP);
				break;
			case AT:
				Annotation();
				break;
			default:
				jj_la1[5] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			s += getToken(0).toString() + " ";
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * Declaration syntax follows.
	 */
	final public void TypeDeclaration() throws ParseException {
		log("TypeDeclaration");
		switch (jj_nt.kind) {
		case SEMICOLON:
			jj_consume_token(SEMICOLON);
			break;
		case ABSTRACT:
		case CLASS:
		case ENUM:
		case FINAL:
		case INTERFACE:
		case NATIVE:
		case PRIVATE:
		case PROTECTED:
		case PUBLIC:
		case STATIC:
		case STRICTFP:
		case SYNCHRONIZED:
		case TRANSIENT:
		case VOLATILE:
		case AT:
			Modifiers();
			switch (jj_nt.kind) {
			case CLASS:
			case INTERFACE:
				ClassOrInterfaceDeclaration();
				break;
			case ENUM:
				EnumDeclaration();
				break;
			case AT:
				AnnotationTypeDeclaration();
				break;
			default:
				jj_la1[6] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			break;
		default:
			jj_la1[7] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
	}

	final public void ClassOrInterfaceDeclaration() throws ParseException {
		log("ClassOrInterfaceDeclaration");
		Token t, t1;
		switch (jj_nt.kind) {
		case CLASS:
			t = jj_consume_token(CLASS);
			break;
		case INTERFACE:
			t = jj_consume_token(INTERFACE);
			break;
		default:
			jj_la1[8] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		t1 = jj_consume_token(IDENTIFIER);
		switch (jj_nt.kind) {
		case LT:
			TypeParameters();
			break;
		default:
			jj_la1[9] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case EXTENDS:
			ExtendsList();
			break;
		default:
			jj_la1[10] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case IMPLEMENTS:
			ImplementsList();
			break;
		default:
			jj_la1[11] = jj_gen;
			;
		}
		if (pToken != t)
			jab.appendNode(pToken.beginLine, t.sumUpX(pToken));

		pToken = getToken(1);
		jab.beginClass(t.beginLine,
		// t.specialTokenToString(), // prefix
				getToken(0).sumUp(t), // content
				t1.image, // name
				""); // extension

		ClassOrInterfaceBody();
		jab.endClass();
	}

	final public void ExtendsList() throws ParseException {
		jj_consume_token(EXTENDS);
		ClassOrInterfaceType();
		label_4: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[12] = jj_gen;
				break label_4;
			}
			jj_consume_token(COMMA);
			ClassOrInterfaceType();
		}
	}

	final public void ImplementsList() throws ParseException {
		jj_consume_token(IMPLEMENTS);
		ClassOrInterfaceType();
		label_5: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[13] = jj_gen;
				break label_5;
			}
			jj_consume_token(COMMA);
			ClassOrInterfaceType();
		}
	}

	final public void EnumDeclaration() throws ParseException {
		log("EnumDeclaration");
		Token t, t1;
		JavaASTCompoundNode cn, ecn;
		t = jj_consume_token(ENUM);
		t1 = jj_consume_token(IDENTIFIER);
		switch (jj_nt.kind) {
		case IMPLEMENTS:
			ImplementsList();
			break;
		default:
			jj_la1[14] = jj_gen;
			;
		}
		if (pToken != t)
			jab.appendNode(pToken.beginLine, t.sumUpX(pToken));

		pToken = getToken(1);
		jab.beginClass(t.beginLine,
		// t.specialTokenToString(), // prefix
				getToken(0).sumUp(t), // content
				t1.image, // name
				""); // extension

		t = jj_consume_token(LBRACE);
		cn = jab.beginCompound(t.beginLine, t.allToString());
		pToken = getToken(1);
		switch (jj_nt.kind) {
		case IDENTIFIER:
			EnumConstant();
			label_6: while (true) {
				if (jj_2_2(2)) {
					;
				} else {
					break label_6;
				}
				jj_consume_token(COMMA);
				EnumConstant();
			}
			break;
		default:
			jj_la1[15] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case COMMA:
			jj_consume_token(COMMA);
			break;
		default:
			jj_la1[16] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case SEMICOLON:
			jj_consume_token(SEMICOLON);
			label_7: while (true) {
				switch (jj_nt.kind) {
				case ABSTRACT:
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case CLASS:
				case DOUBLE:
				case ENUM:
				case FINAL:
				case FLOAT:
				case INT:
				case INTERFACE:
				case LONG:
				case NATIVE:
				case PRIVATE:
				case PROTECTED:
				case PUBLIC:
				case SHORT:
				case STATIC:
				case STRICTFP:
				case SYNCHRONIZED:
				case TRANSIENT:
				case VOID:
				case VOLATILE:
				case IDENTIFIER:
				case LBRACE:
				case SEMICOLON:
				case AT:
				case LT:
					;
					break;
				default:
					jj_la1[17] = jj_gen;
					break label_7;
				}
				ClassOrInterfaceBodyDeclaration();
			}
			break;
		default:
			jj_la1[18] = jj_gen;
			;
		}
		t = jj_consume_token(RBRACE);
		if (pToken != t)
			;
		jab.appendNode(pToken.beginLine, t.sumUpX(pToken));
		pToken = getToken(1);
		jab.endCompound(t.beginLine, t.allToString());
		jab.endClass();
	}

	final public void EnumConstant() throws ParseException {
		log("EnumConstant");
		jj_consume_token(IDENTIFIER);
		switch (jj_nt.kind) {
		case LPAREN:
			Arguments();
			break;
		default:
			jj_la1[19] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case LBRACE:
			ClassOrInterfaceBody();
			break;
		default:
			jj_la1[20] = jj_gen;
			;
		}
	}

	final public void TypeParameters() throws ParseException {
		log("TypeParameters");
		jj_consume_token(LT);
		TypeParameter();
		label_8: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[21] = jj_gen;
				break label_8;
			}
			jj_consume_token(COMMA);
			TypeParameter();
		}
		jj_consume_token(GT);
	}

	final public void TypeParameter() throws ParseException {
		log("TypeParameter");
		jj_consume_token(IDENTIFIER);
		switch (jj_nt.kind) {
		case EXTENDS:
			TypeBound();
			break;
		default:
			jj_la1[22] = jj_gen;
			;
		}
	}

	final public void TypeBound() throws ParseException {
		log("TypeBound");
		jj_consume_token(EXTENDS);
		ClassOrInterfaceType();
		label_9: while (true) {
			switch (jj_nt.kind) {
			case BIT_AND:
				;
				break;
			default:
				jj_la1[23] = jj_gen;
				break label_9;
			}
			jj_consume_token(BIT_AND);
			ClassOrInterfaceType();
		}
	}

	final public void ClassOrInterfaceBody() throws ParseException {
		log("ClassOrInterfaceBody");
		Token t;
		JavaASTCompoundNode cn, ecn;
		t = jj_consume_token(LBRACE);
		cn = jab.beginCompound(t.beginLine, t.allToString());
		pToken = getToken(1);
		label_10: while (true) {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case CLASS:
			case DOUBLE:
			case ENUM:
			case FINAL:
			case FLOAT:
			case INT:
			case INTERFACE:
			case LONG:
			case NATIVE:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case SHORT:
			case STATIC:
			case STRICTFP:
			case SYNCHRONIZED:
			case TRANSIENT:
			case VOID:
			case VOLATILE:
			case IDENTIFIER:
			case LBRACE:
			case SEMICOLON:
			case AT:
			case LT:
				;
				break;
			default:
				jj_la1[24] = jj_gen;
				break label_10;
			}
			ClassOrInterfaceBodyDeclaration();
		}
		t = jj_consume_token(RBRACE);
		if (pToken != t)
			;
		jab.appendNode(pToken.beginLine, t.sumUpX(pToken));
		pToken = getToken(1);
		jab.endCompound(t.beginLine, t.allToString());
	}

	final public void ClassOrInterfaceBodyDeclaration() throws ParseException {
		log("ClassOrInterfaceBodyDeclaration");
		String modifiers = "";
		if (jj_2_5(2)) {
			Initializer();
		} else {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case CLASS:
			case DOUBLE:
			case ENUM:
			case FINAL:
			case FLOAT:
			case INT:
			case INTERFACE:
			case LONG:
			case NATIVE:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case SHORT:
			case STATIC:
			case STRICTFP:
			case SYNCHRONIZED:
			case TRANSIENT:
			case VOID:
			case VOLATILE:
			case IDENTIFIER:
			case AT:
			case LT:
				modifiers = Modifiers();
				switch (jj_nt.kind) {
				case CLASS:
				case INTERFACE:
					ClassOrInterfaceDeclaration();
					break;
				case ENUM:
					EnumDeclaration();
					break;
				case AT:
					AnnotationTypeDeclaration();
					break;
				default:
					jj_la1[25] = jj_gen;
					if (jj_2_3(2147483647)) {
						ConstructorDeclaration(modifiers);
					} else if (jj_2_4(2147483647)) {
						FieldDeclaration();
					} else {
						switch (jj_nt.kind) {
						case BOOLEAN:
						case BYTE:
						case CHAR:
						case DOUBLE:
						case FLOAT:
						case INT:
						case LONG:
						case SHORT:
						case VOID:
						case IDENTIFIER:
						case LT:
							MethodDeclaration(modifiers);
							break;
						default:
							jj_la1[26] = jj_gen;
							jj_consume_token(-1);
							throw new ParseException();
						}
					}
				}
				break;
			case SEMICOLON:
				jj_consume_token(SEMICOLON);
				break;
			default:
				jj_la1[27] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	/*
	 * Variablen(Feld etc.)Deklaration im ClassInterfaceBody Type Variablenname, ... = ...;
	 */
	final public void FieldDeclaration() throws ParseException {
		log("FieldDeclaration");
		String t;
		// Modifiers are already matched in the caller
		t = Type();
		jab.beginDeclaration(t);
		VariableDeclarator();
		label_11: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[28] = jj_gen;
				break label_11;
			}
			jj_consume_token(COMMA);
			VariableDeclarator();
		}
		jj_consume_token(SEMICOLON);
		jab.endDeclaration();
	}

	final public void VariableDeclarator() throws ParseException {
		log("VariableDeclarator");
		VariableDeclaratorId();
		switch (jj_nt.kind) {
		case ASSIGN:
			jab.storeTermData();
			jj_consume_token(ASSIGN);
			VariableInitializer();
			jab.restoreTermData();
			break;
		default:
			jj_la1[29] = jj_gen;
			;
		}
	}

	/*
	 * liefert ID und eventuelle TypeSuffixes
	 */
	final public String VariableDeclaratorId() throws ParseException {
		log("VariableDeclaratorId");
		Token t;
		t = jj_consume_token(IDENTIFIER);
		label_12: while (true) {
			switch (jj_nt.kind) {
			case LBRACKET:
				;
				break;
			default:
				jj_la1[30] = jj_gen;
				break label_12;
			}
			jj_consume_token(LBRACKET);
			jj_consume_token(RBRACKET);
		}
		{
			if (true)
				return t.image;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition VariableInitializer() throws ParseException {
		log("VariableInitializer");
		JavaCondition cond = null;
		switch (jj_nt.kind) {
		case LBRACE:
			ArrayInitializer();
			break;
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case BANG:
		case TILDE:
		case INCR:
		case DECR:
		case PLUS:
		case MINUS:
			cond = Expression(false);
			break;
		default:
			jj_la1[31] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * mgliches Szenario: int a[] = { 1, test(2)>b?1:2}; alle VariablenInitializer mssen vorher zugewiesen werden! im
	 * Classbody ignorieren, da unvorhersagbare Nebeneffekte
	 */
	final public void ArrayInitializer() throws ParseException {
		log("ArrayInitializer");
		jj_consume_token(LBRACE);
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case LBRACE:
		case BANG:
		case TILDE:
		case INCR:
		case DECR:
		case PLUS:
		case MINUS:
			VariableInitializer();
			label_13: while (true) {
				if (jj_2_6(2)) {
					;
				} else {
					break label_13;
				}
				jj_consume_token(COMMA);
				VariableInitializer();
			}
			break;
		default:
			jj_la1[32] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case COMMA:
			jj_consume_token(COMMA);
			break;
		default:
			jj_la1[33] = jj_gen;
			;
		}
		jj_consume_token(RBRACE);
	}

	final public void MethodDeclaration(String modifiers) throws ParseException {
		log("MethodDeclaration");
		Token t1, tx;
		String s, parameters = "";
		t1 = getToken(1);
		switch (jj_nt.kind) {
		case LT:
			TypeParameters();
			break;
		default:
			jj_la1[34] = jj_gen;
			;
		}
		s = ResultType();
		jab.setType(s);
		tx = jj_consume_token(IDENTIFIER);
		parameters = FormalParameters();
		label_14: while (true) {
			switch (jj_nt.kind) {
			case LBRACKET:
				;
				break;
			default:
				jj_la1[35] = jj_gen;
				break label_14;
			}
			jj_consume_token(LBRACKET);
			jj_consume_token(RBRACKET);
		}
		switch (jj_nt.kind) {
		case THROWS:
			jj_consume_token(THROWS);
			NameList();
			break;
		default:
			jj_la1[36] = jj_gen;
			;
		}
		switch (jj_nt.kind) {
		case LBRACE:
			if (pToken != t1)
				jab.appendNode(pToken.beginLine, t1.sumUpX(pToken));
			pToken = getToken(1);

			jab.beginFunction(t1.beginLine,
			// t1.specialTokenToString(), // prefix
					getToken(0).sumUp(t1), // content
					modifiers, tx.image, // name
					parameters); // arguments

			Block(true, false);
			pToken = getToken(1);
			break;
		case SEMICOLON:
			jj_consume_token(SEMICOLON);
			break;
		default:
			jj_la1[37] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
	}

	/*
	 * liefert "(" Type "," Type "..." ")" zurck, fr Funktionsidentifizierung
	 */
	final public String FormalParameters() throws ParseException {
		log("FormalParameters");
		String s = "", s1 = "";

		jab.storeTermData();
		jj_consume_token(LPAREN);
		switch (jj_nt.kind) {
		case ABSTRACT:
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FINAL:
		case FLOAT:
		case INT:
		case LONG:
		case NATIVE:
		case PRIVATE:
		case PROTECTED:
		case PUBLIC:
		case SHORT:
		case STATIC:
		case STRICTFP:
		case SYNCHRONIZED:
		case TRANSIENT:
		case VOLATILE:
		case IDENTIFIER:
		case AT:
			s = FormalParameter();
			label_15: while (true) {
				switch (jj_nt.kind) {
				case COMMA:
					;
					break;
				default:
					jj_la1[38] = jj_gen;
					break label_15;
				}
				jj_consume_token(COMMA);
				s1 = FormalParameter();
				s += "," + s1;
			}
			break;
		default:
			jj_la1[39] = jj_gen;
			;
		}
		jj_consume_token(RPAREN);
		jab.restoreTermData();
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * liefert Type des Parameters zurck Vorkommen: Funktionsdefinition, catch(x)
	 */
	final public String FormalParameter() throws ParseException {
		log("FormalParameter");
		String s = "";
		Modifiers();
		s = Type();
		switch (jj_nt.kind) {
		case ELLIPSIS:
			jj_consume_token(ELLIPSIS);
			s += "...";
			break;
		default:
			jj_la1[40] = jj_gen;
			;
		}
		VariableDeclaratorId();
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	final public void ConstructorDeclaration(String modifiers) throws ParseException {
		log("ConstructorDeclaration");
		Token t, t1;
		JavaASTCompoundNode cn, ecn;
		JavaASTStatementNode endNode;
		String parameters = "";
		t = getToken(1);
		switch (jj_nt.kind) {
		case LT:
			TypeParameters();
			break;
		default:
			jj_la1[41] = jj_gen;
			;
		}
		// Modifiers matched in the caller
		t1 = jj_consume_token(IDENTIFIER);
		parameters = FormalParameters();
		switch (jj_nt.kind) {
		case THROWS:
			jj_consume_token(THROWS);
			NameList();
			break;
		default:
			jj_la1[42] = jj_gen;
			;
		}
		if (pToken != t)
			jab.appendNode(pToken.beginLine, t.sumUpX(pToken));
		pToken = getToken(1);

		jab.beginFunction(t.beginLine,
		// t.specialTokenToString(), // prefix
				getToken(0).sumUp(t), // content
				modifiers, t1.image, // name
				parameters); // arguments

		t = jj_consume_token(LBRACE);
		cn = jab.beginCompound(t.beginLine, t.allToString());
		pToken = getToken(1);
		if (jj_2_7(2147483647)) {
			ExplicitConstructorInvocation();
			/*
			 * Explizite Konstruktoraufruf muss als erstes im Funktionsrumpf stehen, erst danach darf instrumentiert
			 * werden.
			 */
			jab.appendNode(pToken.beginLine, getToken(0).sumUp(pToken));
		} else {
			;
		}
		jab.addFunctionEntryNode(getToken(0).beginLine);
		pToken = getToken(1);
		Statements();
		t = jj_consume_token(RBRACE);
		pToken = getToken(1);

		jab.endFunction(t.beginLine);
		jab.endCompound(t.beginLine, t.allToString());
	}

	final public void ExplicitConstructorInvocation() throws ParseException {
		log("ExplicitConstructorInvocation");
		if (jj_2_9(2147483647)) {
			jj_consume_token(THIS);
			Arguments();
			jj_consume_token(SEMICOLON);
		} else {
			switch (jj_nt.kind) {
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case DOUBLE:
			case FALSE:
			case FLOAT:
			case INT:
			case LONG:
			case NEW:
			case NULL:
			case SHORT:
			case SUPER:
			case THIS:
			case TRUE:
			case VOID:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
				if (jj_2_8(2)) {
					PrimaryExpression();
					jj_consume_token(DOT);
				} else {
					;
				}
				jj_consume_token(SUPER);
				Arguments();
				jj_consume_token(SEMICOLON);
				break;
			default:
				jj_la1[43] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public void Statements() throws ParseException {
		label_16: while (true) {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case ASSERT:
			case BOOLEAN:
			case BREAK:
			case BYTE:
			case CHAR:
			case CLASS:
			case CONTINUE:
			case DO:
			case DOUBLE:
			case FALSE:
			case FINAL:
			case FLOAT:
			case FOR:
			case IF:
			case INT:
			case INTERFACE:
			case LONG:
			case NATIVE:
			case NEW:
			case NULL:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case RETURN:
			case SHORT:
			case STATIC:
			case STRICTFP:
			case SUPER:
			case SWITCH:
			case SYNCHRONIZED:
			case THIS:
			case THROW:
			case TRANSIENT:
			case TRUE:
			case TRY:
			case VOID:
			case VOLATILE:
			case WHILE:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
			case LBRACE:
			case SEMICOLON:
			case AT:
			case INCR:
			case DECR:
				;
				break;
			default:
				jj_la1[44] = jj_gen;
				break label_16;
			}
			BlockStatement();
		}
	}

	final public void Initializer() throws ParseException {
		log("Initializer");
		String s = "";
		Token t = getToken(1);
		if (pToken != getToken(1))
			;
		jab.appendNode(pToken.beginLine, getToken(1).sumUpX(pToken));
		pToken = getToken(1);
		switch (jj_nt.kind) {
		case STATIC:
			jj_consume_token(STATIC);
			s = getToken(0).allToString();
			break;
		default:
			jj_la1[45] = jj_gen;
			;
		}
		jab.beginFunction(t.beginLine, s, // content
				"", "static_class_initializer", // name
				""); // arguments
		jab.clearTerm();
		Block(true, false);
		pToken = getToken(1);
	}

	/*
	 * Type, name and expression syntax follows.
	 */
	final public String Type() throws ParseException {
		log("Type");
		String s = "";
		if (jj_2_10(2)) {
			s = ReferenceType();
		} else {
			switch (jj_nt.kind) {
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case DOUBLE:
			case FLOAT:
			case INT:
			case LONG:
			case SHORT:
				s = PrimitiveType();
				break;
			default:
				jj_la1[46] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	final public String ReferenceType() throws ParseException {
		log("ReferenceType");
		String s = "";
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FLOAT:
		case INT:
		case LONG:
		case SHORT:
			s = PrimitiveType();
			label_17: while (true) {
				jj_consume_token(LBRACKET);
				jj_consume_token(RBRACKET);
				s += "[]";
				if (jj_2_11(2)) {
					;
				} else {
					break label_17;
				}
			}
			break;
		case IDENTIFIER:
			s = ClassOrInterfaceType();
			label_18: while (true) {
				if (jj_2_12(2)) {
					;
				} else {
					break label_18;
				}
				jj_consume_token(LBRACKET);
				jj_consume_token(RBRACKET);
				s += "[]";
			}
			break;
		default:
			jj_la1[47] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	final public String ClassOrInterfaceType() throws ParseException {
		log("ClassOrInterfaceType");
		String s = "", s2;
		Token t;
		t = jj_consume_token(IDENTIFIER);
		if (jj_2_13(2)) {
			s = TypeArguments();
		} else {
			;
		}
		s = t.image + s;
		label_19: while (true) {
			if (jj_2_14(2)) {
				;
			} else {
				break label_19;
			}
			jj_consume_token(DOT);
			t = jj_consume_token(IDENTIFIER);
			s += "." + t.image;
			if (jj_2_15(2)) {
				s2 = TypeArguments();
				s += s2;
			} else {
				;
			}
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	final public String TypeArguments() throws ParseException {
		log("TypeArguments");
		String s, s1 = "", s2;
		jj_consume_token(LT);
		s = TypeArgument();
		label_20: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[48] = jj_gen;
				break label_20;
			}
			jj_consume_token(COMMA);
			s2 = TypeArgument();
			s1 += "," + s2;
		}
		jj_consume_token(GT);
		{
			if (true)
				return "<" + s + s1 + ">";
		}
		throw new Error("Missing return statement in function");
	}

	final public String TypeArgument() throws ParseException {
		log("TypeARgument");
		String s;
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FLOAT:
		case INT:
		case LONG:
		case SHORT:
		case IDENTIFIER:
			s = ReferenceType();
			break;
		case HOOK:
			s = Wildcard();
			break;
		default:
			jj_la1[49] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	final public String Wildcard() throws ParseException {
		log("Wildcard");
		String s = "";
		jj_consume_token(HOOK);
		switch (jj_nt.kind) {
		case EXTENDS:
		case SUPER:
			switch (jj_nt.kind) {
			case EXTENDS:
				jj_consume_token(EXTENDS);
				s = ReferenceType();
				s = "extends" + s;
				break;
			case SUPER:
				jj_consume_token(SUPER);
				s = ReferenceType();
				s = "super" + s;
				break;
			default:
				jj_la1[50] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			break;
		default:
			jj_la1[51] = jj_gen;
			;
		}
		{
			if (true)
				return "?" + s;
		}
		throw new Error("Missing return statement in function");
	}

	final public String PrimitiveType() throws ParseException {
		log("PrimitiveType");
		switch (jj_nt.kind) {
		case BOOLEAN:
			jj_consume_token(BOOLEAN);
			break;
		case CHAR:
			jj_consume_token(CHAR);
			break;
		case BYTE:
			jj_consume_token(BYTE);
			break;
		case SHORT:
			jj_consume_token(SHORT);
			break;
		case INT:
			jj_consume_token(INT);
			break;
		case LONG:
			jj_consume_token(LONG);
			break;
		case FLOAT:
			jj_consume_token(FLOAT);
			break;
		case DOUBLE:
			jj_consume_token(DOUBLE);
			break;
		default:
			jj_la1[52] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return getToken(0).image;
		}
		throw new Error("Missing return statement in function");
	}

	final public String ResultType() throws ParseException {
		log("ResultType");
		Token t;
		String s = "";
		switch (jj_nt.kind) {
		case VOID:
			t = jj_consume_token(VOID);
			{
				if (true)
					return t.image;
			}
			break;
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FLOAT:
		case INT:
		case LONG:
		case SHORT:
		case IDENTIFIER:
			s = Type();
			{
				if (true)
					return s;
			}
			break;
		default:
			jj_la1[53] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		throw new Error("Missing return statement in function");
	}

	/* in PrimPrefix, PackageDeclaration, ImportDeclaration, Annotations, throws */
	final public String Name() throws ParseException {
		log("Name");
		Token t;
		String s;
		t = jj_consume_token(IDENTIFIER);
		s = t.image;
		label_21: while (true) {
			if (jj_2_16(2)) {
				;
			} else {
				break label_21;
			}
			jj_consume_token(DOT);
			t = jj_consume_token(IDENTIFIER);
			s += "." + t.image;
		}
		{
			if (true)
				return s;
		}
		throw new Error("Missing return statement in function");
	}

	/* nur nach "throws" */
	final public void NameList() throws ParseException {
		log("NameList");
		Name();
		label_22: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[54] = jj_gen;
				break label_22;
			}
			jj_consume_token(COMMA);
			Name();
		}
	}

	/*
	 * Expression syntax follows.
	 */
	final public JavaCondition Expression(boolean isCond) throws ParseException {
		log("Expression");
		JavaCondition cond;
		cond = ConditionalExpression(isCond);
		if (jj_2_17(2)) {
			AssignmentOperator();
			Expression(false);
			cond = null;
		} else {
			;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public String AssignmentOperator() throws ParseException {
		expr_log("AssignmentOperator");
		switch (jj_nt.kind) {
		case ASSIGN:
			jj_consume_token(ASSIGN);
			break;
		case STARASSIGN:
			jj_consume_token(STARASSIGN);
			break;
		case SLASHASSIGN:
			jj_consume_token(SLASHASSIGN);
			break;
		case REMASSIGN:
			jj_consume_token(REMASSIGN);
			break;
		case PLUSASSIGN:
			jj_consume_token(PLUSASSIGN);
			break;
		case MINUSASSIGN:
			jj_consume_token(MINUSASSIGN);
			break;
		case LSHIFTASSIGN:
			jj_consume_token(LSHIFTASSIGN);
			break;
		case RSIGNEDSHIFTASSIGN:
			jj_consume_token(RSIGNEDSHIFTASSIGN);
			break;
		case RUNSIGNEDSHIFTASSIGN:
			jj_consume_token(RUNSIGNEDSHIFTASSIGN);
			break;
		case ANDASSIGN:
			jj_consume_token(ANDASSIGN);
			break;
		case XORASSIGN:
			jj_consume_token(XORASSIGN);
			break;
		case ORASSIGN:
			jj_consume_token(ORASSIGN);
			break;
		default:
			jj_la1[55] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return getToken(0).image;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ConditionalExpression(boolean isCond) throws ParseException {
		expr_log("ConditionalExpression");
		JavaCondition cond = null;
		cond = ConditionalOrExpression(isCond);
		switch (jj_nt.kind) {
		case HOOK:
			jj_consume_token(HOOK);
			Expression(false);
			jj_consume_token(COLON);
			Expression(false);
			cond = null;
			break;
		default:
			jj_la1[56] = jj_gen;
			;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ConditionalOrExpression(boolean isCond) throws ParseException {
		expr_log("ConditionalOrExpression");
		Token t1, t2;
		JavaCondition cond1 = null, cond2;
		t1 = getToken(1);
		cond1 = ConditionalAndExpression(isCond);
		label_23: while (true) {
			switch (jj_nt.kind) {
			case SC_OR:
				;
				break;
			default:
				jj_la1[57] = jj_gen;
				break label_23;
			}
			t2 = jj_consume_token(SC_OR);
			cond2 = ConditionalAndExpression(true);
			cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
			cond1.setShortCircuitValue(true);
			jab.setType("boolean");
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ConditionalAndExpression(boolean isCond) throws ParseException {
		expr_log("ConditionalAndExpression");
		Token t1, t2;
		JavaCondition cond1 = null, cond2;
		t1 = getToken(1);
		cond1 = InclusiveOrExpression(isCond);
		label_24: while (true) {
			switch (jj_nt.kind) {
			case SC_AND:
				;
				break;
			default:
				jj_la1[58] = jj_gen;
				break label_24;
			}
			t2 = jj_consume_token(SC_AND);
			cond2 = InclusiveOrExpression(true);
			cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
			cond1.setShortCircuitValue(false);
			jab.setType("boolean");
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition InclusiveOrExpression(boolean isCond) throws ParseException {
		expr_log("InclusiveOrExpression");
		Token t1, t2;
		JavaCondition cond1 = null, cond2;
		String firstType = "", secondType = "";
		t1 = getToken(1);
		cond1 = ExclusiveOrExpression(isCond);
		firstType = jab.getType();
		label_25: while (true) {
			switch (jj_nt.kind) {
			case BIT_OR:
				;
				break;
			default:
				jj_la1[59] = jj_gen;
				break label_25;
			}
			t2 = jj_consume_token(BIT_OR);
			cond2 = ExclusiveOrExpression(isCond);
			secondType = jab.getType();

			// '|' kann boolscher oder bit-Operator sein ! Versuche das zu unterscheiden ...
			if (isCond || firstType.equals("boolean") || secondType.equals("boolean")
					|| (cond1 != null && cond1.isBoolean()) || (cond2 != null && cond2.isBoolean()))
				cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
			// wenn nicht eindeutig boolean, ignoriere ihn, d.h. unbekannter Rckgabewert
			else
				cond1 = null;
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ExclusiveOrExpression(boolean isCond) throws ParseException {
		expr_log("ExclusiveOrExpression");
		Token t1, t2;
		String firstType = "", secondType = "";
		JavaCondition cond1 = null, cond2;
		t1 = getToken(1);
		cond1 = AndExpression(isCond);
		firstType = jab.getType();
		label_26: while (true) {
			switch (jj_nt.kind) {
			case XOR:
				;
				break;
			default:
				jj_la1[60] = jj_gen;
				break label_26;
			}
			t2 = jj_consume_token(XOR);
			cond2 = AndExpression(isCond);
			secondType = jab.getType();
			// '^' kann boolscher oder bit-Operator sein ! Versuche das zu unterscheiden ...
			if (isCond || firstType.equals("boolean") || secondType.equals("boolean")
					|| (cond1 != null && cond1.isBoolean()) || (cond2 != null && cond2.isBoolean()))
				cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
			// wenn nicht eindeutig boolean, ignoriere ihn, d.h. unbekannter Rckgabewert
			else
				cond1 = null;
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition AndExpression(boolean isCond) throws ParseException {
		expr_log("AndExpression");
		Token t1, t2;
		String firstType = "", secondType = "";
		JavaCondition cond1 = null, cond2;
		t1 = getToken(1);
		cond1 = EqualityExpression();
		firstType = jab.getType();
		label_27: while (true) {
			switch (jj_nt.kind) {
			case BIT_AND:
				;
				break;
			default:
				jj_la1[61] = jj_gen;
				break label_27;
			}
			t2 = jj_consume_token(BIT_AND);
			cond2 = EqualityExpression();
			secondType = jab.getType();
			// '&' kann boolscher oder bit-Operator sein ! Versuche das zu unterscheiden ...
			if (isCond || firstType.equals("boolean") || secondType.equals("boolean")
					|| (cond1 != null && cond1.isBoolean()) || (cond2 != null && cond2.isBoolean()))
				cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
			// wenn nicht eindeutig boolean, ignoriere ihn, d.h. unbekannter Rckgabewert
			else
				cond1 = null;
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition EqualityExpression() throws ParseException {
		expr_log("EqualityExpression");
		Token t1, t2;
		JavaCondition cond = null;
		t1 = getToken(1);
		cond = InstanceOfExpression();
		label_28: while (true) {
			switch (jj_nt.kind) {
			case EQ:
			case NE:
				;
				break;
			default:
				jj_la1[62] = jj_gen;
				break label_28;
			}
			switch (jj_nt.kind) {
			case EQ:
				t2 = jj_consume_token(EQ);
				break;
			case NE:
				t2 = jj_consume_token(NE);
				break;
			default:
				jj_la1[63] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			InstanceOfExpression();
			// "==" und "!=" sind keine boolschen, aber relationale Operatoren, d.h. liefern bool
			cond = new JavaCondition(getToken(0).sumUp(t1));
			jab.setType("boolean");
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition InstanceOfExpression() throws ParseException {
		expr_log("InstanceOfExpression");
		JavaCondition cond = null;
		Token t;
		t = getToken(1);
		cond = RelationalExpression();
		switch (jj_nt.kind) {
		case INSTANCEOF:
			jj_consume_token(INSTANCEOF);
			Type();
			jab.setType("boolean");
			cond = new JavaCondition(getToken(0).sumUp(t));
			break;
		default:
			jj_la1[64] = jj_gen;
			;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition RelationalExpression() throws ParseException {
		expr_log("RelationalExpression");
		JavaCondition cond1 = null, cond2 = null;
		Token t1, t2 = null;
		t1 = getToken(1);
		cond1 = ShiftExpression();
		label_29: while (true) {
			switch (jj_nt.kind) {
			case LT:
			case LE:
			case GE:
			case GT:
				;
				break;
			default:
				jj_la1[65] = jj_gen;
				break label_29;
			}
			switch (jj_nt.kind) {
			case LT:
				t2 = jj_consume_token(LT);
				break;
			case GT:
				t2 = jj_consume_token(GT);
				break;
			case LE:
				t2 = jj_consume_token(LE);
				break;
			case GE:
				t2 = jj_consume_token(GE);
				break;
			default:
				jj_la1[66] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			cond2 = ShiftExpression();
			cond1 = new JavaCondition(getToken(0).sumUp(t1));
			jab.setType("boolean");
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ShiftExpression() throws ParseException {
		expr_log("ShiftExpression");
		JavaCondition cond = null;
		Token t = getToken(0);
		cond = AdditiveExpression();
		label_30: while (true) {
			if (jj_2_18(1)) {
				;
			} else {
				break label_30;
			}
			switch (jj_nt.kind) {
			case LSHIFT:
				jj_consume_token(LSHIFT);
				break;
			default:
				jj_la1[67] = jj_gen;
				if (jj_2_19(1)) {
					RSIGNEDSHIFT();
				} else if (jj_2_20(1)) {
					RUNSIGNEDSHIFT();
				} else {
					jj_consume_token(-1);
					throw new ParseException();
				}
			}
			AdditiveExpression();
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition AdditiveExpression() throws ParseException {
		expr_log("AdditiveExpression");
		JavaCondition cond = null;
		cond = MultiplicativeExpression();
		label_31: while (true) {
			switch (jj_nt.kind) {
			case PLUS:
			case MINUS:
				;
				break;
			default:
				jj_la1[68] = jj_gen;
				break label_31;
			}
			switch (jj_nt.kind) {
			case PLUS:
				jj_consume_token(PLUS);
				break;
			case MINUS:
				jj_consume_token(MINUS);
				break;
			default:
				jj_la1[69] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			MultiplicativeExpression();
			cond = null;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition MultiplicativeExpression() throws ParseException {
		expr_log("MultiplicativeExpression");
		JavaCondition cond = null;
		cond = UnaryExpression();
		label_32: while (true) {
			switch (jj_nt.kind) {
			case STAR:
			case SLASH:
			case REM:
				;
				break;
			default:
				jj_la1[70] = jj_gen;
				break label_32;
			}
			switch (jj_nt.kind) {
			case STAR:
				jj_consume_token(STAR);
				break;
			case SLASH:
				jj_consume_token(SLASH);
				break;
			case REM:
				jj_consume_token(REM);
				break;
			default:
				jj_la1[71] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			UnaryExpression();
			cond = null;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition UnaryExpression() throws ParseException {
		expr_log("UnaryExpression");
		JavaCondition cond = null;
		switch (jj_nt.kind) {
		case PLUS:
		case MINUS:
			switch (jj_nt.kind) {
			case PLUS:
				jj_consume_token(PLUS);
				break;
			case MINUS:
				jj_consume_token(MINUS);
				break;
			default:
				jj_la1[72] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			cond = UnaryExpression();
			break;
		case INCR:
			PreIncrementExpression();
			break;
		case DECR:
			PreDecrementExpression();
			break;
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case BANG:
		case TILDE:
			cond = UnaryExpressionNotPlusMinus();
			break;
		default:
			jj_la1[73] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public void PreIncrementExpression() throws ParseException {
		expr_log("PreIncrementExpression");
		jj_consume_token(INCR);
		PrimaryExpression();
	}

	final public void PreDecrementExpression() throws ParseException {
		expr_log("PreDecrementExpression");
		jj_consume_token(DECR);
		PrimaryExpression();
	}

	final public JavaCondition UnaryExpressionNotPlusMinus() throws ParseException {
		expr_log("UnaryExpressionNotPlusMinus");
		JavaCondition cond;
		Token t1, t2;
		switch (jj_nt.kind) {
		case BANG:
		case TILDE:
			switch (jj_nt.kind) {
			case TILDE:
				t1 = jj_consume_token(TILDE);
				break;
			case BANG:
				t1 = jj_consume_token(BANG);
				break;
			default:
				jj_la1[74] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			cond = UnaryExpression();
			if (t1.image == "!") {
				jab.setType("boolean");
				if (cond == null)
					cond = new JavaCondition(getToken(0).sumUp(t1.next));
				cond = new JavaCondition("!", cond);
			}
			break;
		default:
			jj_la1[75] = jj_gen;
			if (jj_2_21(2147483647)) {
				cond = CastExpression();
			} else {
				switch (jj_nt.kind) {
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case DOUBLE:
				case FALSE:
				case FLOAT:
				case INT:
				case LONG:
				case NEW:
				case NULL:
				case SHORT:
				case SUPER:
				case THIS:
				case TRUE:
				case VOID:
				case LONG_LITERAL:
				case INTEGER_LITERAL:
				case FLOATING_POINT_LITERAL:
				case CHARACTER_LITERAL:
				case STRING_LITERAL:
				case IDENTIFIER:
				case LPAREN:
					cond = PostfixExpression();
					break;
				default:
					jj_la1[76] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
			}
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	// This production is to determine lookahead only. The LOOKAHEAD specifications
	// below are not used, but they are there just to indicate that we know about
	// this.
	final public void CastLookahead() throws ParseException {
		if (jj_2_22(2)) {
			jj_consume_token(LPAREN);
			PrimitiveType();
		} else if (jj_2_23(2147483647)) {
			jj_consume_token(LPAREN);
			Type();
			jj_consume_token(LBRACKET);
			jj_consume_token(RBRACKET);
		} else {
			switch (jj_nt.kind) {
			case LPAREN:
				jj_consume_token(LPAREN);
				Type();
				jj_consume_token(RPAREN);
				switch (jj_nt.kind) {
				case TILDE:
					jj_consume_token(TILDE);
					break;
				case BANG:
					jj_consume_token(BANG);
					break;
				case LPAREN:
					jj_consume_token(LPAREN);
					break;
				case IDENTIFIER:
					jj_consume_token(IDENTIFIER);
					break;
				case THIS:
					jj_consume_token(THIS);
					break;
				case SUPER:
					jj_consume_token(SUPER);
					break;
				case NEW:
					jj_consume_token(NEW);
					break;
				case FALSE:
				case NULL:
				case TRUE:
				case LONG_LITERAL:
				case INTEGER_LITERAL:
				case FLOATING_POINT_LITERAL:
				case CHARACTER_LITERAL:
				case STRING_LITERAL:
					Literal();
					break;
				default:
					jj_la1[77] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
				break;
			default:
				jj_la1[78] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public JavaCondition PostfixExpression() throws ParseException {
		expr_log("PostfixExpression");
		JavaCondition cond;
		cond = PrimaryExpression();
		switch (jj_nt.kind) {
		case INCR:
		case DECR:
			switch (jj_nt.kind) {
			case INCR:
				jj_consume_token(INCR);
				break;
			case DECR:
				jj_consume_token(DECR);
				break;
			default:
				jj_la1[79] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			break;
		default:
			jj_la1[80] = jj_gen;
			;
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition CastExpression() throws ParseException {
		expr_log("CastExpression");
		JavaCondition cond;
		Token t1, t2;
		String s;
		if (jj_2_24(2147483647)) {
			jj_consume_token(LPAREN);
			s = Type();
			jj_consume_token(RPAREN);
			UnaryExpression();
		} else {
			switch (jj_nt.kind) {
			case LPAREN:
				jj_consume_token(LPAREN);
				s = Type();
				jj_consume_token(RPAREN);
				UnaryExpressionNotPlusMinus();
				break;
			default:
				jj_la1[81] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
		jab.setType(s);
		{
			if (true)
				return null;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition PrimaryExpression() throws ParseException {
		expr_log("PrimaryExpression");
		JavaCondition cond = null;
		jab.clearTerm();
		cond = PrimaryPrefix();
		label_33: while (true) {
			if (jj_2_25(2)) {
				;
			} else {
				break label_33;
			}
			PrimarySuffix();
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition PrimaryPrefix() throws ParseException {
		expr_log("PrimaryPrefix");
		JavaCondition cond = null;
		String s = "";
		Token t, t1, t2;
		switch (jj_nt.kind) {
		case FALSE:
		case NULL:
		case TRUE:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
			cond = Literal();
			break;
		case THIS:
			jj_consume_token(THIS);
			break;
		case SUPER:
			jj_consume_token(SUPER);
			jj_consume_token(DOT);
			t = jj_consume_token(IDENTIFIER);
			break;
		case LPAREN:
			t1 = jj_consume_token(LPAREN);
			cond = Expression(false);
			t2 = jj_consume_token(RPAREN);
			break;
		case NEW:
			cond = AllocationExpression();
			break;
		default:
			jj_la1[82] = jj_gen;
			if (jj_2_26(2147483647)) {
				s = ResultType();
				jj_consume_token(DOT);
				jj_consume_token(CLASS);

			} else {
				switch (jj_nt.kind) {
				case IDENTIFIER:
					s = Name();
					break;
				default:
					jj_la1[83] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
			}
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition PrimarySuffix() throws ParseException {
		expr_log("PrimarySuffix");
		JavaCondition cond1 = null, cond2;
		Token t, t1, t2;
		switch (jj_nt.kind) {
		case DOT:
			t1 = jj_consume_token(DOT);
			switch (jj_nt.kind) {
			case THIS:
				jj_consume_token(THIS);
				break;
			case SUPER:
				jj_consume_token(SUPER);
				break;
			case NEW:
				cond1 = AllocationExpression();
				break;
			case IDENTIFIER:
			case LT:
				switch (jj_nt.kind) {
				case LT:
					TypeArguments();
					break;
				default:
					jj_la1[84] = jj_gen;
					;
				}
				t = jj_consume_token(IDENTIFIER);
				break;
			default:
				jj_la1[85] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
			break;
		case LBRACKET:
			t1 = jj_consume_token(LBRACKET);
			jab.storeTermData();
			Expression(false);
			t2 = jj_consume_token(RBRACKET);
			jab.restoreTermData();
			break;
		case LPAREN:
			cond1 = Arguments();
			break;
		default:
			jj_la1[86] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition Literal() throws ParseException {
		log("Literal");
		JavaCondition cond = null;
		Token t;
		switch (jj_nt.kind) {
		case INTEGER_LITERAL:
			jj_consume_token(INTEGER_LITERAL);
			jab.appendToType("int");
			break;
		case LONG_LITERAL:
			jj_consume_token(LONG_LITERAL);
			jab.appendToType("long");
			break;
		case FLOATING_POINT_LITERAL:
			jj_consume_token(FLOATING_POINT_LITERAL);
			jab.appendToType("float");
			break;
		case CHARACTER_LITERAL:
			jj_consume_token(CHARACTER_LITERAL);
			jab.appendToType("char");
			break;
		case STRING_LITERAL:
			jj_consume_token(STRING_LITERAL);
			jab.appendToType("String");
			break;
		case FALSE:
		case TRUE:
			cond = BooleanLiteral();
			break;
		case NULL:
			NullLiteral();
			break;
		default:
			jj_la1[87] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition BooleanLiteral() throws ParseException {
		log("BooleanLiteral");
		Token t;
		switch (jj_nt.kind) {
		case TRUE:
			t = jj_consume_token(TRUE);
			jab.appendToType("boolean");
			break;
		case FALSE:
			t = jj_consume_token(FALSE);
			jab.appendToType("boolean");
			break;
		default:
			jj_la1[88] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return new JavaCondition(t.image);
		}
		throw new Error("Missing return statement in function");
	}

	final public void NullLiteral() throws ParseException {
		log("NullLiteral");
		jj_consume_token(NULL);
	}

	final public JavaCondition Arguments() throws ParseException {
		log("Arguments");
		String s = "";
		JavaCondition cond = null;
		Token t1, t2;
		t1 = jj_consume_token(LPAREN);
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case BANG:
		case TILDE:
		case INCR:
		case DECR:
		case PLUS:
		case MINUS:
			cond = ArgumentList();
			break;
		default:
			jj_la1[89] = jj_gen;
			;
		}
		t2 = jj_consume_token(RPAREN);
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition ArgumentList() throws ParseException {
		log("ArgumentList");
		String s = "";
		jab.storeTermData();
		JavaCondition cond1 = null, cond2;
		Token t1, t2;
		t1 = getToken(1);
		cond1 = Expression(false);
		label_34: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[90] = jj_gen;
				break label_34;
			}
			t2 = jj_consume_token(COMMA);
			cond2 = Expression(false);
			cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
		}
		jab.restoreTermData();
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public JavaCondition AllocationExpression() throws ParseException {
		log("AllocationExpression");
		String s, prefix = "";
		JavaCondition cond;
		Token t1, t2;
		ASTNode n;
		if (jj_2_28(2)) {
			t1 = jj_consume_token(NEW);
			s = PrimitiveType();
			t2 = getToken(0);
			cond = ArrayDimsAndInits();
		} else {
			switch (jj_nt.kind) {
			case NEW:
				t1 = jj_consume_token(NEW);
				ClassOrInterfaceType();
				t2 = getToken(0);
				switch (jj_nt.kind) {
				case LT:
					TypeArguments();
					break;
				default:
					jj_la1[91] = jj_gen;
					;
				}
				switch (jj_nt.kind) {
				case LBRACKET:
					cond = ArrayDimsAndInits();
					break;
				case LPAREN:
					cond = Arguments();
					if (jj_2_27(2)) {
						if (pToken != t1) {
							jab.appendNode(pToken.beginLine, t1.sumUpX(pToken));
						}
						pToken = getToken(1);
						jab.beginClass(t1.beginLine, getToken(0).sumUp(t1), // content
								null, // name
								""); // extension

						ClassOrInterfaceBody();
						jab.endClass();
						pToken = getToken(1);
					} else {
						;
					}
					break;
				default:
					jj_la1[92] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
				break;
			default:
				jj_la1[93] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * The third LOOKAHEAD specification below is to parse to PrimarySuffix if there is an expression between the
	 * "[...]".
	 */
	final public JavaCondition ArrayDimsAndInits() throws ParseException {
		log("ArrayDimsAndInits");
		JavaCondition cond = null;
		Token t1, t2;
		if (jj_2_31(2)) {
			label_35: while (true) {
				jab.storeTermData();
				t1 = jj_consume_token(LBRACKET);
				cond = Expression(false);
				t2 = jj_consume_token(RBRACKET);
				jab.restoreTermData();
				if (jj_2_29(2)) {
					;
				} else {
					break label_35;
				}
			}
			label_36: while (true) {
				if (jj_2_30(2)) {
					;
				} else {
					break label_36;
				}
				t1 = jj_consume_token(LBRACKET);
				t2 = jj_consume_token(RBRACKET);
			}
		} else {
			switch (jj_nt.kind) {
			case LBRACKET:
				t1 = getToken(1);
				label_37: while (true) {
					jj_consume_token(LBRACKET);
					jj_consume_token(RBRACKET);
					switch (jj_nt.kind) {
					case LBRACKET:
						;
						break;
					default:
						jj_la1[94] = jj_gen;
						break label_37;
					}
				}
				t2 = getToken(0);
				ArrayInitializer();
				break;
			default:
				jj_la1[95] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
		{
			if (true)
				return cond;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * Statement syntax follows.
	 */
	final public void Statement() throws ParseException {
		log("Statement");
		Token t, t1, t2;
		JavaCondition cond;
		JavaASTStatementNode sn;
		ASTNode n;
		if (jj_2_32(2)) {
			LabeledStatement();
		} else {
			switch (jj_nt.kind) {
			case ASSERT:
				AssertStatement();
				break;
			case LBRACE:
				Block(false, false);
				break;
			case SEMICOLON:
				EmptyStatement();
				break;
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case DOUBLE:
			case FALSE:
			case FLOAT:
			case INT:
			case LONG:
			case NEW:
			case NULL:
			case SHORT:
			case SUPER:
			case THIS:
			case TRUE:
			case VOID:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
			case INCR:
			case DECR:
				t1 = getToken(1);
				sn = jab.appendStatement(t1.beginLine, "");
				pToken = getToken(1);
				cond = StatementExpression();
				t2 = jj_consume_token(SEMICOLON);
				if (pToken == t1) {
					sn.setContent(getToken(0).sumUp(t1));
				} else {
					n = jab.appendNode(t2.beginLine, t2.sumUp(pToken));
					sn.setEndOfStatement(n);
				}
				pToken = getToken(1);
				break;
			case SWITCH:
				SwitchStatement();
				break;
			case IF:
				IfStatement();
				break;
			case WHILE:
				WhileStatement();
				break;
			case DO:
				DoStatement();
				break;
			case FOR:
				ForStatement();
				break;
			case BREAK:
				BreakStatement();
				break;
			case CONTINUE:
				ContinueStatement();
				break;
			case RETURN:
				ReturnStatement();
				break;
			case THROW:
				ThrowStatement();
				break;
			case SYNCHRONIZED:
				SynchronizedStatement();
				break;
			case TRY:
				TryStatement();
				break;
			default:
				jj_la1[96] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public void AssertStatement() throws ParseException {
		log("AssertStatement");
		Token t1, t2;
		Condition cond, cond1 = null;
		jab.enterStatement();
		t1 = jj_consume_token(ASSERT);
		cond = Expression(false);
		switch (jj_nt.kind) {
		case COLON:
			jj_consume_token(COLON);
			cond1 = Expression(false);
			break;
		default:
			jj_la1[97] = jj_gen;
			;
		}
		jj_consume_token(SEMICOLON);
		jab.leaveStatement();
		jab.appendStatement(t1.beginLine, getToken(0).sumUp(t1));
	}

	final public void LabeledStatement() throws ParseException {
		log("LabeledStatement");
		Token t;
		t = jj_consume_token(IDENTIFIER);
		jj_consume_token(COLON);
		jab.appendLabel(t.beginLine, getToken(0).sumUp(t), t.image);
		Statement();
	}

	final public ASTJumpNode Block(boolean isFunctionBlock, boolean isTryBlock) throws ParseException {
		log("Block");
		Token t;
		JavaASTCompoundNode cn, ecn;
		JavaASTStatementNode endNode;
		ASTJumpNode jumpNode;
		t = jj_consume_token(LBRACE);
		jab.beginCompound(t.beginLine, t.allToString());
		pToken = getToken(1);
		if (isFunctionBlock)
			jab.addFunctionEntryNode(getToken(0).beginLine);
		if (isTryBlock)
			jab.setEntryOfTryBlock(getToken(0).beginLine);
		Statements();
		t = jj_consume_token(RBRACE);
		if (isFunctionBlock)
			jab.endFunction(t.beginLine);
		jab.endCompound(t.beginLine, t.allToString());

		if (isTryBlock) {
			jumpNode = jab.appendJump(getToken(0).beginLine, "", "");
			jumpNode.isVirtual(true);
			{
				if (true)
					return jumpNode;
			}
		}
		{
			if (true)
				return null;
		}
		throw new Error("Missing return statement in function");
	}

	final public ASTJumpNode CatchBlock() throws ParseException {
		log("CatchBlock");
		Token t;
		JavaASTCompoundNode cn, ecn;
		JavaASTStatementNode endNode;
		ASTJumpNode jumpNode;
		t = jj_consume_token(LBRACE);
		jab.beginCompound(t.beginLine, t.allToString());
		pToken = getToken(1);
		Statements();
		t = jj_consume_token(RBRACE);
		jab.endCompound(t.beginLine, t.allToString());
		jumpNode = jab.appendJump(getToken(0).beginLine, "", "");
		jumpNode.isVirtual(true);
		{
			if (true)
				return jumpNode;
		}
		throw new Error("Missing return statement in function");
	}

	/*
	 * Classes inside block stametents can only be abstract or final. The semantic must check it.
	 */
	final public void BlockStatement() throws ParseException {
		log("BlockStatement");
		Token t1, t2;
		ASTNode n;
		JavaASTStatementNode sn;
		if (jj_2_33(2147483647)) {
			Modifiers();
			ClassOrInterfaceDeclaration();
		} else if (jj_2_34(2147483647)) {
			t1 = getToken(1);
			sn = jab.appendStatement(t1.beginLine, "");
			pToken = getToken(1);
			VariableDeclarationExpression();
			t2 = jj_consume_token(SEMICOLON);
			if (pToken == t1)
				sn.setContent(getToken(0).sumUp(t1));
			else {
				n = jab.appendNode(t2.beginLine, t2.sumUp(pToken));
				sn.setEndOfStatement(n);
				n.isVirtual(false);
			}
			pToken = getToken(1);
		} else {
			switch (jj_nt.kind) {
			case ASSERT:
			case BOOLEAN:
			case BREAK:
			case BYTE:
			case CHAR:
			case CONTINUE:
			case DO:
			case DOUBLE:
			case FALSE:
			case FLOAT:
			case FOR:
			case IF:
			case INT:
			case LONG:
			case NEW:
			case NULL:
			case RETURN:
			case SHORT:
			case SUPER:
			case SWITCH:
			case SYNCHRONIZED:
			case THIS:
			case THROW:
			case TRUE:
			case TRY:
			case VOID:
			case WHILE:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
			case LBRACE:
			case SEMICOLON:
			case INCR:
			case DECR:
				Statement();
				break;
			default:
				jj_la1[98] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public void VariableDeclarationExpression() throws ParseException {
		log("VariableDeclarationExpression");
		Modifiers();
		Type();
		VariableDeclarator();
		label_38: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[99] = jj_gen;
				break label_38;
			}
			jj_consume_token(COMMA);
			VariableDeclarator();
		}
	}

	final public void EmptyStatement() throws ParseException {
		log("EmptyStatement");
		Token t;
		t = jj_consume_token(SEMICOLON);
		jab.appendStatement(t.beginLine, t.allToString());
	}

	final public JavaCondition StatementExpression() throws ParseException {
		log("StatementExpression");
		JavaCondition cond1 = null, cond2 = null;
		Token t1, t2;
		switch (jj_nt.kind) {
		case INCR:
			PreIncrementExpression();
			break;
		case DECR:
			PreDecrementExpression();
			break;
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
			t1 = getToken(1);
			cond1 = PrimaryExpression();
			switch (jj_nt.kind) {
			case ASSIGN:
			case INCR:
			case DECR:
			case PLUSASSIGN:
			case MINUSASSIGN:
			case STARASSIGN:
			case SLASHASSIGN:
			case ANDASSIGN:
			case ORASSIGN:
			case XORASSIGN:
			case REMASSIGN:
			case LSHIFTASSIGN:
			case RSIGNEDSHIFTASSIGN:
			case RUNSIGNEDSHIFTASSIGN:
				switch (jj_nt.kind) {
				case INCR:
				case DECR:
					switch (jj_nt.kind) {
					case INCR:
						t2 = jj_consume_token(INCR);
						break;
					case DECR:
						t2 = jj_consume_token(DECR);
						break;
					default:
						jj_la1[100] = jj_gen;
						jj_consume_token(-1);
						throw new ParseException();
					}
					break;
				case ASSIGN:
				case PLUSASSIGN:
				case MINUSASSIGN:
				case STARASSIGN:
				case SLASHASSIGN:
				case ANDASSIGN:
				case ORASSIGN:
				case XORASSIGN:
				case REMASSIGN:
				case LSHIFTASSIGN:
				case RSIGNEDSHIFTASSIGN:
				case RUNSIGNEDSHIFTASSIGN:
					AssignmentOperator();
					t2 = getToken(0);
					cond2 = Expression(false);
					cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
					break;
				default:
					jj_la1[101] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
				break;
			default:
				jj_la1[102] = jj_gen;
				;
			}
			break;
		default:
			jj_la1[103] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public void SwitchStatement() throws ParseException {
		log("SwitchStatement");
		Token t;
		JavaASTStatementNode endNode;
		ASTNode entryNode;
		JavaASTCompoundNode cn, ecn;
		Condition cond;
		t = jj_consume_token(SWITCH);
		jj_consume_token(LPAREN);
		cond = Expression(false);
		jj_consume_token(RPAREN);
		jab.beginSwitch(t.beginLine, getToken(0).sumUp(t));
		t = jj_consume_token(LBRACE);
		jab.beginCompound(t.beginLine, t.allToString());
		label_39: while (true) {
			switch (jj_nt.kind) {
			case CASE:
			case _DEFAULT:
				;
				break;
			default:
				jj_la1[104] = jj_gen;
				break label_39;
			}
			SwitchEntry();
		}
		t = jj_consume_token(RBRACE);
		jab.endCompound(t.beginLine, t.allToString());
		jab.endSwitch(t.beginLine);
	}

	final public void SwitchEntry() throws ParseException {
		log("SwitchEntry");
		Token t, t2;
		JavaASTCaseNode caseNode;
		switch (jj_nt.kind) {
		case CASE:
			t = jj_consume_token(CASE);
			Expression(false);
			t2 = jj_consume_token(COLON);
			caseNode = jab.appendCase(t.beginLine, getToken(0).sumUp(t), t2.sumUpX(t.next));
			break;
		case _DEFAULT:
			t = jj_consume_token(_DEFAULT);
			jj_consume_token(COLON);
			caseNode = jab.appendCase(t.beginLine, getToken(0).sumUp(t));
			break;
		default:
			jj_la1[105] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
		Statements();
	}

	final public void IfStatement() throws ParseException {
		log("IfStatement");
		Token t, th1, th2;
		JavaCondition cond = null;
		JavaASTIfNode ifNode;
		ASTNode elseNode = null;
		JavaASTStatementNode trueNode = null, falseNode = null, endNode;
		JavaASTJumpNode jumpNode;
		boolean virtualCompound = false;
		t = jj_consume_token(IF);
		jj_consume_token(LPAREN);
		th1 = getToken(1);
		jab.clearTerm();
		jab.enterStatement();
		cond = Expression(true);
		jab.leaveStatement();
		th2 = getToken(0);
		if (cond == null)
			cond = new JavaCondition(getToken(0).sumUp(th1));
		jj_consume_token(RPAREN);
		ifNode = jab.beginIf(t.beginLine, getToken(0).sumUp(t), cond);
		if (getToken(1).kind != LBRACE) {
			virtualCompound = true;
			// jab.appendCompound(getToken(0).beginLine);
		}
		trueNode = jab.appendStatement(getToken(0).beginLine);
		Statement();
		jumpNode = jab.appendJump(getToken(0).beginLine, "", "");
		jumpNode.isVirtual(true);
		switch (jj_nt.kind) {
		case ELSE:
			t = jj_consume_token(ELSE);
			elseNode = jab.appendNode(t.beginLine, t.allToString());
			falseNode = jab.appendStatement(getToken(0).beginLine);
			Statement();
			break;
		default:
			jj_la1[106] = jj_gen;
			;
		}
		if (elseNode == null) {
			elseNode = jab.appendNode(getToken(0).beginLine);
			falseNode = jab.appendStatement(getToken(0).beginLine);
		}

		ifNode.setTrueNode(trueNode);
		ifNode.setFalseNode(falseNode);
		ifNode.setElseNode(elseNode);
		jab.endIf(getToken(0).beginLine);
		jumpNode.setTarget(ifNode.getEndNode());
	}

	final public void WhileStatement() throws ParseException {
		log("WhileStatement");
		Token t, th1;
		JavaASTIterationNode iterationNode;
		JavaASTStatementNode loopNode;
		ASTNode entryNode, n;
		JavaCondition cond;
		ASTJumpNode jumpNode;
		t = jj_consume_token(WHILE);
		jj_consume_token(LPAREN);
		th1 = getToken(1);
		jab.clearTerm();
		jab.enterStatement();
		cond = Expression(true);
		jab.leaveStatement();
		if (cond == null)
			cond = new JavaCondition(getToken(0).sumUp(th1));
		jj_consume_token(RPAREN);
		n = jab.getLastNode();
		if (n instanceof ASTLabelNode) {
			while (n.getPrevNode() instanceof ASTLabelNode)
				n = n.getPrevNode();
			entryNode = n;
		} else
			entryNode = jab.appendNode(getToken(0).beginLine);

		iterationNode = jab.beginWhileIteration(t.beginLine, getToken(0).sumUp(t), cond);
		loopNode = jab.appendStatement(getToken(0).beginLine);
		iterationNode.setEntryNode(entryNode);
		iterationNode.setLoopNode(loopNode);
		Statement();
		jumpNode = jab.appendJump(getToken(0).beginLine, "", "");
		jumpNode.isVirtual(true);
		jumpNode.setTarget(iterationNode);
		jab.endIteration(getToken(0).beginLine);
	}

	final public void DoStatement() throws ParseException {
		log("DoStatement");
		Token t, t1;
		ASTNode entryNode;
		JavaASTStatementNode loopNode;
		JavaASTDoNode doNode;
		JavaASTIterationNode iterationNode;
		JavaCondition cond = new JavaCondition();
		t = jj_consume_token(DO);
		// entryNode ist der Knoten, der den Eingang in das Konstrukt vermerkt
		// dort wird die ID des IterationNodes geloggt!
		entryNode = jab.getLastNode();
		if (entryNode instanceof ASTLabelNode) {
			while (entryNode.getPrevNode() instanceof ASTLabelNode)
				entryNode = entryNode.getPrevNode();
		} else
			entryNode = jab.appendNode(t.beginLine);

		doNode = jab.beginDoIteration(t.beginLine, getToken(0).sumUp(t));
		loopNode = jab.appendStatement(getToken(0).beginLine);
		Statement();
		t = jj_consume_token(WHILE);
		jj_consume_token(LPAREN);
		t1 = getToken(1);
		jab.enterStatement();
		cond = Expression(true);
		jab.leaveStatement();
		if (cond == null)
			cond = new JavaCondition(getToken(0).sumUp(t1));
		jj_consume_token(RPAREN);
		jj_consume_token(SEMICOLON);
		iterationNode = jab.endDoIteration(t.beginLine, getToken(0).sumUp(t), cond, getToken(0).beginLine);
		iterationNode.setEntryNode(entryNode);
		iterationNode.setDoNode(doNode);
		iterationNode.setLoopNode(loopNode);
		doNode.setIterationNode(iterationNode);
	}

	final public void ForStatement() throws ParseException {
		log("ForStatement");
		Token t, t2, t3 = null, t4 = null;
		Token tf1 = null, tf2;
		String s, s2;
		String forInitString = "", forUpdateString = "";
		ASTNode entryNode;
		JavaASTIterationNode iterationNode;
		JavaASTStatementNode loopNode;
		JavaCondition cond = new JavaCondition();
		ASTJumpNode jumpNode;
		boolean isForEach = false;
		t = jj_consume_token(FOR);
		t2 = jj_consume_token(LPAREN);
		if (jj_2_35(2147483647)) {
			VariableDeclarationExpression();
			jj_consume_token(COLON);
			Expression(false);
			isForEach = true;
		} else {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case DOUBLE:
			case FALSE:
			case FINAL:
			case FLOAT:
			case INT:
			case LONG:
			case NATIVE:
			case NEW:
			case NULL:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case SHORT:
			case STATIC:
			case STRICTFP:
			case SUPER:
			case SYNCHRONIZED:
			case THIS:
			case TRANSIENT:
			case TRUE:
			case VOID:
			case VOLATILE:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
			case SEMICOLON:
			case AT:
			case BANG:
			case TILDE:
			case INCR:
			case DECR:
			case PLUS:
			case MINUS:
				jab.enterStatement();
				tf1 = getToken(1);
				switch (jj_nt.kind) {
				case ABSTRACT:
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case DOUBLE:
				case FALSE:
				case FINAL:
				case FLOAT:
				case INT:
				case LONG:
				case NATIVE:
				case NEW:
				case NULL:
				case PRIVATE:
				case PROTECTED:
				case PUBLIC:
				case SHORT:
				case STATIC:
				case STRICTFP:
				case SUPER:
				case SYNCHRONIZED:
				case THIS:
				case TRANSIENT:
				case TRUE:
				case VOID:
				case VOLATILE:
				case LONG_LITERAL:
				case INTEGER_LITERAL:
				case FLOATING_POINT_LITERAL:
				case CHARACTER_LITERAL:
				case STRING_LITERAL:
				case IDENTIFIER:
				case LPAREN:
				case AT:
				case BANG:
				case TILDE:
				case INCR:
				case DECR:
				case PLUS:
				case MINUS:
					ForInit();
					break;
				default:
					jj_la1[107] = jj_gen;
					;
				}
				tf2 = jj_consume_token(SEMICOLON);
				if (tf1 != tf2)
					forInitString = tf2.sumUpX(tf1);
				switch (jj_nt.kind) {
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case DOUBLE:
				case FALSE:
				case FLOAT:
				case INT:
				case LONG:
				case NEW:
				case NULL:
				case SHORT:
				case SUPER:
				case THIS:
				case TRUE:
				case VOID:
				case LONG_LITERAL:
				case INTEGER_LITERAL:
				case FLOATING_POINT_LITERAL:
				case CHARACTER_LITERAL:
				case STRING_LITERAL:
				case IDENTIFIER:
				case LPAREN:
				case BANG:
				case TILDE:
				case INCR:
				case DECR:
				case PLUS:
				case MINUS:
					t3 = getToken(1);
					cond = Expression(true);
					// Logger.logln(getToken(0).sumUp(t3));
					if (cond == null)
						cond = new JavaCondition(getToken(0).sumUp(t3));
					// cond.print();

					break;
				default:
					jj_la1[108] = jj_gen;
					;
				}
				jj_consume_token(SEMICOLON);
				tf1 = getToken(1);
				switch (jj_nt.kind) {
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case DOUBLE:
				case FALSE:
				case FLOAT:
				case INT:
				case LONG:
				case NEW:
				case NULL:
				case SHORT:
				case SUPER:
				case THIS:
				case TRUE:
				case VOID:
				case LONG_LITERAL:
				case INTEGER_LITERAL:
				case FLOATING_POINT_LITERAL:
				case CHARACTER_LITERAL:
				case STRING_LITERAL:
				case IDENTIFIER:
				case LPAREN:
				case BANG:
				case TILDE:
				case INCR:
				case DECR:
				case PLUS:
				case MINUS:
					ForUpdate();
					break;
				default:
					jj_la1[109] = jj_gen;
					;
				}
				jab.leaveStatement();
				break;
			default:
				jj_la1[110] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
		tf2 = jj_consume_token(RPAREN);
		s = getToken(0).sumUp(t);

		entryNode = jab.getLastNode();
		if (entryNode instanceof ASTLabelNode) {
			while (entryNode.getPrevNode() instanceof ASTLabelNode)
				entryNode = entryNode.getPrevNode();
		} else
			entryNode = jab.appendNode(getToken(0).beginLine);

		if (isForEach) {
			iterationNode = jab.beginForEachIteration(t.beginLine, s);
		} else {
			if (tf1 != tf2)
				forUpdateString = tf2.sumUpX(tf1);

			iterationNode = jab.beginForIteration(t.beginLine, s, forInitString, cond, forUpdateString);
		}
		loopNode = jab.appendStatement(getToken(0).beginLine);

		iterationNode.setLoopNode(loopNode);
		iterationNode.setEntryNode(entryNode);
		iterationNode.isForNode();
		Statement();
		jumpNode = jab.appendJump(getToken(0).beginLine, "", "");
		jumpNode.isVirtual(true);
		jumpNode.setTarget(iterationNode);
		jab.endIteration(getToken(0).beginLine);
	}

	final public void ForInit() throws ParseException {
		log("ForInit");
		if (jj_2_36(2147483647)) {
			VariableDeclarationExpression();
		} else {
			switch (jj_nt.kind) {
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case DOUBLE:
			case FALSE:
			case FLOAT:
			case INT:
			case LONG:
			case NEW:
			case NULL:
			case SHORT:
			case SUPER:
			case THIS:
			case TRUE:
			case VOID:
			case LONG_LITERAL:
			case INTEGER_LITERAL:
			case FLOATING_POINT_LITERAL:
			case CHARACTER_LITERAL:
			case STRING_LITERAL:
			case IDENTIFIER:
			case LPAREN:
			case BANG:
			case TILDE:
			case INCR:
			case DECR:
			case PLUS:
			case MINUS:
				ExpressionList();
				break;
			default:
				jj_la1[111] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public JavaCondition ExpressionList() throws ParseException {
		log("ExpressionList");
		JavaCondition cond1 = null, cond2;
		Token t1, t2;
		t1 = getToken(1);
		cond1 = Expression(false);
		label_40: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[112] = jj_gen;
				break label_40;
			}
			t2 = jj_consume_token(COMMA);
			cond2 = Expression(false);
			cond1 = joinConditions(t1, cond1, t2, cond2, getToken(0));
		}
		{
			if (true)
				return cond1;
		}
		throw new Error("Missing return statement in function");
	}

	final public void ForUpdate() throws ParseException {
		log("ForUpdate");
		ExpressionList();
	}

	final public void BreakStatement() throws ParseException {
		log("BreakStatement");
		Token t, t1;
		String s = "";
		t = jj_consume_token(BREAK);
		switch (jj_nt.kind) {
		case IDENTIFIER:
			t1 = jj_consume_token(IDENTIFIER);
			s = t1.image;
			break;
		default:
			jj_la1[113] = jj_gen;
			;
		}
		jj_consume_token(SEMICOLON);
		jab.appendBreak(t.beginLine, getToken(0).sumUp(t), s);
	}

	final public void ContinueStatement() throws ParseException {
		log("ContinueStatement");
		Token t, t1;
		String s = "";
		JavaASTJumpNode jumpNode;
		t = jj_consume_token(CONTINUE);
		switch (jj_nt.kind) {
		case IDENTIFIER:
			t1 = jj_consume_token(IDENTIFIER);
			s = t1.image;
			break;
		default:
			jj_la1[114] = jj_gen;
			;
		}
		jj_consume_token(SEMICOLON);
		jab.appendContinue(t.beginLine, getToken(0).sumUp(t), s);
	}

	final public void ReturnStatement() throws ParseException {
		log("ReturnStatement");
		Token t1, t2;
		Condition cond = null;
		JavaASTJumpNode jumpNode;
		t1 = jj_consume_token(RETURN);
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case BANG:
		case TILDE:
		case INCR:
		case DECR:
		case PLUS:
		case MINUS:
			jab.enterStatement();
			cond = Expression(false);
			jab.leaveStatement();
			break;
		default:
			jj_la1[115] = jj_gen;
			;
		}
		t2 = jj_consume_token(SEMICOLON);
		jab.appendReturn(t1.beginLine, getToken(0).sumUp(t1), "");
	}

	final public void ThrowStatement() throws ParseException {
		log("ThrowStatement");
		Token t1, t2;
		JavaASTJumpNode jumpNode;
		Condition cond;
		jab.enterStatement();
		t1 = jj_consume_token(THROW);
		cond = Expression(false);
		t2 = jj_consume_token(SEMICOLON);
		jab.leaveStatement();
		jab.appendThrow(t1.beginLine, getToken(0).sumUp(t1), "");
	}

	final public void SynchronizedStatement() throws ParseException {
		log("SynchronizedStatement");
		JavaASTCompoundNode cn, ecn;
		Condition cond;
		Token t1, t2, t3;
		jab.enterStatement();
		t1 = jj_consume_token(SYNCHRONIZED);
		t2 = jj_consume_token(LPAREN);
		cond = Expression(false);
		t3 = jj_consume_token(RPAREN);
		jab.leaveStatement();
		jab.appendNode(t1.beginLine, t3.sumUp(t1));
		Block(false, false);
	}

	final public void TryStatement() throws ParseException {
		log("TryStatement");
		Token t, t2, t3;
		JavaASTTryNode tryNode;
		JavaASTCatchNode catchNode;
		Stack<Object> hstack;
		ASTJumpNode jumpNode;
		ArrayList<ASTJumpNode> jumpNodeList = new ArrayList<ASTJumpNode>();
		t = jj_consume_token(TRY);
		tryNode = jab.beginTry(t.beginLine, t.allToString());
		jumpNode = Block(false, true);
		jab.setEndOfTryBlock();
		jumpNodeList.add(jumpNode);
		label_41: while (true) {
			switch (jj_nt.kind) {
			case CATCH:
				;
				break;
			default:
				jj_la1[116] = jj_gen;
				break label_41;
			}
			t = jj_consume_token(CATCH);
			t2 = jj_consume_token(LPAREN);
			FormalParameter();
			t3 = jj_consume_token(RPAREN);
			catchNode = jab.appendCatch(t.beginLine, getToken(0).sumUp(t), t3.sumUpX(t2.next));
			tryNode.addCatchNode(catchNode);
			jumpNode = CatchBlock();
			jumpNodeList.add(jumpNode);
		}
		switch (jj_nt.kind) {
		case FINALLY:
			t = jj_consume_token(FINALLY);
			catchNode = jab.appendFinally(t.beginLine, getToken(0).sumUp(t));
			tryNode.addCatchNode(catchNode);
			for (ASTJumpNode jn : jumpNodeList) {
				jn.setTarget(catchNode);
			}
			jumpNodeList.clear();
			Block(false, false);
			break;
		default:
			jj_la1[117] = jj_gen;
			;
		}
		jab.endTry(getToken(0).beginLine);
		for (ASTJumpNode jn : jumpNodeList) {
			jn.setTarget(tryNode.getEndNode());
		}
	}

	/*
	 * We use productions to match >>>, >> and > so that we can keep the type declaration syntax with generics clean
	 */
	final public void RUNSIGNEDSHIFT() throws ParseException {
		Token t1, t2, t3;
		if (getToken(1).kind == GT && ((Token.GTToken) getToken(1)).realKind == RUNSIGNEDSHIFT) {

		} else {
			jj_consume_token(-1);
			throw new ParseException();
		}
		t1 = jj_consume_token(GT);
		t2 = jj_consume_token(GT);
		t3 = jj_consume_token(GT);
	}

	final public void RSIGNEDSHIFT() throws ParseException {
		Token t1, t2;
		if (getToken(1).kind == GT && ((Token.GTToken) getToken(1)).realKind == RSIGNEDSHIFT) {

		} else {
			jj_consume_token(-1);
			throw new ParseException();
		}
		t1 = jj_consume_token(GT);
		t2 = jj_consume_token(GT);
	}

	/* Annotation syntax follows. */
	final public void Annotation() throws ParseException {
		if (jj_2_37(2147483647)) {
			NormalAnnotation();
		} else if (jj_2_38(2147483647)) {
			SingleMemberAnnotation();
		} else {
			switch (jj_nt.kind) {
			case AT:
				MarkerAnnotation();
				break;
			default:
				jj_la1[118] = jj_gen;
				jj_consume_token(-1);
				throw new ParseException();
			}
		}
	}

	final public void NormalAnnotation() throws ParseException {
		jj_consume_token(AT);
		Name();
		jj_consume_token(LPAREN);
		switch (jj_nt.kind) {
		case IDENTIFIER:
			MemberValuePairs();
			break;
		default:
			jj_la1[119] = jj_gen;
			;
		}
		jj_consume_token(RPAREN);
	}

	final public void MarkerAnnotation() throws ParseException {
		jj_consume_token(AT);
		Name();
	}

	final public void SingleMemberAnnotation() throws ParseException {
		jj_consume_token(AT);
		Name();
		jj_consume_token(LPAREN);
		MemberValue();
		jj_consume_token(RPAREN);
	}

	final public void MemberValuePairs() throws ParseException {
		MemberValuePair();
		label_42: while (true) {
			switch (jj_nt.kind) {
			case COMMA:
				;
				break;
			default:
				jj_la1[120] = jj_gen;
				break label_42;
			}
			jj_consume_token(COMMA);
			MemberValuePair();
		}
	}

	final public void MemberValuePair() throws ParseException {
		jj_consume_token(IDENTIFIER);
		jj_consume_token(ASSIGN);
		MemberValue();
	}

	final public void MemberValue() throws ParseException {
		switch (jj_nt.kind) {
		case AT:
			Annotation();
			break;
		case LBRACE:
			MemberValueArrayInitializer();
			break;
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case DOUBLE:
		case FALSE:
		case FLOAT:
		case INT:
		case LONG:
		case NEW:
		case NULL:
		case SHORT:
		case SUPER:
		case THIS:
		case TRUE:
		case VOID:
		case LONG_LITERAL:
		case INTEGER_LITERAL:
		case FLOATING_POINT_LITERAL:
		case CHARACTER_LITERAL:
		case STRING_LITERAL:
		case IDENTIFIER:
		case LPAREN:
		case BANG:
		case TILDE:
		case INCR:
		case DECR:
		case PLUS:
		case MINUS:
			ConditionalExpression(false);
			break;
		default:
			jj_la1[121] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
	}

	final public void MemberValueArrayInitializer() throws ParseException {
		jj_consume_token(LBRACE);
		MemberValue();
		label_43: while (true) {
			if (jj_2_39(2)) {
				;
			} else {
				break label_43;
			}
			jj_consume_token(COMMA);
			MemberValue();
		}
		switch (jj_nt.kind) {
		case COMMA:
			jj_consume_token(COMMA);
			break;
		default:
			jj_la1[122] = jj_gen;
			;
		}
		jj_consume_token(RBRACE);
	}

	/* Annotation Types. */
	final public void AnnotationTypeDeclaration() throws ParseException {
		jj_consume_token(AT);
		jj_consume_token(INTERFACE);
		jj_consume_token(IDENTIFIER);
		AnnotationTypeBody();
	}

	final public void AnnotationTypeBody() throws ParseException {
		jj_consume_token(LBRACE);
		label_44: while (true) {
			switch (jj_nt.kind) {
			case ABSTRACT:
			case BOOLEAN:
			case BYTE:
			case CHAR:
			case CLASS:
			case DOUBLE:
			case ENUM:
			case FINAL:
			case FLOAT:
			case INT:
			case INTERFACE:
			case LONG:
			case NATIVE:
			case PRIVATE:
			case PROTECTED:
			case PUBLIC:
			case SHORT:
			case STATIC:
			case STRICTFP:
			case SYNCHRONIZED:
			case TRANSIENT:
			case VOLATILE:
			case IDENTIFIER:
			case SEMICOLON:
			case AT:
				;
				break;
			default:
				jj_la1[123] = jj_gen;
				break label_44;
			}
			AnnotationBodyDeclaration();
		}
		jj_consume_token(RBRACE);
	}

	final public void AnnotationBodyDeclaration() throws ParseException {
		Modifiers();
		switch (jj_nt.kind) {
		case BOOLEAN:
		case BYTE:
		case CHAR:
		case CLASS:
		case DOUBLE:
		case ENUM:
		case FLOAT:
		case INT:
		case INTERFACE:
		case LONG:
		case SHORT:
		case IDENTIFIER:
		case AT:
			if (jj_2_40(2147483647)) {
				AnnotationTypeMemberDeclaration();
			} else {
				switch (jj_nt.kind) {
				case CLASS:
				case INTERFACE:
					ClassOrInterfaceDeclaration();
					break;
				case ENUM:
					EnumDeclaration();
					break;
				case AT:
					AnnotationTypeDeclaration();
					break;
				case BOOLEAN:
				case BYTE:
				case CHAR:
				case DOUBLE:
				case FLOAT:
				case INT:
				case LONG:
				case SHORT:
				case IDENTIFIER:
					FieldDeclaration();
					break;
				default:
					jj_la1[124] = jj_gen;
					jj_consume_token(-1);
					throw new ParseException();
				}
			}
			break;
		case SEMICOLON:
			jj_consume_token(SEMICOLON);
			break;
		default:
			jj_la1[125] = jj_gen;
			jj_consume_token(-1);
			throw new ParseException();
		}
	}

	final public void AnnotationTypeMemberDeclaration() throws ParseException {
		Type();
		jj_consume_token(IDENTIFIER);
		jj_consume_token(LPAREN);
		jj_consume_token(RPAREN);
		switch (jj_nt.kind) {
		case _DEFAULT:
			DefaultValue();
			break;
		default:
			jj_la1[126] = jj_gen;
			;
		}
		jj_consume_token(SEMICOLON);
	}

	final public void DefaultValue() throws ParseException {
		jj_consume_token(_DEFAULT);
		MemberValue();
	}

	final private boolean jj_2_1(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_1();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(0, xla);
		}
	}

	final private boolean jj_2_2(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_2();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(1, xla);
		}
	}

	final private boolean jj_2_3(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_3();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(2, xla);
		}
	}

	final private boolean jj_2_4(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_4();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(3, xla);
		}
	}

	final private boolean jj_2_5(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_5();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(4, xla);
		}
	}

	final private boolean jj_2_6(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_6();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(5, xla);
		}
	}

	final private boolean jj_2_7(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_7();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(6, xla);
		}
	}

	final private boolean jj_2_8(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_8();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(7, xla);
		}
	}

	final private boolean jj_2_9(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_9();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(8, xla);
		}
	}

	final private boolean jj_2_10(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_10();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(9, xla);
		}
	}

	final private boolean jj_2_11(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_11();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(10, xla);
		}
	}

	final private boolean jj_2_12(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_12();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(11, xla);
		}
	}

	final private boolean jj_2_13(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_13();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(12, xla);
		}
	}

	final private boolean jj_2_14(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_14();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(13, xla);
		}
	}

	final private boolean jj_2_15(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_15();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(14, xla);
		}
	}

	final private boolean jj_2_16(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_16();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(15, xla);
		}
	}

	final private boolean jj_2_17(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_17();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(16, xla);
		}
	}

	final private boolean jj_2_18(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_18();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(17, xla);
		}
	}

	final private boolean jj_2_19(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_19();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(18, xla);
		}
	}

	final private boolean jj_2_20(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_20();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(19, xla);
		}
	}

	final private boolean jj_2_21(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_21();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(20, xla);
		}
	}

	final private boolean jj_2_22(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_22();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(21, xla);
		}
	}

	final private boolean jj_2_23(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_23();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(22, xla);
		}
	}

	final private boolean jj_2_24(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_24();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(23, xla);
		}
	}

	final private boolean jj_2_25(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_25();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(24, xla);
		}
	}

	final private boolean jj_2_26(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_26();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(25, xla);
		}
	}

	final private boolean jj_2_27(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_27();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(26, xla);
		}
	}

	final private boolean jj_2_28(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_28();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(27, xla);
		}
	}

	final private boolean jj_2_29(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_29();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(28, xla);
		}
	}

	final private boolean jj_2_30(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_30();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(29, xla);
		}
	}

	final private boolean jj_2_31(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_31();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(30, xla);
		}
	}

	final private boolean jj_2_32(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_32();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(31, xla);
		}
	}

	final private boolean jj_2_33(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_33();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(32, xla);
		}
	}

	final private boolean jj_2_34(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_34();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(33, xla);
		}
	}

	final private boolean jj_2_35(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_35();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(34, xla);
		}
	}

	final private boolean jj_2_36(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_36();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(35, xla);
		}
	}

	final private boolean jj_2_37(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_37();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(36, xla);
		}
	}

	final private boolean jj_2_38(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_38();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(37, xla);
		}
	}

	final private boolean jj_2_39(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_39();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(38, xla);
		}
	}

	final private boolean jj_2_40(int xla) {
		jj_la = xla;
		jj_lastpos = jj_scanpos = token;
		try {
			return !jj_3_40();
		} catch (LookaheadSuccess ls) {
			return true;
		} finally {
			jj_save(39, xla);
		}
	}

	final private boolean jj_3R_49() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_97() {
		if (jj_3R_124())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_125())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_255() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_97())
			return true;
		return false;
	}

	final private boolean jj_3R_260() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(91)) {
			jj_scanpos = xsp;
			if (jj_scan_token(90))
				return true;
		}
		if (jj_3R_238())
			return true;
		return false;
	}

	final private boolean jj_3R_203() {
		if (jj_scan_token(DO))
			return true;
		if (jj_3R_169())
			return true;
		if (jj_scan_token(WHILE))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_244() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_260()) {
			jj_scanpos = xsp;
			if (jj_3R_261()) {
				jj_scanpos = xsp;
				if (jj_3R_262())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_167() {
		if (jj_3R_48())
			return true;
		if (jj_3R_97())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_255()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3_4() {
		if (jj_3R_48())
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_49()) {
				jj_scanpos = xsp;
				break;
			}
		}
		xsp = jj_scanpos;
		if (jj_scan_token(85)) {
			jj_scanpos = xsp;
			if (jj_scan_token(88)) {
				jj_scanpos = xsp;
				if (jj_scan_token(84))
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_47() {
		if (jj_3R_73())
			return true;
		return false;
	}

	final private boolean jj_3R_227() {
		if (jj_scan_token(DECR))
			return true;
		if (jj_3R_53())
			return true;
		return false;
	}

	final private boolean jj_3_3() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_47())
			jj_scanpos = xsp;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_226() {
		if (jj_scan_token(INCR))
			return true;
		if (jj_3R_53())
			return true;
		return false;
	}

	final private boolean jj_3R_155() {
		if (jj_3R_168())
			return true;
		return false;
	}

	final private boolean jj_3R_154() {
		if (jj_3R_167())
			return true;
		return false;
	}

	final private boolean jj_3R_153() {
		if (jj_3R_166())
			return true;
		return false;
	}

	final private boolean jj_3R_242() {
		if (jj_3R_244())
			return true;
		return false;
	}

	final private boolean jj_3R_152() {
		if (jj_3R_165())
			return true;
		return false;
	}

	final private boolean jj_3R_241() {
		if (jj_3R_227())
			return true;
		return false;
	}

	final private boolean jj_3R_240() {
		if (jj_3R_226())
			return true;
		return false;
	}

	final private boolean jj_3R_151() {
		if (jj_3R_164())
			return true;
		return false;
	}

	final private boolean jj_3R_150() {
		if (jj_3R_163())
			return true;
		return false;
	}

	final private boolean jj_3R_239() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(102)) {
			jj_scanpos = xsp;
			if (jj_scan_token(103))
				return true;
		}
		if (jj_3R_238())
			return true;
		return false;
	}

	final private boolean jj_3R_138() {
		if (jj_3R_67())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_150()) {
			jj_scanpos = xsp;
			if (jj_3R_151()) {
				jj_scanpos = xsp;
				if (jj_3R_152()) {
					jj_scanpos = xsp;
					if (jj_3R_153()) {
						jj_scanpos = xsp;
						if (jj_3R_154()) {
							jj_scanpos = xsp;
							if (jj_3R_155())
								return true;
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_238() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_239()) {
			jj_scanpos = xsp;
			if (jj_3R_240()) {
				jj_scanpos = xsp;
				if (jj_3R_241()) {
					jj_scanpos = xsp;
					if (jj_3R_242())
						return true;
				}
			}
		}
		return false;
	}

	final private boolean jj_3_5() {
		if (jj_3R_50())
			return true;
		return false;
	}

	final private boolean jj_3R_259() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(104)) {
			jj_scanpos = xsp;
			if (jj_scan_token(105)) {
				jj_scanpos = xsp;
				if (jj_scan_token(109))
					return true;
			}
		}
		if (jj_3R_238())
			return true;
		return false;
	}

	final private boolean jj_3R_123() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_5()) {
			jj_scanpos = xsp;
			if (jj_3R_138()) {
				jj_scanpos = xsp;
				if (jj_scan_token(84))
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_202() {
		if (jj_scan_token(WHILE))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_169())
			return true;
		return false;
	}

	final private boolean jj_3R_236() {
		if (jj_3R_238())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_259()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_96() {
		if (jj_3R_123())
			return true;
		return false;
	}

	final private boolean jj_3R_243() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(102)) {
			jj_scanpos = xsp;
			if (jj_scan_token(103))
				return true;
		}
		if (jj_3R_236())
			return true;
		return false;
	}

	final private boolean jj_3R_65() {
		if (jj_scan_token(LBRACE))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_96()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_234() {
		if (jj_3R_236())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_243()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_274() {
		if (jj_3R_65())
			return true;
		return false;
	}

	final private boolean jj_3R_130() {
		if (jj_3R_140())
			return true;
		return false;
	}

	final private boolean jj_3R_106() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_105())
			return true;
		return false;
	}

	final private boolean jj_3R_156() {
		if (jj_scan_token(BIT_AND))
			return true;
		if (jj_3R_116())
			return true;
		return false;
	}

	final private boolean jj_3_20() {
		if (jj_3R_60())
			return true;
		return false;
	}

	final private boolean jj_3_19() {
		if (jj_3R_59())
			return true;
		return false;
	}

	final private boolean jj_3R_293() {
		if (jj_scan_token(ELSE))
			return true;
		if (jj_3R_169())
			return true;
		return false;
	}

	final private boolean jj_3R_140() {
		if (jj_scan_token(EXTENDS))
			return true;
		if (jj_3R_116())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_156()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3_18() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(110)) {
			jj_scanpos = xsp;
			if (jj_3_19()) {
				jj_scanpos = xsp;
				if (jj_3_20())
					return true;
			}
		}
		if (jj_3R_234())
			return true;
		return false;
	}

	final private boolean jj_3R_105() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_130())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_273() {
		if (jj_3R_54())
			return true;
		return false;
	}

	final private boolean jj_3R_230() {
		if (jj_3R_234())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_18()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_73() {
		if (jj_scan_token(LT))
			return true;
		if (jj_3R_105())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_106()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(GT))
			return true;
		return false;
	}

	final private boolean jj_3_2() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_46())
			return true;
		return false;
	}

	final private boolean jj_3R_46() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_273())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_274())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_265() {
		if (jj_3R_123())
			return true;
		return false;
	}

	final private boolean jj_3R_237() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(89)) {
			jj_scanpos = xsp;
			if (jj_scan_token(125)) {
				jj_scanpos = xsp;
				if (jj_scan_token(95)) {
					jj_scanpos = xsp;
					if (jj_scan_token(96))
						return true;
				}
			}
		}
		if (jj_3R_230())
			return true;
		return false;
	}

	final private boolean jj_3R_201() {
		if (jj_scan_token(IF))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_169())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_293())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_250() {
		if (jj_scan_token(SEMICOLON))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_265()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_224() {
		if (jj_3R_230())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_237()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_249() {
		if (jj_3R_46())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_2()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_311() {
		if (jj_scan_token(_DEFAULT))
			return true;
		if (jj_scan_token(COLON))
			return true;
		return false;
	}

	final private boolean jj_3R_235() {
		if (jj_scan_token(INSTANCEOF))
			return true;
		if (jj_3R_48())
			return true;
		return false;
	}

	final private boolean jj_3R_310() {
		if (jj_scan_token(CASE))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(COLON))
			return true;
		return false;
	}

	final private boolean jj_3R_248() {
		if (jj_3R_264())
			return true;
		return false;
	}

	final private boolean jj_3R_303() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_310()) {
			jj_scanpos = xsp;
			if (jj_3R_311())
				return true;
		}
		if (jj_3R_107())
			return true;
		return false;
	}

	final private boolean jj_3R_216() {
		if (jj_3R_224())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_235())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_164() {
		if (jj_scan_token(ENUM))
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_248())
			jj_scanpos = xsp;
		if (jj_scan_token(LBRACE))
			return true;
		xsp = jj_scanpos;
		if (jj_3R_249())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(85))
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_250())
			jj_scanpos = xsp;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_272() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_116())
			return true;
		return false;
	}

	final private boolean jj_3R_264() {
		if (jj_scan_token(IMPLEMENTS))
			return true;
		if (jj_3R_116())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_272()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_292() {
		if (jj_3R_303())
			return true;
		return false;
	}

	final private boolean jj_3R_233() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(94)) {
			jj_scanpos = xsp;
			if (jj_scan_token(97))
				return true;
		}
		if (jj_3R_216())
			return true;
		return false;
	}

	final private boolean jj_3R_271() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_116())
			return true;
		return false;
	}

	final private boolean jj_3R_263() {
		if (jj_scan_token(EXTENDS))
			return true;
		if (jj_3R_116())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_271()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_195() {
		if (jj_3R_216())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_233()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_200() {
		if (jj_scan_token(SWITCH))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_scan_token(LBRACE))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_292()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_309() {
		if (jj_3R_57())
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_308() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(100)) {
			jj_scanpos = xsp;
			if (jj_scan_token(101))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_247() {
		if (jj_3R_264())
			return true;
		return false;
	}

	final private boolean jj_3R_302() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_308()) {
			jj_scanpos = xsp;
			if (jj_3R_309())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_246() {
		if (jj_3R_263())
			return true;
		return false;
	}

	final private boolean jj_3R_245() {
		if (jj_3R_73())
			return true;
		return false;
	}

	final private boolean jj_3R_98() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_97())
			return true;
		return false;
	}

	final private boolean jj_3R_221() {
		if (jj_3R_53())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_302())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_163() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(21)) {
			jj_scanpos = xsp;
			if (jj_scan_token(41))
				return true;
		}
		if (jj_scan_token(IDENTIFIER))
			return true;
		xsp = jj_scanpos;
		if (jj_3R_245())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_246())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_247())
			jj_scanpos = xsp;
		if (jj_3R_65())
			return true;
		return false;
	}

	final private boolean jj_3R_229() {
		if (jj_scan_token(BIT_AND))
			return true;
		if (jj_3R_195())
			return true;
		return false;
	}

	final private boolean jj_3R_220() {
		if (jj_3R_227())
			return true;
		return false;
	}

	final private boolean jj_3R_219() {
		if (jj_3R_226())
			return true;
		return false;
	}

	final private boolean jj_3R_173() {
		if (jj_3R_195())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_229()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_199() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_219()) {
			jj_scanpos = xsp;
			if (jj_3R_220()) {
				jj_scanpos = xsp;
				if (jj_3R_221())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_45() {
		if (jj_3R_72())
			return true;
		return false;
	}

	final private boolean jj_3R_198() {
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3_1() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(49)) {
			jj_scanpos = xsp;
			if (jj_scan_token(52)) {
				jj_scanpos = xsp;
				if (jj_scan_token(48)) {
					jj_scanpos = xsp;
					if (jj_scan_token(47)) {
						jj_scanpos = xsp;
						if (jj_scan_token(31)) {
							jj_scanpos = xsp;
							if (jj_scan_token(13)) {
								jj_scanpos = xsp;
								if (jj_scan_token(56)) {
									jj_scanpos = xsp;
									if (jj_scan_token(43)) {
										jj_scanpos = xsp;
										if (jj_scan_token(60)) {
											jj_scanpos = xsp;
											if (jj_scan_token(64)) {
												jj_scanpos = xsp;
												if (jj_scan_token(53)) {
													jj_scanpos = xsp;
													if (jj_3R_45())
														return true;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_67() {
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_1()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_223() {
		if (jj_scan_token(XOR))
			return true;
		if (jj_3R_173())
			return true;
		return false;
	}

	final private boolean jj_3R_68() {
		if (jj_3R_67())
			return true;
		if (jj_3R_48())
			return true;
		if (jj_3R_97())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_98()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_159() {
		if (jj_3R_169())
			return true;
		return false;
	}

	final private boolean jj_3R_162() {
		if (jj_3R_173())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_223()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3_34() {
		if (jj_3R_68())
			return true;
		return false;
	}

	final private boolean jj_3_33() {
		if (jj_3R_67())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(21)) {
			jj_scanpos = xsp;
			if (jj_scan_token(41))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_158() {
		if (jj_3R_68())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_157() {
		if (jj_3R_67())
			return true;
		if (jj_3R_163())
			return true;
		return false;
	}

	final private boolean jj_3R_215() {
		if (jj_scan_token(BIT_OR))
			return true;
		if (jj_3R_162())
			return true;
		return false;
	}

	final private boolean jj_3R_141() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_157()) {
			jj_scanpos = xsp;
			if (jj_3R_158()) {
				jj_scanpos = xsp;
				if (jj_3R_159())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_149() {
		if (jj_3R_162())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_215()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_307() {
		if (jj_scan_token(LBRACE))
			return true;
		if (jj_3R_107())
			return true;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_194() {
		if (jj_scan_token(SC_AND))
			return true;
		if (jj_3R_149())
			return true;
		return false;
	}

	final private boolean jj_3R_136() {
		if (jj_3R_149())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_194()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_290() {
		if (jj_3R_301())
			return true;
		return false;
	}

	final private boolean jj_3R_291() {
		if (jj_scan_token(COLON))
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_172() {
		if (jj_scan_token(SC_OR))
			return true;
		if (jj_3R_136())
			return true;
		return false;
	}

	final private boolean jj_3R_119() {
		if (jj_3R_136())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_172()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_76() {
		if (jj_scan_token(LBRACE))
			return true;
		if (jj_3R_107())
			return true;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_301() {
		if (jj_scan_token(_DEFAULT))
			return true;
		if (jj_3R_71())
			return true;
		return false;
	}

	final private boolean jj_3R_288() {
		if (jj_3R_48())
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_290())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_66() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(COLON))
			return true;
		if (jj_3R_169())
			return true;
		return false;
	}

	final private boolean jj_3R_161() {
		if (jj_scan_token(HOOK))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(COLON))
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3_40() {
		if (jj_3R_48())
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_287() {
		if (jj_3R_167())
			return true;
		return false;
	}

	final private boolean jj_3R_286() {
		if (jj_3R_165())
			return true;
		return false;
	}

	final private boolean jj_3R_86() {
		if (jj_3R_119())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_161())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_285() {
		if (jj_3R_164())
			return true;
		return false;
	}

	final private boolean jj_3R_284() {
		if (jj_3R_163())
			return true;
		return false;
	}

	final private boolean jj_3R_283() {
		if (jj_3R_288())
			return true;
		return false;
	}

	final private boolean jj_3R_197() {
		if (jj_scan_token(ASSERT))
			return true;
		if (jj_3R_58())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_291())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_281() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_283()) {
			jj_scanpos = xsp;
			if (jj_3R_284()) {
				jj_scanpos = xsp;
				if (jj_3R_285()) {
					jj_scanpos = xsp;
					if (jj_3R_286()) {
						jj_scanpos = xsp;
						if (jj_3R_287())
							return true;
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_193() {
		if (jj_3R_210())
			return true;
		return false;
	}

	final private boolean jj_3R_57() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(88)) {
			jj_scanpos = xsp;
			if (jj_scan_token(113)) {
				jj_scanpos = xsp;
				if (jj_scan_token(114)) {
					jj_scanpos = xsp;
					if (jj_scan_token(118)) {
						jj_scanpos = xsp;
						if (jj_scan_token(111)) {
							jj_scanpos = xsp;
							if (jj_scan_token(112)) {
								jj_scanpos = xsp;
								if (jj_scan_token(119)) {
									jj_scanpos = xsp;
									if (jj_scan_token(120)) {
										jj_scanpos = xsp;
										if (jj_scan_token(121)) {
											jj_scanpos = xsp;
											if (jj_scan_token(115)) {
												jj_scanpos = xsp;
												if (jj_scan_token(117)) {
													jj_scanpos = xsp;
													if (jj_scan_token(116))
														return true;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_192() {
		if (jj_3R_209())
			return true;
		return false;
	}

	final private boolean jj_3R_275() {
		if (jj_3R_67())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_281()) {
			jj_scanpos = xsp;
			if (jj_scan_token(84))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_191() {
		if (jj_3R_208())
			return true;
		return false;
	}

	final private boolean jj_3R_266() {
		if (jj_3R_275())
			return true;
		return false;
	}

	final private boolean jj_3R_190() {
		if (jj_3R_207())
			return true;
		return false;
	}

	final private boolean jj_3R_189() {
		if (jj_3R_206())
			return true;
		return false;
	}

	final private boolean jj_3_39() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_71())
			return true;
		return false;
	}

	final private boolean jj_3R_188() {
		if (jj_3R_205())
			return true;
		return false;
	}

	final private boolean jj_3R_251() {
		if (jj_scan_token(LBRACE))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_266()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3_17() {
		if (jj_3R_57())
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_187() {
		if (jj_3R_204())
			return true;
		return false;
	}

	final private boolean jj_3R_186() {
		if (jj_3R_203())
			return true;
		return false;
	}

	final private boolean jj_3R_58() {
		if (jj_3R_86())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_17())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_185() {
		if (jj_3R_202())
			return true;
		return false;
	}

	final private boolean jj_3R_165() {
		if (jj_scan_token(AT))
			return true;
		if (jj_scan_token(INTERFACE))
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_3R_251())
			return true;
		return false;
	}

	final private boolean jj_3R_184() {
		if (jj_3R_201())
			return true;
		return false;
	}

	final private boolean jj_3R_183() {
		if (jj_3R_200())
			return true;
		return false;
	}

	final private boolean jj_3R_182() {
		if (jj_3R_199())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_101() {
		if (jj_3R_86())
			return true;
		return false;
	}

	final private boolean jj_3R_278() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_69())
			return true;
		return false;
	}

	final private boolean jj_3R_126() {
		if (jj_scan_token(LBRACE))
			return true;
		if (jj_3R_71())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_39()) {
				jj_scanpos = xsp;
				break;
			}
		}
		xsp = jj_scanpos;
		if (jj_scan_token(85))
			jj_scanpos = xsp;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_100() {
		if (jj_3R_126())
			return true;
		return false;
	}

	final private boolean jj_3R_99() {
		if (jj_3R_72())
			return true;
		return false;
	}

	final private boolean jj_3R_218() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_217())
			return true;
		return false;
	}

	final private boolean jj_3R_268() {
		if (jj_3R_69())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_278()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_181() {
		if (jj_3R_198())
			return true;
		return false;
	}

	final private boolean jj_3R_71() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_99()) {
			jj_scanpos = xsp;
			if (jj_3R_100()) {
				jj_scanpos = xsp;
				if (jj_3R_101())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_179() {
		if (jj_3R_197())
			return true;
		return false;
	}

	final private boolean jj_3R_180() {
		if (jj_3R_76())
			return true;
		return false;
	}

	final private boolean jj_3_32() {
		if (jj_3R_66())
			return true;
		return false;
	}

	final private boolean jj_3R_217() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(ASSIGN))
			return true;
		if (jj_3R_71())
			return true;
		return false;
	}

	final private boolean jj_3_16() {
		if (jj_scan_token(DOT))
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_69() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_16()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_70() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_scan_token(ASSIGN))
			return true;
		return false;
	}

	final private boolean jj_3R_169() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_32()) {
			jj_scanpos = xsp;
			if (jj_3R_179()) {
				jj_scanpos = xsp;
				if (jj_3R_180()) {
					jj_scanpos = xsp;
					if (jj_3R_181()) {
						jj_scanpos = xsp;
						if (jj_3R_182()) {
							jj_scanpos = xsp;
							if (jj_3R_183()) {
								jj_scanpos = xsp;
								if (jj_3R_184()) {
									jj_scanpos = xsp;
									if (jj_3R_185()) {
										jj_scanpos = xsp;
										if (jj_3R_186()) {
											jj_scanpos = xsp;
											if (jj_3R_187()) {
												jj_scanpos = xsp;
												if (jj_3R_188()) {
													jj_scanpos = xsp;
													if (jj_3R_189()) {
														jj_scanpos = xsp;
														if (jj_3R_190()) {
															jj_scanpos = xsp;
															if (jj_3R_191()) {
																jj_scanpos = xsp;
																if (jj_3R_192()) {
																	jj_scanpos = xsp;
																	if (jj_3R_193())
																		return true;
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_196() {
		if (jj_3R_217())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_218()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_176() {
		if (jj_3R_196())
			return true;
		return false;
	}

	final private boolean jj_3R_95() {
		if (jj_3R_48())
			return true;
		return false;
	}

	final private boolean jj_3R_128() {
		if (jj_scan_token(AT))
			return true;
		if (jj_3R_69())
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_71())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_94() {
		if (jj_scan_token(VOID))
			return true;
		return false;
	}

	final private boolean jj_3_38() {
		if (jj_scan_token(AT))
			return true;
		if (jj_3R_69())
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_64() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_94()) {
			jj_scanpos = xsp;
			if (jj_3R_95())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_129() {
		if (jj_scan_token(AT))
			return true;
		if (jj_3R_69())
			return true;
		return false;
	}

	final private boolean jj_3R_228() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3_37() {
		if (jj_scan_token(AT))
			return true;
		if (jj_3R_69())
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_70()) {
			jj_scanpos = xsp;
			if (jj_scan_token(79))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_222() {
		Token xsp;
		if (jj_3R_228())
			return true;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_228()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_3R_108())
			return true;
		return false;
	}

	final private boolean jj_3R_127() {
		if (jj_scan_token(AT))
			return true;
		if (jj_3R_69())
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_176())
			jj_scanpos = xsp;
		if (jj_scan_token(RPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_104() {
		if (jj_3R_129())
			return true;
		return false;
	}

	final private boolean jj_3_30() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_103() {
		if (jj_3R_128())
			return true;
		return false;
	}

	final private boolean jj_3R_232() {
		if (jj_scan_token(SUPER))
			return true;
		if (jj_3R_55())
			return true;
		return false;
	}

	final private boolean jj_3_29() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_225() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_231()) {
			jj_scanpos = xsp;
			if (jj_3R_232())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_231() {
		if (jj_scan_token(EXTENDS))
			return true;
		if (jj_3R_55())
			return true;
		return false;
	}

	final private boolean jj_3R_102() {
		if (jj_3R_127())
			return true;
		return false;
	}

	final private boolean jj_3_31() {
		Token xsp;
		if (jj_3_29())
			return true;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_29()) {
				jj_scanpos = xsp;
				break;
			}
		}
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_30()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_62() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(15)) {
			jj_scanpos = xsp;
			if (jj_scan_token(20)) {
				jj_scanpos = xsp;
				if (jj_scan_token(17)) {
					jj_scanpos = xsp;
					if (jj_scan_token(51)) {
						jj_scanpos = xsp;
						if (jj_scan_token(40)) {
							jj_scanpos = xsp;
							if (jj_scan_token(42)) {
								jj_scanpos = xsp;
								if (jj_scan_token(33)) {
									jj_scanpos = xsp;
									if (jj_scan_token(26))
										return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_211() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_31()) {
			jj_scanpos = xsp;
			if (jj_3R_222())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_87() {
		return false;
	}

	final private boolean jj_3R_72() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_102()) {
			jj_scanpos = xsp;
			if (jj_3R_103()) {
				jj_scanpos = xsp;
				if (jj_3R_104())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_178() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_85())
			return true;
		return false;
	}

	final private boolean jj_3R_135() {
		if (jj_scan_token(HOOK))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_225())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3_27() {
		if (jj_3R_65())
			return true;
		return false;
	}

	final private boolean jj_3R_88() {
		return false;
	}

	final private boolean jj_3R_59() {
		Token xsp;
		xsp = jj_scanpos;
		lookingAhead = true;
		jj_semLA = getToken(1).kind == GT && ((Token.GTToken) getToken(1)).realKind == RSIGNEDSHIFT;
		lookingAhead = false;
		if (!jj_semLA || jj_3R_87())
			return true;
		if (jj_scan_token(GT))
			return true;
		if (jj_scan_token(GT))
			return true;
		return false;
	}

	final private boolean jj_3R_118() {
		if (jj_3R_135())
			return true;
		return false;
	}

	final private boolean jj_3R_117() {
		if (jj_3R_55())
			return true;
		return false;
	}

	final private boolean jj_3R_85() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_117()) {
			jj_scanpos = xsp;
			if (jj_3R_118())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_60() {
		Token xsp;
		xsp = jj_scanpos;
		lookingAhead = true;
		jj_semLA = getToken(1).kind == GT && ((Token.GTToken) getToken(1)).realKind == RUNSIGNEDSHIFT;
		lookingAhead = false;
		if (!jj_semLA || jj_3R_88())
			return true;
		if (jj_scan_token(GT))
			return true;
		if (jj_scan_token(GT))
			return true;
		if (jj_scan_token(GT))
			return true;
		return false;
	}

	final private boolean jj_3_12() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_214() {
		if (jj_3R_54())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_27())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_213() {
		if (jj_3R_211())
			return true;
		return false;
	}

	final private boolean jj_3_13() {
		if (jj_3R_56())
			return true;
		return false;
	}

	final private boolean jj_3R_56() {
		if (jj_scan_token(LT))
			return true;
		if (jj_3R_85())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_178()) {
				jj_scanpos = xsp;
				break;
			}
		}
		if (jj_scan_token(GT))
			return true;
		return false;
	}

	final private boolean jj_3_11() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_212() {
		if (jj_3R_56())
			return true;
		return false;
	}

	final private boolean jj_3_15() {
		if (jj_3R_56())
			return true;
		return false;
	}

	final private boolean jj_3_14() {
		if (jj_scan_token(DOT))
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_15())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_148() {
		if (jj_scan_token(NEW))
			return true;
		if (jj_3R_116())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_212())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_213()) {
			jj_scanpos = xsp;
			if (jj_3R_214())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_300() {
		if (jj_scan_token(FINALLY))
			return true;
		if (jj_3R_76())
			return true;
		return false;
	}

	final private boolean jj_3R_116() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_13())
			jj_scanpos = xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_14()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3_28() {
		if (jj_scan_token(NEW))
			return true;
		if (jj_3R_62())
			return true;
		if (jj_3R_211())
			return true;
		return false;
	}

	final private boolean jj_3R_84() {
		if (jj_3R_116())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_12()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_133() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_28()) {
			jj_scanpos = xsp;
			if (jj_3R_148())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_299() {
		if (jj_scan_token(CATCH))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_276())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_307())
			return true;
		return false;
	}

	final private boolean jj_3R_83() {
		if (jj_3R_62())
			return true;
		Token xsp;
		if (jj_3_11())
			return true;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_11()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_55() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_83()) {
			jj_scanpos = xsp;
			if (jj_3R_84())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_210() {
		if (jj_scan_token(TRY))
			return true;
		if (jj_3R_76())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_299()) {
				jj_scanpos = xsp;
				break;
			}
		}
		xsp = jj_scanpos;
		if (jj_3R_300())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_134() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_74() {
		if (jj_3R_62())
			return true;
		return false;
	}

	final private boolean jj_3_10() {
		if (jj_3R_55())
			return true;
		return false;
	}

	final private boolean jj_3R_48() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_10()) {
			jj_scanpos = xsp;
			if (jj_3R_74())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_115() {
		if (jj_3R_58())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_134()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_209() {
		if (jj_scan_token(SYNCHRONIZED))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_76())
			return true;
		return false;
	}

	final private boolean jj_3R_82() {
		if (jj_3R_115())
			return true;
		return false;
	}

	final private boolean jj_3R_54() {
		if (jj_scan_token(LPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_82())
			jj_scanpos = xsp;
		if (jj_scan_token(RPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_50() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_75())
			jj_scanpos = xsp;
		if (jj_3R_76())
			return true;
		return false;
	}

	final private boolean jj_3R_75() {
		if (jj_scan_token(STATIC))
			return true;
		return false;
	}

	final private boolean jj_3R_208() {
		if (jj_scan_token(THROW))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_131() {
		if (jj_3R_141())
			return true;
		return false;
	}

	final private boolean jj_3R_107() {
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_131()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3_9() {
		if (jj_scan_token(THIS))
			return true;
		if (jj_3R_54())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_171() {
		if (jj_scan_token(FALSE))
			return true;
		return false;
	}

	final private boolean jj_3_8() {
		if (jj_3R_53())
			return true;
		if (jj_scan_token(DOT))
			return true;
		return false;
	}

	final private boolean jj_3R_170() {
		if (jj_scan_token(TRUE))
			return true;
		return false;
	}

	final private boolean jj_3R_80() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_8())
			jj_scanpos = xsp;
		if (jj_scan_token(SUPER))
			return true;
		if (jj_3R_54())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_79() {
		if (jj_scan_token(THIS))
			return true;
		if (jj_3R_54())
			return true;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_298() {
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_160() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_170()) {
			jj_scanpos = xsp;
			if (jj_3R_171())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_253() {
		if (jj_scan_token(THROWS))
			return true;
		if (jj_3R_268())
			return true;
		return false;
	}

	final private boolean jj_3R_297() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_207() {
		if (jj_scan_token(RETURN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_298())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_52() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_79()) {
			jj_scanpos = xsp;
			if (jj_3R_80())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_147() {
		if (jj_3R_160())
			return true;
		return false;
	}

	final private boolean jj_3R_146() {
		if (jj_scan_token(STRING_LITERAL))
			return true;
		return false;
	}

	final private boolean jj_3R_145() {
		if (jj_scan_token(CHARACTER_LITERAL))
			return true;
		return false;
	}

	final private boolean jj_3R_144() {
		if (jj_scan_token(FLOATING_POINT_LITERAL))
			return true;
		return false;
	}

	final private boolean jj_3R_296() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_206() {
		if (jj_scan_token(CONTINUE))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_297())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3_7() {
		if (jj_3R_52())
			return true;
		return false;
	}

	final private boolean jj_3R_143() {
		if (jj_scan_token(LONG_LITERAL))
			return true;
		return false;
	}

	final private boolean jj_3R_142() {
		if (jj_scan_token(INTEGER_LITERAL))
			return true;
		return false;
	}

	final private boolean jj_3R_254() {
		if (jj_3R_52())
			return true;
		return false;
	}

	final private boolean jj_3R_205() {
		if (jj_scan_token(BREAK))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_296())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		return false;
	}

	final private boolean jj_3R_132() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_142()) {
			jj_scanpos = xsp;
			if (jj_3R_143()) {
				jj_scanpos = xsp;
				if (jj_3R_144()) {
					jj_scanpos = xsp;
					if (jj_3R_145()) {
						jj_scanpos = xsp;
						if (jj_3R_146()) {
							jj_scanpos = xsp;
							if (jj_3R_147()) {
								jj_scanpos = xsp;
								if (jj_scan_token(45))
									return true;
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_313() {
		if (jj_3R_316())
			return true;
		return false;
	}

	final private boolean jj_3R_93() {
		if (jj_3R_54())
			return true;
		return false;
	}

	final private boolean jj_3R_92() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_137() {
		if (jj_3R_56())
			return true;
		return false;
	}

	final private boolean jj_3R_166() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_174())
			jj_scanpos = xsp;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_3R_252())
			return true;
		xsp = jj_scanpos;
		if (jj_3R_253())
			jj_scanpos = xsp;
		if (jj_scan_token(LBRACE))
			return true;
		xsp = jj_scanpos;
		if (jj_3R_254())
			jj_scanpos = xsp;
		if (jj_3R_107())
			return true;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_122() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_137())
			jj_scanpos = xsp;
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_174() {
		if (jj_3R_73())
			return true;
		return false;
	}

	final private boolean jj_3R_317() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_121() {
		if (jj_3R_133())
			return true;
		return false;
	}

	final private boolean jj_3_36() {
		if (jj_3R_67())
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_316() {
		if (jj_3R_58())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_317()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_282() {
		if (jj_scan_token(ELLIPSIS))
			return true;
		return false;
	}

	final private boolean jj_3R_315() {
		if (jj_3R_316())
			return true;
		return false;
	}

	final private boolean jj_3R_276() {
		if (jj_3R_67())
			return true;
		if (jj_3R_48())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_282())
			jj_scanpos = xsp;
		if (jj_3R_124())
			return true;
		return false;
	}

	final private boolean jj_3R_91() {
		if (jj_scan_token(DOT))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(57)) {
			jj_scanpos = xsp;
			if (jj_scan_token(54)) {
				jj_scanpos = xsp;
				if (jj_3R_121()) {
					jj_scanpos = xsp;
					if (jj_3R_122())
						return true;
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_314() {
		if (jj_3R_68())
			return true;
		return false;
	}

	final private boolean jj_3R_63() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_91()) {
			jj_scanpos = xsp;
			if (jj_3R_92()) {
				jj_scanpos = xsp;
				if (jj_3R_93())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3_26() {
		if (jj_3R_64())
			return true;
		if (jj_scan_token(DOT))
			return true;
		if (jj_scan_token(CLASS))
			return true;
		return false;
	}

	final private boolean jj_3R_312() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_314()) {
			jj_scanpos = xsp;
			if (jj_3R_315())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_277() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_276())
			return true;
		return false;
	}

	final private boolean jj_3R_114() {
		if (jj_3R_69())
			return true;
		return false;
	}

	final private boolean jj_3R_267() {
		if (jj_3R_276())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_277()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_120() {
		if (jj_3R_132())
			return true;
		return false;
	}

	final private boolean jj_3R_113() {
		if (jj_3R_64())
			return true;
		if (jj_scan_token(DOT))
			return true;
		if (jj_scan_token(CLASS))
			return true;
		return false;
	}

	final private boolean jj_3R_112() {
		if (jj_3R_133())
			return true;
		return false;
	}

	final private boolean jj_3R_111() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_58())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_256() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_252() {
		if (jj_scan_token(LPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_267())
			jj_scanpos = xsp;
		if (jj_scan_token(RPAREN))
			return true;
		return false;
	}

	final private boolean jj_3R_110() {
		if (jj_scan_token(SUPER))
			return true;
		if (jj_scan_token(DOT))
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		return false;
	}

	final private boolean jj_3R_258() {
		if (jj_3R_76())
			return true;
		return false;
	}

	final private boolean jj_3R_109() {
		if (jj_3R_132())
			return true;
		return false;
	}

	final private boolean jj_3R_81() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_109()) {
			jj_scanpos = xsp;
			if (jj_scan_token(57)) {
				jj_scanpos = xsp;
				if (jj_3R_110()) {
					jj_scanpos = xsp;
					if (jj_3R_111()) {
						jj_scanpos = xsp;
						if (jj_3R_112()) {
							jj_scanpos = xsp;
							if (jj_3R_113()) {
								jj_scanpos = xsp;
								if (jj_3R_114())
									return true;
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3_6() {
		if (jj_scan_token(COMMA))
			return true;
		if (jj_3R_51())
			return true;
		return false;
	}

	final private boolean jj_3_25() {
		if (jj_3R_63())
			return true;
		return false;
	}

	final private boolean jj_3_24() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_62())
			return true;
		return false;
	}

	final private boolean jj_3R_53() {
		if (jj_3R_81())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_25()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_306() {
		if (jj_3R_313())
			return true;
		return false;
	}

	final private boolean jj_3R_257() {
		if (jj_scan_token(THROWS))
			return true;
		if (jj_3R_268())
			return true;
		return false;
	}

	final private boolean jj_3R_305() {
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_280() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_244())
			return true;
		return false;
	}

	final private boolean jj_3R_168() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_175())
			jj_scanpos = xsp;
		if (jj_3R_64())
			return true;
		if (jj_scan_token(IDENTIFIER))
			return true;
		if (jj_3R_252())
			return true;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_256()) {
				jj_scanpos = xsp;
				break;
			}
		}
		xsp = jj_scanpos;
		if (jj_3R_257())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_3R_258()) {
			jj_scanpos = xsp;
			if (jj_scan_token(84))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_175() {
		if (jj_3R_73())
			return true;
		return false;
	}

	final private boolean jj_3R_177() {
		if (jj_3R_51())
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3_6()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3_35() {
		if (jj_3R_68())
			return true;
		if (jj_scan_token(COLON))
			return true;
		return false;
	}

	final private boolean jj_3R_304() {
		if (jj_3R_312())
			return true;
		return false;
	}

	final private boolean jj_3R_279() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_238())
			return true;
		return false;
	}

	final private boolean jj_3R_295() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_304())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		xsp = jj_scanpos;
		if (jj_3R_305())
			jj_scanpos = xsp;
		if (jj_scan_token(SEMICOLON))
			return true;
		xsp = jj_scanpos;
		if (jj_3R_306())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3R_269() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_279()) {
			jj_scanpos = xsp;
			if (jj_3R_280())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_78() {
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_294() {
		if (jj_3R_68())
			return true;
		if (jj_scan_token(COLON))
			return true;
		if (jj_3R_58())
			return true;
		return false;
	}

	final private boolean jj_3R_108() {
		if (jj_scan_token(LBRACE))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_177())
			jj_scanpos = xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(85))
			jj_scanpos = xsp;
		if (jj_scan_token(RBRACE))
			return true;
		return false;
	}

	final private boolean jj_3R_204() {
		if (jj_scan_token(FOR))
			return true;
		if (jj_scan_token(LPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_294()) {
			jj_scanpos = xsp;
			if (jj_3R_295())
				return true;
		}
		if (jj_scan_token(RPAREN))
			return true;
		if (jj_3R_169())
			return true;
		return false;
	}

	final private boolean jj_3R_289() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(100)) {
			jj_scanpos = xsp;
			if (jj_scan_token(101))
				return true;
		}
		return false;
	}

	final private boolean jj_3R_270() {
		if (jj_3R_53())
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_289())
			jj_scanpos = xsp;
		return false;
	}

	final private boolean jj_3_23() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(LBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3R_77() {
		if (jj_3R_108())
			return true;
		return false;
	}

	final private boolean jj_3R_51() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3R_77()) {
			jj_scanpos = xsp;
			if (jj_3R_78())
				return true;
		}
		return false;
	}

	final private boolean jj_3R_90() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(RPAREN))
			return true;
		Token xsp;
		xsp = jj_scanpos;
		if (jj_scan_token(91)) {
			jj_scanpos = xsp;
			if (jj_scan_token(90)) {
				jj_scanpos = xsp;
				if (jj_scan_token(78)) {
					jj_scanpos = xsp;
					if (jj_scan_token(75)) {
						jj_scanpos = xsp;
						if (jj_scan_token(57)) {
							jj_scanpos = xsp;
							if (jj_scan_token(54)) {
								jj_scanpos = xsp;
								if (jj_scan_token(44)) {
									jj_scanpos = xsp;
									if (jj_3R_120())
										return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	final private boolean jj_3R_89() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_48())
			return true;
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3_21() {
		if (jj_3R_61())
			return true;
		return false;
	}

	final private boolean jj_3R_125() {
		if (jj_scan_token(ASSIGN))
			return true;
		if (jj_3R_51())
			return true;
		return false;
	}

	final private boolean jj_3R_139() {
		if (jj_scan_token(LBRACKET))
			return true;
		if (jj_scan_token(RBRACKET))
			return true;
		return false;
	}

	final private boolean jj_3_22() {
		if (jj_scan_token(LPAREN))
			return true;
		if (jj_3R_62())
			return true;
		return false;
	}

	final private boolean jj_3R_61() {
		Token xsp;
		xsp = jj_scanpos;
		if (jj_3_22()) {
			jj_scanpos = xsp;
			if (jj_3R_89()) {
				jj_scanpos = xsp;
				if (jj_3R_90())
					return true;
			}
		}
		return false;
	}

	final private boolean jj_3R_124() {
		if (jj_scan_token(IDENTIFIER))
			return true;
		Token xsp;
		while (true) {
			xsp = jj_scanpos;
			if (jj_3R_139()) {
				jj_scanpos = xsp;
				break;
			}
		}
		return false;
	}

	final private boolean jj_3R_262() {
		if (jj_3R_270())
			return true;
		return false;
	}

	final private boolean jj_3R_261() {
		if (jj_3R_269())
			return true;
		return false;
	}

	public JavaParserTokenManager token_source;
	SimpleCharStream jj_input_stream;
	public Token token, jj_nt;
	private Token jj_scanpos, jj_lastpos;
	private int jj_la;
	public boolean lookingAhead = false;
	private boolean jj_semLA;
	private int jj_gen;
	final private int[] jj_la1 = new int[127];
	static private int[] jj_la1_0;
	static private int[] jj_la1_1;
	static private int[] jj_la1_2;
	static private int[] jj_la1_3;
	static {
		jj_la1_0();
		jj_la1_1();
		jj_la1_2();
		jj_la1_3();
	}

	private static void jj_la1_0() {
		jj_la1_0 = new int[] { 0x0, 0x0, 0x90202000, 0x0, 0x0, 0x80002000, 0x10200000, 0x90202000, 0x200000, 0x0,
				0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9432a000, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x9432a000,
				0x10200000, 0x4128000, 0x9432a000, 0x0, 0x0, 0x0, 0x44128000, 0x44128000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x8412a000, 0x0, 0x0, 0x0, 0x44128000, 0xc6b3e000, 0x0, 0x4128000, 0x4128000, 0x0, 0x4128000,
				0x20000000, 0x20000000, 0x4128000, 0x4128000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44128000, 0x0, 0x0, 0x44128000, 0x40000000, 0x0, 0x0, 0x0,
				0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x40000000, 0x44128000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x4693c000, 0x0, 0x4693c000, 0x0, 0x0, 0x0, 0x0, 0x44128000, 0x1040000, 0x1040000, 0x8000000,
				0xc412a000, 0x44128000, 0x44128000, 0xc412a000, 0x44128000, 0x0, 0x0, 0x0, 0x44128000, 0x80000, 0x0,
				0x0, 0x0, 0x0, 0x44128000, 0x0, 0x9432a000, 0x14328000, 0x14328000, 0x1000000, };
	}

	private static void jj_la1_1() {
		jj_la1_1 = new int[] { 0x4000, 0x40, 0x11338a00, 0x100000, 0x0, 0x11338800, 0x200, 0x11338a00, 0x200, 0x0, 0x0,
				0x20, 0x0, 0x0, 0x20, 0x0, 0x0, 0x913b8f02, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x913b8f02, 0x200,
				0x80080502, 0x913b8f02, 0x0, 0x0, 0x0, 0xa2483502, 0xa2483502, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0,
				0x113b8d02, 0x0, 0x0, 0x8000000, 0xa2483502, 0xf7ffbf16, 0x100000, 0x80502, 0x80502, 0x0, 0x80502,
				0x400000, 0x400000, 0x80502, 0x80080502, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0,
				0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa2483502, 0x0, 0x0, 0xa2483502, 0x22403000, 0x0, 0x0, 0x0, 0x0,
				0x22403000, 0x0, 0x0, 0x2401000, 0x0, 0x20002000, 0x20000000, 0xa2483502, 0x0, 0x0, 0x0, 0x1000, 0x0,
				0x0, 0xe7cc3516, 0x0, 0xe7cc3516, 0x0, 0x0, 0x0, 0x0, 0xa2483502, 0x0, 0x0, 0x0, 0xb37bbd02,
				0xa2483502, 0xa2483502, 0xb37bbd02, 0xa2483502, 0x0, 0x0, 0x0, 0xa2483502, 0x0, 0x1, 0x0, 0x0, 0x0,
				0xa2483502, 0x0, 0x113b8f02, 0x80702, 0x80702, 0x0, };
	}

	private static void jj_la1_2() {
		jj_la1_2 = new int[] { 0x0, 0x0, 0x900001, 0x0, 0x400000, 0x800001, 0x800000, 0x900001, 0x0, 0x2000000, 0x0,
				0x0, 0x200000, 0x200000, 0x0, 0x800, 0x200000, 0x2910801, 0x100000, 0x4000, 0x10000, 0x200000, 0x0,
				0x0, 0x2910801, 0x800000, 0x2000800, 0x2900801, 0x200000, 0x1000000, 0x40000, 0xc014e8c, 0xc014e8c,
				0x200000, 0x2000000, 0x40000, 0x0, 0x110000, 0x200000, 0x800801, 0x0, 0x2000000, 0x0, 0x4e8c, 0x914e8f,
				0x0, 0x0, 0x800, 0x200000, 0x10000800, 0x0, 0x0, 0x0, 0x800, 0x200000, 0x1000000, 0x10000000, 0x0, 0x0,
				0x0, 0x0, 0x0, 0x40000000, 0x40000000, 0x0, 0x82000000, 0x82000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0xc004e8c, 0xc000000, 0xc000000, 0x4e8c, 0xc004e8c, 0x4000, 0x0, 0x0, 0x4000, 0x468c, 0x800, 0x2000000,
				0x2000800, 0x444000, 0x68c, 0x0, 0xc004e8c, 0x200000, 0x2000000, 0x44000, 0x0, 0x40000, 0x40000,
				0x114e8e, 0x20000000, 0x114e8e, 0x200000, 0x0, 0x1000000, 0x1000000, 0x4e8c, 0x0, 0x0, 0x0, 0xc804e8d,
				0xc004e8c, 0xc004e8c, 0xc904e8d, 0xc004e8c, 0x200000, 0x800, 0x800, 0xc004e8c, 0x0, 0x0, 0x800000,
				0x800, 0x200000, 0xc814e8c, 0x200000, 0x900801, 0x800800, 0x900800, 0x0, };
	}

	private static void jj_la1_3() {
		jj_la1_3 = new int[] { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf0, 0xf0, 0x0, 0x0, 0x0, 0x0,
				0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x30, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x3ff8000, 0x0, 0x4, 0x8, 0x800, 0x1000, 0x400, 0x2, 0x2, 0x0, 0x20000001, 0x20000001, 0x4000, 0xc0,
				0xc0, 0x2300, 0x2300, 0xc0, 0xf0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x30, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
				0x0, 0x0, 0xf0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x0, 0x30, 0x0, 0x30, 0x3ff8030, 0x3ff8030, 0x30,
				0x0, 0x0, 0x0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x0, 0x0, 0x0, 0xf0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf0, 0x0,
				0x0, 0x0, 0x0, 0x0, };
	}

	final private JJCalls[] jj_2_rtns = new JJCalls[40];
	private boolean jj_rescan = false;
	private int jj_gc = 0;

	public JavaParser(java.io.InputStream stream) {
		this(stream, null);
	}

	public JavaParser(java.io.InputStream stream, String encoding) {
		try {
			jj_input_stream = new SimpleCharStream(stream, encoding, 1, 1);
		} catch (java.io.UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
		token_source = new JavaParserTokenManager(jj_input_stream);
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	public void ReInit(java.io.InputStream stream) {
		ReInit(stream, null);
	}

	public void ReInit(java.io.InputStream stream, String encoding) {
		try {
			jj_input_stream.ReInit(stream, encoding, 1, 1);
		} catch (java.io.UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
		token_source.ReInit(jj_input_stream);
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	public JavaParser(java.io.Reader stream) {
		jj_input_stream = new SimpleCharStream(stream, 1, 1);
		token_source = new JavaParserTokenManager(jj_input_stream);
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	public void ReInit(java.io.Reader stream) {
		jj_input_stream.ReInit(stream, 1, 1);
		token_source.ReInit(jj_input_stream);
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	public JavaParser(JavaParserTokenManager tm) {
		token_source = tm;
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	public void ReInit(JavaParserTokenManager tm) {
		token_source = tm;
		token = new Token();
		token.next = jj_nt = token_source.getNextToken();
		jj_gen = 0;
		for (int i = 0; i < 127; i++)
			jj_la1[i] = -1;
		for (int i = 0; i < jj_2_rtns.length; i++)
			jj_2_rtns[i] = new JJCalls();
	}

	final private Token jj_consume_token(int kind) throws ParseException {
		Token oldToken = token;
		if ((token = jj_nt).next != null)
			jj_nt = jj_nt.next;
		else
			jj_nt = jj_nt.next = token_source.getNextToken();
		if (token.kind == kind) {
			jj_gen++;
			if (++jj_gc > 100) {
				jj_gc = 0;
				for (int i = 0; i < jj_2_rtns.length; i++) {
					JJCalls c = jj_2_rtns[i];
					while (c != null) {
						if (c.gen < jj_gen)
							c.first = null;
						c = c.next;
					}
				}
			}
			return token;
		}
		jj_nt = token;
		token = oldToken;
		jj_kind = kind;
		throw generateParseException();
	}

	static private final class LookaheadSuccess extends java.lang.Error {
	}

	final private LookaheadSuccess jj_ls = new LookaheadSuccess();

	final private boolean jj_scan_token(int kind) {
		if (jj_scanpos == jj_lastpos) {
			jj_la--;
			if (jj_scanpos.next == null) {
				jj_lastpos = jj_scanpos = jj_scanpos.next = token_source.getNextToken();
			} else {
				jj_lastpos = jj_scanpos = jj_scanpos.next;
			}
		} else {
			jj_scanpos = jj_scanpos.next;
		}
		if (jj_rescan) {
			int i = 0;
			Token tok = token;
			while (tok != null && tok != jj_scanpos) {
				i++;
				tok = tok.next;
			}
			if (tok != null)
				jj_add_error_token(kind, i);
		}
		if (jj_scanpos.kind != kind)
			return true;
		if (jj_la == 0 && jj_scanpos == jj_lastpos)
			throw jj_ls;
		return false;
	}

	final public Token getNextToken() {
		if ((token = jj_nt).next != null)
			jj_nt = jj_nt.next;
		else
			jj_nt = jj_nt.next = token_source.getNextToken();
		jj_gen++;
		return token;
	}

	final public Token getToken(int index) {
		Token t = lookingAhead ? jj_scanpos : token;
		for (int i = 0; i < index; i++) {
			if (t.next != null)
				t = t.next;
			else
				t = t.next = token_source.getNextToken();
		}
		return t;
	}

	private java.util.Vector<int[]> jj_expentries = new java.util.Vector<int[]>();
	private int[] jj_expentry;
	private int jj_kind = -1;
	private int[] jj_lasttokens = new int[100];
	private int jj_endpos;

	private void jj_add_error_token(int kind, int pos) {
		if (pos >= 100)
			return;
		if (pos == jj_endpos + 1) {
			jj_lasttokens[jj_endpos++] = kind;
		} else if (jj_endpos != 0) {
			jj_expentry = new int[jj_endpos];
			for (int i = 0; i < jj_endpos; i++) {
				jj_expentry[i] = jj_lasttokens[i];
			}
			boolean exists = false;
			for (java.util.Enumeration e = jj_expentries.elements(); e.hasMoreElements();) {
				int[] oldentry = (int[]) (e.nextElement());
				if (oldentry.length == jj_expentry.length) {
					exists = true;
					for (int i = 0; i < jj_expentry.length; i++) {
						if (oldentry[i] != jj_expentry[i]) {
							exists = false;
							break;
						}
					}
					if (exists)
						break;
				}
			}
			if (!exists)
				jj_expentries.addElement(jj_expentry);
			if (pos != 0)
				jj_lasttokens[(jj_endpos = pos) - 1] = kind;
		}
	}

	public ParseException generateParseException() {
		jj_expentries.removeAllElements();
		boolean[] la1tokens = new boolean[126];
		for (int i = 0; i < 126; i++) {
			la1tokens[i] = false;
		}
		if (jj_kind >= 0) {
			la1tokens[jj_kind] = true;
			jj_kind = -1;
		}
		for (int i = 0; i < 127; i++) {
			if (jj_la1[i] == jj_gen) {
				for (int j = 0; j < 32; j++) {
					if ((jj_la1_0[i] & (1 << j)) != 0) {
						la1tokens[j] = true;
					}
					if ((jj_la1_1[i] & (1 << j)) != 0) {
						la1tokens[32 + j] = true;
					}
					if ((jj_la1_2[i] & (1 << j)) != 0) {
						la1tokens[64 + j] = true;
					}
					if ((jj_la1_3[i] & (1 << j)) != 0) {
						la1tokens[96 + j] = true;
					}
				}
			}
		}
		for (int i = 0; i < 126; i++) {
			if (la1tokens[i]) {
				jj_expentry = new int[1];
				jj_expentry[0] = i;
				jj_expentries.addElement(jj_expentry);
			}
		}
		jj_endpos = 0;
		jj_rescan_token();
		jj_add_error_token(0, 0);
		int[][] exptokseq = new int[jj_expentries.size()][];
		for (int i = 0; i < jj_expentries.size(); i++) {
			exptokseq[i] = (int[]) jj_expentries.elementAt(i);
		}
		return new ParseException(token, exptokseq, tokenImage);
	}

	final public void enable_tracing() {
	}

	final public void disable_tracing() {
	}

	final private void jj_rescan_token() {
		jj_rescan = true;
		for (int i = 0; i < 40; i++) {
			try {
				JJCalls p = jj_2_rtns[i];
				do {
					if (p.gen > jj_gen) {
						jj_la = p.arg;
						jj_lastpos = jj_scanpos = p.first;
						switch (i) {
						case 0:
							jj_3_1();
							break;
						case 1:
							jj_3_2();
							break;
						case 2:
							jj_3_3();
							break;
						case 3:
							jj_3_4();
							break;
						case 4:
							jj_3_5();
							break;
						case 5:
							jj_3_6();
							break;
						case 6:
							jj_3_7();
							break;
						case 7:
							jj_3_8();
							break;
						case 8:
							jj_3_9();
							break;
						case 9:
							jj_3_10();
							break;
						case 10:
							jj_3_11();
							break;
						case 11:
							jj_3_12();
							break;
						case 12:
							jj_3_13();
							break;
						case 13:
							jj_3_14();
							break;
						case 14:
							jj_3_15();
							break;
						case 15:
							jj_3_16();
							break;
						case 16:
							jj_3_17();
							break;
						case 17:
							jj_3_18();
							break;
						case 18:
							jj_3_19();
							break;
						case 19:
							jj_3_20();
							break;
						case 20:
							jj_3_21();
							break;
						case 21:
							jj_3_22();
							break;
						case 22:
							jj_3_23();
							break;
						case 23:
							jj_3_24();
							break;
						case 24:
							jj_3_25();
							break;
						case 25:
							jj_3_26();
							break;
						case 26:
							jj_3_27();
							break;
						case 27:
							jj_3_28();
							break;
						case 28:
							jj_3_29();
							break;
						case 29:
							jj_3_30();
							break;
						case 30:
							jj_3_31();
							break;
						case 31:
							jj_3_32();
							break;
						case 32:
							jj_3_33();
							break;
						case 33:
							jj_3_34();
							break;
						case 34:
							jj_3_35();
							break;
						case 35:
							jj_3_36();
							break;
						case 36:
							jj_3_37();
							break;
						case 37:
							jj_3_38();
							break;
						case 38:
							jj_3_39();
							break;
						case 39:
							jj_3_40();
							break;
						}
					}
					p = p.next;
				} while (p != null);
			} catch (LookaheadSuccess ls) {
			}
		}
		jj_rescan = false;
	}

	final private void jj_save(int index, int xla) {
		JJCalls p = jj_2_rtns[index];
		while (p.gen > jj_gen) {
			if (p.next == null) {
				p = p.next = new JJCalls();
				break;
			}
			p = p.next;
		}
		p.gen = jj_gen + xla - jj_la;
		p.first = token;
		p.arg = xla;
	}

	static final class JJCalls {
		int gen;
		Token first;
		int arg;
		JJCalls next;
	}

}
